@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -29,15 +29,15 @@ typedef enum
 	INDR_REF,	/* 11 */
 	MLAB_REF,	/* 12 */
 	ILIT_REF,	/* 13 */
-	CDLT_REF,	/* 14 */
-	TEMP_REF,	/* 15 */
+	CDLT_REF,	/* 14 - apparently no longer used */
+	TEMP_REF,	/* 15 - apparently no longer maintained */
 	MFUN_REF,	/* 16 */
 	MNXL_REF,	/* 17 refer to internalentry of child line */
 	TSIZ_REF,	/* 18 ilit refering to size of given triple codegen */
 	OCNT_REF,	/* 19 Offset from Call to Next Triple */
 	CDIDX_REF	/* 20 Denotes index into a table containing a code address */
 } operclass;
-#define VALUED_REF_TYPES 6	/* Types 0-6 are specific types used by alloc_reg() in array references
+#define VALUED_REF_TYPES 6	/* Types 0-5 are specific types used by alloc_reg() in array references
 				 * **** WARNING **** Do NOT reorder
 				 */
 
@@ -68,6 +68,7 @@ typedef struct	mlinestruct
 	struct	tripletype	*externalentry;
 	uint4			line_number;	/* ...operation on this line */
 	boolean_t		table;		/* put in table or not */
+	boolean_t		block_ok;	/* saw argumentless DO or not */
 } mline;
 
 typedef struct	mlabstruct
@@ -88,6 +89,7 @@ typedef struct	mliteralstruct
 					*bl;
 	}			que;
 	INTPTR_T       		rt_addr;
+	int			reference_count;	/* Used in the hash table to track references */
 	mval			v;
 } mliteral;
 
@@ -152,7 +154,7 @@ typedef struct	tripletype
 
 typedef struct
 {
-	unsigned short		octype;
+	uint4		octype;
 } octabstruct;
 
 /* Values for octype */
@@ -167,6 +169,10 @@ typedef struct
 #define OCT_EXPRLEAF	64
 #define OCT_CGSKIP	128
 #define OCT_COERCE	256
+#define OCT_ARITH	512
+#define OCT_UNARY	1024
+#define OCT_NEGATED	2048
+#define OCT_REL	4096
 
 typedef struct
 {
@@ -471,7 +477,7 @@ typedef struct
 }
 
 #define GOOD_FOR  FALSE	/* single level */
-#define BLOWN_FOR TRUE	/* all levels */
+#define BLOWN_FOR (!TREF(xecute_literal_parse))	/* all levels, except only one for our funky friend xecute_literal_parse */
 
 /* Marco to decrement or clear the for_stack, clear the for_temp array
  * and generate code to release run-time malloc'd mvals anchored in the for_saved_indx array
@@ -529,6 +535,93 @@ typedef struct
 	UNICODE_ONLY((mvalptr)->utfcgr_indx = 0xff);	\
 }
 
+/* Macro to put a literal truth value as an operand */
+#define PUT_LITERAL_TRUTH(TV, TRIP_REF)									\
+MBSTART {												\
+	LITREF mval		literal_zero, literal_one;						\
+													\
+	mval	*V;											\
+													\
+	V = (mval *)mcalloc(SIZEOF(mval));								\
+	*V = (TV) ? literal_one : literal_zero;								\
+	assert((1 == literal_one.str.len) && (1 == literal_zero.str.len));				\
+	ENSURE_STP_FREE_SPACE(1);									\
+	*(char *)stringpool.free = *V->str.addr;							\
+	V->str.addr = (char *)stringpool.free;								\
+	stringpool.free += 1;										\
+	put_lit_s(V, TRIP_REF);										\
+} MBEND
+
+/* Macro to decide whether an invocation of unary tail has any promise */
+#define UNARY_TAIL(OPR)											\
+MBSTART {												\
+	assert(TRIP_REF == (OPR)->oprclass);								\
+	if (OCT_UNARY & oc_tab[(OPR)->oprval.tref->opcode].octype)					\
+		unary_tail(OPR);									\
+} MBEND
+
+/* the following structure and macros save and restore parsing state and as of this writing are only used by m_xecute */
+typedef struct
+{
+	boolean_t	source_error_found;
+	int		source_column;
+	int		director_ident_len;
+	int4		block_level;
+	mstr_len_t	source_len;
+	mval		director_mval;
+	unsigned char	ident_buffer[(MAX_MIDENT_LEN * 2) + 1];
+	char		director_token;
+	char		*lexical_ptr;
+	char		window_token;
+} parse_save_block;
+
+#define SAVE_PARSE_STATE(SAVE_PARSE_PTR)									\
+MBSTART {													\
+	SAVE_PARSE_PTR->block_level = TREF(block_level);							\
+	SAVE_PARSE_PTR->director_ident_len = (TREF(director_ident)).len;					\
+	memcpy(SAVE_PARSE_PTR->ident_buffer, (TREF(director_ident)).addr, SAVE_PARSE_PTR->director_ident_len);	\
+	SAVE_PARSE_PTR->director_mval = TREF(director_mval);							\
+	SAVE_PARSE_PTR->director_token = TREF(director_token);							\
+	SAVE_PARSE_PTR->lexical_ptr = TREF(lexical_ptr);							\
+	SAVE_PARSE_PTR->source_column = source_column;								\
+	SAVE_PARSE_PTR->source_error_found = TREF(source_error_found);						\
+	SAVE_PARSE_PTR->source_len = (TREF(source_buffer)).len;							\
+	SAVE_PARSE_PTR->window_token = TREF(window_token);							\
+} MBEND
+
+#define RESTORE_PARSE_STATE(SAVE_PARSE_PTR)								\
+MBSTART {													\
+	TREF(block_level) = SAVE_PARSE_PTR->block_level;							\
+	(TREF(director_ident)).len = SAVE_PARSE_PTR->director_ident_len;					\
+	memcpy((TREF(director_ident)).addr, SAVE_PARSE_PTR->ident_buffer, SAVE_PARSE_PTR->director_ident_len);	\
+	TREF(director_mval) = SAVE_PARSE_PTR->director_mval;							\
+	TREF(director_token) = SAVE_PARSE_PTR->director_token;							\
+	TREF(lexical_ptr) = SAVE_PARSE_PTR->lexical_ptr;							\
+	(TREF(source_buffer)).addr = (char *)&aligned_source_buffer;						\
+	(TREF(source_buffer)).len = SAVE_PARSE_PTR->source_len;							\
+	source_column = SAVE_PARSE_PTR->source_column;								\
+	TREF(source_error_found) = SAVE_PARSE_PTR->source_error_found;						\
+	TREF(window_token) = SAVE_PARSE_PTR->window_token;							\
+} MBEND
+
+#define RETURN_IF_RTS_ERROR							\
+MBSTART {									\
+	if (TREF(rts_error_in_parse))						\
+		return;								\
+} MBEND
+
+#define RETURN_EXPR_IF_RTS_ERROR						\
+MBSTART {									\
+	if (TREF(rts_error_in_parse))						\
+	{									\
+		TREF(rts_error_in_parse) = FALSE;				\
+		DECREMENT_EXPR_DEPTH;						\
+		return EXPR_FAIL;						\
+	}									\
+} MBEND
+
+#define ALREADY_RTERROR (OC_RTERROR == (TREF(curtchain))->exorder.bl->exorder.bl->exorder.bl->opcode)
+
 /* Autorelink enabled platforms pass a different argument to glue code when calling a non-local M
  * routine. Use put_cdlt() to pass addresses of the items and use put_cdidx() when passing an ofset
  * into the linkage table where the items reside. Also macroize the opcode to use.
@@ -544,7 +637,8 @@ typedef struct
 #endif
 
 int		actuallist(oprtype *opr);
-int		bool_expr(boolean_t op, oprtype *addr);
+int		bool_expr(boolean_t sense, oprtype *addr);
+void		bx_boollit(triple *t);
 void		bx_boolop(triple *t, boolean_t jmp_type_one, boolean_t jmp_to_next, boolean_t sense, oprtype *addr);
 void		bx_relop(triple *t, opctype cmp, opctype tst, oprtype *addr);
 void		bx_tail(triple *t, boolean_t sense, oprtype *addr);
@@ -600,8 +694,10 @@ int		f_two_mstrs(oprtype *a, opctype op);
 int		f_two_mval(oprtype *a, opctype op);
 int		f_view(oprtype *a, opctype op);
 int		f_zahandle(oprtype *a, opctype op);
+int		f_zatransform(oprtype *a, opctype op);
 int		f_zcall(oprtype *a, opctype op);
 int		f_zchar(oprtype *a, opctype op);
+int		f_zcollate(oprtype *a, opctype op);
 int		f_zconvert(oprtype *a, opctype op);
 int		f_zdate(oprtype *a, opctype op);
 int		f_zdebug(oprtype *a, opctype op);
@@ -662,9 +758,11 @@ oprtype		put_str(char *pt, mstr_len_t n);
 oprtype		put_tjmp(triple *x);
 oprtype		put_tnxt(triple *x);
 oprtype		put_tref(triple *x);
+boolean_t	resolve_optimize(triple *curtrip);
 int		resolve_ref(int errknt);
 void		resolve_tref(triple *, oprtype *);
 triple		*setcurtchain(triple *x);
+void		unary_tail(oprtype *opr);
 /* VMS uses same code generator as USHBIN so treat as USHBIN for these compiler routines */
 #		if defined(USHBIN_SUPPORTED) || defined(VMS)
 void		shrink_trips(void);
@@ -676,6 +774,7 @@ void		start_fetches(opctype op);
 void		start_for_fetches(void);
 void		tnxtarg(oprtype *a);
 void		tripinit(void);
+boolean_t	unuse_literal(mval *x);
 void		walktree(mvar *n,void (*f)(),char *arg);
 void		wrtcatopt(triple *r, triple ***lpx, triple **lptop);
 int		zlcompile(unsigned char len, unsigned char *addr);		/***type int added***/
