@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -38,7 +38,6 @@ GBLREF short int source_line;
 GBLREF int	source_column;
 
 GBLREF unsigned char		source_file_name[];
-GBLREF unsigned char 		*source_buffer;
 GBLREF src_line_struct 		src_head;
 GBLREF triple			t_orig, *curr_fetch_trip, *curr_fetch_opr;
 GBLREF int4			curr_fetch_count;
@@ -119,23 +118,23 @@ boolean_t compiler_startup(void)
 	{
 		if (-1 == (n = read_source_file()))
 			break;
-		rtn_src_chksum_line(&checksum_ctx, source_buffer, n);
+		rtn_src_chksum_line(&checksum_ctx, (TREF(source_buffer)).addr, n);
 		/* Save the source lines; a check later determines whether to include them in the object file */
 		/* Accumulate list of M source lines */
 		sl = (src_line_struct *)mcalloc(SIZEOF(src_line_struct));
 		dqrins(&src_head, que, sl);
-		sl->str.addr = mcalloc(n + 1);	/* +1 for zero termination */
+		sl->str.addr = mcalloc(n + 1);		/* +1 for zero termination */
 		sl->str.len = n;
 		sl->line = source_line;
-		memcpy(sl->str.addr, source_buffer, n + 1);
+		memcpy(sl->str.addr, (TREF(source_buffer)).addr, n + 1);
 		total_source_len += n;
+		cp = (unsigned char *)((TREF(source_buffer)).addr + n - 1);
+		NEWLINE_TO_NULL(*cp); /* avoid SPOREOL errors due to trailing newlines */
 		if (need_source_lines && creating_list_file && !(cmd_qlf.qlf & CQ_MACHINE_CODE))
 		{	/* list now. for machine_code we intersperse machine code and M code, thus can't list M code yet */
-			NEWLINE_TO_NULL(source_buffer[n - 1]);
 			list_line_number();
-			list_line((char *)source_buffer);
+			list_line((TREF(source_buffer)).addr);
 		}
-		NEWLINE_TO_NULL(source_buffer[n - 1]); /* compiler doesn't like trailing newlines (gives SPOREOL errors) */
 		TREF(source_error_found) = 0;
 		lb_init();
 		if (cmd_qlf.qlf & CQ_CE_PREPROCESS)
