@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -128,8 +128,12 @@
 #include "gv_trigger.h"
 #include "gtm_trigger.h"
 #endif
+#ifdef DEBUG
+#include "wcs_wt.h"
+#endif
 #define DEFAULT_ZERROR_STR	"Unprocessed $ZERROR, see $ZSTATUS"
 #define DEFAULT_ZERROR_LEN	(SIZEOF(DEFAULT_ZERROR_STR) - 1)
+#include "gtm_libaio.h"
 
 GBLDEF	gd_region		*db_init_region;
 GBLDEF	sgmnt_data_ptr_t	cs_data;
@@ -187,7 +191,6 @@ GBLDEF	VSIG_ATOMIC_T	forced_exit;		/* Asynchronous signal/interrupt handler sets
 						 */
 GBLDEF	intrpt_state_t	intrpt_ok_state = INTRPT_OK_TO_INTERRUPT;	/* any other value implies it is not ok to interrupt */
 GBLDEF	unsigned char	*msp,
-			*mubbuf,
 			*restart_ctxt,
 			*stackbase,
 			*stacktop,
@@ -215,13 +218,10 @@ GBLDEF MSTR_CONST(default_sysid, "gtm_sysid");
 GBLDEF	mval		dollar_zgbldir,
 			dollar_zsource = DEFINE_MVAL_STRING(MV_STR, 0, 0, 0, NULL, 0, 0),
 			dollar_zstatus,
-			dollar_zstep = DEFINE_MVAL_STRING(MV_STR | MV_NM | MV_INT | MV_NUM_APPROX, 0, 0, 1, "B", 0, 0),
-			dollar_ztrap,
 			ztrap_pop2level = DEFINE_MVAL_STRING(MV_NM | MV_INT, 0, 0, 0, 0, 0, 0),
 			zstep_action,
 			dollar_system,
 			dollar_estack_delta = DEFINE_MVAL_STRING(MV_STR, 0, 0, 0, NULL, 0, 0),
-			dollar_etrap,
 			dollar_zerror = DEFINE_MVAL_STRING(MV_STR, 0, 0, DEFAULT_ZERROR_LEN, DEFAULT_ZERROR_STR, 0, 0),
 			dollar_zyerror,
 			dollar_ztexit = DEFINE_MVAL_STRING(MV_STR, 0, 0, 0, NULL, 0, 0);
@@ -281,7 +281,6 @@ GBLDEF	boolean_t	block_saved;
 GBLDEF	gtm_chset_t	dse_over_chset = CHSET_M;
 LITDEF	MIDENT_DEF(zero_ident, 0, NULL);		/* the null mident */
 GBLDEF	int4		aligned_source_buffer[MAX_SRCLINE / SIZEOF(int4) + 1];
-GBLDEF	unsigned char	*source_buffer = (unsigned char *)aligned_source_buffer;
 GBLDEF	src_line_struct	src_head;
 GBLDEF	short int	source_line;
 GBLDEF	int		source_column;
@@ -301,8 +300,7 @@ GBLDEF	char		**cmd_arg;
 #ifdef __osf__
 #pragma pointer_size (restore)
 #endif
-GBLDEF	volatile uint4	heartbeat_counter;
-GBLDEF	boolean_t	heartbeat_started;
+GBLDEF	boolean_t	oldjnlclose_started;
 /* DEFERRED EVENTS */
 GBLDEF	bool		licensed = TRUE;
 
@@ -317,9 +315,13 @@ GBLDEF	void			(*tp_timeout_start_timer_ptr)(int4 tmout_sec) = tp_start_timer_dum
 GBLDEF	void			(*tp_timeout_clear_ptr)(void) = tp_clear_timeout_dummy;
 GBLDEF	void			(*tp_timeout_action_ptr)(void) = tp_timeout_action_dummy;
 GBLDEF	void			(*ctrlc_handler_ptr)() = ctrlc_handler_dummy;
-GBLDEF	int			(*op_open_ptr)(mval *v, mval *p, int t, mval *mspace) = op_open_dummy;
+GBLDEF	int			(*op_open_ptr)(mval *v, mval *p, mval *t, mval *mspace) = op_open_dummy;
 GBLDEF	void			(*unw_prof_frame_ptr)(void) = unw_prof_frame_dummy;
-GBLDEF	void			(*heartbeat_timer_ptr)(void);	/* Initialized only in gtm_startup() */
+/* Initialized only in gtm_startup() */
+GBLDEF	void			(*jnl_file_close_timer_ptr)(void);
+GBLDEF	void			(*fake_enospc_ptr)(void);
+GBLDEF	void			(*simple_timeout_timer_ptr)(TID tid, int4 hd_len, boolean_t **timedout);
+
 #ifdef UNICODE_SUPPORTED
 GBLDEF	u_casemap_t 		gtm_strToTitle_ptr;		/* Function pointer for gtm_strToTitle */
 #endif
@@ -360,15 +362,13 @@ GBLDEF	int			gv_fillfactor = 100,
 GBLDEF	uint4			update_array_size,
 				cumul_update_array_size;	/* the current total size of the update array */
 GBLDEF	kill_set		*kill_set_tail;
-GBLDEF	boolean_t		pool_init;
+GBLDEF	int			pool_init;
 GBLDEF	boolean_t		is_src_server;
 GBLDEF	boolean_t		is_rcvr_server;
 GBLDEF	jnl_format_buffer	*non_tp_jfb_ptr;
 GBLDEF	boolean_t		dse_running;
-GBLDEF	jnlpool_addrs		jnlpool;
-GBLDEF	jnlpool_ctl_ptr_t	jnlpool_ctl;
-GBLDEF	sm_uc_ptr_t		jnldata_base;
-GBLDEF	int4			jnlpool_shmid = INVALID_SHMID;
+GBLDEF	jnlpool_addrs_ptr_t	jnlpool;
+GBLDEF	jnlpool_addrs_ptr_t	jnlpool_head;
 GBLDEF	recvpool_addrs		recvpool;
 GBLDEF	int			recvpool_shmid = INVALID_SHMID;
 GBLDEF	int			gtmsource_srv_count;
@@ -395,7 +395,6 @@ GBLDEF	char			*cli_err_str_ptr;
 GBLDEF	boolean_t		gtm_pipe_child;
 GBLDEF	io_desc			*gtm_err_dev;
 /* this array is indexed by file descriptor */
-GBLDEF	boolean_t		*lseekIoInProgress_flags;
 /* Latch variable for Unix implementations. Used in SUN and HP */
 GBLDEF	global_latch_t		defer_latch;
 GBLDEF	int			num_additional_processors;
@@ -415,7 +414,6 @@ GBLDEF	sgmnt_addrs		*kip_csa;
 GBLDEF	boolean_t		need_kip_incr;
 GBLDEF	int			merge_args;
 GBLDEF	merge_glvn_ptr		mglvnp;
-GBLDEF	int			ztrap_form;
 GBLDEF	boolean_t		ztrap_new;
 GBLDEF	int4			wtfini_in_prog;
 #ifdef DEBUG
@@ -713,7 +711,7 @@ GBLDEF	boolean_t	gvdupsetnoop = TRUE;	/* if TRUE, duplicate SETs do not change G
 						 * incremented and logical SET journal records will be written. By default, this
 						 * behavior is turned ON. GT.M has a way of turning it off with a VIEW command.
 						 */
-GBLDEF boolean_t	gtm_fullblockwrites;	/* Do full (not partial) database block writes T/F */
+GBLDEF  int4		gtm_fullblockwrites;	/* Do full (not partial) 1. file system block writes, or 2. database block writes */
 GBLDEF	volatile boolean_t	in_wcs_recover;	/* TRUE if in "wcs_recover", used by "bt_put" and "generic_exit_handler" */
 GBLDEF	boolean_t	in_gvcst_incr;		/* set to TRUE by gvcst_incr, set to FALSE by gvcst_put
 						 * distinguishes to gvcst_put, if the current db operation is a SET or $INCR */
@@ -734,7 +732,6 @@ GBLDEF	mvax		*mvaxtab,*mvaxtab_end;
 GBLDEF	mlabel		*mlabtab;
 GBLDEF	mline		mline_root;
 GBLDEF	mline		*mline_tail;
-GBLDEF	short int	block_level;
 GBLDEF	triple		t_orig;
 GBLDEF	int		mvmax, mlmax, mlitmax;
 static	char		routine_name_buff[SIZEOF(mident_fixed)], module_name_buff[SIZEOF(mident_fixed)];
@@ -857,6 +854,10 @@ GBLDEF	void		(*cache_table_relobjs)(void);   /* Function pointer to call cache_t
 GBLDEF	ch_ret_type	(*ht_rhash_ch)();		/* Function pointer to hashtab_rehash_ch */
 GBLDEF	ch_ret_type	(*jbxm_dump_ch)();		/* Function pointer to jobexam_dump_ch */
 GBLDEF	ch_ret_type	(*stpgc_ch)();			/* Function pointer to stp_gcol_ch */
+#ifdef DEBUG
+GBLDEF	ch_ret_type	(*t_ch_fnptr)();		/* Function pointer to t_ch */
+GBLDEF	ch_ret_type	(*dbinit_ch_fnptr)();		/* Function pointer to dbinit_ch */
+#endif
 GBLDEF	cache_rec_ptr_t	pin_fail_cr;			/* Pointer to the cache-record that we failed while pinning */
 GBLDEF	cache_rec	pin_fail_cr_contents;		/* Contents of the cache-record that we failed while pinning */
 GBLDEF	cache_rec_ptr_t	pin_fail_twin_cr;		/* Pointer to twin of the cache-record that we failed to pin */
@@ -900,6 +901,11 @@ GBLDEF	sigset_t	blockalrm;
 GBLDEF	sigset_t	block_ttinout;
 GBLDEF	sigset_t	block_sigsent;	/* block all signals that can be sent externally
 					  (SIGINT, SIGQUIT, SIGTERM, SIGTSTP, SIGCONT) */
+GBLDEF  sigset_t	block_worker;	/* block all signals for use by the linux AIO worker thread, except for a few
+					 * fatal signals that can be internally generated inside the thread. This way
+					 * any signal externally sent always gets handled by the main process and not
+					 * by the worker thread.
+					 */
 GBLDEF  char            *gtm_core_file;
 GBLDEF  char            *gtm_core_putenv;
 #ifdef __MVS__
@@ -931,7 +937,7 @@ GBLDEF	boolean_t	stringpool_unexpandable;	/* Set to TRUE by any function for a s
 							 * collections or expansions.
 							 */
 GBLDEF	boolean_t	donot_INVOKE_MUMTSTART;		/* Set to TRUE whenever an implicit TSTART is done in gvcst_put/kill as
-							 * part of an explicit + trigger update. In this situation, we dont expect
+							 * part of an explicit + trigger update. In this situation, we don't expect
 							 * MUM_TSTART macro to be invoked at all (see skip_INVOKE_RESTART below
 							 * for description on why this is needed). So we keep this debug-only
 							 * flag turned on throughout the gvcst_put/kill. An assert in
@@ -962,14 +968,13 @@ GBLDEF	mval		*dollar_ztdata,
 			*dollar_ztupdate,
 			*dollar_ztvalue,
 			dollar_ztwormhole = DEFINE_MVAL_STRING(MV_STR, 0, 0, 0, NULL, 0, 0),
-			dollar_ztslate = DEFINE_MVAL_STRING(MV_STR, 0, 0, 0, NULL, 0, 0),
-			gtm_trigger_etrap;  		/* Holds $ETRAP value for inside trigger */
+			dollar_ztslate = DEFINE_MVAL_STRING(MV_STR, 0, 0, 0, NULL, 0, 0);
 GBLDEF	int		tprestart_state;		/* When triggers restart, multiple states possible. See tp_restart.h */
 GBLDEF	boolean_t	skip_INVOKE_RESTART;		/* set to TRUE if caller of op_tcommit/t_retry does not want it to
 							 * use the INVOKE_RESTART macro (which uses an rts_error to trigger
-							 * the restart) instead return code. The reason we dont want to do
+							 * the restart) instead return code. The reason we don't want to do
 							 * rts_error is that this is an implicit tstart situation where we
-							 * did not do opp_tstart.s so we dont want control to be transferred
+							 * did not do opp_tstart.s so we don't want control to be transferred
 							 * using the MUM_TSTART macro by mdb_condition_handler (which assumes
 							 * opp_tstart.s invocation).
 							 */
@@ -1015,7 +1020,7 @@ GBLDEF	boolean_t	in_mu_rndwn_file;		/* TRUE if we are in mu_rndwn_file (holding
 #endif
 GBLDEF	char		gvcst_search_clue;
 /* The following are replication related global variables. Ideally if we had a repl_gbls_t structure (like jnl_gbls_t)
- * this would be a member in that. But since we dont have one and since we need to initialize this specificially to a
+ * this would be a member in that. But since we don't have one and since we need to initialize this specificially to a
  * non-zero value (whereas usually everything else accepts a 0 default value), this is better kept as a separate global
  * variable instead of inside a global variable structure.
  */
@@ -1040,18 +1045,21 @@ GBLDEF  repl_conn_info_t        *this_side, *remote_side;
 GBLDEF	seq_num			gtmsource_save_read_jnl_seqno;
 GBLDEF	gtmsource_state_t	gtmsource_state = GTMSOURCE_DUMMY_STATE;
 GBLDEF	boolean_t	gv_play_duplicate_kills;	/* A TRUE value implies KILLs of non-existent nodes will continue to
-							 * write jnl records and increment the db curr_tn even though they dont
+							 * write jnl records and increment the db curr_tn even though they don't
 							 * touch any GDS blocks in the db (i.e. treat it as a duplicate kill).
 							 * Set to TRUE for the update process & journal recovery currently.
 							 * Set to FALSE otherwise.
 							 */
-GBLDEF	boolean_t	donot_fflush_NULL;		/* Set to TRUE whenever we dont want gtm_putmsg to fflush(NULL). BYPASSOK
+GBLDEF	boolean_t	donot_fflush_NULL;		/* Set to TRUE whenever we don't want gtm_putmsg to fflush(NULL). BYPASSOK
 							 * As of Jan 2012, mu_rndwn_all is the only user of this functionality.
 							 */
 GBLDEF	boolean_t	jnlpool_init_needed;		/* TRUE if jnlpool_init should be done at database init time (eg., for
 							 * anticipatory freeze supported configurations). The variable is set
 							 * explicitly by interested commands (eg., MUPIP REORG).
 							 */
+GBLDEF	char		repl_instfilename[MAX_FN_LEN + 1];	/* save first instance */
+GBLDEF	char		repl_inst_name[MAX_INSTNAME_LEN];	/* for syslog */
+GBLDEF	gd_addr		*repl_inst_from_gld;		/* if above obtained from directory */
 GBLDEF	boolean_t	span_nodes_disallowed; 		/* Indicates whether spanning nodes are not allowed. For example,
 							 * they are not allowed for GT.CM OMI and GNP. */
 GBLDEF	boolean_t	argumentless_rundown;
@@ -1066,7 +1074,10 @@ GBLDEF	boolean_t	in_jnl_file_autoswitch;		/* Set to TRUE for a short window insi
 							 * to autoswitch; used by jnl_write. */
 #ifdef GTM_PTHREAD
 GBLDEF	pthread_t	gtm_main_thread_id;		/* ID of the main GT.M thread. */
-GBLDEF	boolean_t	gtm_main_thread_id_set;		/* Indicates whether the thread ID is set. */
+GBLDEF	boolean_t	gtm_main_thread_id_set;		/* Indicates whether the thread ID is set. This is not set just for a jvm
+							 * process but also otherwise. This is necessary now for the linux kernel
+							 * interface to AIO.
+							 */
 GBLDEF	boolean_t	gtm_jvm_process;		/* Indicates whether we are running with JVM or stand-alone. */
 #endif
 GBLDEF	size_t		gtm_max_storalloc;		/* Maximum that GTM allows to be allocated - used for testing */
@@ -1076,6 +1087,10 @@ GBLDEF	boolean_t	ipv4_only;			/* If TRUE, only use AF_INET. Reflects the value o
 GBLDEF void (*stx_error_fptr)(int in_error, ...);	/* Function pointer for stx_error() so gtm_utf8.c can avoid pulling
 							 * stx_error() into gtmsecshr, and thus just about everything else as well.
 							 */
+GBLDEF void (*stx_error_va_fptr)(int in_error, va_list args);	/* Function pointer for stx_error() so rts_error can avoid pulling
+								 * stx_error() into gtmsecshr, and thus just about everything else
+								 * as well.
+								 */
 GBLDEF void (*show_source_line_fptr)(boolean_t warn);	/* Func pointer for show_source_line() - same purpose as stx_error_fptr */
 #ifdef GTM_TLS
 GBLDEF	gtm_tls_ctx_t	*tls_ctx;			/* Process private pointer to SSL/TLS context. Any SSL/TLS connections that
@@ -1138,4 +1153,33 @@ GBLDEF	int		gtm_db_counter_sem_incr;	/* Value used to bump the counter semaphore
 							 * Default is 1. Higher values exercise the ERANGE code better
 							 * when the ftok/access/jnlpool counter semaphore overflows.
 							 */
+GBLDEF	boolean_t	forw_recov_lgtrig_only;		/* TRUE if jgbl.forw_phase_recovery is TRUE AND the current TP transaction
+							 * being played consists of only *LGTRIG* records (no SET/KILL etc.)
+							 * Used by an assert in op_tcommit.
+							 */
+GBLDEF	boolean_t	in_mu_cre_file;			/* TRUE only if inside "mu_cre_file" function (used by an assert).
+							 * This is MUPIP CREATE only (db does not exist at that point and so
+							 * threads are unlikely there) AND is dbg-only and hence okay to be a
+							 * gbldef instead of a threadgbldef.
+							 */
+GBLDEF	enum dbg_wtfini_lcnt_t	dbg_wtfini_lcnt;	/* "lcnt" value for WCS_OPS_TRACE tracking purposes. This is dbg-only
+							 * and hence it is okay to be a gbldef instead of a threadgbldef.
+							 */
 #endif
+GBLDEF	sgm_info	*sgm_info_ptr;
+GBLDEF	tp_region	*tp_reg_free_list;	/* Ptr to list of tp_regions that are unused */
+GBLDEF  tp_region	*tp_reg_list;		/* Ptr to list of tp_regions for this transaction */
+
+#ifdef USE_LIBAIO
+GBLDEF  char 		*aio_shim_errstr;	/* If an error occurred (mostly but not limited to EAGAIN),
+						 * what triggered it?
+						 */
+GBLDEF	char		io_setup_errstr[IO_SETUP_ERRSTR_ARRAYSIZE];
+							/* The original nr_events used by the client is necessary
+							 * to understand why io_setup() failed on occasion. We set
+							 * up an error string of the form io_setup(nr_events).
+							 */
+#endif
+GBLDEF	void		(*mupip_exit_fp)(int4 errnum);	/* Function pointer to mupip_exit() in MUPIP but points to a routine
+							 * that assert fails if run from non-MUPIP builds.
+							 */
