@@ -1,6 +1,6 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;								;
-; Copyright (c) 2001-2015 Fidelity National Information 	;
+; Copyright (c) 2001-2017 Fidelity National Information		;
 ; Services, Inc. and/or its subsidiaries. All rights reserved.	;
 ;								;
 ;	This source code contains the intellectual property	;
@@ -11,12 +11,10 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 gdeinit: ;set up local variables and arrays
 GDEINIT
-	i $ZVER'["VMS" view "BADCHAR"
+	view "BADCHAR"
 	s renpref=""
 	s log=0,logfile="GDELOG.LOG",BOL=""
 	s ZERO=$c(0),ONE=$c(1),TRUE=ONE,FALSE=ZERO,TAB=$c(9)
-	s endian("VAX","VMS")=FALSE,glo("VMS")=1024
-	s endian("AXP","VMS")=FALSE,endian("AXP","OSF1")=FALSE,glo("VMS")=1024,glo("OSF1")=1024
 	s endian("x86","SCO")=FALSE,endian("x86","UWIN")=FALSE,endian("x86","Linux")=FALSE,endian("x86","CYGWIN")=FALSE
 	s endian("x86_64","Linux")=FALSE
 	s glo("SCO")=384,glo("UWIN")=1024,glo("Linux")=1024,glo("CYGWIN")=1024
@@ -84,8 +82,7 @@ GDEINIT
 	s typevalue("STR2NUM","TACCMETH","BG")=0
 	s typevalue("STR2NUM","TACCMETH","MM")=1
 	s nommbi=1              ; this is used in GDEVERIF and should be removed along with the code when support is added
-	d UNIX:ver'="VMS"
-	d VMS:ver="VMS"
+	d UNIX
 	d syntabi
 ;
 	; Explanation of some of the SIZEOF local variable subscripts.
@@ -95,38 +92,29 @@ GDEINIT
 	. s SIZEOF("am_offset")=332		; --> offset of "acc_meth" field in the "gd_segment" structure
 	. s SIZEOF("file_spec")=256		; --> maximum size (in bytes) of a file name specified in gde command line
 	. s SIZEOF("gd_header")=16		; --> 16-byte header structure at offset 0 of .gld (12 byte label, 4-byte filesize)
-	. s SIZEOF("gd_contents")=76		; --> size of the "gd_addr" structure (defined in gdsfhead.h)
+	. s SIZEOF("gd_contents")=80		; --> size of the "gd_addr" structure (defined in gdsfhead.h)
 	. s SIZEOF("gd_map")=16			; --> size of the "gd_binding" structure (defined in gdsfhead.h)
-	. if ver'="VMS" d
-	. . s SIZEOF("gd_region")=372		; --> size of the "gd_region"  structure (defined in gdsfhead.h)
-	. . s SIZEOF("gd_region_padding")=0	; --> padding at end of "gd_region" structure (4-bytes for 64-bit platforms)
-	. . s SIZEOF("gd_segment")=364		; --> size of the "gd_segment" structure (defined in gdsfhead.h)
-	. e  d
-	. . s SIZEOF("gd_region")=348		; --> size of the "gd_region"  structure (defined in gdsfhead.h)
-	. . ; SIZEOF("gd_region_padding")	 is not used in VMS
-	. . s SIZEOF("gd_segment")=356		; --> size of the "gd_segment" structure (defined in gdsfhead.h)
+	. s SIZEOF("gd_region")=412		; --> size of the "gd_region"  structure (defined in gdsfhead.h)
+	. s SIZEOF("gd_region_padding")=0	; --> padding at end of "gd_region" structure (4-bytes for 64-bit platforms)
+	. s SIZEOF("gd_segment")=368		; --> size of the "gd_segment" structure (defined in gdsfhead.h)
 	e  d
 	. s SIZEOF("am_offset")=340		; --> offset of "acc_meth" field in the "gd_segment" structure
 	. s SIZEOF("file_spec")=256		; --> maximum size (in bytes) of a file name specified in gde command line
 	. s SIZEOF("gd_header")=16		; --> 16-byte header structure at offset 0 of .gld (12 byte label, 4-byte filesize)
-	. s SIZEOF("gd_contents")=112		; --> size of the "gd_addr" structure (defined in gdsfhead.h)
+	. s SIZEOF("gd_contents")=120		; --> size of the "gd_addr" structure (defined in gdsfhead.h)
 	. s SIZEOF("gd_map")=24			; --> size of the "gd_binding" structure (defined in gdsfhead.h)
-	. s SIZEOF("gd_region")=384		; --> size of the "gd_region"  structure (defined in gdsfhead.h)
+	. s SIZEOF("gd_region")=424		; --> size of the "gd_region"  structure (defined in gdsfhead.h)
 	. s SIZEOF("gd_region_padding")=4	; --> padding at end of "gd_region" structure (4-bytes for 64-bit platforms)
 	. s SIZEOF("gd_segment")=384		; --> size of the "gd_segment" structure (defined in gdsfhead.h)
+	d Init^GDEINITSZ			; --> current gvstats size comes from routine generated by scantypedefs
 	s SIZEOF("gd_gblname")=40
+	s SIZEOF("gd_inst_info")=SIZEOF("file_spec")	; --> size of the "gd_inst_info" structure (defined in gdsfhead.h)
 	s SIZEOF("mident")=32
 	s SIZEOF("blk_hdr")=16
-	i ver'="VMS" d
-	. s SIZEOF("rec_hdr")=4	;GTM-6941
-	e  d
-	. s SIZEOF("rec_hdr")=3
+	s SIZEOF("rec_hdr")=4	;GTM-6941
 	s SIZEOF("dsk_blk")=512
-	i ver'="VMS" d
-	. s SIZEOF("max_str")=1048576
-	e  d
-	. s SIZEOF("max_str")=32767
-	s SIZEOF("reg_jnl_deq")=4				; not used on VMS
+	s SIZEOF("max_str")=1048576
+	s SIZEOF("reg_jnl_deq")=4
 	s MAXNAMLN=SIZEOF("mident")-1,MAXREGLN=32,MAXSEGLN=32	; maximum name length allowed is 31 characters
 	s PARNAMLN=31,PARREGLN=31,PARSEGLN=31
 	s MAXSTRLEN=(2**20)					; needs to be equal to MAX_STRLEN in mdef.h at all times
@@ -142,75 +130,75 @@ GDEINIT
 	s tokens("(")="TKLPAREN",tokens("TKLPAREN")="left parenthesis"
 	s tokens(")")="TKRPAREN",tokens("TKRPAREN")="right parenthesis"
 	s tokens("!")="TKEXCLAM",tokens("TKEXCLAM")="exclamation point"
-	i ver'="VMS" s tokens("-")="TKDASH",tokens("TKDASH")="dash"
-	e            s tokens("/")="TKSLASH",tokens("TKSLASH")="slash"
+	s tokens("-")="TKDASH",tokens("TKDASH")="dash"
 	s tokens("")="TKEOL"	; for parsing purposes
 ; tokendelim is used for parsing; it is defined for all characters that can terminate a valid token during parsing
 	for c=" ",TAB,"=",",",")","" s tokendelim(c)=""
-	; in case of VMS, we want the "/" qualifier separator to be treated as a delimiter even if not preceded by white space
-	; this is how VMS tools work. Whereas in the Unix shell command line, space is by default the separator for multiple
-	; qualifiers. Hence GDE also is similarly different between Unix vs VMS in terms of separator handling ("-" vs "/").
-	i ver="VMS" s tokendelim("/")=""
+	; In the Unix shell command line, space is by default the separator for multiple qualifiers.
+	; GDE handles separators similarly.
 ; spacedelim is used for parsing when we want to terminate token parsing only if we see a white space (a subset of "tokendelim")
-; this is needed in case we are parsing say a filename and we dont want a "-" or "=" in the file name to terminate the parse.
+; this is needed in case we are parsing say a filename and we don't want a "-" or "=" in the file name to terminate the parse.
 	for c=" ",TAB,"" s spacedelim(c)=""
-	i ver="VMS" s spacedelim("/")=""
 	k c
 ; maximums and mimimums
 ; gblname
 	s mingnam("COLLATION")=0
 	s maxgnam("COLLATION")=255
+; instance
+	s maxinst("FILE")=SIZEOF("file_spec")
 ; region
-	s minreg("ALLOCATION")=$s(ver'="VMS":200,1:10)
+	s minreg("ALLOCATION")=200
 	s minreg("BEFORE_IMAGE")=0,minreg("COLLATION_DEFAULT")=0,minreg("STDNULLCOLL")=0
 	s minreg("EXTENSION")=0
-	i ver'="VMS" d
-	. s minreg("AUTOSWITCHLIMIT")=16384
-	. s minreg("ALIGNSIZE")=4096			; geq RECORD_SIZE
-	. s minreg("EPOCH_INTERVAL")=1
-	. s minreg("SYNC_IO")=0
-	. s minreg("YIELD_LIMIT")=0
-	. s minreg("INST_FREEZE_ON_ERROR")=0
-	. s minreg("BUFFER_SIZE")=2307
-	. s minreg("QDBRUNDOWN")=0
-	. s minreg("EPOCHTAPER")=0
-	. s minreg("RECORD_SIZE")=0
-	e  d
-	. s minreg("RECORD_SIZE")=SIZEOF("rec_hdr")+4
+	s minreg("AUTOSWITCHLIMIT")=16384
+	s minreg("ALIGNSIZE")=4096			; geq RECORD_SIZE
+	s minreg("EPOCH_INTERVAL")=1
+	s minreg("SYNC_IO")=0
+	s minreg("YIELD_LIMIT")=0
+	s minreg("INST_FREEZE_ON_ERROR")=0
+	s minreg("BUFFER_SIZE")=2307
+	s minreg("QDBRUNDOWN")=0
+	s minreg("EPOCHTAPER")=0
+	s minreg("AUTODB")=0
+	s minreg("STATS")=0
+	s minreg("LOCK_CRIT")=0
+	s minreg("RECORD_SIZE")=0
 	s minreg("JOURNAL")=0,minreg("KEY_SIZE")=3,minreg("NULL_SUBSCRIPTS")=0
 	s maxreg("ALLOCATION")=TWO(24),maxreg("BEFORE_IMAGE")=1
 	s maxreg("COLLATION_DEFAULT")=255,maxreg("STDNULLCOLL")=1
-	i ver="VMS" do
-	. s maxreg("EXTENSION")=HEX(4)-1
-	. s maxreg("BUFFER_SIZE")=2000
-	. s maxreg("KEY_SIZE")=255
-	e  d
-	. s maxreg("EXTENSION")=1073741823
-	. s maxreg("AUTOSWITCHLIMIT")=8388607
-	. s maxreg("ALIGNSIZE")=4194304
-	. s maxreg("EPOCH_INTERVAL")=32767
-	. s maxreg("SYNC_IO")=1
-	. s maxreg("YIELD_LIMIT")=2048
-	. s maxreg("INST_FREEZE_ON_ERROR")=1
-	. s maxreg("BUFFER_SIZE")=32768
-	. s maxreg("QDBRUNDOWN")=1
-	. s maxreg("EPOCHTAPER")=1
-	. s maxreg("KEY_SIZE")=1019	; = max value of KEY->end that returns TRUE for CAN_APPEND_HIDDEN_SUBS(KEY) in gdsfhead.h
+	s maxreg("EXTENSION")=1073741823
+	s maxreg("AUTOSWITCHLIMIT")=8388607
+	s maxreg("ALIGNSIZE")=4194304
+	s maxreg("EPOCH_INTERVAL")=32767
+	s maxreg("SYNC_IO")=1
+	s maxreg("YIELD_LIMIT")=2048
+	s maxreg("INST_FREEZE_ON_ERROR")=1
+	s maxreg("BUFFER_SIZE")=32768
+	s maxreg("QDBRUNDOWN")=1
+	s maxreg("EPOCHTAPER")=1
+	s maxreg("AUTODB")=1
+	s maxreg("STATS")=1
+	s maxreg("LOCK_CRIT")=1
+	s maxreg("KEY_SIZE")=1019	; = max value of KEY->end that returns TRUE for CAN_APPEND_HIDDEN_SUBS(KEY) in gdsfhead.h
 	s maxreg("JOURNAL")=1,maxreg("NULL_SUBSCRIPTS")=2
 	s maxreg("RECORD_SIZE")=SIZEOF("max_str")
+	; Store defaults in "dflreg". "tmpreg" serves this purpose for the most part but it is user-editable (using the
+	; TEMPLATE command) but for things where we want to use the GT.M-default value (not user-edited default value)
+	; we use "dflreg". Currently the only parameter we care about is KEY_SIZE which is used for setting statsdb characteristics
+	s dflreg("KEY_SIZE")=64	; should be maintained in parallel with STATSDB_KEY_SIZE in gdsfhead.h
 ; segments
 	; First define segment characteristics (minimum and maximum) that are identical to BG and MM access methods
 	; Then define overrides specific to BG and MM
 	n minsegcommon,maxsegcommon
 	s minsegcommon("ALLOCATION")=10
 	s maxsegcommon("ALLOCATION")=TWO(27)
-	i ver'="VMS" s maxsegcommon("ALLOCATION")=TWO(30)-TWO(25) ; supports 992M blocks for UNIX only
+	s maxsegcommon("ALLOCATION")=TWO(30)-TWO(25) ; supports 992M blocks
 	s minsegcommon("BLOCK_SIZE")=SIZEOF("dsk_blk")
 	s maxsegcommon("BLOCK_SIZE")=HEX(4)-SIZEOF("dsk_blk")
 	s minsegcommon("EXTENSION_COUNT")=0
 	s maxsegcommon("EXTENSION_COUNT")=HEX(4)-1
 	s minsegcommon("LOCK_SPACE")=10
-	s maxsegcommon("LOCK_SPACE")=65536
+	s maxsegcommon("LOCK_SPACE")=262144
 	s minsegcommon("MUTEX_SLOTS")=64	; keep this in sync with MIN_CRIT_ENTRY in gdsbt.h
 	s maxsegcommon("MUTEX_SLOTS")=32768	; keep this in sync with MAX_CRIT_ENTRY in gdsbt.h
 	s minsegcommon("RESERVED_BYTES")=0
@@ -220,7 +208,7 @@ GDEINIT
 	m maxseg("BG")=maxsegcommon	; copy over all common stuff into BG access method first
 	; now add BG specific overrides
 	s minseg("BG","GLOBAL_BUFFER_COUNT")=64
-	i (gtm64=TRUE) s maxseg("BG","GLOBAL_BUFFER_COUNT")=2147483647 ; 2G-1
+	i (gtm64=TRUE) s maxseg("BG","GLOBAL_BUFFER_COUNT")=2097151 ; GTM64_WC_MAX_BUFFS defined in gdsbt.h
 	e  s maxseg("BG","GLOBAL_BUFFER_COUNT")=65536
 ; mm
 	m minseg("MM")=minsegcommon	; copy over all common stuff into MM access method first
@@ -232,7 +220,7 @@ GDEINIT
 	; This is particularly needed for fields that are only available in more recent .gld formats
 	; So if we are reading an older format .gld file, we can use these as the default values.
 	s defseg("ALLOCATION")=100
-	s defseg("BLOCK_SIZE")=1024
+	s defseg("BLOCK_SIZE")=4096
 	s defseg("BUCKET_SIZE")=""
 	s defseg("DEFER_ALLOCATE")=1
 	s defseg("EXTENSION_COUNT")=100
@@ -241,7 +229,8 @@ GDEINIT
 	s defseg("RESERVED_BYTES")=0
 	s defseg("LOCK_SPACE")=40
 	s defseg("WINDOW_SIZE")=""
-	i ver'="VMS" s defseg("ENCRYPTION_FLAG")=0
+	s defseg("ENCRYPTION_FLAG")=0
+	s defseg("ASYNCIO")=0
 	q
 
 ;-----------------------------------------------------------------------------------------------------------------------------------
@@ -260,8 +249,11 @@ syntabi:
 	s syntab("ADD","REGION","STDNULLCOLL")="NEGATABLE"
 	s syntab("ADD","REGION","DYNAMIC_SEGMENT")="REQUIRED"
 	s syntab("ADD","REGION","DYNAMIC_SEGMENT","TYPE")="TSEGMENT"
-	i ver'="VMS" s syntab("ADD","REGION","EPOCHTAPER")="NEGATABLE"
-	i ver'="VMS" s syntab("ADD","REGION","INST_FREEZE_ON_ERROR")="NEGATABLE"
+	s syntab("ADD","REGION","EPOCHTAPER")="NEGATABLE"
+	s syntab("ADD","REGION","AUTODB")="NEGATABLE"
+	s syntab("ADD","REGION","STATS")="NEGATABLE"
+	s syntab("ADD","REGION","LOCK_CRIT")="NEGATABLE"
+	s syntab("ADD","REGION","INST_FREEZE_ON_ERROR")="NEGATABLE"
 	s syntab("ADD","REGION","JOURNAL")="NEGATABLE,REQUIRED,LIST"
 	s syntab("ADD","REGION","JOURNAL","ALLOCATION")="REQUIRED"
 	s syntab("ADD","REGION","JOURNAL","ALLOCATION","TYPE")="TNUMBER"
@@ -278,7 +270,7 @@ syntabi:
 	s syntab("ADD","REGION","KEY_SIZE","TYPE")="TNUMBER"
 	s syntab("ADD","REGION","NULL_SUBSCRIPTS")="NEGATABLE,REQUIRED"
 	s syntab("ADD","REGION","NULL_SUBSCRIPTS","TYPE")="TNULLSUB"
-	i ver'="VMS" s syntab("ADD","REGION","QDBRUNDOWN")="NEGATABLE"
+	s syntab("ADD","REGION","QDBRUNDOWN")="NEGATABLE"
 	s syntab("ADD","REGION","RECORD_SIZE")="REQUIRED"
 	s syntab("ADD","REGION","RECORD_SIZE","TYPE")="TNUMBER"
 	s syntab("ADD","SEGMENT")=""
@@ -286,13 +278,14 @@ syntabi:
 	s syntab("ADD","SEGMENT","ACCESS_METHOD","TYPE")="TACCMETH"
 	s syntab("ADD","SEGMENT","ALLOCATION")="REQUIRED"
 	s syntab("ADD","SEGMENT","ALLOCATION","TYPE")="TNUMBER"
+	s syntab("ADD","SEGMENT","ASYNCIO")="NEGATABLE"
 	s syntab("ADD","SEGMENT","BLOCK_SIZE")="REQUIRED"
 	s syntab("ADD","SEGMENT","BLOCK_SIZE","TYPE")="TNUMBER"
 	s syntab("ADD","SEGMENT","BUCKET_SIZE")="REQUIRED"
 	s syntab("ADD","SEGMENT","BUCKET_SIZE","TYPE")="TNUMBER"
 	s syntab("ADD","SEGMENT","DEFER")="NEGATABLE"
 	s syntab("ADD","SEGMENT","DEFER_ALLOCATE")="NEGATABLE"
- 	i ver'="VMS" s syntab("ADD","SEGMENT","ENCRYPTION_FLAG")="NEGATABLE"
+ 	s syntab("ADD","SEGMENT","ENCRYPTION_FLAG")="NEGATABLE"
 	s syntab("ADD","SEGMENT","EXTENSION_COUNT")="REQUIRED"
 	s syntab("ADD","SEGMENT","EXTENSION_COUNT","TYPE")="TNUMBER"
 	s syntab("ADD","SEGMENT","FILE_NAME")="REQUIRED"
@@ -310,6 +303,9 @@ syntabi:
 	s syntab("CHANGE","GBLNAME")=""
 	s syntab("CHANGE","GBLNAME","COLLATION")="REQUIRED"
 	s syntab("CHANGE","GBLNAME","COLLATION","TYPE")="TNUMBER"
+	s syntab("CHANGE","INSTANCE")=""
+	s syntab("CHANGE","INSTANCE","FILE_NAME")="REQUIRED"
+	s syntab("CHANGE","INSTANCE","FILE_NAME","TYPE")="TFSPEC"
 	s syntab("CHANGE","NAME")=""
 	s syntab("CHANGE","NAME","REGION")="REQUIRED"
 	s syntab("CHANGE","NAME","REGION","TYPE")="TREGION"
@@ -319,8 +315,11 @@ syntabi:
 	s syntab("CHANGE","REGION","STDNULLCOLL")="NEGATABLE"
 	s syntab("CHANGE","REGION","DYNAMIC_SEGMENT")="REQUIRED"
 	s syntab("CHANGE","REGION","DYNAMIC_SEGMENT","TYPE")="TSEGMENT"
-	i ver'="VMS" s syntab("CHANGE","REGION","EPOCHTAPER")="NEGATABLE"
-	i ver'="VMS" s syntab("CHANGE","REGION","INST_FREEZE_ON_ERROR")="NEGATABLE"
+	s syntab("CHANGE","REGION","EPOCHTAPER")="NEGATABLE"
+	s syntab("CHANGE","REGION","AUTODB")="NEGATABLE"
+	s syntab("CHANGE","REGION","STATS")="NEGATABLE"
+	s syntab("CHANGE","REGION","LOCK_CRIT")="NEGATABLE"
+	s syntab("CHANGE","REGION","INST_FREEZE_ON_ERROR")="NEGATABLE"
 	s syntab("CHANGE","REGION","JOURNAL")="NEGATABLE,REQUIRED,LIST"
 	s syntab("CHANGE","REGION","JOURNAL","ALLOCATION")="REQUIRED"
 	s syntab("CHANGE","REGION","JOURNAL","ALLOCATION","TYPE")="TNUMBER"
@@ -337,7 +336,7 @@ syntabi:
 	s syntab("CHANGE","REGION","KEY_SIZE","TYPE")="TNUMBER"
 	s syntab("CHANGE","REGION","NULL_SUBSCRIPTS")="NEGATABLE,REQUIRED"
 	s syntab("CHANGE","REGION","NULL_SUBSCRIPTS","TYPE")="TNULLSUB"
-	i ver'="VMS" s syntab("CHANGE","REGION","QDBRUNDOWN")="NEGATABLE"
+	s syntab("CHANGE","REGION","QDBRUNDOWN")="NEGATABLE"
 	s syntab("CHANGE","REGION","RECORD_SIZE")="REQUIRED"
 	s syntab("CHANGE","REGION","RECORD_SIZE","TYPE")="TNUMBER"
 	s syntab("CHANGE","SEGMENT")=""
@@ -345,13 +344,14 @@ syntabi:
 	s syntab("CHANGE","SEGMENT","ACCESS_METHOD","TYPE")="TACCMETH"
 	s syntab("CHANGE","SEGMENT","ALLOCATION")="REQUIRED"
 	s syntab("CHANGE","SEGMENT","ALLOCATION","TYPE")="TNUMBER"
+	s syntab("CHANGE","SEGMENT","ASYNCIO")="NEGATABLE"
 	s syntab("CHANGE","SEGMENT","BLOCK_SIZE")="REQUIRED"
 	s syntab("CHANGE","SEGMENT","BLOCK_SIZE","TYPE")="TNUMBER"
 	s syntab("CHANGE","SEGMENT","BUCKET_SIZE")="REQUIRED"
 	s syntab("CHANGE","SEGMENT","BUCKET_SIZE","TYPE")="TNUMBER"
 	s syntab("CHANGE","SEGMENT","DEFER")="NEGATABLE"
 	s syntab("CHANGE","SEGMENT","DEFER_ALLOCATE")="NEGATABLE"
-	i ver'="VMS" s syntab("CHANGE","SEGMENT","ENCRYPTION_FLAG")="NEGATABLE"
+	s syntab("CHANGE","SEGMENT","ENCRYPTION_FLAG")="NEGATABLE"
 	s syntab("CHANGE","SEGMENT","EXTENSION_COUNT")="REQUIRED"
 	s syntab("CHANGE","SEGMENT","EXTENSION_COUNT","TYPE")="TNUMBER"
 	s syntab("CHANGE","SEGMENT","FILE_NAME")="REQUIRED"
@@ -375,14 +375,16 @@ syntabi:
 	s syntab("TEMPLATE","REGION","JOURNAL")="NEGATABLE,REQUIRED,LIST"
 	s syntab("TEMPLATE","REGION","JOURNAL","ALLOCATION")="REQUIRED"
 	s syntab("TEMPLATE","REGION","JOURNAL","ALLOCATION","TYPE")="TNUMBER"
-	i ver'="VMS" d
-	. s syntab("TEMPLATE","REGION","INST_FREEZE_ON_ERROR")="NEGATABLE"
-	. s syntab("TEMPLATE","REGION","JOURNAL","AUTOSWITCHLIMIT")="REQUIRED"
-	. s syntab("TEMPLATE","REGION","JOURNAL","AUTOSWITCHLIMIT","TYPE")="TNUMBER"
+	s syntab("TEMPLATE","REGION","INST_FREEZE_ON_ERROR")="NEGATABLE"
+	s syntab("TEMPLATE","REGION","JOURNAL","AUTOSWITCHLIMIT")="REQUIRED"
+	s syntab("TEMPLATE","REGION","JOURNAL","AUTOSWITCHLIMIT","TYPE")="TNUMBER"
 	s syntab("TEMPLATE","REGION","JOURNAL","BUFFER_SIZE")="REQUIRED"
 	s syntab("TEMPLATE","REGION","JOURNAL","BUFFER_SIZE","TYPE")="TNUMBER"
 	s syntab("TEMPLATE","REGION","JOURNAL","BEFORE_IMAGE")="NEGATABLE"
-	i ver'="VMS" s syntab("TEMPLATE","REGION","EPOCHTAPER")="NEGATABLE"
+	s syntab("TEMPLATE","REGION","EPOCHTAPER")="NEGATABLE"
+	s syntab("TEMPLATE","REGION","AUTODB")="NEGATABLE"
+	s syntab("TEMPLATE","REGION","STATS")="NEGATABLE"
+	s syntab("TEMPLATE","REGION","LOCK_CRIT")="NEGATABLE"
 	s syntab("TEMPLATE","REGION","JOURNAL","EXTENSION")="REQUIRED"
 	s syntab("TEMPLATE","REGION","JOURNAL","EXTENSION","TYPE")="TNUMBER"
 	s syntab("TEMPLATE","REGION","JOURNAL","FILE_NAME")="REQUIRED"
@@ -391,7 +393,7 @@ syntabi:
 	s syntab("TEMPLATE","REGION","KEY_SIZE","TYPE")="TNUMBER"
 	s syntab("TEMPLATE","REGION","NULL_SUBSCRIPTS")="NEGATABLE,REQUIRED"
 	s syntab("TEMPLATE","REGION","NULL_SUBSCRIPTS","TYPE")="TNULLSUB"
-	i ver'="VMS" s syntab("TEMPLATE","REGION","QDBRUNDOWN")="NEGATABLE"
+	s syntab("TEMPLATE","REGION","QDBRUNDOWN")="NEGATABLE"
 	s syntab("TEMPLATE","REGION","RECORD_SIZE")="REQUIRED"
 	s syntab("TEMPLATE","REGION","RECORD_SIZE","TYPE")="TNUMBER"
 	s syntab("TEMPLATE","SEGMENT")=""
@@ -399,13 +401,14 @@ syntabi:
 	s syntab("TEMPLATE","SEGMENT","ACCESS_METHOD","TYPE")="TACCMETH"
 	s syntab("TEMPLATE","SEGMENT","ALLOCATION")="REQUIRED"
 	s syntab("TEMPLATE","SEGMENT","ALLOCATION","TYPE")="TNUMBER"
+	s syntab("TEMPLATE","SEGMENT","ASYNCIO")="NEGATABLE"
 	s syntab("TEMPLATE","SEGMENT","BLOCK_SIZE")="REQUIRED"
 	s syntab("TEMPLATE","SEGMENT","BLOCK_SIZE","TYPE")="TNUMBER"
 	s syntab("TEMPLATE","SEGMENT","BUCKET_SIZE")="REQUIRED"
 	s syntab("TEMPLATE","SEGMENT","BUCKET_SIZE","TYPE")="TNUMBER"
 	s syntab("TEMPLATE","SEGMENT","DEFER")="NEGATABLE"
 	s syntab("TEMPLATE","SEGMENT","DEFER_ALLOCATE")="NEGATABLE"
-	i ver'="VMS" s syntab("TEMPLATE","SEGMENT","ENCRYPTION_FLAG")="NEGATABLE"
+	s syntab("TEMPLATE","SEGMENT","ENCRYPTION_FLAG")="NEGATABLE"
 	s syntab("TEMPLATE","SEGMENT","EXTENSION_COUNT")="REQUIRED"
 	s syntab("TEMPLATE","SEGMENT","EXTENSION_COUNT","TYPE")="TNUMBER"
 	s syntab("TEMPLATE","SEGMENT","FILE_NAME")="REQUIRED"
@@ -445,6 +448,7 @@ syntabi:
 	s syntab("SHOW","COMMANDS","FILE")="OPTIONAL"
 	s syntab("SHOW","COMMANDS","FILE","TYPE")="TFSPEC"
 	s syntab("SHOW","GBLNAME")=""
+	s syntab("SHOW","INSTANCE")=""
 	s syntab("SHOW","MAP")=""
 	s syntab("SHOW","MAP","REGION")="REQUIRED"
 	s syntab("SHOW","MAP","REGION","TYPE")="TREGION"
@@ -461,25 +465,9 @@ syntabi:
 	s syntab("VERIFY","SEGMENT")=""
 	s syntab("VERIFY","TEMPLATE")=""
 	q
-VMS
-	s endian=FALSE
-	s hdrlab="GTCGBLDIR010"		; must be concurrently maintained in gbldirnam.h!!!
-	s tfile="GTM$GBLDIR"
-	s accmeth="\BG\MM\USER"
-	s typevalue("STR2NUM","TACCMETH","USER")=2
-	s helpfile="GTM$HELP:GDE.HLB"
-	s defdb="MUMPS"
-	s defgld="MUMPS.GLD",defgldext=".GLD"
-	s defreg="$DEFAULT"
-	s defseg="$DEFAULT"
-	s dbfilpar=".1AN.1""-"".1""_"".1"":"".1""$"".1""["".1""]"".1""<"".1"">"".1""."".1"";"""
-	s filexfm="$tr(filespec,lower,upper)"
-	s sep="TKSLASH"
-	q
-
 UNIX:
-	s hdrlab="GTCGBDUNX010"         ; must be concurrently maintained in gbldirnam.h!!!
-	i (gtm64=TRUE) s hdrlab="GTCGBDUNX110" ; the high order digit is a 64-bit flag
+	s hdrlab="GTCGBDUNX012"         ; must be concurrently maintained in gbldirnam.h!!!
+	i (gtm64=TRUE) s hdrlab="GTCGBDUNX112" ; the high order digit is a 64-bit flag
 	s tfile="$gtmgbldir"
 	s accmeth="\BG\MM"
 	s helpfile="$gtm_dist/gdehelp.gld"
