@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2010-2016 Fidelity National Information	*
+ * Copyright (c) 2010-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -35,6 +35,7 @@
 THREADGBLDEF(grabbing_crit, 			gd_region *)			/* Region currently grabbing crit in (if any) */
 
 /* Compiler */
+THREADGBLDEF(block_level,			int4)				/* used to check embedded subroutine levels */
 THREADGBLDEF(boolchain,				triple)				/* anchor for chain used by bx_boolop  */
 THREADGBLDEF(boolchain_ptr,			triple *)			/* pointer to anchor for chain used by bx_boolop  */
 THREADGBLDEF(bool_targ_anchor,			tbp)				/* anchor of targ chain for bool relocation */
@@ -62,11 +63,11 @@ THREADGBLDEF(last_source_column,		int)				/* parser tracker */
 THREADGBLDEF(lexical_ptr,			char *)				/* parser character position */
 THREADGBLDEF(linkage_first,			struct linkage_entry *)		/* Start of linkage (extern) list this routine */
 THREADGBLDEF(linkage_last,			struct linkage_entry *)		/* Last added entry */
-THREADGBLDEF(max_advancewindow_line,		int4)				/* the maximum source line length */
 #ifdef USHBIN_SUPPORTED
 THREADGBLDEF(objhash_state,			hash128_state_t)		/* Seed value - progressive hash of object file */
 #endif
 THREADGBLDEF(pos_in_chain,			triple)				/* anchor used to restart after a parsing error */
+THREADGBLDEF(rts_error_in_parse,		boolean_t)			/* flag to stop parsing current line */
 THREADGBLDEF(s2n_intlit, 			boolean_t)			/* type info from s2n for advancewindow */
 THREADGBLDEF(routine_source_offset,		uint4)				/* offset of M source within literal text pool */
 THREADGBLDEF(saw_side_effect,			boolean_t)			/* need side effect handling other than naked */
@@ -76,6 +77,7 @@ THREADGBLDEF(side_effect_base,			boolean_t *)			/* anchor side effect array: bin
 THREADGBLDEF(side_effect_depth,			uint4)				/* current high water of side effect expr array */
 THREADGBLDEF(side_effect_handling,		int)				/* side effect handling in actuallists, function
 										 * args & non-boolean binary operator operands */
+THREADGBLDEF(source_buffer,			mstr)				/* source line buffer control */
 THREADGBLDEF(source_error_found,		int4)				/* flag to partially defer compiler error */
 THREADGBLDEF(temp_subs,				boolean_t)			/* flag temp storing of subscripts to preserve
 										 * current evaluation */
@@ -83,11 +85,11 @@ THREADGBLDEF(trigger_compile_and_link,		boolean_t)			/* A trigger compilation/li
 THREADGBLDEF(window_ident,			mstr)				/* current scanner mident from advancewindow */
 THREADGBLDEF(window_mval,			mval)				/* current scanner mval from advancewindow */
 THREADGBLDEF(window_token,			char)				/* current scanner token from advancewindow */
+THREADGBLDEF(xecute_literal_parse,		boolean_t)			/* flag TRUE when trying what its name says */
 
 /* Database */
-THREADGBLDEF(dbinit_max_hrtbt_delta,		uint4)				/* max heartbeats before we bail out in db_init */
+THREADGBLDEF(dbinit_max_delta_secs,		uint4)				/* max time before we bail out in db_init */
 THREADGBLDEF(dollar_zmaxtptime, 		int4)				/* tp timeout in seconds */
-THREADGBLDEF(donot_commit,			boolean_t)			/* debug-only - see gdsfhead.h for purpose */
 THREADGBLDEF(donot_write_inctn_in_wcs_recover,	boolean_t)			/* TRUE if wcs_recover should NOT write INCTN */
 THREADGBLDEF(gbuff_limit,			mval)				/* holds a GTM_POOLLIMIT value for REORG or DBG */
 THREADGBLDEF(gd_targ_tn,			trans_num)			/* number that is incremented for every gvcst_spr*
@@ -102,6 +104,8 @@ THREADGBLDEF(gd_targ_reg_array,			trans_num *)			/* Indicates which regions are
 THREADGBLDEF(gd_targ_reg_array_size,		uint4)				/* Size of current gd_targ_reg_array allocation.
 										 * Non-zero only if at least one spanning global
 										 * has been seen at time of gld open.
+										 * Note: the dimension is the # of regions involved
+										 * and not the # of bytes allocated in the array.
 										 */
 THREADGBLDEF(gd_targ_addr,			gd_addr *)			/* current global directory reference. Needed by
 										 * name level $order or $zprevious to know inside
@@ -169,7 +173,7 @@ THREADGBLDEF(redo_rootsrch_ctxt,		redo_root_search_context)	/* context to be sav
 										 * gvcst_redo_root_search */
 THREADGBLDEF(semwait2long,			volatile boolean_t)		/* Waited too long for a semaphore */
 THREADGBLDEF(skip_file_corrupt_check,		boolean_t)			/* skip file_corrupt check in grab_crit */
-THREADGBLDEF(tpnotacidtime,			int4)				/* limit for long non-ACID ops in transactions */
+THREADGBLDEF(tpnotacidtime,			mval)				/* limit for long non-ACID ops in transactions */
 THREADGBLDEF(tp_restart_count,			uint4)				/* tp_restart counter */
 THREADGBLDEF(tp_restart_dont_counts,		int4)				/* tp_restart count adjustment; NOTE: DEBUG only */
 THREADGBLDEF(tp_restart_entryref,		mval)				/* tp_restart position for reporting */
@@ -181,6 +185,7 @@ THREADGBLDEF(wcs_recover_done,			boolean_t)			/* TRUE if wcs_recover was ever in
 										 * process. */
 
 /* Local variables */
+THREADGBLDEF(curr_symval_cycle,			unsigned int)			/* When curr_symval is changed, counter is bumped */
 THREADGBLDEF(in_op_fnnext,			boolean_t)			/* set TRUE by op_fnnext; FALSE by op_fnorder */
 THREADGBLDEF(local_collseq,			collseq *)			/* pointer to collation algorithm for lvns */
 THREADGBLDEF(local_collseq_stdnull,		boolean_t)			/* flag temp controlling empty-string subscript
@@ -190,6 +195,9 @@ THREADGBLDEF(local_coll_nums_as_strings,	boolean_t)			/* flag controlling whethe
 										 * evaluate to numbers are treated like numbers
 										 * (collating before strings) or like strings in
 										 * local collations */
+THREADGBLDEF(lvmon_active,			boolean_t)			/* TRUE when local var monitoring is active */
+THREADGBLDEF(lvmon_vars_anchor,			lvmon_var *)			/* Anchor for lv monitoring structure */
+THREADGBLDEF(lvmon_vars_count,			int)				/* Count of lvmon_vars at lvmon_vars_anchor */
 THREADGBLDEF(lv_null_subs,			int)				/* set in gtm_env_init_sp() */
 THREADGBLDEF(max_lcl_coll_xform_bufsiz,		int)				/* max size of local collation buffer,which extends
 										 * from 32K each time the buffer overflows */
@@ -209,10 +217,13 @@ THREADGBLFPTR(create_fatal_error_zshow_dmp_fptr, void, 		(void))		/* Fptr for gt
 THREADGBLDEF(disable_sigcont,			boolean_t)			/* indicates whether the SIGCONT signal
 										 * is allowed internally */
 THREADGBLDEF(dollar_zcompile,			mstr)				/* compiler qualifiers */
+THREADGBLDEF(dollar_etrap,			mval)				/* $etrap - standard error action */
 THREADGBLDEF(dollar_zmode,			mval)				/* run mode indicator */
 THREADGBLDEF(dollar_zonlnrlbk,			int)				/* ISV (incremented for every online rollback) */
 THREADGBLDEF(dollar_zclose,			int)				/* ISV (set to close status for PIPE device) */
 THREADGBLDEF(dollar_zroutines,			mstr)				/* routine search list */
+THREADGBLDEF(dollar_zstep,			mval)				/* $zstep - zstep action */
+THREADGBLDEF(dollar_ztrap,			mval)				/* $ztrap - recursive try error action */
 THREADGBLDEF(error_on_jnl_file_lost,		unsigned int)			/* controls error handling done by jnl_file_lost.
 										 * 0 (default) : Turn off journaling and continue.
 										 * 1 : Keep journaling on, throw rts_error */
@@ -221,7 +232,7 @@ THREADGBLDEF(fnzsearch_sub_mval,		mval)				/* op_fnzsearch subscript constuctor
 THREADGBLDEF(fnzsearch_nullsubs_sav,		int)				/* op_fnzsearch temp for null subs control */
 THREADGBLDEF(fnzsearch_globbuf_ptr,		glob_t *)			/* op_fnzsearch temp for pointing to glob results */
 THREADGBLDEF(glvn_pool_ptr,			glvn_pool *)			/* Pointer to the glvn pool */
-#if defined(GTMDBGFLAGS_ENABLED)
+#ifdef GTMDBGFLAGS_ENABLED
 THREADGBLDEF(gtmdbgflags,			int)
 THREADGBLDEF(gtmdbgflags_freq,			int)
 THREADGBLDEF(gtmdbgflags_freq_cntr,		int)
@@ -232,6 +243,9 @@ THREADGBLDEF(gtm_environment_init,		boolean_t)			/* indicates GT.M development e
 										 * than a production environment */
 THREADGBLFPTR(gtm_sigusr1_handler,		void, 		(void))		/* SIGUSR1 signal handler function ptr */
 THREADGBLDEF(gtm_linktmpdir,			mstr)				/* Directory to use for relinkctl files */
+THREADGBLDEF(gtm_strpllim,			int4)				/* if non-zero, sets limit on stringpool */
+THREADGBLDEF(gtm_strpllimwarned,		boolean_t)			/* already hit limit on stringpool  */
+THREADGBLDEF(gtm_trigger_etrap,			mval)				/* $etrap - for use in triggers */
 THREADGBLDEF(gtm_trctbl_cur,			trctbl_entry *)			/* Current gtm trace table entry */
 THREADGBLDEF(gtm_trctbl_end,			trctbl_entry *)			/* End of gtm trace table (last entry + 1) */
 THREADGBLDEF(gtm_trctbl_groups,			unsigned int)			/* Trace group mask (max 31 groups) */
@@ -241,6 +255,7 @@ THREADGBLDEF(gtmprompt,				mstr)				/* mstr pointing to prombuf containing the G
 										 * prompt */
 THREADGBLDEF(gtmsecshr_comkey,			unsigned int)			/* Hashed version key for gtmsecshr communications
 										 * eliminates cross-version issues */
+THREADGBLDEF(gvcst_statsDB_open_ch_active,	boolean_t)			/* Condition handler is active */
 THREADGBLDEF(in_zwrite,				boolean_t)			/* ZWrite is active */
 THREADGBLDEF(lab_lnr,				lnr_tabent **)			/* Passes address from op_rhd_ext to op_extcall etc.
 										 * Points into either lab_proxy or linkage table
@@ -270,6 +285,7 @@ THREADGBLDEF(mprof_reclaim_cnt,			int)				/* Amount of mem to reclaim after unw_
 THREADGBLDEF(mprof_stack_curr_frame, 		mprof_stack_frame *)		/* Pointer to the last frame on the mprof stack */
 THREADGBLDEF(mprof_stack_next_frame, 		mprof_stack_frame *)		/* Pointer to the next frame to be put on the
 										 * mprof stack */
+THREADGBLDEF(mu_cre_file_openrc,		int)				/* 0 if success otherwise holds errno after open */
 #ifdef AUTORELINK_SUPPORTED
 THREADGBLDEF(open_relinkctl_list,		open_relinkctl_sgm *)		/* Anchor for open relinkctl list; similar to
 										 * open_shlib_root */
@@ -281,6 +297,7 @@ THREADGBLDEF(gtm_autorelink_keeprtn,		boolean_t)			/* do not let go of objects i
 THREADGBLDEF(open_shlib_root,			open_shlib *)			/* Anchor for open shared library list */
 THREADGBLDEF(parm_pool_ptr,			parm_pool *)			/* Pointer to the parameter pool */
 THREADGBLDEF(parms_cnt,                         unsigned int)                   /* Parameters count */
+THREADGBLDEF(statsdb_fnerr_reason,		int)				/* Failure code for "gvcst_set_statsdb_fname" */
 THREADGBLAR1DEF(zpeek_regname,			char,		NAME_ENTRY_SZ)	/* Last $ZPEEK() region specified */
 THREADGBLDEF(zpeek_regname_len,			int)				/* Length of zpeekop_regname */
 THREADGBLDEF(zpeek_reg_ptr,			gd_region *)			/* Resolved pointer for zpeekop_regname */
@@ -295,6 +312,8 @@ THREADGBLDEF(save_zhist,			zro_hist *)			/* Temp storage for zro_hist blk so con
 THREADGBLDEF(set_zroutines_cycle,		uint4)				/* Informs us if we changed $ZROUTINES between
 										 * linking a routine and invoking it
 										 */
+THREADGBLDEF(statsDB_init_defer_anchor,		statsDB_deferred_init_que_elem *) /* Anchor point for deferred init of statsDBs */
+THREADGBLDEF(statshare_opted_in,		boolean_t)			/* Flag when true shared stats collection active */
 THREADGBLDEF(trans_code_pop,			mval *)				/* trans_code holder for $ZTRAP popping */
 THREADGBLDEF(view_ydirt_str,			char *)				/* op_view working storage for ydir* ops */
 THREADGBLDEF(view_ydirt_str_len,		int4)				/* Part of op_view working storage for ydir* ops */
@@ -302,6 +321,7 @@ THREADGBLDEF(zdate_form,			int4)				/* Control for default $zdate() format */
 THREADGBLAR1DEF(zintcmd_active,			zintcmd_active_info,	ZINTCMD_LAST)	/* Interrupted timed commands */
 THREADGBLDEF(zro_root,				zro_ent *)			/* Anchor for zroutines structure entry array */
 THREADGBLDEF(zsearch_var,			lv_val *)			/* UNIX $zsearch() lookup variable */
+THREADGBLDEF(ztrap_form,			int4)				/* ztrap type indicator */
 THREADGBLDEF(poll_fds_buffer,			char *)				/* Buffer for poll() argument */
 THREADGBLDEF(poll_fds_buffer_size,		size_t)				/* Current allocated size of poll_fds_buffer */
 THREADGBLDEF(socket_handle_counter,		int)				/* Counter for generated socket handles */
@@ -402,6 +422,16 @@ THREADGBLDEF(expand_prev_key,			boolean_t)	/* Want gvcst_search_blk/gvcst_search
 								 * "gvcst_expand_key" to determine prev_key after the search.
 								 */
 THREADGBLDEF(gtm_autorelink_ctlmax,		uint4)		/* Maximum number of routines allowed for auterelink */
+/* Each process that opens a database file with O_DIRECT (which happens if asyncio=TRUE) needs to do
+ * writes from a buffer that is aligned at the filesystem-blocksize level. We ensure this in database shared
+ * memory global buffers where each buffer is guaranteed aligned at OS_PAGE_SIZE as private memory buffers that are
+ * used (e.g. dbfilop etc.). All the private memory usages will use the global variable "dio_buff.aligned".
+ * It is guaranteed to be OS_PAGE_SIZE aligned. "dio_buff.unaligned_size" holds the size of the allocated buffer
+ * and it has enough space to hold one GDS block (max db blocksize across ALL dbs opened by this process till now)
+ * with OS_PAGE_SIZE padding. "dio_buff.unaligned" points to the beginning of this unaligned buffer and
+ * "dio_buff.aligned" points to an offset within this unaligned buffer that is also OS_PAGE_SIZE aligned.
+ */
+THREADGBLDEF(dio_buff,				dio_buff_t)
 #ifdef GTM_TRIGGER
 THREADGBLDEF(gvt_triggers_read_this_tn,		boolean_t)			/* if non-zero, indicates triggers were read for
 										 * at least one gv_target in this transaction.
@@ -427,8 +457,25 @@ THREADGBLDEF(ztrigbuffLen,			int)				/* Length of used ztrigbuff buffer */
 THREADGBLDEF(ztrig_use_io_curr_device,		boolean_t)	/* Use current IO device instead of stderr/util_out_print */
 #endif
 
+THREADGBLDEF(in_ext_call,			boolean_t)	/* Indicates we are in an external call */
+
+/* linux AIO related data */
+#ifdef 	USE_LIBAIO
+THREADGBLDEF(gtm_aio_nr_events,			uint4)		/* Indicates the value of the nr_events parameter suggested for
+								 * use by io_setup().
+								 */
+#endif
+THREADGBLDEF(crit_reg_count,			int4)		/* A count of the number of regions/jnlpools where this process
+								 * has crit
+								 */
+THREADGBLDEF(ok_to_see_statsdb_regs,		boolean_t)	/* FALSE implies statsdb regions are hidden at "gd_load" time */
+THREADGBLDEF(was_open_reg_seen,			boolean_t)	/* TRUE => there is at least one region with reg->was_open = TRUE */
+THREADGBLDEF(nontp_jbuf_rsrv,			jbuf_rsrv_struct_t *)	/* Pointer to structure corresponding to reservations
+									 * on the journal buffer for current non-TP transaction.
+									 */
 /* Debug values */
 #ifdef DEBUG
+THREADGBLDEF(donot_commit,			boolean_t)			/* debug-only - see gdsfhead.h for purpose */
 THREADGBLDEF(continue_proc_cnt,			int)				/* Used by whitebox secshr test to count time
 										 * process was continued. */
 THREADGBLDEF(gtm_test_fake_enospc,		boolean_t)			/*  DEBUG-only option to enable/disable anticipatory
@@ -467,3 +514,8 @@ THREADGBLDEF(fork_without_child_wait,		boolean_t)	/*  we did a FORK but did not
 								 *  inherited shm so shm_nattch could be higher than we expect.
 								 */
 #endif	/* #ifdef DEBUG */
+/* (DEBUG_ONLY relevant points reproduced from the comment at the top of this file)
+ *   5. It is important for ANY DEBUG_ONLY fields to go at the VERY END. Failure to do this breaks gtmpcat.
+ *   6. If a DEBUG_ONLY array is declared whose dimension is a macro, then it is necessary, for gtmpcat to work,
+ *      that the macro shouldn't be defined as DEBUG_ONLY.
+ */
