@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -18,10 +18,7 @@
 #include "gtm_stdio.h"
 #include "gtm_inet.h"	/* Required for gtmsource.h */
 
-#ifdef VMS
-#include <descrip.h> /* Required for gtmsource.h */
-#endif
-
+#include "gtmio.h"
 #include "cdb_sc.h"
 #include "gdsroot.h"
 #include "gdskill.h"
@@ -44,15 +41,16 @@
 #include "interlock.h"
 #include <rtnhdr.h>
 #include "stack_frame.h"
-#ifdef GTM_TRIGGER
-# include "gv_trigger.h"
-# include "gtm_trigger.h"
-# include "gv_trigger_protos.h"
-# include "mv_stent.h"
-# include "stringpool.h"
-#endif
+#include "gv_trigger.h"
+#include "gtm_trigger.h"
+#include "gv_trigger_protos.h"
+#include "mv_stent.h"
+#include "stringpool.h"
+#include "gtm_trigger_trc.h"
 #include "tp_frame.h"
 #include "tp_restart.h"
+#include "is_file_identical.h"
+#include "anticipatory_freeze.h"
 
 /* Include prototypes */
 #include "gvcst_kill_blk.h"
@@ -76,6 +74,7 @@
 #include "have_crit.h"
 #include "error.h"
 #include "gtmimagename.h" /* needed for spanning nodes */
+#include "gtm_repl_multi_inst.h" /* for DISALLOW_MULTIINST_UPDATE_IN_TP */
 
 GBLREF	gd_region		*gv_cur_region;
 GBLREF	gv_key			*gv_currkey, *gv_altkey;
@@ -87,11 +86,13 @@ GBLREF	uint4			dollar_tlevel;
 GBLREF	sgmnt_addrs		*cs_addrs;
 GBLREF	sgmnt_data_ptr_t	cs_data;
 GBLREF	sgm_info		*sgm_info_ptr;
+GBLREF	sgm_info		*first_sgm_info;
 GBLREF	unsigned char		cw_set_depth;
 GBLREF	unsigned int		t_tries;
 GBLREF	boolean_t		need_kip_incr;
 GBLREF	uint4			update_trans;
-GBLREF	jnlpool_addrs		jnlpool;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool_head;
 GBLREF	sgmnt_addrs		*kip_csa;
 GBLREF	boolean_t		skip_dbtriggers;	/* see gbldefs.c for description of this global */
 GBLREF	stack_frame		*frame_pointer;
@@ -111,7 +112,7 @@ GBLREF	uint4			update_array_size; /* needed for the ENSURE_UPDATE_ARRAY_SPACE ma
 GBLREF	jnl_gbls_t		jgbl;
 GBLREF	boolean_t		donot_INVOKE_MUMTSTART;
 #endif
-UNIX_ONLY(GBLREF	boolean_t 		span_nodes_disallowed;)
+GBLREF	boolean_t 		span_nodes_disallowed;
 
 error_def(ERR_TPRETRY);
 error_def(ERR_GVKILLFAIL);
@@ -122,9 +123,6 @@ LITREF	mval	*fndata_table[2][2];
 #endif
 LITREF	mval	literal_batch;
 
-#define SKIP_ASSERT_TRUE	TRUE
-#define SKIP_ASSERT_FALSE	FALSE
-
 #define	GOTO_RETRY(CDB_STATUS, SKIP_ASSERT)							\
 {												\
 	assert((CDB_STAGNATE > t_tries) || IS_FINAL_RETRY_CODE(CDB_STATUS) || SKIP_ASSERT);	\
@@ -155,9 +153,15 @@ void	gvcst_kill(boolean_t do_subtree)
 		if (!spanstat)
 			return;
 	}
-	VMS_ONLY(assert(FALSE));
-#	ifdef UNIX
 	RTS_ERROR_IF_SN_DISALLOWED;
+#	ifdef DEBUG
+	/* ^%Y* should never be killed (op_gvkill/op_gvzwithdraw would have issued a ERR_PCTYRESERVED error in that case).
+	 * The only exception is if we are removing a ^%YGS record from the statsdb (caller "gvcst_remove_statsDB_linkage"
+	 * but in that case we would have reg->read_only set to FALSE for a statsdb region name. Account for that.
+	 */
+	assert((RESERVED_NAMESPACE_LEN > gv_currkey->end) || (0 != MEMCMP_LIT(gv_currkey->base, RESERVED_NAMESPACE))
+		|| (IS_STATSDB_REGNAME(gv_cur_region) && !gv_cur_region->read_only));
+#	endif
 	oldend = gv_currkey->end;
 	/* Almost certainly have a spanning node to zkill. So start a TP transaction to deal with it. */
 	if (!dollar_tlevel)
@@ -188,7 +192,6 @@ void	gvcst_kill(boolean_t do_subtree)
 		REVERT; /* remove our condition handler */
 	}
 	assert(save_dollar_tlevel == dollar_tlevel);
-#	endif
 }
 
 void	gvcst_kill2(boolean_t do_subtree, boolean_t *span_status, boolean_t killing_chunks)
@@ -255,7 +258,7 @@ void	gvcst_kill2(boolean_t do_subtree, boolean_t *span_status, boolean_t killing
 			ztwormhole_used = FALSE;
 		}
 	)
-	JNLPOOL_INIT_IF_NEEDED(csa, csd, cnl);
+	JNLPOOL_INIT_IF_NEEDED(csa, csd, cnl, SCNDDBNOUPD_CHECK_TRUE);
 	if (!dollar_tlevel)
 	{
 		kill_set_head.next_kill_set = NULL;
@@ -269,6 +272,7 @@ void	gvcst_kill2(boolean_t do_subtree, boolean_t *span_status, boolean_t killing
 		prev_update_trans = sgm_info_ptr->update_trans;
 	assert(('\0' != gv_currkey->base[0]) && gv_currkey->end);
 	DBG_CHECK_GVTARGET_GVCURRKEY_IN_SYNC(CHECK_CSA_TRUE);
+	DISALLOW_MULTIINST_UPDATE_IN_TP(dollar_tlevel, jnlpool_head, csa, first_sgm_info, FALSE);
 	T_BEGIN_SETORKILL_NONTP_OR_TP(ERR_GVKILLFAIL);
 	assert(NULL != update_array);
 	assert(NULL != update_array_ptr);
@@ -457,7 +461,7 @@ void	gvcst_kill2(boolean_t do_subtree, boolean_t *span_status, boolean_t killing
 research:
 		if (gvt_root)
 		{
-#if defined(DEBUG) && defined(UNIX)
+			#if defined(DEBUG)
 			if (gtm_white_box_test_case_enabled && (WBTEST_ANTIFREEZE_GVKILLFAIL == gtm_white_box_test_case_number))
 			{
 				cdb_status = cdb_sc_blknumerr;
@@ -466,7 +470,7 @@ research:
 				 */
 				GOTO_RETRY(cdb_status, SKIP_ASSERT_TRUE);
 			}
-#endif
+			#endif
 			if (cdb_sc_normal != (cdb_status = gvcst_search(gv_currkey, NULL)))
 				GOTO_RETRY(cdb_status, SKIP_ASSERT_FALSE);
 			assert(gv_altkey->top == gv_currkey->top);
@@ -482,7 +486,6 @@ research:
 				base[++end] = KEY_DELIMITER;
 			} else
 			{
-#				ifdef UNIX
 				target_key_size = gv_currkey->end + 1;
 				bh = &gv_target->hist.h[0];
 				key_exists = (target_key_size == bh->curr_rec.match);
@@ -525,7 +528,6 @@ research:
 						}
 					}
 				}
-#				endif
 				if (killing_chunks)
 				{	/* Second call of gvcst_kill2 within TP transaction in gvcst_kill
 					 * Kill all hidden subscripts...
