@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -13,7 +13,9 @@
 #include "mdef.h"
 
 #include "gtm_string.h"
+#include "gtm_stdio.h"
 
+#include "gtmio.h"
 #include "gdsroot.h"
 #include "gdsblk.h"
 #include "gtm_facility.h"
@@ -39,9 +41,8 @@
 #include "get_spec.h"
 #include "collseq.h"
 #include "gtmimagename.h"
-#ifdef UNIX
 #include "error.h"
-#endif
+#include "io.h"
 
 GBLREF	gv_key		*gv_currkey, *gv_altkey;
 GBLREF	int4		gv_keysize;
@@ -54,7 +55,6 @@ GBLREF	unsigned int	t_tries;
 GBLREF	gv_namehead	*reset_gv_target;
 GBLREF	boolean_t	mu_reorg_process;
 GBLREF	boolean_t	mupip_jnl_recover;
-#ifdef UNIX
 # ifdef DEBUG
 GBLREF	boolean_t	is_rcvr_server;
 GBLREF	boolean_t	is_src_server;
@@ -67,7 +67,6 @@ GBLREF	trans_num	start_tn;
 GBLREF	uint4		update_trans;
 GBLREF	inctn_opcode_t	inctn_opcode;
 GBLREF	uint4		t_err;
-#endif
 #ifdef GTM_TRIGGER
 GBLREF	boolean_t		skip_INVOKE_RESTART;
 #endif
@@ -104,7 +103,6 @@ static	mstr	global_collation_mstr;
 	TRIG_TP_SET_SGM;									\
 }
 
-#ifdef UNIX
 #define SAVE_ROOTSRCH_ENTRY_STATE								\
 {												\
 	int				idx;							\
@@ -129,7 +127,7 @@ static	mstr	global_collation_mstr;
 	 * t_begin that it is 0.								\
 	 */											\
 	update_trans = 0;									\
-	inctn_opcode = 0;									\
+	inctn_opcode = inctn_invalid_op;							\
 	rootsrch_ctxt_ptr->t_err = t_err;							\
 	rootsrch_ctxt_ptr->hold_onto_crit = cs_addrs->hold_onto_crit;				\
 	if (CDB_STAGNATE <= t_tries)								\
@@ -211,7 +209,6 @@ void gvcst_redo_root_search()
 	RESTORE_ROOTSRCH_ENTRY_STATE;
 	REVERT;
 }
-#endif
 
 enum cdb_sc gvcst_root_search(boolean_t donot_restart)
 {
@@ -230,7 +227,7 @@ enum cdb_sc gvcst_root_search(boolean_t donot_restart)
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
-	assert((dba_bg == REG_ACC_METH(gv_cur_region)) || (dba_mm == REG_ACC_METH(gv_cur_region)));
+	assert(IS_REG_BG_OR_MM(gv_cur_region));
 	SET_GV_ALTKEY_TO_GBLNAME_FROM_GV_CURRKEY;	/* set up gv_altkey to be just the gblname */
 	save_targ = gv_target;
 	/* Check if "gv_target->gvname" matches "gv_altkey->base". If not, there is a name mismatch (out-of-design situation).
