@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2003-2016 Fidelity National Information	*
+ * Copyright (c) 2003-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -33,6 +33,7 @@
 #include "filestruct.h"
 #include "jnl.h"
 #include "gtmio.h"
+#include "sgtm_putmsg.h"
 #include "eintr_wrappers.h"
 #include "repl_msg.h"
 #include "gtmsource.h"
@@ -44,27 +45,34 @@
 #include "get_fs_block_size.h"
 #include "anticipatory_freeze.h"
 
-GBLREF	jnlpool_ctl_ptr_t	jnlpool_ctl;
-GBLREF	boolean_t		pool_init;
+GBLREF	int			pool_init;
 GBLREF  jnl_process_vector      *prc_vec;
 GBLREF	jnl_gbls_t		jgbl;
 GBLREF	uint4			mu_reorg_encrypt_in_prog;
 
+error_def(ERR_CRYPTJNLMISMATCH);
 error_def(ERR_FILEIDMATCH);
-error_def(ERR_JNLOPNERR);
-error_def(ERR_JNLRDERR);
 error_def(ERR_JNLBADRECFMT);
+error_def(ERR_JNLOPNERR);
+error_def(ERR_JNLPREVRECOV);
+error_def(ERR_JNLRDERR);
 error_def(ERR_JNLRECTYPE);
+error_def(ERR_JNLSWITCHRETRY);
 error_def(ERR_JNLTRANSGTR);
 error_def(ERR_JNLTRANSLSS);
-error_def(ERR_JNLWRERR);
 error_def(ERR_JNLVSIZE);
+error_def(ERR_JNLWRERR);
 error_def(ERR_PREMATEOF);
-error_def(ERR_JNLPREVRECOV);
-error_def(ERR_CRYPTJNLMISMATCH);
+
+#define RETURN_AND_SET_JPC(ERR, ERR2, BUF)	\
+MBSTART {					\
+	jpc->status = ERR2;			\
+	jpc->err_str = BUF;			\
+	return ERR;				\
+} MBEND
 
 /* note: returns 0 on success */
-uint4 jnl_file_open_common(gd_region *reg, off_jnl_t os_file_size)
+uint4 jnl_file_open_common(gd_region *reg, off_jnl_t os_file_size, char *buff)
 {
 	sgmnt_addrs		*csa;
 	sgmnt_data_ptr_t	csd;
@@ -76,7 +84,8 @@ uint4 jnl_file_open_common(gd_region *reg, off_jnl_t os_file_size)
 	unsigned char		*eof_rec_buffer;
 	unsigned char		eof_rec[(DISK_BLOCK_SIZE * 2) + MAX_IO_BLOCK_SIZE];
 	off_jnl_t		adjust;
-	uint4			jnl_fs_block_size, read_write_size, read_size;
+	uint4			end_of_data, jnl_fs_block_size, read_write_size, read_size;
+	int4			status;
 	gtm_uint64_t		header_virtual_size;
 
 	csa = &FILE_INFO(reg)->s_addrs;
@@ -84,6 +93,7 @@ uint4 jnl_file_open_common(gd_region *reg, off_jnl_t os_file_size)
 	jpc = csa->jnl;
 	jb = jpc->jnl_buff;
 	jpc->status = jpc->status2 = SS_NORMAL;
+	assert(NULL != buff);
 	jnl_fs_block_size = get_fs_block_size(jpc->channel);
 	/* check that the filesystem block size is a power of 2 as we do a lot of calculations below assuming this is the case */
 	assert(!(jnl_fs_block_size & (jnl_fs_block_size - 1)));
@@ -107,65 +117,70 @@ uint4 jnl_file_open_common(gd_region *reg, off_jnl_t os_file_size)
 		 * entire unaligned journal file size is lesser than the aligned journal file header size.
 		 */
 		assert(ERR_PREMATEOF == jpc->status);
-		return ERR_JNLRDERR;
+		return ERR_JNLRDERR; /* Has one !AD parameter, the journal file, which jnl_send_oper() provides */
 	}
 	/* Check if the header format matches our format. Cannot access any fields inside header unless this matches */
 	CHECK_JNL_FILE_IS_USABLE(header, jpc->status, FALSE, 0, NULL);	/* FALSE => NO gtm_putmsg even if errors */
 	if (SS_NORMAL != jpc->status)
-		return ERR_JNLOPNERR;
-	adjust = header->end_of_data & (jnl_fs_block_size - 1);
-	/* Read the journal JRT_EOF at header->end_of_data offset.
+	{
+		sgtm_putmsg(buff, VARLSTCNT(7) ERR_JNLOPNERR, 4, JNL_LEN_STR(csd), DB_LEN_STR(reg), jpc->status);
+		RETURN_AND_SET_JPC(ERR_JNLOPNERR, jpc->status, buff);
+	}
+	end_of_data = header->end_of_data;
+	adjust = end_of_data & (jnl_fs_block_size - 1);
+	/* Read the journal JRT_EOF at end_of_data offset.
 	 * Make sure the buffer being read to is big enough and that as part of the read,
 	 * we never touch touch the journal file header territory.
 	 */
 	read_size = ROUND_UP2((EOF_RECLEN + adjust), jnl_fs_block_size);
 	assert(eof_rec_buffer + read_size <= ARRAYTOP(eof_rec));
-	assert(header->end_of_data - adjust >= JNL_HDR_LEN);
-	DO_FILE_READ(jpc->channel, header->end_of_data - adjust, eof_rec_buffer, read_size, jpc->status, jpc->status2);
+	assert(end_of_data - adjust >= JNL_HDR_LEN);
+	DO_FILE_READ(jpc->channel, end_of_data - adjust, eof_rec_buffer, read_size, jpc->status, jpc->status2);
 	if (SS_NORMAL != jpc->status)
-	{
-		return ERR_JNLRDERR;
-	}
+		return ERR_JNLRDERR; /* Has one !AD parameter, the journal file, which jnl_send_oper() provides */
 	if (header->prev_recov_end_of_data)
 	{
 		/* not possible for run time. In case it happens user must fix it */
-		jpc->status = ERR_JNLPREVRECOV;
-		return ERR_JNLOPNERR;
+		sgtm_putmsg(buff, VARLSTCNT(7) ERR_JNLOPNERR, 4, JNL_LEN_STR(csd), DB_LEN_STR(reg), ERR_JNLPREVRECOV);
+		RETURN_AND_SET_JPC(ERR_JNLOPNERR, ERR_JNLPREVRECOV, buff);
 	}
 	if (!is_gdid_file_identical(&FILE_ID(reg), (char *)header->data_file_name, header->data_file_name_length))
 	{
 		rts_error_csa(CSA_ARG(csa) VARLSTCNT(7) ERR_JNLOPNERR, 4, JNL_LEN_STR(csd), DB_LEN_STR(reg), ERR_FILEIDMATCH);
-		assert(FALSE);	/* we dont expect the rts_error in the line above to return */
+		assert(FALSE);	/* we don't expect the rts_error in the line above to return */
 		return ERR_JNLOPNERR;
 	}
 	memcpy(&eof_record, (unsigned char *)eof_rec_buffer + adjust, EOF_RECLEN);
 	if (JRT_EOF != eof_record.prefix.jrec_type)
 	{
-		jpc->status = ERR_JNLRECTYPE;
-		return ERR_JNLOPNERR;
+		sgtm_putmsg(buff, VARLSTCNT(7) ERR_JNLOPNERR, 4, JNL_LEN_STR(csd), DB_LEN_STR(reg), ERR_JNLRECTYPE);
+		RETURN_AND_SET_JPC(ERR_JNLOPNERR, ERR_JNLRECTYPE, buff);
 	}
 	if (eof_record.prefix.tn != csd->trans_hist.curr_tn)
 	{
 		if (eof_record.prefix.tn < csd->trans_hist.curr_tn)
-			jpc->status = ERR_JNLTRANSLSS;
+			status = ERR_JNLTRANSLSS;
 		else
-			jpc->status = ERR_JNLTRANSGTR;
-		return ERR_JNLOPNERR;
+			status = ERR_JNLTRANSGTR;
+		sgtm_putmsg(buff, VARLSTCNT(10) ERR_JNLOPNERR, 4, JNL_LEN_STR(csd), DB_LEN_STR(reg),
+				status, 2, &eof_record.prefix.tn, &csd->trans_hist.curr_tn);
+		RETURN_AND_SET_JPC(ERR_JNLOPNERR, status, buff);
 	}
 	if (eof_record.suffix.suffix_code != JNL_REC_SUFFIX_CODE ||
 		eof_record.suffix.backptr != eof_record.prefix.forwptr)
 	{
-		jpc->status = ERR_JNLBADRECFMT;
-		return ERR_JNLOPNERR;
+		sgtm_putmsg(buff, VARLSTCNT(11) ERR_JNLOPNERR, 4, JNL_LEN_STR(csd), DB_LEN_STR(reg),
+				ERR_JNLBADRECFMT, 3, JNL_LEN_STR(csd), adjust);
+		RETURN_AND_SET_JPC(ERR_JNLOPNERR, ERR_JNLBADRECFMT, buff);
 	}
 	if (!mu_reorg_encrypt_in_prog && !SAME_ENCRYPTION_SETTINGS(header, csd))
 	{	/* We expect encryption settings in the journal to be in sync with those in the file header. The only exception is
 		 * MUPIP REORG -ENCRYPT, which switches the journal file upon changing encryption-specific fields in the file
 		 * header, thus temporarily violating this expectation.
 		 */
-		send_msg_csa(CSA_ARG(csa) VARLSTCNT(6) ERR_CRYPTJNLMISMATCH, 4, JNL_LEN_STR(csd), DB_LEN_STR(reg));
-		jpc->status = ERR_CRYPTJNLMISMATCH;
-		return ERR_JNLOPNERR;
+		sgtm_putmsg(buff, VARLSTCNT(12) ERR_JNLOPNERR, 4, JNL_LEN_STR(csd), DB_LEN_STR(reg),
+				ERR_CRYPTJNLMISMATCH, 4, JNL_LEN_STR(csd), DB_LEN_STR(reg));
+		RETURN_AND_SET_JPC(ERR_JNLOPNERR, ERR_CRYPTJNLMISMATCH, buff);
 	}
 	assert(header->eov_tn == eof_record.prefix.tn);
 	header->eov_tn = eof_record.prefix.tn;
@@ -179,10 +194,10 @@ uint4 jnl_file_open_common(gd_region *reg, off_jnl_t os_file_size)
 	if ((ROUND_UP2((header_virtual_size * DISK_BLOCK_SIZE), jnl_fs_block_size) < os_file_size)
 		|| (header->jnl_deq && 0 != ((header_virtual_size - header->jnl_alq) % header->jnl_deq)))
 	{
-		send_msg_csa(CSA_ARG(csa) VARLSTCNT(8) ERR_JNLVSIZE, 6, JNL_LEN_STR(csd), header->virtual_size,
-			 header->jnl_alq, header->jnl_deq, os_file_size, jnl_fs_block_size);
-		jpc->status = ERR_JNLVSIZE;
-		return ERR_JNLOPNERR;
+		sgtm_putmsg(buff, VARLSTCNT(14) ERR_JNLOPNERR, 4, JNL_LEN_STR(csd), DB_LEN_STR(reg),
+				ERR_JNLVSIZE, 7, JNL_LEN_STR(csd), header->virtual_size,
+					header->jnl_alq, header->jnl_deq, os_file_size, jnl_fs_block_size);
+		RETURN_AND_SET_JPC(ERR_JNLOPNERR, ERR_JNLVSIZE, buff);
 	}
 	/* For performance reasons (to be able to do aligned writes to the journal file), we need to ensure the journal buffer
 	 * address is filesystem-block-size aligned in Unix. Although this is needed only in case of sync_io/direct-io, we ensure
@@ -195,7 +210,8 @@ uint4 jnl_file_open_common(gd_region *reg, off_jnl_t os_file_size)
 											+ JNL_SHARE_SIZE(csd)));
 	assert((sm_uc_ptr_t)jb == ((sm_uc_ptr_t)csa->nl + NODE_LOCAL_SPACE(csd) + JNL_NAME_EXP_SIZE));
 	jb->last_eof_written = header->last_eof_written;
-	jb->freeaddr = jb->dskaddr = jb->fsync_dskaddr = header->end_of_data;
+	jb->freeaddr = jb->dskaddr = jb->fsync_dskaddr = end_of_data;
+	SET_JBP_RSRV_FREEADDR(jb, end_of_data);	/* sets both jb->rsrv_freeaddr & jb->rsrv_free */
 	jb->post_epoch_freeaddr = jb->end_of_data_at_open = jb->freeaddr;
 	jb->fs_block_size = jnl_fs_block_size;
 	/* The following is to make sure that the data in jnl_buffer is aligned with the data in the
@@ -203,7 +219,7 @@ uint4 jnl_file_open_common(gd_region *reg, off_jnl_t os_file_size)
 	 * alignment with respect to jb->size implies alignment with jnl_fs_block_size.
 	 */
 	assert(0 == (jb->size % jnl_fs_block_size));
-	jb->free = jb->dsk = header->end_of_data % jb->size;
+	jb->free = jb->dsk = end_of_data % jb->size;
 	SET_LATCH_GLOBAL(&jb->fsync_in_prog_latch, LOCK_AVAILABLE);
 	SET_LATCH_GLOBAL(&jb->io_in_prog_latch, LOCK_AVAILABLE);
 	assert(0 == (jnl_fs_block_size % DISK_BLOCK_SIZE));
@@ -220,8 +236,17 @@ uint4 jnl_file_open_common(gd_region *reg, off_jnl_t os_file_size)
 	jb->before_images = header->before_images;
 	jb->epoch_tn = eof_record.prefix.tn;
 	csd->jnl_checksum = header->checksum;
-	assert(4 == SIZEOF(header->alignsize));	/* so we can use the 32bit version of ceil_log2 */
-	jb->log2_of_alignsize = ceil_log2_32bit(header->alignsize);
+	assert(4 == SIZEOF(header->alignsize));
+	assert(((gtm_uint64_t)JNL_MAX_ALIGNSIZE * 512) < (gtm_uint64_t)MAXUINT4); /* ensure 4-byte "alignsize" will not overflow */
+	jb->alignsize = header->alignsize;
+	/* Use "gtm_uint64_t" typecast below to avoid 4G overflow issues with the ROUND_UP2 */
+	jb->next_align_addr = (uint4)(ROUND_UP2(((gtm_uint64_t)jb->freeaddr + MIN_ALIGN_RECLEN), header->alignsize)
+													- MIN_ALIGN_RECLEN);
+	SET_LATCH_GLOBAL(&jb->phase2_commit_latch, LOCK_AVAILABLE);
+	jb->phase2_commit_index1 = jb->phase2_commit_index2 = 0;
+	/* The below 2 lines are relied upon by "mutex_salvage" */
+	jb->phase2_commit_array[JNL_PHASE2_COMMIT_ARRAY_SIZE - 1].start_freeaddr = jb->end_of_data_at_open;
+	jb->phase2_commit_array[JNL_PHASE2_COMMIT_ARRAY_SIZE - 1].tot_jrec_len = 0;
 	assert(header->autoswitchlimit == csd->autoswitchlimit);
 	assert(header->jnl_alq == csd->jnl_alq);
 	assert(header->jnl_deq == csd->jnl_deq);
@@ -233,6 +258,10 @@ uint4 jnl_file_open_common(gd_region *reg, off_jnl_t os_file_size)
 	jb->next_epoch_time = (uint4)(MID_TIME(prc_vec->jpv_time) + jb->epoch_interval);
 	jb->max_jrec_len = header->max_jrec_len;
 	memcpy(&header->who_opened, prc_vec, SIZEOF(jnl_process_vector));
+	if (header->is_not_latest_jnl)
+	{	/* Magic message to indicate that we should try a switch without cutting links. */
+		RETURN_AND_SET_JPC(ERR_JNLSWITCHRETRY, 0, buff);
+	}
 	header->crash = TRUE;	/* in case this processes is crashed, this will remain TRUE */
 	JNL_DO_FILE_WRITE(csa, csd->jnl_file_name, jpc->channel, 0, header, read_write_size, jpc->status, jpc->status2);
 	if (SS_NORMAL != jpc->status)
