@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2011 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -25,11 +26,6 @@ void lke_help(void);
 void lke_show(void);
 void lke_show_memory(mlk_shrblk_ptr_t bhead, char *prefix);
 
-#ifdef VMS
-void lke(void);
-#else
 CONDITION_HANDLER(lke_ctrlc_handler);
-#endif
-void lke_setgdr(void);
 
 #endif /* __LKE_H__ */
