@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -325,21 +325,15 @@ int m_set(void)
 				put->operand[1] = put_ilit(indir_set);
 				return TRUE;
 			}
-			if (TREF(side_effect_handling))
-			{	/* save and restore the variable lookup for true left-to-right evaluation */
-				INSERT_INDSAVGLVN(control_slot, v, ANY_SLOT, 0);	/* 0 flag to defer global reference */
-				if (!used_glvn_slot)
-				{
-					used_glvn_slot = TRUE;
-					first_control_slot = control_slot;
-				}
-				put = maketriple(OC_STOGLVN);
-				put->operand[0] = control_slot;
-			} else
-			{	/* quick and dirty old way */
-				put = maketriple(OC_INDSET);
-				put->operand[0] = v;
+			/* save and restore the variable lookup for true left-to-right evaluation */
+			INSERT_INDSAVGLVN(control_slot, v, ANY_SLOT, 0);	/* 0 flag to defer global reference */
+			if (!used_glvn_slot)
+			{
+				used_glvn_slot = TRUE;
+				first_control_slot = control_slot;
 			}
+			put = maketriple(OC_STOGLVN);
+			put->operand[0] = control_slot;
 			put->operand[1] = resptr;
 			dqins(targchain.exorder.bl, exorder, put);
 			break;
@@ -384,7 +378,6 @@ int m_set(void)
 					assert(OC_RTERROR == tmp->opcode);
 					dqdel(tmp, exorder);
 					dqins(targchain.exorder.bl, exorder, tmp);
-					CHKTCHAIN(&targchain);
 				}
 				break;
 			}
@@ -403,7 +396,6 @@ int m_set(void)
 				assert(OC_RTERROR == tmp->opcode);
 				dqdel(tmp, exorder);
 				dqins(targchain.exorder.bl, exorder, tmp);
-				CHKTCHAIN(&targchain);
 				advancewindow();	/* skip past the function name */
 				advancewindow();	/* skip past the left paren */
 				/* Parse the remaining arguments until corresponding RIGHT-PAREN/SPACE/EOL is reached */
@@ -507,7 +499,6 @@ int m_set(void)
 				s->operand[0] = put_tref(get);
 				/* Code to fetch args for target triple are on targchain. Put get there now too. */
 				dqins(targchain.exorder.bl, exorder, get);
-				CHKTCHAIN(&targchain);
 				if (!is_extract)
 				{	/* Set $[z]piece */
 					delimiter = newtriple(OC_PARAMETER);
@@ -670,7 +661,6 @@ int m_set(void)
 			{
 				dqins(targchain.exorder.bl, exorder, s);
 				dqins(targchain.exorder.bl, exorder, put);
-				CHKTCHAIN(&targchain);
 				/* Put result operand on the chain. End of chain depends on whether or not
 				 * we are calling the shortcut or the full set-piece code
 				 */
