@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -92,11 +92,13 @@ typedef unsigned int 	uint4;		/* 4-byte unsigned integer */
 #define FIFO_PERMISSION		010666 /* fifo with RW permissions for owner, group, other */
 
 #include <inttypes.h>
+#include <stdarg.h>
 #include "mdefsa.h"
 #include "gtm_common_defs.h"
 #include <mdefsp.h>
 #include "gtm_sizeof.h"
 #include "gtm_threadgbl.h"
+
 /* Anchor for thread-global structure rather than individual global vars */
 GBLREF void	*gtm_threadgbl;		/* Accessed through TREF macro in gtm_threadgbl.h */
 
@@ -291,6 +293,7 @@ typedef UINTPTR_T uintszofptr_t;
 #define MAX_DIGITS_IN_EXP       2       /* maximum number of decimal digits in an exponent */
 #define MAX_HOST_NAME_LEN	256
 #define MAX_LONG_IN_DOUBLE	0xFFFFFFFFFFFFF /*Max Fraction part in IEEE double format*/
+#define MAX_INT_IN_BYTE		255
 
 #ifndef _AIX
 #	ifndef __sparc
@@ -488,6 +491,30 @@ mval *underr_strict(mval *start, ...);
 /* Note MV_FORCE_CANONICAL currently only used in op_add() when vars are known to be defined so no MV_FORCE_DEFINED()
    macro has been added. If uses are added, this needs to be revisited. 01/2008 se
 */
+#define MV_FORCE_MSTIMEOUT(TMV, TMS, NOTACID)	/* requires a flock of include files especially for TP */		\
+MBSTART {					/* also requires threaddef DCL and SETUP*/				\
+	GBLREF uint4		dollar_tlevel;										\
+	GBLREF uint4		dollar_trestart;									\
+															\
+															\
+	MV_FORCE_NUM(TMV);												\
+	if (NO_M_TIMEOUT == TMV->m[1])											\
+		TMS = NO_M_TIMEOUT;											\
+	else														\
+	{														\
+		assert(MV_BIAS >= 1000);	/* if formats change scale may need attention */			\
+		/* negative becomes 0 larger than MAXPOSINT4 caps to MAXPOSINT4 */					\
+		if (!(TMV->mvtype & MV_INT))										\
+			TMV->e += 3;	/* for non-ints, bump exponent to get millisecs from MV_FORCE_INT */		\
+		TMS = ((TMV->mvtype & MV_INT) ? TMV->m[1] : MIN(MAXPOSINT4, MV_FORCE_INT(TMV)));			\
+		if (!(TMV->mvtype & MV_INT))										\
+			TMV->e -= 3;	/* if we messed with the exponent, restore it to its original value */		\
+		if (0 > TMS)												\
+			TMS = 0;											\
+	}														\
+	if ((TREF(tpnotacidtime)).m[1] < TMS)										\
+		TPNOTACID_CHECK(NOTACID);										\
+} MBEND
 #define MV_FORCE_CANONICAL(X)	((((X)->mvtype & MV_NM) == 0 ? s2n(X) : 0 ) \
 				 ,((X)->mvtype & MV_NUM_APPROX ? (X)->mvtype &= MV_NUM_MASK : 0 ))
 #define MV_IS_NUMERIC(X)	(((X)->mvtype & MV_NM) != 0)
@@ -636,6 +663,7 @@ void dec_err(int4 msgnum, ...);
 #error unsupported platform
 #endif
 void stx_error(int in_error, ...);
+void stx_error_va(int in_error, va_list args);
 void ins_errtriple(int4 in_error);
 
 int4 timeout2msec(int4 timeout);
@@ -668,18 +696,18 @@ int4 timeout2msec(int4 timeout);
 #define	DB_LEN_STR(reg)			(reg)->dyn.addr->fname_len, (reg)->dyn.addr->fname
 #define	REG_STR_LEN(reg)		(reg)->rname, (reg)->rname_len
 #define	REG_LEN_STR(reg)		(reg)->rname_len, (reg)->rname
+#define	SEG_LEN_STR(seg)		(seg)->sname_len, (seg)->sname
 #define	JNL_STR_LEN(csd)		(csd)->jnl_file_name, (csd)->jnl_file_len
 #define	JNL_LEN_STR(csd)		(csd)->jnl_file_len, (csd)->jnl_file_name
 
 #define	FAB_LEN_STR(fab)		(fab)->fab$b_fns, (fab)->fab$l_fna
 /* *********************************************************************************************************** */
 
-
 #ifdef DEBUG
 /* Original debug code has been removed since it was superfluous and did not work on all platforms. SE 03/01 */
-# define SET_TRACEABLE_VAR(var,value) var = value;
+# define SET_TRACEABLE_VAR(var,value) var = value
 #else
-# define SET_TRACEABLE_VAR(var,value) var = value;
+# define SET_TRACEABLE_VAR(var,value) var = value
 #endif
 
 /* If this is unix, we have a faster sleep for short sleeps ( < 1 second) than doing a hiber start.
@@ -693,41 +721,28 @@ void m_usleep(int useconds);
 #endif
 
 #ifdef UNIX
-#	define UNIX_ONLY(X)		X
-#	define UNIX_ONLY_COMMA(X)	X,
+#	define UNIX_ONLY(X)			X
+#	define UNIX_ONLY_COMMA(X)		X,
+#	define UNSUPPORTED_PLATFORM_CHECK
+	/* z/OS is not able to have dynamic file extensions while running in MM access mode
+	 * If multiple processes are accessing the same mapped file, and one process needs to extend/remap the file,
+	 * all the other processes must also unmap the file.
+	 * This same comment is in the test framework in set_gtm_machtype.csh.  If this comment is updated, also update the other.
+	 */
+#	if !defined(__MVS__)
+#		define MM_FILE_EXT_OK
+#	else
+#		undef MM_FILE_EXT_OK
+#	endif
 #else
 #	define UNIX_ONLY(X)
 #	define UNIX_ONLY_COMMA(X)
-#endif
-
-/* z/OS is not able to have dynamic file extensions while running in MM access mode
- * If multiple processes are accessing the same mapped file, and one process needs to extend/remap the file,
- * all the other processes must also unmap the file.
- *
- * This same comment is in the test framework in set_gtm_machtype.csh.  If this comment is updated, also update the other.
- */
-#ifdef UNIX
-#	if !defined(__MVS__)
-#	define MM_FILE_EXT_OK
-#	else
-#	undef MM_FILE_EXT_OK
-#	endif
-#endif
-
-#ifdef VMS
-#	define VMS_ONLY(X)		X
-#	define VMS_ONLY_COMMA(X)	X,
-#else
-#	define VMS_ONLY(X)
-#	define VMS_ONLY_COMMA(X)
-#endif
-
-#if (defined(UNIX) || defined(VMS))
-#	define UNSUPPORTED_PLATFORM_CHECK
-#else
 #	define UNSUPPORTED_PLATFORM_CHECK	#error UNSUPPORTED PLATFORM
 #endif
 
+#define VMS_ONLY(X)
+#define VMS_ONLY_COMMA(X)
+
 /* Note the macros below refer to the UNIX Shared Binary Support. Because the
  * support is *specifically* for the UNIX platform, "NON_USHBIN_ONLY()" will
  * also be true for VMS even though that platform does have shared binary support
@@ -860,9 +875,9 @@ typedef struct
 #define	IS_PTR_IN_RANGE(ptr, ptr_lo, ptr_hi)								\
 	(((sm_uc_ptr_t)(ptr) >= (sm_uc_ptr_t)(ptr_lo)) && ((sm_uc_ptr_t)(ptr) < (sm_uc_ptr_t)(ptr_hi)))
 
-#define	IS_PTR_2BYTE_ALIGNED(ptr)	(0 == (((uintszofptr_t)ptr) % 2))
-#define	IS_PTR_4BYTE_ALIGNED(ptr)	(0 == (((uintszofptr_t)ptr) % 4))
-#define	IS_PTR_8BYTE_ALIGNED(ptr)	(0 == (((uintszofptr_t)ptr) % 8))
+#define	IS_PTR_2BYTE_ALIGNED(ptr)	(0 == (((uintszofptr_t)ptr) & 0x1))
+#define	IS_PTR_4BYTE_ALIGNED(ptr)	(0 == (((uintszofptr_t)ptr) & 0x3))
+#define	IS_PTR_8BYTE_ALIGNED(ptr)	(0 == (((uintszofptr_t)ptr) & 0x7))
 
 #ifdef DB64
 # ifdef __osf__
@@ -1423,6 +1438,8 @@ typedef gtm_uint64_t	gtm_off_t;
 #define MAXUINT8	((gtm_uint64_t)-1)
 #define MAXUINT4	((uint4)-1)
 #define MAXUINT2	((unsigned short)-1)
+
+#define MAXINT4		(MAXUINT4/2)
 #define	MAXINT2		(MAXUINT2/2)
 
 /* On platforms that support native 8 byte operations (such as Alpha), an assignment to an 8 byte field is atomic. On other
@@ -1730,19 +1747,6 @@ typedef enum
 		SPRINTF((char *)out + i, "%02X", (unsigned char)in[i/2]);	\
 }
 
-#ifdef UNIX
-#	define	GTM_SNAPSHOT
-#	define	NON_GTM_SNAPSHOT_ONLY(X)
-#	define	GTM_SNAPSHOT_ONLY(X)		X
-#else
-#	define	NON_GTM_SNAPSHOT_ONLY(X)	X
-#	define	GTM_SNAPSHOT_ONLY(X)
-#endif
-
-#define	GTM_TRUNCATE
-#define	NON_GTM_TRUNCATE_ONLY(X)
-#define	GTM_TRUNCATE_ONLY(X)		X
-
 /* Currently triggers are supported only on UNIX */
 #if defined(UNIX)
 #	define	GTM_TRIGGER
