@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -60,7 +60,6 @@
 #include "gtmsource.h"
 #include "do_shmat.h"		/* for do_shmat() prototype */
 #include "mutex.h"
-#include "heartbeat_timer.h"
 #include "gtm_file_stat.h"
 #include "util.h"
 #include "gtm_caseconv.h"
@@ -72,6 +71,7 @@
 #include "mu_getlst.h"
 #include "mu_outofband_setup.h"
 #include "gtmmsg.h"
+#include "wcs_backoff.h"
 #include "wcs_sleep.h"
 #include "wcs_flu.h"
 #include "trans_log_name.h"
@@ -98,7 +98,6 @@ GBLREF 	bool		error_mupip;
 GBLREF 	bool		file_backed_up;
 GBLREF 	bool		incremental;
 GBLREF 	bool		online;
-GBLREF 	uchar_ptr_t	mubbuf;
 GBLREF 	int4		mubmaxblk;
 GBLREF	tp_region	*grlist;
 GBLREF 	tp_region 	*halt_ptr;
@@ -125,11 +124,11 @@ GBLREF  int		process_exiting;		/* Process is on it's way out */
 
 GBLREF	boolean_t		jnlpool_init_needed;
 GBLREF	backup_reg_list		*mu_repl_inst_reg_list;
-GBLREF	jnlpool_addrs		jnlpool;
-GBLREF	jnlpool_ctl_ptr_t	jnlpool_ctl;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool_head;
 GBLREF	uint4			mutex_per_process_init_pid;
 GBLREF	boolean_t		holds_sem[NUM_SEM_SETS][NUM_SRC_SEMS];
-GBLREF	boolean_t		pool_init;
+GBLREF	int			pool_init;
 
 LITREF char             gtm_release_name[];
 LITREF int4             gtm_release_name_len;
@@ -148,6 +147,7 @@ error_def(ERR_FILEPARSE);
 error_def(ERR_FREEZECTRL);
 error_def(ERR_JNLCREATE);
 error_def(ERR_JNLDISABLE);
+error_def(ERR_JNLFILOPN);
 error_def(ERR_JNLFNF);
 error_def(ERR_JNLNOCREATE);
 error_def(ERR_JNLPOOLSETUP);
@@ -159,8 +159,10 @@ error_def(ERR_MUNOSTRMBKUP);
 error_def(ERR_MUPCLIERR);
 error_def(ERR_MUSELFBKUP);
 error_def(ERR_NOTRNDMACC);
+error_def(ERR_OFRZACTIVE);
 error_def(ERR_PERMGENFAIL);
 error_def(ERR_PREVJNLLINKCUT);
+error_def(ERR_REPLINSTUNDEF);
 error_def(ERR_REPLJNLCNFLCT);
 error_def(ERR_REPLPOOLINST);
 error_def(ERR_REPLREQROLLBACK);
@@ -168,6 +170,7 @@ error_def(ERR_REPLSTATE);
 error_def(ERR_REPLSTATEERR);
 error_def(ERR_SYSCALL);
 error_def(ERR_TEXT);
+error_def(ERR_FILENAMETOOLONG);
 ZOS_ONLY(error_def(ERR_BADTAG);)
 
 static char	* const jnl_parms[] =
@@ -219,7 +222,7 @@ void mupip_backup(void)
 	char			*tempfilename, *ptr;
 	uint4			status, ret, kip_count;
 	unsigned short		s_len, length;
-	int4			size, crit_counter, save_errno, rv;
+	int4			size, crit_counter, save_errno, rv, orig_buff_size;
 	uint4			ustatus, reg_count;
 	trans_num		tn;
 	shmpool_buff_hdr_ptr_t	sbufh_p;
@@ -243,22 +246,25 @@ void mupip_backup(void)
 	jnl_tm_t		save_gbl_jrec_time;
 	gd_region		*r_save, *reg;
 	int			sync_io_status;
-	boolean_t		sync_io, sync_io_specified, wait_for_zero_kip;
-	boolean_t		dummy_ftok_counter_halted, repl_inst_available;
+	boolean_t		sync_io, sync_io_specified;
+	boolean_t		dummy_ftok_counter_halted;
+	void			*repl_inst_available;
 	struct stat		stat_buf;
 	int			fstat_res, fclose_res, tmpfd;
 	gd_segment		*seg;
-	char			instfilename[MAX_FN_LEN + 1], *errptr, scndry_msg[OUT_BUFF_SIZE];
+	char			*errptr, scndry_msg[OUT_BUFF_SIZE];
 	unsigned int		full_len;
 	unix_db_info		*udi;
 	sgmnt_addrs		*csa;
 	repl_inst_hdr		repl_instance, *save_inst_hdr;
-	unsigned char		*cmdptr, command[MAX_FN_LEN * 2 + 5]; /* 5 == SIZEOF("cp") + 2 (space) + 1 (NULL) */
+	unsigned char		*cmdptr;
+	unsigned char 		command[(GTM_PATH_MAX) * 2 + 5]; /*2 files in the cmd, 5 == SIZEOF("cp") + 2 (space) + 1 (NULL)*/
 	struct shmid_ds		shm_buf;
 	struct semid_ds		semstat;
 	union semun		semarg;
 	int4			shm_id, sem_id;
 	replpool_identifier	replpool_id;
+	DEBUG_ONLY(jnlpool_addrs_ptr_t	jnlpool_save);
 	sm_uc_ptr_t		start_addr;
 	int			user_id;
 	int			group_id;
@@ -266,6 +272,7 @@ void mupip_backup(void)
 	uint4			*kip_pids_arr_ptr;
 	seq_num			jnl_seqno;
 	char			time_str[CTIME_BEFORE_NL + 2];	/* for GET_CUR_TIME macro */
+	struct perm_diag_data	pdd;
 	ZOS_ONLY(int		realfiletag;)
 
 	/* ==================================== STEP 1. Initialization ======================================= */
@@ -489,6 +496,8 @@ void mupip_backup(void)
 	size = ROUND_UP(SIZEOF_FILE_HDR_MAX, DISK_BLOCK_SIZE);
 	ESTABLISH(mu_freeze_ch);
 	tempfilename = tempdir_full.addr = tempdir_full_buffer;
+	orig_buff_size = SIZEOF(tempdir_full_buffer);
+	tempdir_full.len = orig_buff_size;
 	if (TRUE == online)
 	{
 		tempdir_log.addr = GTM_BAK_TEMPDIR_LOG_NAME;
@@ -517,7 +526,7 @@ void mupip_backup(void)
 		file->addr[file->len] = '\0';
 		if (mu_ctrly_occurred || mu_ctrlc_occurred)
 			break;
-		if ((dba_bg != rptr->reg->dyn.addr->acc_meth) && (dba_mm != rptr->reg->dyn.addr->acc_meth))
+		if (!IS_REG_BG_OR_MM(rptr->reg))
 		{
 			util_out_print("Region !AD is not a BG or MM databases", TRUE, REG_LEN_STR(rptr->reg));
 			rptr->not_this_time = give_up_before_create_tempfile;
@@ -530,7 +539,7 @@ void mupip_backup(void)
 			continue;
 		}
 		gv_cur_region = rptr->reg;
-		gvcst_init(gv_cur_region);
+		gvcst_init(gv_cur_region, NULL);
 		if (gv_cur_region->was_open)
 		{
 			gv_cur_region->open = FALSE;
@@ -578,9 +587,19 @@ void mupip_backup(void)
 					ptr--;
 				if (ptr > rptr->backup_file.addr)
 				{
-					memcpy(tempdir_trans_buffer, rptr->backup_file.addr,
-						(tempdir_trans.len = INTCAST(ptr - rptr->backup_file.addr + 1)));
-					tempdir_trans_buffer[tempdir_trans.len] = '\0';
+					tempdir_trans.len = INTCAST(ptr - rptr->backup_file.addr + 1);
+					if (SIZEOF(tempdir_trans_buffer) >= tempdir_trans.len)
+					{
+						memcpy(tempdir_trans_buffer, rptr->backup_file.addr, tempdir_trans.len);
+						tempdir_trans_buffer[tempdir_trans.len] = '\0';
+					}
+					else
+					{
+						gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_FILEPARSE, 2,
+						  rptr->backup_file.len, rptr->backup_file.addr);
+                                		mubclnup(rptr, need_to_del_tempfile);
+                                		mupip_exit(ERR_FILENAMETOOLONG);
+					}
 				} else
 				{
 					tempdir_trans_buffer[0] = '.';
@@ -588,6 +607,8 @@ void mupip_backup(void)
 					tempdir_trans.len = 1;
 				}
 			}
+			 /* Reset length , since gtm_file_stat expects the full buffer length as input */
+                        tempdir_full.len = orig_buff_size;
 			/* verify the accessibility of the tempdir */
 			if (FILE_STAT_ERROR == (fstat_res = gtm_file_stat(&tempdir_trans, NULL, &tempdir_full, FALSE, &ustatus)))
 			{
@@ -596,6 +617,14 @@ void mupip_backup(void)
 				mubclnup(rptr, need_to_del_tempfile);
 				mupip_exit(ustatus);
 			}
+			/*creating temp filename here , check if we have the required space*/
+			if (orig_buff_size <= (tempdir_full.len + strlen(tempnam_prefix) + SIZEOF(uint4)))
+			{
+				gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_FILEPARSE, 2, tempdir_trans.len,
+						tempdir_trans.addr);
+				mubclnup(rptr, need_to_del_tempfile);
+				mupip_exit(ERR_FILENAMETOOLONG);
+			}
 			SPRINTF(tempfilename + tempdir_full.len,"/%s_XXXXXX",tempnam_prefix);
 			MKSTEMP(tempfilename, rptr->backup_fd);
 			if (FD_INVALID == rptr->backup_fd)
@@ -652,7 +681,21 @@ void mupip_backup(void)
 			/* give temporary files the group and permissions as other shared resources - like journal files */
 			FSTAT_FILE(((unix_db_info *)(gv_cur_region->dyn.addr->file_cntl->file_info))->fd, &stat_buf, fstat_res);
 			if (-1 != fstat_res)
-				gtm_permissions(&stat_buf, &user_id, &group_id, &perm, PERM_FILE);
+				if (!gtm_permissions(&stat_buf, &user_id, &group_id, &perm, PERM_FILE, &pdd))
+				{
+					send_msg_csa(CSA_ARG(cs_addrs) VARLSTCNT(6+PERMGENDIAG_ARG_COUNT)
+						ERR_PERMGENFAIL, 4, RTS_ERROR_STRING("backup file"),
+						RTS_ERROR_STRING(
+							((unix_db_info *)(gv_cur_region->dyn.addr->file_cntl->file_info))->fn),
+						PERMGENDIAG_ARGS(pdd));
+					gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(6+PERMGENDIAG_ARG_COUNT)
+						ERR_PERMGENFAIL, 4, RTS_ERROR_STRING("backup file"),
+						RTS_ERROR_STRING(
+							((unix_db_info *)(gv_cur_region->dyn.addr->file_cntl->file_info))->fn),
+						PERMGENDIAG_ARGS(pdd));
+					mubclnup(rptr, need_to_del_tempfile);
+					mupip_exit(EPERM);
+				}
 			/* Setup new group and permissions if indicated by the security rules.  Use
 			 * 0770 anded with current mode for the new mode if masked permission selected.
 			 */
@@ -669,7 +712,7 @@ void mupip_backup(void)
 			}
 		} else
 		{
-			while (REG_ALREADY_FROZEN == region_freeze(gv_cur_region, TRUE, FALSE, FALSE))
+			while (REG_ALREADY_FROZEN == region_freeze(gv_cur_region, TRUE, FALSE, FALSE, FALSE, FALSE))
 			{
 				hiber_start(1000);
 				if ((TRUE == mu_ctrly_occurred) || (TRUE == mu_ctrlc_occurred))
@@ -705,43 +748,51 @@ void mupip_backup(void)
 		/* Attach to jnlpool if backup of replication instance file is needed and "jnlpool_init" did not happen as part of
 		 * "gvcst_init" (e.g. if CUSTOM_ERRORS_AVAILABLE is FALSE).
 		 */
-		repl_inst_available = REPL_INST_AVAILABLE;
+		repl_inst_available = REPL_INST_AVAILABLE(NULL);
 		if (!pool_init && repl_inst_available)
-			jnlpool_init(GTMRELAXED, (boolean_t)FALSE, (boolean_t *)NULL);	/* will set "pool_init" if successful */
+			jnlpool_init(GTMRELAXED, (boolean_t)FALSE, (boolean_t *)NULL, NULL);
 		if (!pool_init)
-		{
-			if (NULL == jnlpool.jnlpool_dummy_reg)
+		{	/* pool_init is only set if the source server has setup shared memory */
+			assert(NULL != jnlpool);
+			if (!repl_inst_available)
+				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_REPLINSTUNDEF);
+			if (!jnlpool)
+			{	/* jnlpool_init should have provided but be safe */
+				jnlpool = malloc(SIZEOF(jnlpool_addrs));
+				memset((uchar_ptr_t)jnlpool, 0, SIZEOF(jnlpool_addrs));
+				assert(NULL == jnlpool_head);
+			}
+			if (NULL == jnlpool->jnlpool_dummy_reg)
 			{
 				r_save = gv_cur_region;
 				mu_gv_cur_reg_init();
-				jnlpool.jnlpool_dummy_reg = reg = gv_cur_region;
+				jnlpool->jnlpool_dummy_reg = reg = gv_cur_region;
 				gv_cur_region = r_save;
 				ASSERT_IN_RANGE(MIN_RN_LEN, SIZEOF(JNLPOOL_DUMMY_REG_NAME) - 1, MAX_RN_LEN);
 				MEMCPY_LIT(reg->rname, JNLPOOL_DUMMY_REG_NAME);
 				reg->rname_len = STR_LIT_LEN(JNLPOOL_DUMMY_REG_NAME);
 				reg->rname[reg->rname_len] = '\0';
-				assertpro(repl_inst_get_name(instfilename, &full_len, MAX_FN_LEN + 1, issue_rts_error));
-					/* rts_error should have been issued by repl_inst_get_name in case of 0 return */
 				udi = FILE_INFO(reg);
 				seg = reg->dyn.addr;
-				memcpy((char *)seg->fname, instfilename, full_len);
+				memcpy((char *)seg->fname, replpool_id.instfilename, full_len);
 				udi->fn = (char *)seg->fname;
 				seg->fname_len = full_len;
 				seg->fname[full_len] = '\0';
 				udi->ftok_semid = INVALID_SEMID;
 			} else
 			{	/* Possible if jnlpool_init did mu_gv_cur_reg_init but returned prematurely due to NOJNLPOOL */
-				assert(!jnlpool.jnlpool_dummy_reg->open);
-				reg = jnlpool.jnlpool_dummy_reg;
+				assert(!jnlpool->jnlpool_dummy_reg->open);
+				reg = jnlpool->jnlpool_dummy_reg;
 				/* Since mu_gv_cur_reg_init is already done, ensure that the reg->rname is correct */
 				assert(0 == MEMCMP_LIT(reg->rname, JNLPOOL_DUMMY_REG_NAME));
 				assert(reg->rname_len == STR_LIT_LEN(JNLPOOL_DUMMY_REG_NAME));
 				assert('\0' == reg->rname[reg->rname_len]);
 				udi = FILE_INFO(reg);
 			}
+			DEBUG_ONLY(jnlpool_save = jnlpool);
 			do
 			{
-				if (!ftok_sem_get(jnlpool.jnlpool_dummy_reg, TRUE, REPLPOOL_ID, FALSE, &dummy_ftok_counter_halted))
+				if (!ftok_sem_get(jnlpool->jnlpool_dummy_reg, TRUE, REPLPOOL_ID, FALSE, &dummy_ftok_counter_halted))
 				{
 					gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_JNLPOOLSETUP);
 					error_mupip = TRUE;
@@ -760,18 +811,19 @@ void mupip_backup(void)
 				 * journal pool. Attach to it after releasing the ftok lock. We expect this situation to be
 				 * rare that we keep retrying.
 				 */
-				ftok_sem_release(jnlpool.jnlpool_dummy_reg, udi->counter_ftok_incremented, TRUE);
-				jnlpool_init(GTMRELAXED, (boolean_t)FALSE, (boolean_t *)NULL);
+				ftok_sem_release(jnlpool->jnlpool_dummy_reg, udi->counter_ftok_incremented, TRUE);
+				jnlpool_init(GTMRELAXED, (boolean_t)FALSE, (boolean_t *)NULL, NULL);
 					/* "jnlpool_init" will set "pool_init" if successful */
 				if (pool_init)
 					break;
+				assert(jnlpool_save == jnlpool);	/* should be reusing incomplete structure */
 			} while (TRUE);
 		}
 		if (pool_init)
 		{
-			udi = FILE_INFO(jnlpool.jnlpool_dummy_reg);
+			udi = FILE_INFO(jnlpool->jnlpool_dummy_reg);
 			assert(INVALID_SEMID != udi->ftok_semid);
-			if (!ftok_sem_lock(jnlpool.jnlpool_dummy_reg, FALSE))
+			if (!ftok_sem_lock(jnlpool->jnlpool_dummy_reg, FALSE))
 			{
 				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_JNLPOOLSETUP);
 				error_mupip = TRUE;
@@ -779,14 +831,14 @@ void mupip_backup(void)
 			}
 			/* (Re-)read the instance file header now that we have the ftok lock */
 			repl_inst_read(udi->fn, (off_t)0, (sm_uc_ptr_t)&repl_instance, SIZEOF(repl_inst_hdr));
-			if (jnlpool_ctl->ftok_counter_halted)
+			if (jnlpool->jnlpool_ctl->ftok_counter_halted)
 				udi->counter_ftok_incremented = FALSE;	/* so we do not inadvertently delete the ftok semaphore */
 		}
-		assert(NULL != jnlpool.jnlpool_dummy_reg);
-		assert(!pool_init || (NULL != jnlpool_ctl));
+		assert(NULL != jnlpool->jnlpool_dummy_reg);
+		assert(!pool_init || ((NULL != jnlpool) && (NULL != jnlpool->jnlpool_ctl)));
 		shm_id = repl_instance.jnlpool_shmid;
 		sem_id = repl_instance.jnlpool_semid;
-		udi = FILE_INFO(jnlpool.jnlpool_dummy_reg);
+		udi = FILE_INFO(jnlpool->jnlpool_dummy_reg);
 		if (INVALID_SEMID != sem_id)
 		{
 			assert(pool_init);
@@ -860,6 +912,18 @@ repl_inst_bkup_done1:
 				grab_crit(gv_cur_region);
 				DEBUG_ONLY(reg_count++);
 			}
+			if (FROZEN_CHILLED(cs_addrs))
+			{	/* While a FREEZE -ONLINE was in place, all processes exited, leaving the
+				 * shared memory up. Either autorelease, if enabled, or error out.
+				 */
+				DO_CHILLED_AUTORELEASE(cs_addrs, cs_data);
+				if (FROZEN_CHILLED(cs_addrs))
+				{
+					gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_OFRZACTIVE, 2, DB_LEN_STR(gv_cur_region));
+					rptr->not_this_time = give_up_before_create_tempfile;
+					continue;
+				}
+			}
 			/* We will be releasing crit if KIP is set, so don't update jgbl.gbl_jrec_time now */
 			if (!kip_count)
 			{
@@ -868,18 +932,17 @@ repl_inst_bkup_done1:
 		}
 	}
 	/* If we have KILLs in progress on any of the regions, wait a maximum of 1 minute(s) for those to finish. */
-	wait_for_zero_kip = (online && kip_count);
-	for (crit_counter = 1; wait_for_zero_kip; )
-	{	/* Release all the crits before going into the wait loop */
+	for (crit_counter = 1; online && kip_count; )
+	{	/* The purpose of this loop is to wait for kip to clear on all regions, waiting for a maximum of about a minute.
+		 * This is complicated by the fact that in spite of setting INCR_INHIBIT_KILLS above, a process in crit for a
+		 * "fourth" retry may ignote that, and set the kip flag while we are waiting
+		 */
 		DEBUG_ONLY(nocritrptr = NULL;)
 		for (rptr = (backup_reg_list *)(grlist); NULL != rptr; rptr = rptr->fPtr)
-		{
+		{	/* Release all the crits before going into the wait loop */
 			if (rptr->not_this_time > keep_going)
 				continue;
 			TP_CHANGE_REG(rptr->reg);
-			/* It is possible to not hold crit on some regions if we are in the second or higher iteration
-			 * of the outer for loop (the one with the loop variable wait_for_zero_kip).
-			 */
 			if (cs_addrs->now_crit)
 			{	/* once we encountered a region in the list that we did not hold crit on, we should also
 				 * not hold crit on all later regions in the list. assert that.
@@ -894,7 +957,7 @@ repl_inst_bkup_done1:
 		}
 		/* Wait for a maximum of 1 minute on all the regions for KIP to reset */
 		for (rptr = (backup_reg_list *)(grlist); NULL != rptr; rptr = rptr->fPtr)
-		{
+		{	/* multiple regions might have kip, but waiting on any waits on all, so just do this for 1 fixed time */
 			if (rptr->not_this_time > keep_going)
 				continue;
 			TP_CHANGE_REG(rptr->reg);
@@ -910,49 +973,23 @@ repl_inst_bkup_done1:
 				GET_C_STACK_FOR_KIP(kip_pids_arr_ptr, crit_counter, MAX_CRIT_TRY, 1, MAX_KIP_PID_SLOTS);
 				wcs_sleep(crit_counter);
 			}
-		}
-		if (debug_mupip)
-		{
-			GET_CUR_TIME(time_str);
-			util_out_print("!/MUPIP INFO: !AD : Done with kill-in-prog wait on ALL databases", TRUE,
-				CTIME_BEFORE_NL, time_str);
+			if (debug_mupip)
+			{
+				GET_CUR_TIME(time_str);
+				util_out_print("!/MUPIP INFO: !AD : Done with kill-in-prog wait on !AD", TRUE,
+					CTIME_BEFORE_NL, time_str, DB_LEN_STR(gv_cur_region));
+			}
+			if (MAX_CRIT_TRY < crit_counter)
+				break;
 		}
 		/* Since we have waited a while for KIP to get reset, get current time again to make it more accurate */
 		SET_GBL_JREC_TIME;
-		/* Most code in this for-loop is similar to the previous for loop where we grab_crit;
-		 * but most of the times the second for loop will never be executed (because KIP will be
-		 * zero on all database files) and in some rare cases, it is okay to take the hit of an
-		 * extra rel_crit/wait-for-kip/grab_crit sequence.
-		 */
-		wait_for_zero_kip = (MAX_CRIT_TRY > crit_counter);
-		kip_count = 0;
-		for (rptr = (backup_reg_list *)(grlist); NULL != rptr; rptr = rptr->fPtr)
+		for (kip_count = 0, rptr = (backup_reg_list *)(grlist); NULL != rptr; rptr = rptr->fPtr)
 		{
 			if (rptr->not_this_time > keep_going)
 				continue;
 			TP_CHANGE_REG(rptr->reg);
 			grab_crit(gv_cur_region);
-			if (cs_data->kill_in_prog)
-			{	/* It is possible for this to happen in case a concurrent GT.M process is in its 4th retry.
-				 * In that case, it will not honor the inhibit_kills flag since it holds crit and therefore
-				 * could have set kill-in-prog to a non-zero value while we were outside of crit.
-				 * Check if we have waited 1 minute until now. If not, release crit and continue the wait.
-				 * If waited already, then proceed with the backup. The reasoning is that once the GT.M process
-				 * that is in the final retry finishes off the second part of the M-kill, it will not start
-				 * a new transaction in the first try which is outside of crit so will honor the inhibit-kills
-				 * flag and therefore not increment the kill_in_prog counter any more until backup is done.
-				 * So we could be waiting for at most 1 kip increment per concurrent process that is updating
-				 * the database. We expect these kills to be complete within 1 minute.
-				 */
-				if (wait_for_zero_kip)
-				{
-					kip_count++;
-					break;
-				}
-				assert(!kip_count);
-				GET_C_STACK_FOR_KIP(kip_pids_arr_ptr, crit_counter, MAX_CRIT_TRY, 2, MAX_KIP_PID_SLOTS);
-				gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_BACKUPKILLIP, 2, DB_LEN_STR(gv_cur_region));
-			}
 			/* Now that we have crit, check if this region is actively journaled and if gbl_jrec_time needs to be
 			 * adjusted (to ensure time ordering of journal records within this region's journal file).
 			 * This needs to be done BEFORE writing any journal records for this region. The value of
@@ -960,9 +997,18 @@ repl_inst_bkup_done1:
 			 * regions so all regions will have same eov/bov timestamps.
 			 */
 			UPDATE_GBL_JREC_TIME;
+			if (cs_data->kill_in_prog)
+			{
+				if (MAX_CRIT_TRY < crit_counter)
+					gtm_putmsg_csa(CSA_ARG(cs_addrs) VARLSTCNT(4) ERR_BACKUPKILLIP,
+						       2, DB_LEN_STR(gv_cur_region));
+				else	/* lets wait some more */
+				{
+					kip_count++;
+					break;
+				}
+			}
 		}
-		if (0 == kip_count)
-			break;	/* all regions have zero kill-in-prog so we can break out of this loop unconditionally */
 	}
 	assert(reg_count == have_crit(CRIT_HAVE_ANY_REG | CRIT_ALL_REGIONS));
 	DEBUG_ONLY(save_gbl_jrec_time = jgbl.gbl_jrec_time;)
@@ -980,9 +1026,10 @@ repl_inst_bkup_done1:
 				 */
 				goto repl_inst_bkup_done2;
 			}
-			assert(udi == FILE_INFO(jnlpool.jnlpool_dummy_reg));
-			seg = jnlpool.jnlpool_dummy_reg->dyn.addr; /* re-initialize (for pool_init == TRUE case) */
+			assert(udi == FILE_INFO(jnlpool->jnlpool_dummy_reg));
+			seg = jnlpool->jnlpool_dummy_reg->dyn.addr; /* re-initialize (for pool_init == TRUE case) */
 			cmdptr = &command[0];
+			assert(SIZEOF(command) >= (5 + seg->fname_len + mu_repl_inst_reg_list->backup_file.len));
 			memcpy(cmdptr, "cp ", 3);
 			cmdptr += 3;
 			memcpy(cmdptr, seg->fname, seg->fname_len);
@@ -1026,20 +1073,20 @@ repl_inst_bkup_done1:
 					 */
 					csa = &udi->s_addrs;
 					assert(csa->critical
-						== (mutex_struct_ptr_t)((sm_uc_ptr_t)jnlpool.jnlpool_ctl + JNLPOOL_CTL_SIZE));
-					grab_lock(jnlpool.jnlpool_dummy_reg, TRUE, ASSERT_NO_ONLINE_ROLLBACK);
-					jnl_seqno = jnlpool.jnlpool_ctl->jnl_seqno;
-					if (repl_instance.num_histinfo != jnlpool.repl_inst_filehdr->num_histinfo)
+						== (mutex_struct_ptr_t)((sm_uc_ptr_t)jnlpool->jnlpool_ctl + JNLPOOL_CTL_SIZE));
+					grab_lock(jnlpool->jnlpool_dummy_reg, TRUE, ASSERT_NO_ONLINE_ROLLBACK);
+					jnl_seqno = jnlpool->jnlpool_ctl->jnl_seqno;
+					if (repl_instance.num_histinfo != jnlpool->repl_inst_filehdr->num_histinfo)
 					{
-						repl_instance = *jnlpool.repl_inst_filehdr;
-						rel_lock(jnlpool.jnlpool_dummy_reg);
+						repl_instance = *jnlpool->repl_inst_filehdr;
+						rel_lock(jnlpool->jnlpool_dummy_reg);
 						continue;	/* redo the "cp" */
 					}
-					save_inst_hdr = jnlpool.repl_inst_filehdr;
-					jnlpool.repl_inst_filehdr = &repl_instance;
+					save_inst_hdr = jnlpool->repl_inst_filehdr;
+					jnlpool->repl_inst_filehdr = &repl_instance;
 					assert(0 != jnl_seqno);
 					/* All the cleanup we want is exactly done by the "repl_inst_histinfo_truncate" function.
-					 * But we dont want to clean the instance file. We want to instead clean the backed up
+					 * But we don't want to clean the instance file. We want to instead clean the backed up
 					 * instance file. To that effect, temporarily change "udi->fn" to reflect the backed up
 					 * file so all the changes get flushed there. Restore it after the function call.
 					 */
@@ -1050,13 +1097,13 @@ repl_inst_bkup_done1:
 					COPY_JCTL_STRMSEQNO_TO_INSTHDR_IF_NEEDED;
 					repl_inst_histinfo_truncate(jnl_seqno);	/* Flush updated file header to
 										 * backed up instance file */
-					rel_lock(jnlpool.jnlpool_dummy_reg);
+					rel_lock(jnlpool->jnlpool_dummy_reg);
 					udi->fn = (char *)seg->fname; /* Restore */
-					/* Now that instance file truncate is done, restore jnlpool.repl_inst_filehdr to its
+					/* Now that instance file truncate is done, restore jnlpool->repl_inst_filehdr to its
 					 * original value
 					 */
 					assert(NULL != save_inst_hdr);
-					jnlpool.repl_inst_filehdr = save_inst_hdr;
+					jnlpool->repl_inst_filehdr = save_inst_hdr;
 				} else
 				{	/* We are guaranteed that NO one is actively accessing the instance file. So, no need to
 					 * truncate the backed up instance file.
@@ -1075,7 +1122,7 @@ repl_inst_bkup_done2:
 					assertpro(SS_NORMAL == rel_sem_immediate(SOURCE, JNL_POOL_ACCESS_SEM));
 				}
 				if (udi->grabbed_ftok_sem)
-					ftok_sem_release(jnlpool.jnlpool_dummy_reg, udi->counter_ftok_incremented, TRUE);
+					ftok_sem_release(jnlpool->jnlpool_dummy_reg, udi->counter_ftok_incremented, TRUE);
 			}
 			if (!error_mupip)
 			{
@@ -1159,7 +1206,7 @@ repl_inst_bkup_done2:
 							!keep_prev_link || !JNL_ENABLED(cs_data)) ? TRUE : FALSE;
 						if (JNL_ENABLED(cs_data) && (0 != cs_addrs->nl->jnl_file.u.inode))
 						{	/* Note: following will again call wcs_flu() */
-							if (SS_NORMAL != (status = set_jnl_file_close(SET_JNL_FILE_CLOSE_BACKUP)))
+							if (SS_NORMAL != (status = set_jnl_file_close()))
 							{
 								util_out_print("!/Journal file !AD not closed:",
 									TRUE, jnl_info.jnl_len, jnl_info.jnl);
@@ -1186,6 +1233,7 @@ repl_inst_bkup_done2:
 								error_mupip = TRUE;
 								continue;
 							}
+							jnl_set_cur_prior(gv_cur_region, cs_addrs, cs_data);
 							jnl_info.no_rename = (FILE_NOT_FOUND == jnl_fstat);
 						}
 						wcs_flu(WCSFLU_FSYNC_DB | WCSFLU_FLUSH_HDR | WCSFLU_MSYNC_DB);
@@ -1268,6 +1316,9 @@ repl_inst_bkup_done2:
 									4, JNL_LEN_STR(cs_data), DB_LEN_STR(gv_cur_region));
 							fc = gv_cur_region->dyn.addr->file_cntl;
 							fc->op = FC_WRITE;
+							/* Note: cs_data points to shared memory and is already aligned
+							 * appropriately even if db was opened using O_DIRECT.
+							 */
 							fc->op_buff = (sm_uc_ptr_t)cs_data;
 							fc->op_len = SGMNT_HDR_LEN;
 							fc->op_pos = 1;
@@ -1365,16 +1416,18 @@ repl_inst_bkup_done2:
 			/*
 			 * For backed up database we want to change some file header fields.
 			 */
-			rptr->backup_hdr->freeze = 0;
+			rptr->backup_hdr->freeze = FALSE;
 			rptr->backup_hdr->image_count = 0;
-			rptr->backup_hdr->kill_in_prog = 0;
+			if (rptr->backup_hdr->kill_in_prog)
+				rptr->backup_hdr->abandoned_kills = TRUE;
+			rptr->backup_hdr->kill_in_prog = FALSE;
 			memset(rptr->backup_hdr->machine_name, 0, MAX_MCNAMELEN);
 			rptr->backup_hdr->repl_state = repl_closed;
 			rptr->backup_hdr->semid = INVALID_SEMID;
 			rptr->backup_hdr->shmid = INVALID_SHMID;
 			rptr->backup_hdr->gt_sem_ctime.ctime = 0;
 			rptr->backup_hdr->gt_shm_ctime.ctime = 0;
-			if (jnl_options[jnl_off] || bkdbjnl_off_specified)
+			if ((jnl_options[jnl_off] || bkdbjnl_off_specified) && (jnl_notallowed != gv_cur_region->jnl_state))
 				rptr->backup_hdr->jnl_state = jnl_closed;
 			if (jnl_options[jnl_disable] || bkdbjnl_disable_specified)
 			{
@@ -1389,7 +1442,6 @@ repl_inst_bkup_done2:
 					LEN_AND_STR(jnl_state_lit[rptr->backup_hdr->jnl_state]));
 		}
 		ENABLE_AST
-		mubbuf = (uchar_ptr_t)malloc(BACKUP_READ_SIZE);
 		for (rptr = (backup_reg_list *)(grlist);  NULL != rptr;  rptr = rptr->fPtr)
 		{
 			if (rptr->not_this_time > keep_going)
@@ -1416,7 +1468,6 @@ repl_inst_bkup_done2:
 			if (mu_ctrly_occurred || mu_ctrlc_occurred)
 				break;
 		}
-		free(mubbuf);
 	} else
 	{
 		mubclnup((backup_reg_list *)halt_ptr, need_to_rel_crit);
