@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -93,16 +93,6 @@ error_def(ERR_SYSCALL);
 	murgbl.extr_buff[extract_len++] = '\\';				\
 }
 
-/* extract jnl record "rec" using extraction routine "extract" into file "file_info" (extract/lost/broken transaction files) */
-#define EXTRACT_JNLREC(jctl, rec, extract, file_info, status)		\
-{									\
-	pini_list_struct	*plst;					\
-									\
-	status = mur_get_pini(jctl, (rec)->prefix.pini_addr, &plst);	\
-	if (SS_NORMAL == status)					\
-		(*extract)(jctl, (file_info), (rec), plst);		\
-}
-
 #define	EXTPID(plst)					\
 {							\
 	EXTINT(plst->jpv.jpv_pid);			\
@@ -174,16 +164,16 @@ error_def(ERR_SYSCALL);
 
 #define IS_VALID_LEN_FROM_PREFIX(JREC, JFH)							\
 (	/* length within range */								\
-	(ROUND_DOWN2((JREC)->prefix.forwptr, JNL_REC_START_BNDRY) == (JREC)->prefix.forwptr) &&	\
-	(JREC)->prefix.forwptr > MIN_JNLREC_SIZE  &&						\
-	(JREC)->prefix.forwptr <= (JFH)->max_jrec_len						\
+	(ROUND_DOWN2((JREC)->prefix.forwptr, JNL_REC_START_BNDRY) == (JREC)->prefix.forwptr)	\
+		&& (MIN_JNLREC_SIZE <= (JREC)->prefix.forwptr)					\
+		&& ((JREC)->prefix.forwptr <= (JFH)->max_jrec_len)				\
 )
 
 #define IS_VALID_LEN_FROM_SUFFIX(SUFFIX, JFH)							\
 ( /* length within range */									\
-	(ROUND_DOWN2((SUFFIX)->backptr, JNL_REC_START_BNDRY) == (SUFFIX)->backptr) &&		\
-  	(SUFFIX)->backptr > MIN_JNLREC_SIZE &&							\
-	(SUFFIX)->backptr <= (JFH)->max_jrec_len						\
+	(ROUND_DOWN2((SUFFIX)->backptr, JNL_REC_START_BNDRY) == (SUFFIX)->backptr)		\
+  		&& (MIN_JNLREC_SIZE <= (SUFFIX)->backptr)					\
+		&& ((SUFFIX)->backptr <= (JFH)->max_jrec_len)					\
 )
 
 #define IS_VALID_LINKS(JREC)													\
@@ -246,7 +236,7 @@ error_def(ERR_SYSCALL);
 			}											\
 		} else												\
 		{	/* Recovery on this database was previously interrupted. Although we know which		\
-			 * stream seqno was specified for this round of rollback, we dont know which		\
+			 * stream seqno was specified for this round of rollback, we don't know which		\
 			 * streams were specified in previous interrupted rollbacks. So set all the		\
 			 * stream #s in the file header based on murgbl.resync_strm_seqno[] which has		\
 			 * already been computed taking into account ALL region file headers.			\
@@ -348,7 +338,10 @@ enum rec_fence_type
 {
 	NOFENCE = 0,
 	TPFENCE = 1,
-	ZTPFENCE = 2
+	ZTPFENCE = 2,
+	NULLFENCE = 3	/* i.e. a multi-region TP transaction whose TSET/TKILL/USET/UKILL journal records got replaced
+			 * as NULL records by "jnl_phase2_salvage" due to kill9 of process before jnl records were written.
+			 */
 };
 
 enum broken_type
@@ -426,7 +419,7 @@ typedef struct multi_element_struct
 	jnl_tm_t			time;
 	uint4				partner;	/* # of unmatched regions involved in TP/ZTP */
 	uint4				tot_partner;	/* Total # of regions originally involved in TP/ZTP */
-	enum rec_fence_type 		fence;		/* NOFENCE or TPFENCE or ZTPFENCE */
+	enum rec_fence_type 		fence;		/* NOFENCE or TPFENCE or ZTPFENCE or NULLFENCE */
 	struct multi_element_struct	*next;
 } multi_struct;
 
@@ -639,6 +632,7 @@ typedef struct reg_ctl_list_struct
 	struct jnlext_multi_struct *jnlext_shm_list[TOT_EXTR_TYPES];	/* valid only if "rctl->this_pid_is_owner" is FALSE */
 	int			extr_fn_len_orig[TOT_EXTR_TYPES];	/* fn_len before the region-name suffix was added */
 	boolean_t		last_jext_logical_rec[TOT_EXTR_TYPES];/* Whether corresponding last_jext_rec[] is logical record */
+	dio_buff_t		dio_buff;	/* Used for O_DIRECT IO in case this region has asyncio=TRUE */
 } reg_ctl_list;
 
 typedef struct redirect_list_struct
@@ -789,7 +783,7 @@ typedef struct jnlext_multi_struct
 						 * = 2*n+1 for *SET*,*KILL*,*LGTRIG*,*ZTRIG rectypes where this is the nth
 						 *	journaled update (across all regions) in this TP.
 						 * = 2*n for *ZTWORM* rectype. We need this 2n+1 vs 2n distinction because
-						 *	ZTWORM type of updates dont inherit a unique update_num at the time of
+						 *	ZTWORM type of updates don't inherit a unique update_num at the time of
 						 *	the TP transaction. They just inherit the update_num of the following
 						 *	*SET*,*KILL* rectype.
 						 * = 2**31-1 (max-possible-value) for TCOM record.
@@ -1207,21 +1201,23 @@ typedef struct {
 #define	RESOLVED_TRUE	TRUE
 
 /* The below macro is similar to code in gtm_pthread_init_key.c */
-#define	MUR_SET_MULTI_PROC_KEY(RCTL, PTR)			\
-{								\
-	gd_region	*reg;					\
-								\
-	assert(multi_proc_in_use);				\
-	reg = RCTL->gd;						\
-	if (reg->rname_len)					\
-	{							\
-		PTR = &reg->rname[0];				\
-		assert('\0' == PTR[reg->rname_len]);		\
-	} else							\
-	{							\
-		PTR = &reg->dyn.addr->fname[0];			\
-		assert('\0' == PTR[reg->dyn.addr->fname_len]);	\
-	}							\
+#define	MUR_SET_MULTI_PROC_KEY(RCTL, PTR)					\
+{										\
+	gd_region	*reg;							\
+										\
+	assert(multi_proc_in_use);						\
+	reg = RCTL->gd;								\
+	if (!reg->owning_gd->is_dummy_gbldir)					\
+	{									\
+		assert(reg->rname_len);						\
+		PTR = &reg->rname[0];						\
+		assert('\0' == PTR[reg->rname_len]);				\
+	} else									\
+	{									\
+		assert(!memcmp(reg->rname, "DEFAULT", reg->rname_len));		\
+		PTR = &reg->dyn.addr->fname[0];					\
+		assert('\0' == PTR[reg->dyn.addr->fname_len]);			\
+	}									\
 }
 
 #define	IN_MUR_CLOSE_FILES_FALSE	FALSE
