@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2015 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -14,7 +14,9 @@
 
 #include <stdarg.h>
 #include "gtm_string.h"
+#include "gtm_stdio.h"
 
+#include "gtmio.h"
 #include "gdsroot.h"
 #include "gtm_facility.h"
 #include "fileinfo.h"
@@ -23,6 +25,8 @@
 #include "dpgbldir.h"
 #include "filestruct.h"
 #include "jnl.h"
+#include "repl_msg.h"		/* for gtmsource.h */
+#include "gtmsource.h"		/* for jnlpool_addrs_ptr_t */
 #include "view.h"
 #include "stringpool.h"
 #include "cmd_qlf.h"
@@ -46,10 +50,11 @@
 #include "gvnh_spanreg.h"
 #include "targ_alloc.h"
 #include "change_reg.h"
-#ifdef UNIX
 # include "gtmlink.h"
-#endif
 #include "gtm_ctype.h"		/* for ISDIGIT_ASCII macro */
+#include "gvn2gds.h"
+#include "io.h"
+#include "interlock.h"
 
 GBLREF spdesc		stringpool;
 GBLREF int4		cache_hits, cache_fails;
@@ -61,6 +66,7 @@ GBLREF gd_region	*gv_cur_region;
 GBLREF gv_namehead	*gv_target;
 GBLREF gv_namehead	*reset_gv_target;
 GBLREF jnl_fence_control jnl_fence_ctl;
+GBLREF jnlpool_addrs_ptr_t	jnlpool;
 GBLREF bool		undef_inhibit;
 GBLREF int4		break_message_mask;
 GBLREF command_qualifier cmd_qlf;
@@ -73,11 +79,12 @@ GBLREF int		gv_fillfactor;
 GBLREF int4		gtm_max_sockets;
 GBLREF gv_key		*gv_currkey;
 GBLREF boolean_t	is_gtm_chset_utf8;
-UNIX_ONLY(GBLREF	boolean_t		dmterm_default;)
+GBLREF	uint4		process_id;
+GBLREF mstr		extnam_str;
+GBLREF	boolean_t	dmterm_default;
 
 error_def(ERR_COLLATIONUNDEF);
 error_def(ERR_GBLNOMAPTOREG);
-error_def(ERR_GVSUBSERR);
 error_def(ERR_VIEWFN);
 error_def(ERR_VIEWGVN);
 
@@ -85,7 +92,6 @@ LITREF	gtmImageName	gtmImageNames[];
 LITREF	mstr		relink_allowed_mstr[];
 LITREF	mval		literal_zero;
 LITREF	mval		literal_one;
-LITREF	mval		literal_null;
 
 #define		MM_RES			"MM"
 #define		BG_RES			"BG"
@@ -97,22 +103,6 @@ LITREF	mval		literal_null;
 #define		STATS_MAX_DIGITS	MAX_DIGITS_IN_INT8
 #define		STATS_KEYWD_SIZE	(3 + 1 + 1)	/* 3 character mnemonic, colon and comma */
 
-STATICFNDCL unsigned char *gvn2gds(mval *gvn, gv_key *gvkey, int act);
-
-#define	COPY_ARG_TO_STRINGPOOL(DST, KEYEND, KEYSTART)			\
-{									\
-	int	keylen;							\
-									\
-	keylen = (unsigned char *)KEYEND - (unsigned char *)(KEYSTART);	\
-	ENSURE_STP_FREE_SPACE(keylen);					\
-	assert(stringpool.top - stringpool.free >= keylen);		\
-	memcpy(stringpool.free, KEYSTART, keylen);			\
-	DST->mvtype = MV_STR;						\
-	DST->str.len = keylen;						\
-	DST->str.addr = (char *)stringpool.free;			\
-	stringpool.free += keylen;					\
-}
-
 #define STATS_PUT_PARM(TXT, CNTR, BASE)					\
 {									\
 	MEMCPY_LIT(stringpool.free, TXT);				\
@@ -123,9 +113,8 @@ STATICFNDCL unsigned char *gvn2gds(mval *gvn, gv_key *gvkey, int act);
 	assert(stringpool.free <= stringpool.top);			\
 }
 
-void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
+void	op_fnview(int numarg, mval *dst, ...)
 {
-	VMS_ONLY(int	numarg;)
 	boolean_t	save_transform;
 	gv_key		save_currkey[DBKEYALLOC(MAX_KEY_SZ)];
 	unsigned char	*key;
@@ -207,7 +196,21 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			n = cache_fails;
 			break;
 		case VTK_SPSIZE:
-			n = (int)(stringpool.top - stringpool.base);
+			commastr.len = 1;
+			commastr.addr = ",";
+			ENSURE_STP_FREE_SPACE((STATS_MAX_DIGITS * 3) + 2);
+			MV_FORCE_MVAL(dst, (int)(stringpool.top - stringpool.base));
+			MV_FORCE_STR(dst);
+			dst->mvtype = vtp->restype;
+			s2pool_concat(dst, &commastr);
+			arg2 = &tmpmval;
+			MV_FORCE_MVAL(arg2, (int)(stringpool.free - stringpool.base));
+			MV_FORCE_STR(arg2);
+			s2pool_concat(dst, &arg2->str);
+			s2pool_concat(dst, &commastr);
+			MV_FORCE_MVAL(arg2, (int)(stringpool.top - stringpool.invokestpgcollevel));
+			MV_FORCE_STR(arg2);
+			s2pool_concat(dst, &arg2->str);
 			break;
 		case VTK_GDSCERT:
 			if (certify_all_blocks)
@@ -219,7 +222,7 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			assert(gd_header);
 			assert(parmblk.gv_ptr);
 			if (!parmblk.gv_ptr->open)
-				gv_init_reg(parmblk.gv_ptr);
+				gv_init_reg(parmblk.gv_ptr, NULL);
 			assert(parmblk.gv_ptr->open);
 			switch (REG_ACC_METH(parmblk.gv_ptr))
 			{
@@ -274,7 +277,7 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			assert(gd_header);
 			assert(parmblk.gv_ptr);
 			if (!parmblk.gv_ptr->open)
-				gv_init_reg(parmblk.gv_ptr);
+				gv_init_reg(parmblk.gv_ptr, NULL);
 			tmpstr.addr = (char *)parmblk.gv_ptr->dyn.addr->fname;
 			tmpstr.len = parmblk.gv_ptr->dyn.addr->fname_len;
 			s2pool(&tmpstr);
@@ -291,7 +294,12 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 		case VTK_GVNEXT:
 			assert(gd_header);
 			if (arg1->str.len)
-				parmblk.gv_ptr++;
+			{
+				for (reg = parmblk.gv_ptr + 1;
+				     ((reg - gd_header->regions) < gd_header->n_regions) && IS_STATSDB_REG(reg); reg++)
+					;
+				parmblk.gv_ptr = reg;
+			}
 			if (parmblk.gv_ptr - gd_header->regions >= gd_header->n_regions)
 				dst->str.len = 0;
 			else
@@ -307,7 +315,7 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			if (parmblk.gv_ptr)
 			{
 				if (!parmblk.gv_ptr->open)
-					gv_init_reg(parmblk.gv_ptr);
+					gv_init_reg(parmblk.gv_ptr, NULL);
 				csa = &FILE_INFO(parmblk.gv_ptr)->s_addrs;
 				if (NULL != csa->hdr)
 					n = csa->hdr->jnl_state;
@@ -321,7 +329,7 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			if (parmblk.gv_ptr)
 			{
 				if (!parmblk.gv_ptr->open)
-					gv_init_reg(parmblk.gv_ptr);
+					gv_init_reg(parmblk.gv_ptr, NULL);
 				csa = &FILE_INFO(parmblk.gv_ptr)->s_addrs;
 				if (NULL != csa->hdr)
 					view_jnlfile(dst, parmblk.gv_ptr);
@@ -330,6 +338,21 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			} else
 				dst->str.len = 0;
 			break;
+		case VTK_JNLPOOL:
+			if (jnlpool)
+			{
+				reg = jnlpool->jnlpool_dummy_reg;
+				if (reg && reg->dyn.addr)
+				{
+					tmpstr.addr = (char *)reg->dyn.addr->fname;
+					tmpstr.len = reg->dyn.addr->fname_len;
+					s2pool(&tmpstr);
+					dst->str = tmpstr;
+				} else
+					dst->str.len = 0;
+			} else
+				dst->str.len = 0;
+			break;
 		case VTK_JNLTRANSACTION:
 			n = jnl_fence_ctl.level;
 			break;
@@ -354,25 +377,78 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			assert(NULL != gd_header);	/* view_arg_convert would have done this for VTK_POOLLIMIT */
 			reg = parmblk.gv_ptr;
 			if (!reg->open)
-				gv_init_reg(reg);
+				gv_init_reg(reg, NULL);
 			csa = &FILE_INFO(reg)->s_addrs;
 			n = csa->gbuff_limit;
 			break;
+#ifdef TESTPOLLCRIT
+		case VTK_GRABCRIT:
+		case VTK_RELCRIT:
+		case VTK_GRABLOCK:
+		case VTK_RELLOCK:
+		case VTK_GRABJNLPH2:
+		case VTK_RELJNLPH2:
+		case VTK_RELJNLPOOLPH2:
+		case VTK_GRABJNLPOOLPH2:
+		case VTK_GRABJNLQIO:
+		case VTK_RELJNLQIO:
+			reg = parmblk.gv_ptr;
+			if (!reg->open)
+				gv_init_reg(reg, NULL);
+			csa = &FILE_INFO(reg)->s_addrs;
+			if (NULL != csa->hdr)
+			{
+				if (VTK_GRABCRIT == vtp->keycode)
+					grab_crit(reg);
+				else if (VTK_RELCRIT == vtp->keycode)
+					rel_crit(reg);
+				else if (VTK_GRABLOCK == vtp->keycode)
+				{
+					assert(NULL != jnlpool);
+					if (jnlpool)
+						grab_lock(jnlpool->jnlpool_dummy_reg, TRUE, ASSERT_NO_ONLINE_ROLLBACK);
+				} else if (VTK_RELLOCK == vtp->keycode)
+				{
+					assert(NULL != jnlpool);
+					if (jnlpool)
+						rel_lock(jnlpool->jnlpool_dummy_reg);
+				} else if (VTK_GRABJNLPH2 == vtp->keycode)
+					grab_latch(&csa->jnl->jnl_buff->phase2_commit_latch, GRAB_LATCH_INDEFINITE_WAIT);
+				else if (VTK_RELJNLPH2 == vtp->keycode)
+					rel_latch(&csa->jnl->jnl_buff->phase2_commit_latch);
+				else if (VTK_GRABJNLPOOLPH2 == vtp->keycode)
+				{
+					assert((NULL != jnlpool) && (NULL != jnlpool->jnlpool_ctl));
+					if (jnlpool && jnlpool->jnlpool_ctl)
+						grab_latch(&jnlpool->jnlpool_ctl->phase2_commit_latch, GRAB_LATCH_INDEFINITE_WAIT);
+				} else if (VTK_RELJNLPOOLPH2 == vtp->keycode)
+				{
+					assert((NULL != jnlpool) && (NULL != jnlpool->jnlpool_ctl));
+					if (jnlpool && jnlpool->jnlpool_ctl)
+						rel_latch(&jnlpool->jnlpool_ctl->phase2_commit_latch);
+				} else if (VTK_GRABJNLQIO == vtp->keycode)
+				{
+					while (!GET_SWAPLOCK(&csa->jnl->jnl_buff->io_in_prog_latch))
+						SHORT_SLEEP(1);
+				} else if (VTK_RELJNLQIO == vtp->keycode)
+					RELEASE_SWAPLOCK(&csa->jnl->jnl_buff->io_in_prog_latch);
+			}
+			break;
+#endif
 		case VTK_PROBECRIT:
 			assert(NULL != gd_header);	/* view_arg_convert would have done this for VTK_POOLLIMIT */
 			reg = parmblk.gv_ptr;
 			if (!reg->open)
-				gv_init_reg(reg);
+				gv_init_reg(reg, NULL);
 			csa = &FILE_INFO(reg)->s_addrs;
 			if (NULL != csa->hdr)
 			{
-				UNIX_ONLY(csa->crit_probe = TRUE);
+				csa->crit_probe = TRUE;
 				grab_crit(reg);
-				UNIX_ONLY(csa->crit_probe = FALSE);
+				csa->crit_probe = FALSE;
 				if (!WBTEST_ENABLED(WBTEST_HOLD_CRIT_ENABLED))
 					rel_crit(reg);
 				dst->str.len = 0;
-#				ifdef UNIX
 				ENSURE_STP_FREE_SPACE(n_probecrit_rec_types * (STATS_MAX_DIGITS + STATS_KEYWD_SIZE));
 				dst->str.addr = (char *)stringpool.free;
 				/* initialize csa->proberit_rec.p_crit_success field from cnl->gvstats_rec */
@@ -383,7 +459,6 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 				assert(stringpool.free < stringpool.top);
 				/* subtract one to remove extra trailing comma delimiter */
 				dst->str.len = INTCAST((char *)stringpool.free - dst->str.addr - 1);
-#				endif
 			}
 			break;
 		case VTK_REGION:
@@ -399,9 +474,10 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			key = gvn2gds(arg1, gvkey, n);
 			assert(key > &gvkey->base[0]);
 			assert(gvkey->end == key - &gvkey->base[0] - 1);
-			start_map = gv_srch_map(gd_header, (char *)&gvkey->base[0], gvkey->end - 1); /* -1 to remove trailing 0 */
+			/* -1 usage in "gv_srch_map" calls below is to remove trailing 0 */
+			start_map = gv_srch_map(gd_header, (char *)&gvkey->base[0], gvkey->end - 1, SKIP_BASEDB_OPEN_FALSE);
 			GVKEY_INCREMENT_ORDER(gvkey);
-			end_map = gv_srch_map(gd_header, (char *)&gvkey->base[0], gvkey->end - 1); /* -1 to remove trailing 0 */
+			end_map = gv_srch_map(gd_header, (char *)&gvkey->base[0], gvkey->end - 1, SKIP_BASEDB_OPEN_FALSE);
 			BACK_OFF_ONE_MAP_ENTRY_IF_EDGECASE(gvkey->base, gvkey->end - 1, end_map);
 			INCREMENT_GD_TARG_TN(gd_targ_tn);	/* takes a copy of incremented "TREF(gd_targ_tn)"
 								 * into local variable "gd_targ_tn" */
@@ -413,6 +489,7 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			commastr.addr = ",";
 			for (map = start_map; map <= end_map; map++)
 			{
+				OPEN_BASEREG_IF_STATSREG(map);
 				reg = map->reg.addr;
 				GET_REG_INDEX(gd_header, reg_start, reg, reg_index);	/* sets "reg_index" */
 				assert((NULL == tn_array) || (TREF(gd_targ_reg_array_size) > reg_index));
@@ -451,7 +528,7 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 		case VTK_BLFREE:
 			assert(gd_header);
 			if (!parmblk.gv_ptr->open)
-				gv_init_reg(parmblk.gv_ptr);
+				gv_init_reg(parmblk.gv_ptr, NULL);
 			csa = &FILE_INFO(parmblk.gv_ptr)->s_addrs;
 			if (NULL != csa->hdr)
 				n = csa->hdr->trans_hist.free_blocks;
@@ -461,7 +538,7 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 		case VTK_BLTOTAL:
 			assert(gd_header);
 			if (!parmblk.gv_ptr->open)
-				gv_init_reg(parmblk.gv_ptr);
+				gv_init_reg(parmblk.gv_ptr, NULL);
 			csa = &FILE_INFO(parmblk.gv_ptr)->s_addrs;
 			if (NULL != csa->hdr)
 			{
@@ -473,7 +550,7 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 		case VTK_FREEZE:
 			assert(gd_header);
 			if (!parmblk.gv_ptr->open)
-				gv_init_reg(parmblk.gv_ptr);
+				gv_init_reg(parmblk.gv_ptr, NULL);
 			csa = &FILE_INFO(parmblk.gv_ptr)->s_addrs;
 			if (NULL != csa->hdr)
 				n = csa->hdr->freeze;
@@ -483,7 +560,7 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 		case VTK_GVSTATS:
 			assert(gd_header);
 			if (!parmblk.gv_ptr->open)
-				gv_init_reg(parmblk.gv_ptr);
+				gv_init_reg(parmblk.gv_ptr, NULL);
 			csa = &FILE_INFO(parmblk.gv_ptr)->s_addrs;
 			if (NULL != csa->hdr)
 			{
@@ -573,7 +650,9 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			}
 			break;
 		case VTK_YDIRTREE:
+			n = extnam_str.len;		/* internal use of op_gvname should not disturb extended reference */
 			op_gvname(VARLSTCNT(1) parmblk.value);
+			extnam_str.len = n;
 			if (NULL != arg2)
 			{
 				view_arg_convert(vtp, VTP_DBREGION, arg2, &parmblk2, IS_DOLLAR_VIEW_TRUE);
@@ -597,50 +676,9 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			RESET_GV_TARGET(DO_GVT_GVKEY_CHECK);
 			break;
 		case VTK_YGDS2GVN:
-			if (NULL != arg2)
-			{
-				n = mval2i(arg2);
-				if (0 != n)
-				{
-					csp = ready_collseq(n);
-					if (NULL == csp)
-					{
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_COLLATIONUNDEF, 1, n);
-						break;
-					}
-				} else
-					csp = NULL;	/* Do not issue COLLATIONUNDEF for 0 collation */
-			}
-			/* Temporarily repoint global variables "gv_target" and "transform".
-			 * They are needed by format_targ_key/gvsub2str "transform" and "gv_target->collseq".
-			 */
-			save_transform = TREF(transform);
-			assert(save_transform);
-			TREF(transform) = TRUE;
-			reset_gv_target = gv_target;
-			gv_target = &temp_gv_target;
-			memset(gv_target, 0, SIZEOF(gv_namehead));
-			if (NULL != arg2)
-				gv_target->collseq = csp;
-			assert(MV_IS_STRING(arg1));
-			gvkey = &save_currkey[0];
-			gvkey->prev = 0;
-			gvkey->top = DBKEYSIZE(MAX_KEY_SZ);
-			if ((gvkey->top < arg1->str.len) || (2 > arg1->str.len)
-					|| (KEY_DELIMITER != arg1->str.addr[arg1->str.len-1])
-					|| (KEY_DELIMITER != arg1->str.addr[arg1->str.len-2]))
-				*dst = literal_null;
-			else
-			{
-				memcpy(&gvkey->base[0], arg1->str.addr, arg1->str.len);
-				DEBUG_ONLY(gvkey->end = arg1->str.len - 1;)	/* for an assert in format_targ_key */
-				if (0 == (c = format_targ_key(&buff[0], MAX_ZWR_KEY_SZ, gvkey, FALSE)))
-					c = &buff[MAX_ZWR_KEY_SZ - 1];
-				COPY_ARG_TO_STRINGPOOL(dst, c, &buff[0]);
-			}
-			/* Restore global variables "gv_target" and "transform" back to their original state */
-			RESET_GV_TARGET(DO_GVT_GVKEY_CHECK);
-			TREF(transform) = save_transform;
+			n = (NULL != arg2) ? mval2i(arg2) : 0;
+			key = gds2gvn(arg1, &buff[0], n);
+			COPY_ARG_TO_STRINGPOOL(dst, key, &buff[0]);
 			break;
 		case VTK_YGVN2GDS:
 			n = (NULL != arg2) ? mval2i(arg2) : 0;
@@ -664,7 +702,7 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			n = 0;
 			assert(gd_header);
 			if (!parmblk.gv_ptr->open)
-				gv_init_reg(parmblk.gv_ptr);
+				gv_init_reg(parmblk.gv_ptr, NULL);
 			reg = parmblk.gv_ptr;
 			if (dba_cm == REG_ACC_METH(reg))
 				n = ((link_info *)reg->dyn.addr->cm_blk->usr)->buffer_size;
@@ -673,7 +711,7 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			n = 0;
 			assert(gd_header);
 			if (!parmblk.gv_ptr->open)
-				gv_init_reg(parmblk.gv_ptr);
+				gv_init_reg(parmblk.gv_ptr, NULL);
 			reg = parmblk.gv_ptr;
 			if (dba_cm == REG_ACC_METH(reg))
 				n = ((link_info *)reg->dyn.addr->cm_blk->usr)->buffered_count;
@@ -682,7 +720,7 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 			n = 0;
 			assert(gd_header);
 			if (!parmblk.gv_ptr->open)
-				gv_init_reg(parmblk.gv_ptr);
+				gv_init_reg(parmblk.gv_ptr, NULL);
 			reg = parmblk.gv_ptr;
 			if (dba_cm == REG_ACC_METH(reg))
 				n = ((link_info *)reg->dyn.addr->cm_blk->usr)->buffer_used;
@@ -719,7 +757,6 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 		case VTK_LOGNONTP:
 			n = TREF(nontprestart_log_delta);
 			break;
-#		ifdef UNIX
 		case VTK_JNLERROR:
 			n = TREF(error_on_jnl_file_lost);
 			break;
@@ -731,7 +768,9 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 		case VTK_DMTERM:
 			n = dmterm_default;
 			break;
-#		endif
+		case VTK_STATSHARE:
+			n = TREF(statshare_opted_in) ? TRUE : FALSE;
+			break;
 		default:
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_VIEWFN);
 	}
@@ -739,278 +778,3 @@ void	op_fnview(UNIX_ONLY_COMMA(int numarg) mval *dst, ...)
 	if (MV_NM == vtp->restype)
 		MV_FORCE_MVAL(dst, n);
 }
-
-/* Converts a GVN in string representation into a key in subscript representation.
- * Note: This code is very similar to "is_canonic_name()". With some effort, they might even be merged into one.
- */
-STATICFNDEF unsigned char *gvn2gds(mval *gvn, gv_key *gvkey, int act)
-{
-	boolean_t	save_transform, is_zchar;
-	collseq		*csp;
-	gd_region	tmpreg, *save_gv_cur_region;
-	gv_namehead	temp_gv_target;
-	int		quotestate, clen;
-	int4		num;
-	unsigned char	strbuff[MAX_KEY_SZ + 1], *key, *key_start, *key_top, *c, *c_top, ch, *str, *str_top, *c1, *c2, *numptr;
-	char		fnname[6];	/* to hold the function name $CHAR or $ZCHAR */
-	char		number[32];	/* to hold a codepoint numeric value for $char or $zchar; 32 digits is more than enough */
-	mval		tmpmval, *mvptr, dollarcharmval;
-	DCL_THREADGBL_ACCESS;
-
-	SETUP_THREADGBL_ACCESS;
-	if (0 != act)
-	{
-		csp = ready_collseq(act);
-		if (NULL == csp)
-			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_COLLATIONUNDEF, 1, act);
-	} else
-		csp = NULL;	/* Do not issue COLLATIONUNDEF for 0 collation */
-	assert(MV_IS_STRING(gvn));
-	c = (unsigned char *)gvn->str.addr;
-	c_top = c + gvn->str.len;
-	key_start = &gvkey->base[0];
-	key = key_start;
-	if ((c >= c_top) || ('^' != *c++))
-		return key;
-	gvkey->prev = 0;
-	gvkey->top = DBKEYSIZE(MAX_KEY_SZ);
-	key_top = key_start + gvkey->top;
-	/* Parse GBLNAME */
-	for ( ; (c < c_top) && (key < key_top); )
-	{
-		if ('(' == *c)
-			break;
-		*key++ = *c++;
-	}
-	if (key >= key_top)
-		return key_start;
-	if (c == c_top)
-	{
-		*key++ = KEY_DELIMITER;
-		gvkey->end = key - key_start;
-		*key++ = KEY_DELIMITER;
-		return key;
-	}
-	assert('(' == *c);
-	c++; /* skip past "(" */
-	*key++ = KEY_DELIMITER;
-	gvkey->end = key - key_start;
-	str = &strbuff[0];
-	str_top = str + ARRAYSIZE(strbuff);
-	/* Temporarily repoint global variables "gv_cur_region", "gv_target" and "transform".
-	 * They are needed by mval2subsc for the following
-	 *	"transform", "gv_target->nct", "gv_target->collseq" and "gv_cur_region->std_null_coll"
-	 */
-	save_transform = TREF(transform);
-	assert(save_transform);
-	TREF(transform) = TRUE;
-	reset_gv_target = gv_target;
-	gv_target = &temp_gv_target;
-	memset(gv_target, 0, SIZEOF(gv_namehead));
-	gv_target->collseq = csp;
-	save_gv_cur_region = gv_cur_region;
-	gv_cur_region = &tmpreg;
-	memset(gv_cur_region, 0, SIZEOF(gd_region));
-	/* Note that the primary caller of the YGVN2GDS functionality is going to be GDE for globals that
-	 * span regions. And since such globals need to reside in regions that have standard null collation
-	 * defined, we set the std_null_coll field to TRUE above.
-	 */
-	gv_cur_region->std_null_coll = TRUE;
-	/* Parse subscripts */
-	quotestate = 0;
-	for ( ; (c < c_top) && (str < str_top); c++)
-	{
-		ch = *c;
-		switch (quotestate)
-		{
-			case 0:
-			case 6:
-				if (('"' == ch) || ('$' == ch))
-				{	/* start of a string subscript */
-					if (0 == quotestate)
-					{
-						tmpmval.mvtype = (MV_STR | MV_NUM_APPROX);
-							/* MV_NUM_APPROX needed by mval2subsc to skip val_iscan call */
-						str = &strbuff[0];
-						tmpmval.str.addr = (char *)str;
-					}
-					quotestate = ('"' == ch) ? 1 : 3;
-				} else if (6 == quotestate)
-				{	/* Defer rts_error until after global variables "gv_cur_region" etc. are restored. */
-					quotestate = -1;/* error in input */
-					c = c_top;	/* do not parse remaining input as subscripted gvn is complete now */
-				} else
-				{	/* quotestate is 0, in this case this is the start of a number */
-					quotestate = 4;	/* start of a number */
-					tmpmval.mvtype = MV_STR;
-					tmpmval.str.addr = (char *)c;
-				}
-				break;
-			case 1:
-				if ('"' == ch)
-					quotestate = 2;
-				else
-					*str++ = ch;	/* and quotestate stays at 1 */
-				break;
-			case 2:
-			case 9:
-				if ((2 == quotestate) && ('"' == ch))
-				{
-					*str++ = '"';
-					quotestate = 1;
-					break;
-				} else if (')' == ch)
-					quotestate = 5;
-				else if (',' == ch)
-					quotestate = 0;
-				else if ('_' == ch)
-				{
-					quotestate = 6;
-					break;
-				} else
-				{	/* Defer rts_error until after global variables "gv_cur_region" etc. are restored. */
-					quotestate = -1;	/* error in input */
-					c = c_top;		/* force break from for loop */
-					break;
-				}
-				assert((')' == ch) || (',' == ch));
-				tmpmval.str.len = str - (unsigned char *)tmpmval.str.addr;
-				DEBUG_ONLY(TREF(skip_mv_num_approx_assert) = TRUE;)
-				mval2subsc(&tmpmval, gvkey, gv_cur_region->std_null_coll);
-				DEBUG_ONLY(TREF(skip_mv_num_approx_assert) = FALSE;)
-				assert(gvkey->end < gvkey->top); /* else GVSUBOFLOW error would have been issued */
-				key = &gvkey->base[gvkey->end];
-				break;
-			case 3:
-				/* Allow only one of $C( or $CHAR( or $ZCH( or $ZCHAR( */
-				c1 = c;
-				for ( ; (c < c_top) && ('(' != *c); c++)
-					;
-				if (c == c_top)
-					break;
-				clen = c - c1;
-				if (clen >= ARRAYSIZE(fnname))
-				{
-					c = c_top;	/* force break from for loop */
-					break;		/* bad function name. issue error after breaking from for loop */
-				}
-				for (c2 = (unsigned char *)&fnname[0]; c1 < c; c2++, c1++)
-					*c2 = TOUPPER(*c1);
-				if (!MEMCMP_LIT(fnname, "ZCHAR") || !MEMCMP_LIT(fnname, "ZCH"))
-					is_zchar = 1;
-				else if (!MEMCMP_LIT(fnname, "CHAR") || !MEMCMP_LIT(fnname, "C"))
-					is_zchar = 0;
-				else
-				{
-					c = c_top;	/* force break from for loop */
-					break;		/* bad function name. issue error after breaking from for loop */
-				}
-				assert('(' == *c);
-				quotestate = 7;
-				break;
-			case 4:
-				if (',' == ch)
-					quotestate = 0;
-				else if (')' == ch)
-					quotestate = 5;
-				else
-					break;
-				tmpmval.str.len = c - (unsigned char *)tmpmval.str.addr;
-				mvptr = &tmpmval;
-				MV_FORCE_NUM(mvptr);
-				if (MVTYPE_IS_NUM_APPROX(tmpmval.mvtype))
-				{	/* User specified either a non-numeric or an imprecise numeric.
-					 * Defer rts_error until after global variables "gv_cur_region" etc. are restored.
-					 */
-					quotestate = -1;/* error in input */
-					c = c_top;	/* do not parse remaining input as subscripted gvn is complete now */
-					break;
-				}
-				mval2subsc(&tmpmval, gvkey, gv_cur_region->std_null_coll);
-				assert(gvkey->end < gvkey->top); /* else GVSUBOFLOW error would have been issued */
-				key = &gvkey->base[gvkey->end];
-				break;
-			case 5:
-				/* Defer rts_error until after global variables "gv_cur_region" etc. are restored. */
-				quotestate = -1;/* error in input */
-				c = c_top;	/* do not parse remaining input as subscripted gvn is complete now */
-				break;
-			case 7:
-				if (!ISDIGIT_ASCII(ch))
-				{	/* Not an ascii numeric digit.
-					 * Defer rts_error until after global variables "gv_cur_region" etc. are restored.
-					 */
-					quotestate = -1;/* error in input */
-					c = c_top;	/* do not parse remaining input as subscripted gvn is complete now */
-					break;
-				}
-				numptr = c;	/* record start of number */
-				quotestate = 8;
-				break;
-			case 8:
-				if (')' == ch)
-					quotestate = 9;
-				else if (',' == ch)
-					quotestate = 7;
-				else if (!ISDIGIT_ASCII(ch))
-				{	/* Not an ascii numeric digit
-					 * Defer rts_error until after global variables "gv_cur_region" etc. are restored.
-					 */
-					quotestate = -1;/* error in input */
-					c = c_top;	/* do not parse remaining input as subscripted gvn is complete now */
-					break;
-				} else
-					break;	/* continue processing numeric argument to $c or $zch */
-				/* end of the $c() number. find its zchar value */
-				if ((c - numptr) >= ARRAYSIZE(number))
-				{	/* number specified to $c or $zch is more than 32 digits long. error out */
-					c = c_top;	/* force break from for loop */
-					break;		/* bad function name. issue error after breaking from for loop */
-				}
-				memcpy(number, numptr, c - numptr);
-				number[c - numptr] = '\0';
-				num = (int4)STRTOUL(number, NULL, 10);
-				if (0 > num)
-				{	/* number is negative. issue error */
-					c = c_top;	/* force break from for loop */
-					break;		/* bad function name. issue error after breaking from for loop */
-				}
-#				ifdef UNICODE_SUPPORTED
-				if (!is_zchar && is_gtm_chset_utf8)
-					op_fnchar(2, &dollarcharmval, num);
-				else
-#				endif
-					op_fnzchar(2, &dollarcharmval, num);
-				assert(MV_IS_STRING(&dollarcharmval));
-				if (dollarcharmval.str.len)
-				{
-					if (str + dollarcharmval.str.len > str_top)
-					{	/* String overflows capacity.
-						 * Defer rts_error until after global variables "gv_cur_region" etc. are restored.
-						 */
-						quotestate = -1;/* error in input */
-						c = c_top; /* do not parse remaining input as subscripted gvn is complete now */
-						break;
-					}
-					memcpy(str, dollarcharmval.str.addr, dollarcharmval.str.len);
-					str += dollarcharmval.str.len;
-				}
-				break;
-			default:
-				assertpro(FALSE && quotestate);
-				break;
-		}
-	}
-	/* Restore global variables "gv_cur_region", "gv_target" and "transform" back to their original state */
-	gv_cur_region = save_gv_cur_region;
-	RESET_GV_TARGET(DO_GVT_GVKEY_CHECK);
-	TREF(transform) = save_transform;
-	if ((str >= str_top) || !CAN_APPEND_HIDDEN_SUBS(gvkey))
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_GVSUBOFLOW, 0, ERR_GVIS, 2, gvn->str.len, gvn->str.addr);
-	if (5 == quotestate)
-		*key++ = KEY_DELIMITER;	/* add double terminating null byte */
-	else
-		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_GVSUBSERR);
-	assert(key <= key_top);
-	return key;
-}
