@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001 Sanchez Computer Associates, Inc.	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -17,12 +18,18 @@
 GBLREF bool		remlkreq;
 GBLREF unsigned short	lks_this_cmd;
 
+error_def(ERR_BADLOCKNEST);
+
 void op_lkinit(void)
 {
+	if (lks_this_cmd)
+	{
+		lks_this_cmd = 0;
+		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_BADLOCKNEST);
+	}
 	if (remlkreq)
 	{
 		gvcmz_clrlkreq();
 		remlkreq = FALSE;
 	}
-	lks_this_cmd = 0;
 }
