@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -13,13 +13,14 @@
 #include "mdef.h"
 
 #include <stdarg.h>
-#if defined(UNIX) && defined(DEBUG)
+#ifdef DEBUG
 # include "gtm_syslog.h"	/* Needed for white box case in VTK_STORDUMP */
 #endif
 #include "gtm_string.h"
 #include "gtm_stdio.h"
 
 #include "gtmio.h"
+#include "io.h"
 #include "util.h"
 #include "have_crit.h"
 #include "gdsroot.h"
@@ -62,13 +63,11 @@
 # include "gv_trigger.h"
 # include "gtm_trigger.h"
 #endif
-#ifdef UNIX
-# include "wbox_test_init.h"
-# include "mutex.h"
-# include "gtmlink.h"
-# ifdef DEBUG
-#  include "gtmsecshr.h"
-# endif
+#include "wbox_test_init.h"
+#include "mutex.h"
+#include "gtmlink.h"
+#ifdef DEBUG
+# include "gtmsecshr.h"
 #endif
 #ifdef AUTORELINK__SUPPORTED
 # include "relinkctl.h"
@@ -76,7 +75,13 @@
 #include "gtmimagename.h"
 #include "cache.h"
 #include "set_gbuff_limit.h"	/* Needed for set_gbuff_limit() */
+#include "interlock.h"
+#include "wcs_backoff.h"
+#include "wcs_wt.h"
+#include "localvarmonitor.h"
+#include "is_file_identical.h"	/* Needed for JNLPOOL_INIT_IF_NEEDED */
 
+STATICFNDCL void lvmon_release(void);
 STATICFNDCL void view_dbop(unsigned char keycode, viewparm *parmblkptr, mval *thirdarg);
 
 GBLREF	volatile int4 		db_fsync_in_prog;
@@ -104,9 +109,11 @@ GBLREF	uint4			gtmDebugLevel;
 GBLREF	boolean_t		lvmon_enabled;
 GBLREF	spdesc			stringpool;
 GBLREF	boolean_t		is_updproc;
-GBLREF	pid_t			process_id;
+GBLREF	uint4			process_id;
 GBLREF	uint4			dollar_tlevel;
-UNIX_ONLY(GBLREF	boolean_t		dmterm_default;)
+GBLREF	boolean_t		dmterm_default;
+GBLREF mstr			extnam_str;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool;
 
 error_def(ERR_ACTRANGE);
 error_def(ERR_COLLATIONUNDEF);
@@ -116,6 +123,7 @@ error_def(ERR_GBLNOMAPTOREG);
 error_def(ERR_INVZDIRFORM);
 error_def(ERR_ISOLATIONSTSCHN);
 error_def(ERR_JNLFLUSH);
+error_def(ERR_TPNOSTATSHARE);
 error_def(ERR_PATLOAD);
 error_def(ERR_PATTABNOTFND);
 error_def(ERR_REQDVIEWPARM);
@@ -145,7 +153,7 @@ error_def(ERR_ZDEFACTIVE);
 			gvnh->gvname.var_name.addr, gvnh->noisolation, status);					\
 }
 
-void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
+void	op_view(int numarg, mval *keyword, ...)
 {
 	boolean_t		dbgdmpenabled, old_bool, was_crit, was_skip_gtm_putmsg;
 	char			*chptr;
@@ -157,11 +165,12 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 	gv_namehead		*gvnh;
 	hash_table_mname	*table;
 	ht_ent_mname		*tabent, *table_base_orig, *topent;
-	int			clrlen, lcnt, lct, icnt, ncol, nct, status, table_size_orig;
+	int			acnt, clrlen, lcnt, lct, icnt, ncol, nct, size, status, table_size_orig;
 	int4			testvalue, tmpzdefbufsiz;
 	jnl_buffer_ptr_t	jb;
 	lv_blk			*lvbp;
 	lv_val			*lv, *lvp, *lvp_top;
+	lvmon_var		*lvmon_var_p, *lvmon_vars_base;
 	mstr			tmpstr;
 	mval			*arg, *nextarg, outval;
 	noisolation_element	*gvnh_entry;
@@ -176,7 +185,6 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 	open_relinkctl_sgm	*linkctl;
 	relinkrec_t		*linkrec;
 #	endif
-	VMS_ONLY(int		numarg;)
 	static readonly char msg1[] = "Caution: Database Block Certification Has Been ";
 	static readonly char lv_msg1[] =
 		"Caution: GT.M reserved local variable string pointer duplicate check diagnostic has been";
@@ -188,7 +196,6 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 
 	SETUP_THREADGBL_ACCESS;
 	VAR_START(var, keyword);
-	VMS_ONLY(va_count(numarg));
 	jnl_status = 0;
 	assertpro(1 <= numarg);
 	MV_FORCE_STR(keyword);
@@ -239,14 +246,12 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 		case VTK_POOLLIMIT:
 			view_dbop(vtp->keycode, &parmblk, (numarg > 1) ? va_arg(var, mval *) : (mval *)NULL);
 			break;
-#		ifdef UNIX
 		case VTK_DMTERM:
 			dmterm_default = TRUE;
 			break;
 		case VTK_NODMTERM:
 			dmterm_default = FALSE;
 			break;
-#		endif
 		case VTK_FULLBOOL:
 		case VTK_FULLBOOLWARN:
 		case VTK_NOFULLBOOL:
@@ -273,9 +278,6 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 		case VTK_GVDUPSETNOOP:
 			gvdupsetnoop = (0 != MV_FORCE_INT(parmblk.value));
 			break;
-		case VTK_LVDUPCHECK:
-			/* This feature is not needed any more. This is a noop now */
-			break;
 		case VTK_LVNULLSUBS:
 			TREF(lv_null_subs) = LVNULLSUBS_OK;
 			break;
@@ -285,9 +287,8 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 		case VTK_NEVERLVNULLSUBS:
 			TREF(lv_null_subs) = LVNULLSUBS_NEVER;
 			break;
-#		ifndef VMS
 		case VTK_JNLERROR:
-			/* In case of update process, dont let this variable be user-controlled. We always want to error out
+			/* In case of update process, don't let this variable be user-controlled. We always want to error out
 			 * if we are about to invoke "jnl_file_lost". This way we will force the operator to fix whatever
 			 * caused the jnl_file_lost invocation (e.g. permissions, disk space issues etc.) and restart
 			 * the receiver server so replication can resume from wherever we last left. On the other hand,
@@ -304,7 +305,6 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 				view_dbop(vtp->keycode, &parmblk, (mval *)NULL);
 			}
 			break;
-#		endif
 		case VTK_JNLWAIT:
 			/* Go through all regions that could have possibly been open across all global directories */
 			if (!dollar_tlevel)
@@ -371,7 +371,7 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 						case dba_mm:
 						case dba_bg:
 							csa = &FILE_INFO(reg)->s_addrs;
-							memset(&csa->gvstats_rec, 0, SIZEOF(gvstats_rec_t));
+							memset((char *)csa->gvstats_rec_p, 0, SIZEOF(gvstats_rec_t));
 							break;
 						case dba_cm:
 						case dba_usr:
@@ -438,7 +438,9 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 			 * will fail if the entry is not present.  So make sure that the global exists before
 			 * a YDIRTREE update is performed
 			 */
+			size = extnam_str.len;		/* internal use of op_gvname should not disturb extended reference */
 			op_gvname(VARLSTCNT(1) parmblk.value);
+			extnam_str.len = size;
 			arg = (numarg > 1) ? va_arg(var, mval *) : NULL;
 			if (NULL != arg)
 			{
@@ -609,20 +611,7 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 					turn_tracing_off(NULL);
 			}
 			break;
-		case VTK_ZDIR_FORM:
-			VMS_ONLY(
-				if (NULL != arg)
-				{
-					testvalue = MV_FORCE_INT(parmblk.value);
-					if (!IS_VALID_ZDIR_FORM(testvalue))
-					{
-						va_end(var);
-						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(3) ERR_INVZDIRFORM, 1, testvalue);
-					}
-				} else
-					testvalue = ZDIR_FORM_FULLPATH;
-				zdir_form = testvalue;
-			)
+		case VTK_ZDIR_FORM:			/* Vestigial remnant from VMS - to be removed in the future */
 			break;
 		case VTK_FILLFACTOR:
 			testvalue = MV_FORCE_INT(parmblk.value);
@@ -659,7 +648,7 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 			DEFER_BASE_REL_HASHTAB(table, FALSE);
 			break;
 		case VTK_STORDUMP:
-#			if defined(DEBUG) && defined(UNIX)
+#			ifdef DEBUG
 			if (gtm_white_box_test_case_enabled
 				&& (WBTEST_HOLD_CRIT_TILL_LCKALERT == gtm_white_box_test_case_number))
 			{	/* Hold crit for a long enough interval to generate lock alert which then does a continue_proc */
@@ -734,11 +723,9 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 		case VTK_NOLOGNONTP:
 			TREF(nontprestart_log_delta) = 0;
 			break;
-#		ifdef UNIX
 		case VTK_LINK:
 			init_relink_allowed(&parmblk.value->str);
 			break;
-#		endif
 #		ifdef DEBUG_ALIAS
 		case VTK_LVMONOUT:
 			als_lvmon_output();
@@ -779,6 +766,55 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 			op_wteol(1);
 			break;
 #		endif
+		case VTK_STATSHARE:
+			if (!TREF(statshare_opted_in))
+			{	/* Don't both to opt-in if already opted-in - it just confuses things */
+				if (0 < dollar_tlevel)
+				{	/* Can't do this in TP */
+					va_end(var);
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_TPNOSTATSHARE);
+				}
+				gvcst_statshare_optin();
+			}
+			break;
+		case VTK_NOSTATSHARE:
+			if (TREF(statshare_opted_in))
+			{	/* Don't both to opt-out if already opted-out - it just confuses things */
+				if (0 < dollar_tlevel)
+				{	/* Can't do this in TP */
+					va_end(var);
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_TPNOSTATSHARE);
+				}
+				gvcst_statshare_optout();
+			}
+			break;
+		case VTK_LVMON:
+			if (0 < numarg)
+			{	/* A variable list was supplied - Need a new table but first free any existing old table first */
+				if (NULL != TREF(lvmon_vars_anchor))
+					lvmon_release();			/* Table exists. Free allocated var names/values */
+				TREF(lvmon_vars_count) = numarg;		/* Elements in this table */
+				size = numarg * SIZEOF(lvmon_var);		/* Byte size of new table */
+				lvmon_vars_base = TREF(lvmon_vars_anchor) = malloc(size);	/* Allocate new table */
+				memset(lvmon_vars_base, 0, size);		/* Clear new table */
+				for (acnt = numarg, lvmon_var_p = lvmon_vars_base;  0 < acnt; acnt--, lvmon_var_p++)
+				{	/* Load up new table from args given */
+					MV_FORCE_STR(arg);
+					lvmon_var_p->lvmv.var_name.len = arg->str.len;
+					lvmon_var_p->lvmv.var_name.addr = malloc(arg->str.len);
+					memcpy(lvmon_var_p->lvmv.var_name.addr, arg->str.addr, arg->str.len);
+					COMPUTE_HASH_MNAME(&lvmon_var_p->lvmv);	/* Set hash value for var name */
+					if (1 < acnt)				/* If another var to fetch, do so */
+						arg = va_arg(var, mval *);
+				}
+				TREF(lvmon_active) = TRUE;			/* We are active now */
+			} else
+			{
+				if (NULL != TREF(lvmon_vars_anchor))
+					lvmon_release();			/* No vars specified - free what have if any */
+				TREF(lvmon_active) = FALSE;			/* No monitoring active now */
+			}
+			break;
 		default:
 			va_end(var);
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_VIEWCMD);
@@ -787,21 +823,64 @@ void	op_view(UNIX_ONLY_COMMA(int numarg) mval *keyword, ...)
 	return;
 }
 
-void view_dbop(unsigned char keycode, viewparm *parmblkptr, mval *thirdarg)
+/* Routine to locate the structure associated with local variable monitoring across given points in GTM code
+ * and release all of:
+ *   1. The malloc'd variable names.
+ *   2. The malloc'd variable values.
+ *   3. The array of lvmon_vars structures anchored at TREF(lvmon_var_anchor).
+ */
+STATICFNDEF void lvmon_release(void)
+{
+	int		acnt;
+	lvmon_var	*lvmon_var_p, *lvmon_vars_base;
+	lvmon_value_ent	*lvmon_val_ent_p;
+	int		ecnt;
+	DCL_THREADGBL_ACCESS;
+
+	SETUP_THREADGBL_ACCESS;
+	for (acnt = TREF(lvmon_vars_count), lvmon_var_p = TREF(lvmon_vars_anchor);
+	     0 < acnt;
+	     acnt--, lvmon_var_p++)
+	{
+		assert(NULL != lvmon_var_p->lvmv.var_name.addr);
+		if (NULL != lvmon_var_p->lvmv.var_name.addr)
+		{
+			free(lvmon_var_p->lvmv.var_name.addr);	/* Free var name */
+			lvmon_var_p->lvmv.var_name.addr = NULL;
+		}
+		for (ecnt = MAX_LVMON_VALUES, lvmon_val_ent_p = &lvmon_var_p->values[0];
+		     0 < ecnt;
+		     ecnt--, lvmon_val_ent_p++)
+		{
+			assert(NULL != lvmon_val_ent_p->varvalue.addr);
+			if (NULL != lvmon_val_ent_p->varvalue.addr)
+			{
+				free(lvmon_val_ent_p->varvalue.addr);
+				lvmon_val_ent_p->varvalue.addr = NULL;
+			}
+		}
+	}
+	free(TREF(lvmon_vars_anchor));		/* Release the old table */
+	TREF(lvmon_vars_anchor) = NULL;
+}
+
+STATICFNDEF void view_dbop(unsigned char keycode, viewparm *parmblkptr, mval *thirdarg)
 {
 	boolean_t		was_crit;
 	gd_region		*reg, *r_top, *save_reg;
+	jnlpool_addrs_ptr_t	save_jnlpool;
 	int			icnt, lcnt, save_errno, status;
 	int4			nbuffs;
 	jnl_buffer_ptr_t	jb;
 	sgmnt_addrs		*csa;
 	sgmnt_data_ptr_t	csd;
-	uint4			jnl_status, dummy_errno;
-	UNIX_ONLY(unix_db_info	*udi;)
+	uint4			jnl_status, dummy_errno, wcsflu_parms;
+	unix_db_info		*udi;
 
 	if (NULL == gd_header)		/* Open gbldir */
 		gvinit();
 	save_reg = gv_cur_region;
+	save_jnlpool = jnlpool;
 	if (NULL == parmblkptr->gv_ptr)
 	{	/* Operate on all regions */
 		reg = gd_header->regions;
@@ -810,24 +889,30 @@ void view_dbop(unsigned char keycode, viewparm *parmblkptr, mval *thirdarg)
 		r_top = reg = parmblkptr->gv_ptr;
 	for (; reg <= r_top; reg++)
 	{
+		if (IS_STATSDB_REG(reg))
+			continue;	/* Skip statsdb regions for the VIEW command */
 		if (!reg->open)
-			gv_init_reg(reg);
+			gv_init_reg(reg, NULL);
 		TP_CHANGE_REG(reg);
+		/* note that the jnlpool needs to be initialized and validated for options which could write to the database
+		 * so instance freeze can be honored.  No data is writtern just file header, etc. so OK on secondary
+		 */
 		switch(keycode)
 		{
 			case VTK_DBFLUSH:
 				if (!reg->read_only)
 				{
+					JNLPOOL_INIT_IF_NEEDED(cs_addrs, cs_data, cs_addrs->nl, SCNDDBNOUPD_CHECK_FALSE);
 					nbuffs = (NULL != thirdarg) ? MV_FORCE_INT(thirdarg) : cs_addrs->nl->wcs_active_lvl;
-					JNL_ENSURE_OPEN_WCS_WTSTART(cs_addrs, reg, nbuffs, dummy_errno);
+					JNL_ENSURE_OPEN_WCS_WTSTART(cs_addrs, reg, nbuffs, NULL, FALSE, dummy_errno);
 				}
 				break;
 			case VTK_DBSYNC:
-#				ifdef UNIX
 				if (!reg->read_only)
 				{
 					csa = cs_addrs;
 					udi = FILE_INFO(reg);
+					JNLPOOL_INIT_IF_NEEDED(cs_addrs, cs_data, cs_addrs->nl, SCNDDBNOUPD_CHECK_FALSE);
 					DB_FSYNC(reg, udi, csa, db_fsync_in_prog, save_errno);
 					if (0 != save_errno)
 					{
@@ -836,12 +921,12 @@ void view_dbop(unsigned char keycode, viewparm *parmblkptr, mval *thirdarg)
 						save_errno = 0;
 					}
 				}
-#				endif
 				break;
 			case VTK_EPOCH:
 			case VTK_FLUSH:
-				if (!reg->read_only)
+				if (!reg->read_only  && !FROZEN_CHILLED(cs_addrs))
 				{
+					JNLPOOL_INIT_IF_NEEDED(cs_addrs, cs_data, cs_addrs->nl, SCNDDBNOUPD_CHECK_FALSE);
 					ENSURE_JNL_OPEN(cs_addrs, gv_cur_region);
 					/* We should NOT invoke wcs_recover here because it's possible we are in the final retry
 					 * of a TP transaction. In this case, we likely have pointers to non-dirty global buffers
@@ -850,16 +935,19 @@ void view_dbop(unsigned char keycode, viewparm *parmblkptr, mval *thirdarg)
 					 * TPFAIL error because we are already in the final retry. By passing the WCSFLU_IN_COMMIT
 					 * bit, we instruct wcs_flu to avoid wcs_recover.
 					 */
-					wcs_flu(WCSFLU_FLUSH_HDR | WCSFLU_WRITE_EPOCH | WCSFLU_IN_COMMIT | WCSFLU_SPEEDUP_NOBEFORE);
+					wcsflu_parms = WCSFLU_FLUSH_HDR | WCSFLU_WRITE_EPOCH | WCSFLU_SPEEDUP_NOBEFORE;
+					if (dollar_tlevel)
+						wcsflu_parms |= WCSFLU_IN_COMMIT;
+					wcs_flu(wcsflu_parms);
 				}
 				break;
 			case VTK_GVSRESET:
 				change_reg();
 				if (!reg->read_only)
 					CLRGVSTATS(cs_addrs);
-				memset((char *)&cs_addrs->gvstats_rec, 0, SIZEOF(gvstats_rec_t));	/* Always process-private */
+				/* Reset process stats in either process-private storage or a shared statsDB record */
+				memset((char *)cs_addrs->gvstats_rec_p, 0, SIZEOF(gvstats_rec_t));
 				break;
-#			ifndef VMS
 			case VTK_JNLERROR:
 				if (!reg->read_only)
 				{
@@ -877,26 +965,27 @@ void view_dbop(unsigned char keycode, viewparm *parmblkptr, mval *thirdarg)
 					}
 				}
 				break;
-#				endif
 			case VTK_JNLFLUSH:
 				csa = cs_addrs;
 				csd = csa->hdr;
 				if (JNL_ENABLED(csd))
 				{
+					JNLPOOL_INIT_IF_NEEDED(cs_addrs, cs_data, cs_addrs->nl, SCNDDBNOUPD_CHECK_FALSE);
 					was_crit = csa->now_crit;
 					if (!was_crit)
 						grab_crit(reg);
 					if (JNL_ENABLED(csd))
 					{
-						jnl_status = jnl_ensure_open();
+						jnl_status = jnl_ensure_open(reg, csa);
 						if (0 == jnl_status)
 						{
 							jb = csa->jnl->jnl_buff;
 							if (SS_NORMAL == (jnl_status = jnl_flush(reg)))
 							{
 								assert(jb->dskaddr == jb->freeaddr);
-								UNIX_ONLY(jnl_fsync(reg, jb->dskaddr));
-								UNIX_ONLY(assert(jb->freeaddr == jb->fsync_dskaddr));
+								assert(jb->freeaddr == jb->rsrv_freeaddr);
+								jnl_fsync(reg, jb->dskaddr);
+								assert(jb->freeaddr == jb->fsync_dskaddr);
 							} else
 							{
 								send_msg_csa(CSA_ARG(csa) VARLSTCNT(9) ERR_JNLFLUSH, 2,
@@ -922,5 +1011,7 @@ void view_dbop(unsigned char keycode, viewparm *parmblkptr, mval *thirdarg)
 	}
 	gv_cur_region = save_reg;
 	change_reg();
+	if (save_jnlpool && (save_jnlpool != jnlpool))
+		jnlpool = save_jnlpool;
 	return;
 }
