@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -112,10 +112,10 @@ void op_zprint(mval *rtn, mval *start_label, int start_int_exp, mval *end_label,
 		null_str.mvtype = MV_STR;
 		null_str.str.len = 0;
 		stat2 = get_src_line(rtn, &null_str, 1, &src2, NULL);
+		assert(NULL != src2);
 		/* number of lines less one for duplicated zero'th line and one due
 		   to termination condition being <=
 		*/
-		assert((INTPTR_T)src2 > 0);
 		src2 += rtn_vector->lnrtab_len - 2;
 	}
 	if (stat1 & CHECKSUMFAIL)
