@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -9,7 +9,7 @@
  *	the license, please stop and do not read further.	*
  *								*
  ****************************************************************/
-
+/* defines opcodes and their associated octypes */
 
 
 
@@ -21,14 +21,14 @@ OPCODE_DEF(OC_NOOP, (OCT_CGSKIP))
 OPCODE_DEF(OC_PARAMETER, (OCT_CGSKIP))
 OPCODE_DEF(OC_VAR, (OCT_MVADDR | OCT_MVAL | OCT_CGSKIP | OCT_EXPRLEAF))
 OPCODE_DEF(OC_LIT, (OCT_MVAL | OCT_CGSKIP | OCT_EXPRLEAF))
-OPCODE_DEF(OC_ADD, (OCT_MVAL))
-OPCODE_DEF(OC_SUB, (OCT_MVAL))
-OPCODE_DEF(OC_MUL, (OCT_MVAL))
-OPCODE_DEF(OC_DIV, (OCT_MVAL))
-OPCODE_DEF(OC_IDIV, (OCT_MVAL))
-OPCODE_DEF(OC_MOD, (OCT_MVAL))
-OPCODE_DEF(OC_NEG, (OCT_MVAL))
-OPCODE_DEF(OC_FORCENUM, (OCT_MVAL))
+OPCODE_DEF(OC_ADD, (OCT_MVAL | OCT_ARITH))
+OPCODE_DEF(OC_SUB, (OCT_MVAL | OCT_ARITH))
+OPCODE_DEF(OC_MUL, (OCT_MVAL | OCT_ARITH))
+OPCODE_DEF(OC_DIV, (OCT_MVAL | OCT_ARITH))
+OPCODE_DEF(OC_IDIV, (OCT_MVAL | OCT_ARITH))
+OPCODE_DEF(OC_MOD, (OCT_MVAL | OCT_ARITH))
+OPCODE_DEF(OC_NEG, (OCT_MVAL | OCT_UNARY))
+OPCODE_DEF(OC_FORCENUM, (OCT_MVAL | OCT_UNARY))
 OPCODE_DEF(OC_CAT, (OCT_MVAL))
 OPCODE_DEF(OC_SRCHINDX, (OCT_MVADDR | OCT_MVAL | OCT_EXPRLEAF))
 OPCODE_DEF(OC_GETINDX, (OCT_MVADDR | OCT_MVAL | OCT_EXPRLEAF))
@@ -74,26 +74,25 @@ OPCODE_DEF(OC_FNZSEA, (OCT_MVAL | OCT_EXPRLEAF))
 OPCODE_DEF(OC_FNZSETPRV, (OCT_MVAL | OCT_EXPRLEAF))
 OPCODE_DEF(OC_SVGET, (OCT_MVAL | OCT_EXPRLEAF))
 OPCODE_DEF(OC_SVPUT, (OCT_NULL))
-OPCODE_DEF(OC_LT, (OCT_BOOL))
-OPCODE_DEF(OC_NLT, (OCT_BOOL))
-OPCODE_DEF(OC_GT, (OCT_BOOL))
-OPCODE_DEF(OC_NGT, (OCT_BOOL))
-OPCODE_DEF(OC_EQU, (OCT_BOOL))
-OPCODE_DEF(OC_NEQU, (OCT_BOOL))
-OPCODE_DEF(OC_CONTAIN, (OCT_BOOL))
-OPCODE_DEF(OC_NCONTAIN, (OCT_BOOL))
-OPCODE_DEF(OC_FOLLOW, (OCT_BOOL))
-OPCODE_DEF(OC_NFOLLOW, (OCT_BOOL))
-OPCODE_DEF(OC_PATTERN, (OCT_BOOL))
-OPCODE_DEF(OC_NPATTERN, (OCT_BOOL))
+OPCODE_DEF(OC_LT, (OCT_BOOL | OCT_REL))
+OPCODE_DEF(OC_NLT, (OCT_BOOL | OCT_REL | OCT_NEGATED))
+OPCODE_DEF(OC_GT, (OCT_BOOL | OCT_REL))
+OPCODE_DEF(OC_NGT, (OCT_BOOL | OCT_REL | OCT_NEGATED))
+OPCODE_DEF(OC_EQU, (OCT_BOOL | OCT_REL))
+OPCODE_DEF(OC_NEQU, (OCT_BOOL | OCT_REL | OCT_NEGATED))
+OPCODE_DEF(OC_CONTAIN, (OCT_BOOL | OCT_REL))
+OPCODE_DEF(OC_NCONTAIN, (OCT_BOOL | OCT_REL | OCT_NEGATED))
+OPCODE_DEF(OC_FOLLOW, (OCT_BOOL | OCT_REL))
+OPCODE_DEF(OC_NFOLLOW, (OCT_BOOL | OCT_REL | OCT_NEGATED))
+OPCODE_DEF(OC_PATTERN, (OCT_BOOL | OCT_REL))
+OPCODE_DEF(OC_NPATTERN, (OCT_BOOL | OCT_REL | OCT_NEGATED))
 OPCODE_DEF(OC_AND, (OCT_BOOL))
-OPCODE_DEF(OC_NAND, (OCT_BOOL))
+OPCODE_DEF(OC_NAND, (OCT_BOOL | OCT_NEGATED))
 OPCODE_DEF(OC_OR, (OCT_BOOL))
-OPCODE_DEF(OC_NOR, (OCT_BOOL))
-OPCODE_DEF(OC_COM, (OCT_BOOL))
+OPCODE_DEF(OC_NOR, (OCT_BOOL | OCT_NEGATED))
+OPCODE_DEF(OC_COM, (OCT_BOOL | OCT_UNARY))
 OPCODE_DEF(OC_BREAK, (OCT_NULL))
 OPCODE_DEF(OC_CLOSE, (OCT_NULL))
-OPCODE_DEF(OC_HALT, (OCT_NULL))
 OPCODE_DEF(OC_HANG, (OCT_NULL))
 OPCODE_DEF(OC_JOB, (OCT_NULL))
 OPCODE_DEF(OC_KILL, (OCT_NULL))
@@ -152,12 +151,12 @@ OPCODE_DEF(OC_INDSET, (OCT_NULL))
 OPCODE_DEF(OC_GVEXTNAM, (OCT_NULL))
 OPCODE_DEF(OC_INDNAME, (OCT_MVAL | OCT_EXPRLEAF))
 OPCODE_DEF(OC_INDTEXT, (OCT_MVAL | OCT_EXPRLEAF))
-OPCODE_DEF(OC_COMVAL, (OCT_MVAL | OCT_COERCE))
+OPCODE_DEF(OC_COMVAL, (OCT_MVAL | OCT_COERCE | OCT_UNARY))
 OPCODE_DEF(OC_GVSAVTARG, (OCT_MVAL))
 OPCODE_DEF(OC_GVRECTARG, (OCT_NULL))
 OPCODE_DEF(OC_COMINT, (OCT_MINT | OCT_COERCE))
 OPCODE_DEF(OC_FNTRANSLATE, (OCT_MVAL | OCT_EXPRLEAF))
-OPCODE_DEF(OC_COBOOL, (OCT_BOOL | OCT_COERCE))
+OPCODE_DEF(OC_COBOOL, (OCT_BOOL | OCT_COERCE | OCT_UNARY))
 OPCODE_DEF(OC_BOOLINIT, (OCT_MINT | OCT_EXPRLEAF))
 OPCODE_DEF(OC_BOOLFINI, (OCT_NULL))
 OPCODE_DEF(OC_SETTEST, (OCT_NULL))
@@ -165,7 +164,7 @@ OPCODE_DEF(OC_CLRTEST, (OCT_NULL))
 OPCODE_DEF(OC_FORLOOP, (OCT_NULL))
 OPCODE_DEF(OC_JMPTSET, (OCT_JUMP))
 OPCODE_DEF(OC_JMPTCLR, (OCT_JUMP))
-OPCODE_DEF(OC_NUMCMP, (OCT_BOOL))
+OPCODE_DEF(OC_NUMCMP, (OCT_BOOL | OCT_REL))
 OPCODE_DEF(OC_JMPEQU, (OCT_JUMP))
 OPCODE_DEF(OC_JMPNEQ, (OCT_JUMP))
 OPCODE_DEF(OC_JMPGTR, (OCT_JUMP))
@@ -243,7 +242,7 @@ OPCODE_DEF(OC_TSTART, (OCT_NULL))
 OPCODE_DEF(OC_TROLLBACK, (OCT_NULL))
 OPCODE_DEF(OC_TRESTART, (OCT_NULL))
 OPCODE_DEF(OC_TCOMMIT, (OCT_NULL))
-OPCODE_DEF(OC_EXP, (OCT_MVAL))
+OPCODE_DEF(OC_EXP, (OCT_MVAL | OCT_ARITH))
 OPCODE_DEF(OC_FNGET2, (OCT_MVAL | OCT_EXPRLEAF))
 OPCODE_DEF(OC_INDINCR, (OCT_MVAL | OCT_EXPRLEAF))
 OPCODE_DEF(OC_FNNAME, (OCT_MVAL | OCT_EXPRLEAF))
@@ -264,8 +263,8 @@ OPCODE_DEF(OC_FNZBITAND, (OCT_MVAL | OCT_EXPRLEAF))
 OPCODE_DEF(OC_FNZBITOR, (OCT_MVAL | OCT_EXPRLEAF))
 OPCODE_DEF(OC_FNZBITXOR, (OCT_MVAL | OCT_EXPRLEAF))
 OPCODE_DEF(OC_FGNLOOKUP, (OCT_CDADDR))
-OPCODE_DEF(OC_SORTS_AFTER, (OCT_BOOL))
-OPCODE_DEF(OC_NSORTS_AFTER, (OCT_BOOL))
+OPCODE_DEF(OC_SORTS_AFTER, (OCT_BOOL | OCT_REL))
+OPCODE_DEF(OC_NSORTS_AFTER, (OCT_BOOL | OCT_REL | OCT_NEGATED))
 OPCODE_DEF(OC_FNGVGET1, (OCT_MVAL | OCT_EXPRLEAF))
 OPCODE_DEF(OC_FNGET1, (OCT_MVAL | OCT_EXPRLEAF))
 OPCODE_DEF(OC_SETP1, (OCT_MVAL | OCT_EXPRLEAF))
@@ -346,3 +345,7 @@ OPCODE_DEF(OC_FNZSOCKET, (OCT_MVAL | OCT_EXPRLEAF))
 OPCODE_DEF(OC_FNZSYSLOG, (OCT_MVAL | OCT_EXPRLEAF))
 OPCODE_DEF(OC_ZRUPDATE, (OCT_NULL))
 OPCODE_DEF(OC_CDIDX, (OCT_MINT | OCT_CGSKIP | OCT_EXPRLEAF))
+OPCODE_DEF(OC_FNZCOLLATE, (OCT_MVAL | OCT_EXPRLEAF))
+OPCODE_DEF(OC_FNZATRANSFORM, (OCT_MVAL | OCT_EXPRLEAF))
+OPCODE_DEF(OC_LASTOPCODE, (OCT_CGSKIP))
+/* insert new opcodes before OC_LASTOPCODE */
