@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -314,6 +314,10 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 						 * 	fixed part from the indefinite part.
 						 */
 						split_atom = TRUE;
+						if ((count >= (MAX_PATTERN_ATOMS - 1)) ||
+								(atom_map >= (MAX_PATTERN_ATOMS -2)))
+			                         	return ERR_PATMAXLEN;
+
 					} else
 					{
 						infinite = TRUE;
@@ -740,6 +744,7 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 				if (pattern_mask & PATM_STRLIT)
 				{
 					memset(&exp_temp[0], 0, SIZEOF(exp_temp));
+					assert(atom_map < MAX_PATTERN_ATOMS);
 					min[atom_map] = lower_bound;
 					max[atom_map] = upper_bound;
 					size[atom_map] = strlit.bytelen;
@@ -810,6 +815,7 @@ int patstr(mstr *instr, ptstr *obj, unsigned char **relay)
 							atom_map--;
 						}
 					}
+					assert(atom_map < MAX_PATTERN_ATOMS);
 					min[atom_map] = lower_bound;
 					max[atom_map] = upper_bound;
 					size[atom_map] = 1;
