@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -62,6 +62,7 @@
 #include "opcode.h"
 #include "glvn_pool.h"
 #include "iormdef.h"
+#include "localvarmonitor.h"
 
 #ifndef STP_MOVE
 GBLDEF int	indr_stp_low_reclaim_passes = 0;
@@ -87,8 +88,8 @@ GBLREF io_log_name		*io_root_log_name;
 GBLREF lvzwrite_datablk		*lvzwrite_block;
 GBLREF mliteral			literal_chain;
 GBLREF mstr			*comline_base, **stp_array;
-GBLREF mval			dollar_etrap, dollar_system, dollar_zerror, dollar_zgbldir, dollar_zstatus, dollar_zstep;
-GBLREF mval			dollar_ztrap, dollar_zyerror, zstep_action, dollar_zinterrupt, dollar_zsource, dollar_ztexit;
+GBLREF mval			dollar_system, dollar_zerror, dollar_zgbldir, dollar_zstatus;
+GBLREF mval			dollar_zyerror, zstep_action, dollar_zinterrupt, dollar_zsource, dollar_ztexit;
 GBLREF mv_stent			*mv_chain;
 GBLREF sgm_info			*first_sgm_info;
 GBLREF spdesc			indr_stringpool, rts_stringpool, stringpool;
@@ -103,36 +104,31 @@ GBLREF int4			LVGC_interval;				/* dead data GC done every LVGC_interval stringp
 GBLREF boolean_t		suspend_lvgcol;
 GBLREF hash_table_str		*complits_hashtab;
 GBLREF mval			*alias_retarg;
+GBLREF io_pair			*io_std_device;
 GTMTRIG_ONLY(GBLREF mval 	dollar_ztwormhole;)
 DEBUG_ONLY(GBLREF   boolean_t	ok_to_UNWIND_in_exit_handling;)
-UNIX_ONLY(GBLREF io_pair	*io_std_device;)
+
 
 OS_PAGE_SIZE_DECLARE
 
 static mstr			**topstr, **array, **arraytop;
 
 error_def(ERR_STPEXPFAIL);
+error_def(ERR_STPCRIT);
+error_def(ERR_STPOFLOW);
 
 /* See comment inside LV_NODE_KEY_STPG_ADD macro for why the ASSERT_LV_NODE_MSTR_EQUIVALENCE macro does what it does */
-#ifdef UNIX
-# define	ASSERT_LV_NODE_MSTR_EQUIVALENCE									\
-{														\
+#define ASSERT_LV_NODE_MSTR_EQUIVALENCE										\
+MBSTART {													\
 	assert(OFFSETOF(lvTreeNode, key_len) - OFFSETOF(lvTreeNode, key_mvtype) == OFFSETOF(mstr, len));	\
 	assert(SIZEOF(((lvTreeNode *)NULL)->key_len) == SIZEOF(((mstr *)NULL)->len));				\
 	assert(OFFSETOF(lvTreeNode, key_addr) - OFFSETOF(lvTreeNode, key_mvtype) == OFFSETOF(mstr, addr));	\
 	assert(SIZEOF(((lvTreeNode *)NULL)->key_addr) == SIZEOF(((mstr *)NULL)->addr));				\
-}
-#elif defined(VMS)
-# define	ASSERT_LV_NODE_MSTR_EQUIVALENCE									\
-{														\
-	assert(SIZEOF(((lvTreeNode *)NULL)->key_len) == SIZEOF(((mstr *)NULL)->len));				\
-	assert(OFFSETOF(lvTreeNode, key_addr) - OFFSETOF(lvTreeNode, key_len) == OFFSETOF(mstr, addr));		\
-	assert(SIZEOF(((lvTreeNode *)NULL)->key_addr) == SIZEOF(((mstr *)NULL)->addr));				\
-}
-#endif
+} MBEND
 
 #define	LV_NODE_KEY_STPG_ADD(NODE)											\
-{	/* NODE is of type "lvTreeNode *" or "lvTreeNodeNum *".								\
+MBSTART {														\
+	/* NODE is of type "lvTreeNode *" or "lvTreeNodeNum *".								\
 	 * Only if it is a "lvTreeNode *" (MV_STR bit is set in this case only)						\
 	 * should we go ahead with the STPG_ADD.									\
 	 */														\
@@ -157,7 +153,7 @@ error_def(ERR_STPEXPFAIL);
 		UNIX_ONLY(MSTR_STPG_ADD((mstr *)(void *)&NODE->key_mvtype));						\
 		VMS_ONLY(MSTR_STPG_ADD((mstr *)(void *)&NODE->key_len));						\
 	}														\
-}
+} MBEND
 
 #define	MVAL_STPG_ADD(MVAL1)							\
 {										\
@@ -350,13 +346,9 @@ error_def(ERR_STPEXPFAIL);
 			if (HTENT_VALID_ADDR(tabent_addr, zwr_alias_var, zav))					\
 			{											\
 				x = &zav->zwr_var;								\
-				/* Regular varnames are already accounted for in other ways so			\
-				 * we need to avoid putting this mstr into the process array twice.		\
-				 * The only var names we need worry about are $ZWRTACxxx so make		\
-				 * simple check for var names starting with '$'.				\
-				 */										\
-				if (x->len && ('$' == x->addr[0]))						\
-					MSTR_STPG_ADD(x);							\
+				/* Protect MSTRs we are using to process the zwrite */				\
+				assert(x->len); /* if it is valid it should have a non-zero length */		\
+				MSTR_STPG_ADD(x);								\
 			}											\
 		}												\
 	}													\
@@ -369,7 +361,7 @@ error_def(ERR_STPEXPFAIL);
 #else
 #define DBGSTPGCOL(x)
 #endif
-static void expand_stp(unsigned int new_size)	/* BYPASSOK */
+static void expand_stp(size_t new_size)	/* BYPASSOK */
 {
 	if (retry_if_expansion_fails)
 		ESTABLISH(stp_gcol_ch);
@@ -428,7 +420,7 @@ void stp_vfy_mval(void)
 	}
 }
 
-boolean_t is_stp_space_available(int space_needed)
+boolean_t is_stp_space_available(ssize_t space_needed)
 {
 	/* Asserts added here are tested every time any module in the codebase does stringpool space checks */
 	assert(!stringpool_unusable);
@@ -456,15 +448,16 @@ void mv_parse_tree_collect(mvar *node)
 void stp_move(char *stp_move_from, char *stp_move_to)
 #else
 /* garbage collect and create enough space for space_asked bytes */
-void stp_gcol(int space_asked)	/* BYPASSOK */
+void stp_gcol(size_t space_asked)	/* BYPASSOK */
 #endif
 {
 #	ifdef STP_MOVE
 	int			space_asked = 0, stp_move_count = 0;
 #	endif
 	unsigned char		*strpool_base, *straddr, *tmpaddr, *begaddr, *endaddr;
-	int			index, space_needed, fixup_cnt, tmplen, totspace;
-	long			space_before_compact, space_after_compact, blklen, delta, space_reclaim, padlen;
+	int			index, fixup_cnt;
+	ssize_t			space_before_compact, space_after_compact, blklen, delta, space_reclaim, padlen;
+	ssize_t			totspace, space_needed, tmplen, tmplen2;
 	io_log_name		*l;		/* logical name pointer		*/
 	lv_blk			*lv_blk_ptr;
 	lv_val			*lvp, *lvlimit;
@@ -480,7 +473,7 @@ void stp_gcol(int space_asked)	/* BYPASSOK */
 	int			*low_reclaim_passes;
 	int			*incr_factor;
 	int			killcnt;
-	long			stp_incr;
+	ssize_t			stp_incr;
 	ht_ent_objcode 		*tabent_objcode, *topent;
 	ht_ent_mname		*tabent_mname, *topent_mname;
 	ht_ent_addr		*tabent_addr, *topent_addr;
@@ -541,10 +534,12 @@ void stp_gcol(int space_asked)	/* BYPASSOK */
 		DBGRFCT((stderr, "stp_gcol: lvgcol check bypassed for lack of aliasness\n"));
 	}
 #	endif	/* DEBUG_REFCNT */
+	if (IS_LVMON_ACTIVE && (stringpool.base == rts_stringpool.base))
+		lvmon_pull_values(1);			/* Pull set of values into index 1 if rts stringpool */
 #	endif	/* !STP_MOVE */
 	assert(stringpool.free >= stringpool.base);
 	assert(stringpool.free <= stringpool.top);
-	assert(stringpool.top - stringpool.free < space_asked || space_asked == 0);
+	assert(stringpool.invokestpgcollevel <= stringpool.top);
         assert(CHK_BOUNDARY_ALIGNMENT(stringpool.top) == 0);
 	/* stp_vfy_mval(); / * uncomment to debug lv corruption issues.. */
 #	ifdef STP_MOVE
@@ -661,15 +656,6 @@ void stp_gcol(int space_asked)	/* BYPASSOK */
 			for (xnewvar = symtab->xnew_var_list; xnewvar; xnewvar = xnewvar->next)
 				MSTR_STPG_ADD(&xnewvar->key.var_name);
 		}
-#		ifdef VMS
-		if (NULL != (TREF(rt_name_tbl)).base)
-		{	/* Keys for $TEXT source hash table can live in stringpool */
-			for (tabent_mname = (TREF(rt_name_tbl)).base, topent_mname = (TREF(rt_name_tbl)).top;
-			    tabent_mname < topent_mname; tabent_mname++)
-				if (HTENT_VALID_MNAME(tabent_mname, routine_source, rsptr))
-					MSTR_STPG_ADD(&tabent_mname->key.var_name);
-		}
-#		endif
 		if (x = comline_base)
 			for (index = MAX_RECALL; index > 0 && x->len; index--, x++)
 			{	/* These strings are guaranteed to be in the stringpool so use MSTR_STPG_PUT macro directly
@@ -683,19 +669,20 @@ void stp_gcol(int space_asked)	/* BYPASSOK */
 			LVZWRITE_BLOCK_GC(lvzwrblk);
 		}
 		ZWRHTAB_GC(zwrhtab);
-		for (l = io_root_log_name; 0 != l; l = l->next)
+		for (l = io_root_log_name; NULL != l; l = l->next)
 		{
+			if ((NULL == l->iod) || (n_io_dev_types == l->iod->type))
+			{
+				assert(FALSE);
+				continue;       /* skip it on pro */
+			}
 			if ((IO_ESC != l->dollar_io[0]) && (l->iod->trans_name == l))
 			{
 				MSTR_STPG_ADD(&l->iod->error_handler);
-#				ifdef UNIX
-				/* if this is on UNIX and it is a split $principal, protect
-				 * error_handler defined on the output side
-				 */
+				/* If this is a split $principal, protect error_handler defined on the output side */
 				if ((l->iod->pair.in != l->iod->pair.out)
 				    && (l->iod->pair.out == io_std_device->out))
 					MSTR_STPG_ADD(&l->iod->pair.out->error_handler);
-#				endif
 				rm_ptr = (rm == l->iod->type) ? (d_rm_struct *)l->iod->dev_sp : NULL;
 				if (NULL != rm_ptr)
 				{	/* Protect the IVs and KEYs as needed. */
@@ -712,14 +699,14 @@ void stp_gcol(int space_asked)	/* BYPASSOK */
 				}
 			}
 		}
-		MVAL_STPG_ADD(&dollar_etrap);
+		MVAL_STPG_ADD(&(TREF(dollar_etrap)));
 		MVAL_STPG_ADD(&dollar_system);
 		MVAL_STPG_ADD(&dollar_zsource);
-		MVAL_STPG_ADD(&dollar_ztrap);
+		MVAL_STPG_ADD(&(TREF(dollar_ztrap)));
 		MVAL_STPG_ADD(&dollar_zstatus);
 		MVAL_STPG_ADD(&dollar_zgbldir);
 		MVAL_STPG_ADD(&dollar_zinterrupt);
-		MVAL_STPG_ADD(&dollar_zstep);
+		MVAL_STPG_ADD(&(TREF(dollar_zstep)));
 		MVAL_STPG_ADD(&zstep_action);
 		MVAL_STPG_ADD(&dollar_zerror);
 		MVAL_STPG_ADD(&dollar_ztexit);
@@ -925,8 +912,7 @@ void stp_gcol(int space_asked)	/* BYPASSOK */
 			assert(0 < tmplen);
 			if (tmpaddr + tmplen > straddr) /* if it is not a proper substring of previous one */
 			{
-				int tmplen2;
-				tmplen2 = ((tmpaddr >= straddr) ? tmplen : (int)(tmpaddr + tmplen - straddr));
+				tmplen2 = ((tmpaddr >= straddr) ? tmplen : (ssize_t)(tmpaddr + tmplen - straddr));
 				assert(0 < tmplen2);
 				totspace += tmplen2;
 				if (mstr_native_align)
@@ -945,7 +931,7 @@ void stp_gcol(int space_asked)	/* BYPASSOK */
 	assert(mstr_native_align || space_after_compact >= space_before_compact);
 #	endif
 	space_reclaim = space_after_compact - space_before_compact; /* this can be -ve, if alignment causes expansion */
-	space_needed -= (int)space_after_compact;
+	space_needed -= (ssize_t)space_after_compact;
 	DBGSTPGCOL((stderr, "space_needed=%i\n", space_needed));
 	/* After compaction if less than 31.25% of space is avail, consider it a low reclaim pass */
 	if (STP_LOWRECLAIM_LEVEL(stringpool.top - stringpool.base) > space_after_compact) /* BYPASSOK */
@@ -995,7 +981,11 @@ void stp_gcol(int space_asked)	/* BYPASSOK */
 			expansion_failed = FALSE;	/* will be set to TRUE by condition handler if can't get memory */
 			assert((stp_incr + stringpool.top - stringpool.base) >= (space_needed + blklen));
 			DBGSTPGCOL((stderr, "incr_factor=%i stp_incr=%i space_needed=%i\n", *incr_factor, stp_incr, space_needed));
-			expand_stp((unsigned int)(stp_incr + stringpool.top - stringpool.base));
+			if ((TREF(gtm_strpllimwarned)) /* previously warned */
+				&& (0 < TREF(gtm_strpllim)) /* watching a stp limit */
+				&& ((stp_incr + stringpool.top - stringpool.base) > TREF(gtm_strpllim))) /* expanding larger */
+					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_STPOFLOW);
+			expand_stp((ssize_t)(stp_incr + stringpool.top - stringpool.base));
 #			ifdef DEBUG
 			/* If expansion failed and stp_gcol_ch did an UNWIND and we were already in exit handling code,
 			 * ok_to_UNWIND_in_exit_handling would have been set to avoid an assert in UNWIND macro. Now that
@@ -1093,8 +1083,25 @@ void stp_gcol(int space_asked)	/* BYPASSOK */
 	}
 	assert(stringpool.free >= stringpool.base);
 	assert(stringpool.free <= stringpool.top);
+	stringpool.invokestpgcollevel = (STP_SPACE_USED_MULTIPLIER * (space_asked + stringpool.free - stringpool.base))
+		+ stringpool.base;
+	stringpool.invokestpgcollevel = (((stringpool.invokestpgcollevel - stringpool.base) < STP_GCOL_TRIGGER_FLOOR)
+		|| (stringpool.invokestpgcollevel > stringpool.top))
+		? stringpool.top : stringpool.invokestpgcollevel;
+	assert(stringpool.invokestpgcollevel <= stringpool.top);
 #	ifndef STP_MOVE
 	assert(stringpool.top - stringpool.free >= space_asked);
-#	endif
+	if (IS_LVMON_ACTIVE && (stringpool.base == rts_stringpool.base))
+	{	/* Do monitoring only for runtime stringpool */
+		lvmon_pull_values(2);				/* Pull values of monitored vars into index 2 */
+		lvmon_compare_value_slots(1, 2);		/* Make sure they are the same */
+	}
+#	endif	/* !STP_MOVE */
+	if ((0 < TREF(gtm_strpllim)) /* monitoring stp limit */
+		&& ((stringpool.top - stringpool.base) > TREF(gtm_strpllim))) /* past the stp limit */
+	{
+		TREF(gtm_strpllimwarned) =  TRUE;
+		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_STPCRIT);
+	}
 	return;
 }
