@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -34,35 +34,30 @@
 #include "have_crit.h"
 #include "min_max.h"
 #include "tp_frame.h"
-#ifdef GTM_TRIGGER
 #include <rtnhdr.h>
+#include "gtm_trigger_trc.h"
 #include "gv_trigger.h"		/* for TP_INVALIDATE_TRIGGER_CYCLES_IF_NEEDED macro */
 #include "util.h"		/* for TP_ZTRIGBUFF_PRINT macro */
-#endif
 
 GBLREF	sgmnt_data_ptr_t	cs_data;
 GBLREF	sgmnt_addrs		*cs_addrs;
 GBLREF	ua_list			*curr_ua, *first_ua;
 GBLREF	uint4			dollar_trestart;
+GBLREF	boolean_t		dollar_ztrigger_invoked;
 GBLREF	sgm_info		*first_tp_si_by_ftok; /* List of participating regions in the TP transaction sorted on ftok order */
 GBLREF	jnl_gbls_t		jgbl;
 GBLREF	jnl_fence_control	jnl_fence_ctl;
 GBLREF	trans_num		local_tn;
 GBLREF	global_tlvl_info	*global_tlvl_info_head;
 GBLREF	buddy_list		*global_tlvl_info_list;
-GBLREF	block_id		gtm_tp_allocation_clue;	/* block# hint to start allocation for created blocks in TP */
 GBLREF	gd_region		*gv_cur_region;
 GBLREF	gv_namehead		*gv_target_list, *gvt_tp_list;
 GBLREF	sgm_info		*sgm_info_ptr, *first_sgm_info;
 GBLREF	int			process_exiting;
-GBLREF	block_id		tp_allocation_clue;
 GBLREF	char			*update_array, *update_array_ptr;
 GBLREF	uint4			update_array_size, cumul_update_array_size;
 GBLREF	tp_frame		*tp_pointer;
 GBLREF	uint4			update_trans;
-#ifdef VMS
-GBLREF	boolean_t		tp_has_kill_t_cse; /* cse->mode of kill_t_write or kill_t_create got created in this transaction */
-#endif
 #ifdef DEBUG
 GBLREF	unsigned int		t_tries;
 #endif
@@ -70,7 +65,7 @@ GBLREF	unsigned int		t_tries;
 error_def(ERR_MEMORY);
 error_def(ERR_VMSMEMORY);
 
-void	tp_clean_up(boolean_t rollback_flag)
+void	tp_clean_up(tp_cleanup_state clnup_state)
 {
 	gv_namehead	*gvnh, *blk_target;
 	sgm_info	*si, *next_si;
@@ -97,7 +92,7 @@ void	tp_clean_up(boolean_t rollback_flag)
 	assert((NULL != first_sgm_info) || (0 == cw_stagnate.size) || cw_stagnate_reinitialized);
 		/* if no database activity, cw_stagnate should be uninitialized or reinitialized */
 #	ifdef DEBUG
-	if (rollback_flag)
+	if (TP_COMMIT != clnup_state)
 		TREF(donot_commit) = FALSE;
 	assert(!TREF(donot_commit));
 #	endif
@@ -106,14 +101,24 @@ void	tp_clean_up(boolean_t rollback_flag)
 		update_trans = 0;
 	} else
 		assert(!update_trans);
+	/* Reset trigger state now. Trigger cycles, dollar_ztrigger_invoked and gvt_triggers_read_this_tn are reset only in
+	 * TP_INVALIDATE_TRIGGER_CYCLES_IF_NEEDED. Trigger cycles and dollar_ztrigger_invoked are different from other TP level
+	 * artifacts in that they are maintained through an incremental rollback and restarts so that future trigger operations in
+	 * the current transaction recognize the invalid trigger cycles and reload triggers as necessary. Trigger cycles and
+	 * dollar_ztrigger_invoked are only reset upon a commit or a complete rollback.
+	 * Unlike other cleanup, trigger reset is not made conditional upon the existence of first_sgm_info because it is possible
+	 * for the Update Process to restart and immediately issue a complete rollback while handling an ONLINE ROLLBACK.
+	 * Please see the comment for TP_INVALIDATE_TRIGGER_CYCLES_IF_NEEDED to understand how trigger cycles and friends are
+	 * handled across commit, rollback and restarts.
+	 */
+	TP_INVALIDATE_TRIGGER_CYCLES_IF_NEEDED(clnup_state);
 	if (NULL != first_sgm_info)
 	{	/* It is possible that first_ua is NULL at this point due to a prior call to tp_clean_up() that failed in
 		 * malloc() of tmp_ua->update_array. This is possible because we might have originally had two chunks of
 		 * update_arrays each x-bytes in size and we freed them up and requested 2x-bytes of contiguous storage
 		 * and we might error out on that malloc attempt (though this is very improbable).
 		 */
-		if ((NULL != first_ua) && (NULL != first_ua->next_ua)
-		    && !process_exiting && (UNIX_ONLY(ERR_MEMORY) VMS_ONLY(ERR_VMSMEMORY) != error_condition))
+		if ((NULL != first_ua) && (NULL != first_ua->next_ua) && !process_exiting && (ERR_MEMORY != error_condition))
 		{	/* if the original update array was too small, make a new larger one */
 			/* tmp_update_array_size is used below instead of the global variables (update_array_size,
 			 * first_ua->update_array_size or cumul_update_array_size) to handle error returns from	malloc()
@@ -156,7 +161,7 @@ void	tp_clean_up(boolean_t rollback_flag)
 			}
 		}
 		RESET_UPDATE_ARRAY; /* do not use CHECK_AND_RESET_UPDATE_ARRAY since we are in TP and will fail the check there */
-		if (rollback_flag) 		/* Rollback invalidates clues in all targets used by this transaction */
+		if (TP_COMMIT != clnup_state) 		/* Rollback invalidates clues in all targets used by this transaction */
 		{
 			for (gvnh = gvt_tp_list; NULL != gvnh; gvnh = gvnh->next_tp_gvnh)
 			{
@@ -172,11 +177,10 @@ void	tp_clean_up(boolean_t rollback_flag)
 				/* Cleanup any block-split info (of created block #) in gvtarget histories */
 				TP_CLEANUP_GVNH_SPLIT_IF_NEEDED(gvnh, 0);
 			}
-			GTMTRIG_ONLY(TP_INVALIDATE_TRIGGER_CYCLES_IF_NEEDED(FALSE, FALSE));
 #			ifdef DEBUG
 			if (!process_exiting)
 			{	/* Ensure that we did not miss out on resetting clue for any gvtarget.
-				 * Dont do this if the process is cleaning up the TP transaction as part of exit handling
+				 * Don't do this if the process is cleaning up the TP transaction as part of exit handling
 				 * as the tp_clean_up invocation could be due to an interrupt (MUPIP STOP etc.) and we cannot
 				 * be sure what state the mainline code was when it was interrupted. Thankfully, the clue
 				 * will be used only as part of the next transaction. Since the process is in the process of
@@ -191,18 +195,7 @@ void	tp_clean_up(boolean_t rollback_flag)
 			}
 #			endif
 			local_tn++;	/* to effectively invalidate first_tp_srch_status of all gv_targets */
-			tp_allocation_clue = gtm_tp_allocation_clue;	/* Reset clue to what it was at beginning of transaction */
-		} else
-		{
-			GTMTRIG_ONLY(TP_INVALIDATE_TRIGGER_CYCLES_IF_NEEDED(FALSE, TRUE));
-			gtm_tp_allocation_clue = tp_allocation_clue;	/* Update tp allocation clue for next transaction to skip
-									 * past values used in this transaction now that this one
-									 * is successfully committed.
-									 */
 		}
-		GTMTRIG_ONLY(assert(!TREF(gvt_triggers_read_this_tn));)
-		/* Assert that for all regions, we have csa->db_dztrigger_cycle = 0 or 1 (based on commit/restart/rollback) */
-		GTMTRIG_ONLY(TP_ASSERT_ZTRIGGER_CYCLE_RESET;)
 		for (si = first_sgm_info;  si != NULL;  si = next_si)
 		{
 			TP_TEND_CHANGE_REG(si);
@@ -214,6 +207,7 @@ void	tp_clean_up(boolean_t rollback_flag)
 					FREE_KILL_SET(ks);
 					si->kill_set_tail = NULL;
 					si->kill_set_head = NULL;
+					assert((NULL == si->kip_csa) || (!gv_cur_region->open || gv_cur_region->was_open));
 				}
 				if (NULL != si->jnl_head)
 				{
@@ -227,7 +221,7 @@ void	tp_clean_up(boolean_t rollback_flag)
 				 * inctn_tp_upd_no_logical_rec) was written. So reset cs_addrs->next_fenced unconditionally.
 				 */
 				cs_addrs->next_fenced = NULL;
-				if (FALSE == rollback_flag)
+				if (TP_COMMIT == clnup_state)
 				{	/* Non-rollback case (op_tcommit) validates clues in the targets we are updating */
 					sgm_info_ptr = si;	/* for tp_get_cw to work */
 					is_mm = (dba_mm == gv_cur_region->dyn.addr->acc_meth);
@@ -292,7 +286,7 @@ void	tp_clean_up(boolean_t rollback_flag)
 							{
 								cr = cse->cr;
 								assert(NULL != cr);
-								UNIX_ONLY(assert((NULL == t1->cr) || (t1->cr == cr)));
+								assert((NULL == t1->cr) || (t1->cr == cr));
 								if (cr != t1->cr)
 								{
 									t1->cr = cr;
@@ -348,7 +342,7 @@ void	tp_clean_up(boolean_t rollback_flag)
 				si->first_cw_set = si->last_cw_set = si->first_cw_bitmap = NULL;
 				si->cw_set_depth = 0;
 				si->update_trans = 0;
-			} else if (rollback_flag)
+			} else if (TP_COMMIT != clnup_state)
 				REINITIALIZE_LIST(si->tlvl_info_list);		/* reinitialize the tlvl_info buddy_list */
 #			ifdef DEBUG
 			/* Verify that all fields that were reset in the if code above are already at the reset value.
@@ -385,20 +379,13 @@ void	tp_clean_up(boolean_t rollback_flag)
 #		ifdef DEBUG
 		if (!process_exiting)
 		{	/* Ensure that we did not miss out on clearing any gv_target->root which had chain.flag set.
-			 * Dont do this if the process is cleaning up the TP transaction as part of exit handling
+			 * Don't do this if the process is cleaning up the TP transaction as part of exit handling
 			 * Also use this opportunity to check that non-zero clues for BG contain non-null cr in histories.
-			 * In addition, check that the list of multi-level block numbers (involved in the most recent split
-			 * operations) stored in the gv_target are valid block #s.
 			 */
 			for (gvnh = gv_target_list; NULL != gvnh; gvnh = gvnh->next_gvnh)
 			{
 				chain1 = *(off_chain *)&gvnh->root;
 				assert(!chain1.flag);
-				for (level = 0; level < ARRAYSIZE(gvnh->last_split_blk_num); level++)
-				{
-					chain1 = *(off_chain *)&gvnh->last_split_blk_num[level];
-					assert(!chain1.flag);
-				}
 				/* If there was a gvnh->write_local_tn, we could assert that if ever that field was updated
 				 * in this transaction, then gvnh->root better be non-zero. Otherwise gvnh could have been
 				 * used only for reads in this TP and in that case it is ok for the root to be 0.
@@ -407,7 +394,6 @@ void	tp_clean_up(boolean_t rollback_flag)
 				{	/* check that gv_target->root falls within total blocks range */
 					csa = gvnh->gd_csa;
 					assert(NULL != csa);
-					NON_GTM_TRUNCATE_ONLY(assert(gvnh->root < csa->ti->total_blks));
 					assert(!IS_BITMAP_BLK(gvnh->root));
 				}
 				if (gvnh->clue.end)
@@ -428,41 +414,35 @@ void	tp_clean_up(boolean_t rollback_flag)
 #		endif
 		jgbl.cumul_jnl_rec_len = 0;
 		jgbl.tp_ztp_jnl_upd_num = 0;
-		GTMTRIG_ONLY(
-			/* reset jgbl.prev_ztworm_ptr as we are now ready to start a new transaction
-			 * and thus need to write new ztwormhole records if needed
-			 */
-			jgbl.prev_ztworm_ptr = NULL;
-		)
+		/* reset jgbl.prev_ztworm_ptr as we are now ready to start a new transaction
+		 * and thus need to write new ztwormhole records if needed
+		 */
+		jgbl.prev_ztworm_ptr = NULL;
 		DEBUG_ONLY(jgbl.cumul_index = jgbl.cu_jnl_index = 0;)
 		global_tlvl_info_head = NULL;
 		REINITIALIZE_LIST(global_tlvl_info_list);		/* reinitialize the global_tlvl_info buddy_list */
 		gvt_tp_list = NULL;
 		CWS_RESET; /* reinitialize the hashtable before restarting/committing the TP transaction */
 	}	/* if (any database work in the transaction) */
-	VMS_ONLY(tp_has_kill_t_cse = FALSE;)
 	sgm_info_ptr = NULL;
 	first_sgm_info = NULL;
 	/* ensure that we don't have crit on any region at the end of a TP transaction (be it GT.M or MUPIP). The only exception
 	 * is ONLINE ROLLBACK or MUPIP TRIGGER -UPGRADE which holds crit for the entire duration
 	 */
-	assert((CDB_STAGNATE == t_tries) || (0 == have_crit(CRIT_HAVE_ANY_REG))
-						UNIX_ONLY(|| jgbl.onlnrlbk || TREF(in_trigger_upgrade)));
+	assert((CDB_STAGNATE == t_tries) || (0 == have_crit(CRIT_HAVE_ANY_REG)) || jgbl.onlnrlbk || TREF(in_trigger_upgrade));
 	/* Now that this transaction try is done (need to start a fresh try in case of a restart; in case of commit the entire
 	 * transaction is done) ensure first_tp_si_by_ftok is NULL at end of tp_clean_up as this field is relied upon by
 	 * secshr_db_clnup and t_commit_cleanup to determine if we have an ongoing transaction. In case of a successfully
-	 * committing transaction (rollback_flag == FALSE), this should be guaranteed already. So we might need to do the reset
-	 * only in case rollback_flag == TRUE but since that is an if condition which involves a pipeline break we avoid it by
+	 * committing transaction (clnup_state == TP_COMMIT), this should be guaranteed already. So we might need to do the reset
+	 * only in case clnup_state != TP_COMMIT but since that is an if condition which involves a pipeline break we avoid it by
 	 * doing the set to NULL unconditionally.
 	 */
-	assert(rollback_flag || (NULL == first_tp_si_by_ftok));
+	assert(TP_COMMIT != (clnup_state) || (NULL == first_tp_si_by_ftok));
 	first_tp_si_by_ftok = NULL;
 	ENABLE_INTERRUPTS(INTRPT_IN_TP_CLEAN_UP, prev_intrpt_state);	/* Allow MUPIP STOP/signals from now on */
-#	ifdef GTM_TRIGGER
-	if (!rollback_flag)
+	if (TP_COMMIT == clnup_state)
 	{
 		TP_ZTRIGBUFF_PRINT; /* TP is committed so print $ZTRIGGER/MUPIP-TRIGGER activity output in this tn */
 	} else
 		TREF(ztrigbuffLen) = 0;
-#	endif
 }
