@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2012-2015 Fidelity National Information 	*
+ * Copyright (c) 2012-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -63,8 +63,7 @@ error_def(ERR_REPLINSTFROZEN);
 error_def(ERR_TEXT);
 error_def(ERR_INSTFRZDEFER);
 
-GBLREF	jnlpool_addrs		jnlpool;
-GBLREF	jnlpool_ctl_ptr_t	jnlpool_ctl;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool;
 GBLREF	boolean_t		is_src_server;
 GBLREF	boolean_t		holds_sem[NUM_SEM_SETS][NUM_SRC_SEMS];
 #ifdef DEBUG
@@ -120,7 +119,8 @@ STATICFNDEF int		get_mnemonic_offset(hash_table_str **err_hashtab, char *mnemoni
 	{	/* create and populate hash-table for future lookups */
 		tmp_err_hashtab = (hash_table_str *)malloc(SIZEOF(hash_table_str));
 		DEBUG_ONLY(tmp_err_hashtab->base = NULL);
-		init_hashtab_str(tmp_err_hashtab, 0, HASHTAB_NO_COMPACT, HASHTAB_NO_SPARE_TABLE);
+		init_hashtab_str(tmp_err_hashtab, (msg_top - msg_beg) * (100.0 / HT_LOAD_FACTOR),
+				HASHTAB_NO_COMPACT, HASHTAB_NO_SPARE_TABLE);
 		assert(tmp_err_hashtab->base);
 		for (msg_info = (err_msg *)msg_beg; msg_info < msg_top; msg_info++)
 		{
@@ -151,8 +151,8 @@ boolean_t	is_anticipatory_freeze_needed(sgmnt_addrs *csa, int msg_id)
 {
 	const err_ctl		*ctl;
 	int			idx;
+	jnlpool_addrs_ptr_t	local_jnlpool;
 
-	assert(NULL != jnlpool.jnlpool_ctl);
 	/* Certain error messages should NOT trigger a freeze even if they are so configured in the custom errors file as they might
 	 * result in instance freezes that can be set indefinitely. Currently, we know of at least 3 such messages:
 	 * 1. ENOSPCQIODEFER and INSTFRZDEFER : To ensure we don't set anticipatory freeze if we don't/can't hold crit
@@ -170,8 +170,10 @@ boolean_t	is_anticipatory_freeze_needed(sgmnt_addrs *csa, int msg_id)
 	if (NULL != ctl)
 	{
 		GET_MSG_IDX(msg_id, ctl, idx);
-		assert(idx < ARRAYSIZE(jnlpool_ctl->merrors_array));
-		if (jnlpool_ctl->merrors_array[idx] & AFREEZE_MASK)
+		local_jnlpool = csa->jnlpool ? csa->jnlpool : jnlpool;
+		assert(local_jnlpool && local_jnlpool->jnlpool_ctl);
+		assert(idx < ARRAYSIZE(local_jnlpool->jnlpool_ctl->merrors_array));
+		if (local_jnlpool->jnlpool_ctl->merrors_array[idx] & AFREEZE_MASK)
 			return TRUE;
 	}
 	return FALSE;
@@ -183,38 +185,40 @@ void		set_anticipatory_freeze(sgmnt_addrs *csa, int msg_id)
 	boolean_t			was_crit;
 	sgmnt_addrs			*repl_csa;
 	const err_msg			*msginfo;
-#	ifdef DEBUG
-	qw_off_t			write_addr;
-	uint4				write;
-#	endif
+	jnlpool_addrs_ptr_t		save_jnlpool;
 
 	assert(is_anticipatory_freeze_needed(csa, msg_id));
-	DEBUG_ONLY(
-		write_addr = jnlpool_ctl->write_addr;
-		write = jnlpool_ctl->write;
-	)
-	assert(write == write_addr % jnlpool_ctl->jnlpool_size);
-	repl_csa = &FILE_INFO(jnlpool.jnlpool_dummy_reg)->s_addrs;
+	save_jnlpool = jnlpool;
+	if (csa->jnlpool && (csa->jnlpool != jnlpool))
+		jnlpool = csa->jnlpool;
+	assert(jnlpool && jnlpool->jnlpool_ctl);
+	assert(jnlpool->jnlpool_dummy_reg);	/* other asserts in is anticipatory_freeze_needed */
+	repl_csa = &FILE_INFO(jnlpool->jnlpool_dummy_reg)->s_addrs;
 	assert(NULL != repl_csa);
 	was_crit = repl_csa->now_crit;
 	if (!was_crit)
 	{
 		if (csa->now_crit)
-			grab_lock(jnlpool.jnlpool_dummy_reg, TRUE, GRAB_LOCK_ONLY);
-		else if (FALSE == grab_lock(jnlpool.jnlpool_dummy_reg, FALSE, GRAB_LOCK_ONLY))
+			grab_lock(jnlpool->jnlpool_dummy_reg, TRUE, GRAB_LOCK_ONLY);
+		else if (FALSE == grab_lock(jnlpool->jnlpool_dummy_reg, FALSE, GRAB_LOCK_ONLY))
 		{
 			MSGID_TO_ERRMSG(msg_id, msginfo);
 			send_msg_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_INSTFRZDEFER, 4, LEN_AND_STR(msginfo->tag),
 				     REG_LEN_STR(csa->region));
+			if (save_jnlpool != jnlpool)
+				jnlpool = save_jnlpool;
 			return;
 		}
 	}
 	/* Now that we hold necessary locks, set the freeze and the comment field */
-	jnlpool.jnlpool_ctl->freeze = TRUE;
-	GENERATE_INST_FROZEN_COMMENT(jnlpool.jnlpool_ctl->freeze_comment, SIZEOF(jnlpool.jnlpool_ctl->freeze_comment), msg_id);
+	jnlpool->jnlpool_ctl->freeze = TRUE;
+	GENERATE_INST_FROZEN_COMMENT(jnlpool->jnlpool_ctl->freeze_comment,
+			SIZEOF(jnlpool->jnlpool_ctl->freeze_comment), msg_id);
 	/* TODO : Do we need a SHM_WRITE_MEMORY_BARRIER ? */
 	if (!was_crit)
-		rel_lock(jnlpool.jnlpool_dummy_reg);
+		rel_lock(jnlpool->jnlpool_dummy_reg);
+	if (save_jnlpool != jnlpool)
+		jnlpool = save_jnlpool;
 }
 
 /* initialize jnlpool_ctl->merrors_array to set up the list of errors that should trigger anticipatory freeze errors */
@@ -234,7 +238,8 @@ boolean_t		init_anticipatory_freeze_errors()
 	 * to check if cmerrors/cmierrors also need to be included in this list or not.
 	 */
 	assert(IS_MUPIP_IMAGE); 				/* is_src_server is not initialized at this point */
-	assert(jnlpool_ctl && !jnlpool_ctl->pool_initialized);	/* should be invoked BEFORE the journal pool is fully-initialized */
+	/* invoke when not previously initialized */
+	assert(jnlpool && jnlpool->jnlpool_ctl && !jnlpool->jnlpool_ctl->instfreeze_environ_inited);
 	assert(holds_sem[SOURCE][JNL_POOL_ACCESS_SEM]);		/* should hold journal pool access control semaphore */
 	/* Now, read the custom errors file and populate the journal pool */
 	custom_err_file = TREF(gtm_custom_errors);
@@ -315,7 +320,7 @@ boolean_t		init_anticipatory_freeze_errors()
 				gtm_putmsg_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_CUSTERRNOTFND, 2, mnemonic_len, mnemonic_buf);
 				return FALSE;
 			}
-			jnlpool_ctl->merrors_array[offset] |= AFREEZE_MASK; /* duplicate entries are not considered an error */
+			jnlpool->jnlpool_ctl->merrors_array[offset] |= AFREEZE_MASK; /* duplicate entries are not an error */
 		}
 		assert(ISSPACE_ASCII(*buffptr) || (COMMENT_DELIMITER == *buffptr));
 		if (EOL_REACHED == (buffptr = scan_space(handle, buff, buffptr, buff_top)))
@@ -359,7 +364,7 @@ boolean_t		init_anticipatory_freeze_errors()
 				custom_err_file.addr, save_errno);
 		return FALSE;
 	}
-	jnlpool_ctl->instfreeze_environ_inited = TRUE;
+	jnlpool->jnlpool_ctl->instfreeze_environ_inited = TRUE;
 	return TRUE;
 }
 
@@ -371,13 +376,15 @@ void clear_fake_enospc_if_master_dead(void)
 	sgmnt_addrs			*csa;
 
 	assert(!multi_thread_in_use);	/* fake-enospc would not have been set if in threaded-code */
-	if((jnlpool_ctl->jnlpool_creator_pid != process_id) && !is_proc_alive(jnlpool_ctl->jnlpool_creator_pid, 0))
+	assert(jnlpool && jnlpool->jnlpool_ctl);
+	if((jnlpool->jnlpool_ctl->jnlpool_creator_pid != process_id)
+		&& !is_proc_alive(jnlpool->jnlpool_ctl->jnlpool_creator_pid, 0))
 	{
 		for (addr_ptr = get_next_gdr(NULL); addr_ptr; addr_ptr = get_next_gdr(addr_ptr))
 		{
 			for (r_local = addr_ptr->regions, r_top = r_local + addr_ptr->n_regions; r_local < r_top; r_local++)
 			{
-				if ((dba_bg != r_local->dyn.addr->acc_meth) && (dba_mm != r_local->dyn.addr->acc_meth))
+				if (!IS_REG_BG_OR_MM(r_local))
 					continue;
 				csa = REG2CSA(r_local);
 				if ((NULL != csa) && (NULL != csa->nl))
