@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -12,5 +13,8 @@
 #ifndef _FAKE_ENOSPC_H
 #define _FAKE_ENOSPC_H
 
+#define ENOSPC_INIT_DURATION	(8 * MILLISECS_IN_SEC)
+
 void fake_enospc(void);
+void handle_deferred_syslog(void);
 #endif
