@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2006-2016 Fidelity National Information	*
+ * Copyright (c) 2006-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -56,10 +56,8 @@
 #include "sgtm_putmsg.h"
 #include "gt_timer.h"
 #include "ftok_sem_incrcnt.h"
-#include "init_secshr_addrs.h"
 #include "mutex.h"
 #include "fork_init.h"
-#include "heartbeat_timer.h"
 #include "gtmio.h"
 
 GBLDEF	boolean_t		gtmrecv_fetchreysnc;
@@ -79,7 +77,7 @@ GBLREF	boolean_t		is_rcvr_server;
 GBLREF	int			gtmrecv_srv_count;
 GBLREF	uint4			log_interval;
 GBLREF	boolean_t		holds_sem[NUM_SEM_SETS][NUM_SRC_SEMS];
-GBLREF	jnlpool_addrs		jnlpool;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool;
 GBLREF	IN_PARMS		*cli_lex_in_ptr;
 GBLREF	uint4			mutex_per_process_init_pid;
 LITREF	gtmImageName		gtmImageNames[];
@@ -129,25 +127,26 @@ int gtmrecv(void)
 				gtmImageNames[image_type].imageNameLen, gtmImageNames[image_type].imageName);
 	if (gtmrecv_options.start || gtmrecv_options.shut_down)
 	{
-		jnlpool_init(GTMRECEIVE, (boolean_t)FALSE, (boolean_t *)NULL);
-		assert(NULL != jnlpool.repl_inst_filehdr);
-		assert(NULL != jnlpool.jnlpool_ctl);
+		jnlpool_init(GTMRECEIVE, (boolean_t)FALSE, (boolean_t *)NULL, NULL);
+		assert(NULL != jnlpool);
+		assert(NULL != jnlpool->repl_inst_filehdr);
+		assert(NULL != jnlpool->jnlpool_ctl);
 		/* If -UPDATERESYNC was specified without an empty instance file, error out. The only exception is if
 		 * the receiver is a root primary (updates enabled) supplementary instance and the source is non-supplementary.
 		 * In this case, since the non-supplementary stream is being ADDED to an existing instance file, there is no
 		 * requirement that the current instance file be empty. In fact, in that case we expect it to be non-empty
 		 * as one history record would have been written by the source server that brought up the root primary instance.
 		 */
-		if (gtmrecv_options.updateresync && jnlpool.repl_inst_filehdr->num_histinfo
-				&& !(jnlpool.repl_inst_filehdr->is_supplementary && !jnlpool.jnlpool_ctl->upd_disabled))
+		if (gtmrecv_options.updateresync && jnlpool->repl_inst_filehdr->num_histinfo
+				&& !(jnlpool->repl_inst_filehdr->is_supplementary && !jnlpool->jnlpool_ctl->upd_disabled))
 			/* replication instance file is NOT empty. Issue error */
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_UPDSYNC2MTINS);
 		if (gtmrecv_options.noresync)
 		{	/* If -NORESYNC was specified on a non-supplementary receiver instance, issue an error */
-			if (!jnlpool.repl_inst_filehdr->is_supplementary)
+			if (!jnlpool->repl_inst_filehdr->is_supplementary)
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_NORESYNCSUPPLONLY);
 			/* If -NORESYNC was specified on a receiver instance where updates are disabled, issue an error */
-			if (jnlpool.jnlpool_ctl->upd_disabled)
+			if (jnlpool->jnlpool_ctl->upd_disabled)
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_NORESYNCUPDATERONLY);
 		}
 	}
@@ -266,7 +265,7 @@ int gtmrecv(void)
 				assert(4 == SIZEOF(updresync_inst_hdr.num_histinfo)); /* so need to use GTM_BYTESWAP_32 */
 				updresync_inst_hdr.num_histinfo = GTM_BYTESWAP_32(updresync_inst_hdr.num_histinfo);
 			}
-			if (jnlpool.repl_inst_filehdr->is_supplementary && !updresync_inst_hdr.is_supplementary)
+			if (jnlpool->repl_inst_filehdr->is_supplementary && !updresync_inst_hdr.is_supplementary)
 			{	/* Do one check for non-supplementary -> supplementary connection using -updateresync.
 				 * This is because this use of -updateresync is different than the other connection usages.
 				 */
@@ -296,7 +295,7 @@ int gtmrecv(void)
 			{	/* The input instance file is supplementary. Allow it only if the current instance is
 				 * supplementary and is not a root primary.
 				 */
-				if (!jnlpool.repl_inst_filehdr->is_supplementary)
+				if (!jnlpool->repl_inst_filehdr->is_supplementary)
 				{
 					rel_sem(RECV, RECV_SERV_OPTIONS_SEM);
 					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_UPDSYNCINSTFILE, 0, ERR_TEXT, 2,
@@ -304,16 +303,16 @@ int gtmrecv(void)
 							  	" to match current instance"));
 					assert(FALSE);
 				}
-				if (!jnlpool.jnlpool_ctl->upd_disabled)
+				if (!jnlpool->jnlpool_ctl->upd_disabled)
 				{
 					rel_sem(RECV, RECV_SERV_OPTIONS_SEM);
 					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_UPDSYNCINSTFILE, 0, ERR_TEXT, 2,
 						  LEN_AND_LIT("Input instance file must be non-supplementary"
 							  	" as current instance is a supplementary root primary"));
 				}
-			} else if (jnlpool.repl_inst_filehdr->is_supplementary)
+			} else if (jnlpool->repl_inst_filehdr->is_supplementary)
 			{
-				if (jnlpool.jnlpool_ctl->upd_disabled)
+				if (jnlpool->jnlpool_ctl->upd_disabled)
 				{	/* The input instance file is non-supplementary. Allow it only if the current
 					 * instance is non-supplementary or if it is a supplementary root primary.
 					 */
@@ -328,7 +327,7 @@ int gtmrecv(void)
 					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_INITORRESUME);
 				}
 			}
-			if (!jnlpool.repl_inst_filehdr->is_supplementary || jnlpool.jnlpool_ctl->upd_disabled)
+			if (!jnlpool->repl_inst_filehdr->is_supplementary || jnlpool->jnlpool_ctl->upd_disabled)
 			{
 				if (gtmrecv_options.resume_specified)
 				{
@@ -399,15 +398,12 @@ int gtmrecv(void)
 	is_rcvr_server = TRUE;
 	process_id = getpid();
 	OPERATOR_LOG_MSG;
-	/* Reinvoke secshr related initialization with the child's pid */
-	INVOKE_INIT_SECSHR_ADDRS;
 	/* Initialize mutex socket, memory semaphore etc. before any "grab_lock" is done by this process on the journal pool.
 	 * Note that the initialization would already have been done by the parent receiver startup command but we need to
 	 * redo the initialization with the child process id.
 	 */
 	assert(mutex_per_process_init_pid && mutex_per_process_init_pid != process_id);
 	mutex_per_process_init();
-	START_HEARTBEAT_IF_NEEDED;
 	/* Take a copy of the fact whether an -UPDATERESYNC was specified or not. Note this down in shared memory.
 	 * We will clear the shared memory copy as soon as the first history record gets written to the instance file
 	 * after the first time this receiver connects to a source. Future connects of this same receiver with the same
@@ -468,25 +464,25 @@ int gtmrecv(void)
 			  RTS_ERROR_LITERAL("Receiver server error in setsid"), errno);
 	gtm_event_log_init();
 	gtmrecv_local->recv_serv_pid = process_id;
-	assert(NULL != jnlpool.jnlpool_ctl);
-	jnlpool.jnlpool_ctl->gtmrecv_pid = process_id;
+	assert((NULL != jnlpool) && (NULL != jnlpool->jnlpool_ctl));
+	jnlpool->jnlpool_ctl->gtmrecv_pid = process_id;
 	gtmrecv_local->listen_port = gtmrecv_options.listen_port;
 	/* Log receiver server startup command line first */
 	repl_log(gtmrecv_log_fp, TRUE, TRUE, "%s %s\n", cli_lex_in_ptr->argv[0], cli_lex_in_ptr->in_str);
 
-	assert(NULL != jnlpool.repl_inst_filehdr);
+	assert((NULL != jnlpool) && (NULL != jnlpool->repl_inst_filehdr));
 	SPRINTF(tmpmsg, "GTM Replication Receiver Server with Pid [%d] started on replication instance [%s]",
-		process_id, jnlpool.repl_inst_filehdr->inst_info.this_instname);
+		process_id, jnlpool->repl_inst_filehdr->inst_info.this_instname);
 	sgtm_putmsg(print_msg, VARLSTCNT(4) ERR_REPLINFO, 2, LEN_AND_STR(tmpmsg));
 	repl_log(gtmrecv_log_fp, TRUE, TRUE, print_msg);
 	repl_log(gtmrecv_log_fp, TRUE, TRUE, "Attached to existing jnlpool with shmid = [%d] and semid = [%d]\n",
-			jnlpool.repl_inst_filehdr->jnlpool_shmid, jnlpool.repl_inst_filehdr->jnlpool_semid);
+			jnlpool->repl_inst_filehdr->jnlpool_shmid, jnlpool->repl_inst_filehdr->jnlpool_semid);
 	gtm_event_log(GTM_EVENT_LOG_ARGC, "MUPIP", "REPLINFO", print_msg);
 	if (recvpool_ctl->fresh_start)
 	{
 		QWASSIGNDW(recvpool_ctl->jnl_seqno, 0); /* Update process will initialize this to a non-zero value */
 		repl_log(gtmrecv_log_fp, TRUE, TRUE, "Created recvpool with shmid = [%d] and semid = [%d]\n",
-				jnlpool.repl_inst_filehdr->recvpool_shmid, jnlpool.repl_inst_filehdr->recvpool_semid);
+				jnlpool->repl_inst_filehdr->recvpool_shmid, jnlpool->repl_inst_filehdr->recvpool_semid);
 	} else
 	{	/* Coming up after a crash, reset Update process read.  This is done by setting gtmrecv_local->restart.
 		 * This will trigger update process to reset recvpool_ctl->jnl_seqno too.
@@ -498,7 +494,7 @@ int gtmrecv(void)
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(8) ERR_SYSCALL, 5,
 					LEN_AND_LIT("pthread_cond_signal"), CALLFROM, status, 0);
 		repl_log(gtmrecv_log_fp, TRUE, TRUE, "Attached to existing recvpool with shmid = [%d] and semid = [%d]\n",
-				jnlpool.repl_inst_filehdr->recvpool_shmid, jnlpool.repl_inst_filehdr->recvpool_semid);
+				jnlpool->repl_inst_filehdr->recvpool_shmid, jnlpool->repl_inst_filehdr->recvpool_semid);
 	}
 	save_upd_status = upd_proc_local->upd_proc_shutdown;
 	for (upd_start_attempts = 0;
