@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2006-2016 Fidelity National Information	*
+ * Copyright (c) 2006-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -63,9 +63,8 @@ GBLREF	qw_num			repl_recv_data_recvd;
 GBLREF	qw_num			repl_recv_data_processed;
 GBLREF	repl_msg_ptr_t		gtmrecv_msgp;
 GBLREF	uchar_ptr_t		repl_filter_buff;
-GBLREF	jnlpool_addrs		jnlpool;
-GBLREF	jnlpool_ctl_ptr_t	jnlpool_ctl;
-GBLREF	boolean_t		pool_init;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool;
+GBLREF	int			pool_init;
 
 error_def(ERR_SYSCALL);
 
@@ -143,17 +142,17 @@ int gtmrecv_end1(boolean_t auto_shutdown)
 		repl_log(stderr, TRUE, TRUE, "Error detaching from Receive Pool : %s\n", STRERROR(save_errno));
 	}
 	recvpool.recvpool_ctl = NULL;
-	assert((NULL != jnlpool_ctl) && (jnlpool_ctl == jnlpool.jnlpool_ctl));
-	if (NULL != jnlpool.jnlpool_ctl)
+	assert((NULL != jnlpool) && (NULL != jnlpool->jnlpool_ctl));
+	if ((NULL != jnlpool) && (NULL != jnlpool->jnlpool_ctl))
 	{	/* Reset fields that might have been initialized by the receiver server after connecting to the primary.
 		 * It is ok not to hold the journal pool lock while updating jnlpool_ctl fields since this will be the
 		 * only process updating those fields.
 		 */
-		jnlpool.jnlpool_ctl->primary_instname[0] = '\0';
-		jnlpool.jnlpool_ctl->gtmrecv_pid = 0;
-		jnlpool_seqno = jnlpool.jnlpool_ctl->jnl_seqno;
+		jnlpool->jnlpool_ctl->primary_instname[0] = '\0';
+		jnlpool->jnlpool_ctl->gtmrecv_pid = 0;
+		jnlpool_seqno = jnlpool->jnlpool_ctl->jnl_seqno;
 		for (idx = 0; idx < MAX_SUPPL_STRMS; idx++)
-			jnlpool_strm_seqno[idx] = jnlpool.jnlpool_ctl->strm_seqno[idx];
+			jnlpool_strm_seqno[idx] = jnlpool->jnlpool_ctl->strm_seqno[idx];
 		/* Also take this opportunity to detach from the journal pool except in the auto_shutdown case. This is because
 		 * the fields "jnlpool_ctl->repl_inst_filehdr->recvpool_semid" and "jnlpool_ctl->repl_inst_filehdr->recvpool_shmid"
 		 * need to be reset by "gtmrecv_jnlpool_reset" (called from "gtmrecv_shutdown") which is invoked a little later.
@@ -163,15 +162,9 @@ int gtmrecv_end1(boolean_t auto_shutdown)
 		 */
 		if (!auto_shutdown && !INST_FREEZE_ON_ERROR_POLICY)
 		{
-			JNLPOOL_SHMDT(status, save_errno);
+			JNLPOOL_SHMDT(jnlpool, status, save_errno);
 			if (0 > status)
 				repl_log(stderr, TRUE, TRUE, "Error detaching from Journal Pool : %s\n", STRERROR(save_errno));
-			jnlpool.jnlpool_ctl = jnlpool_ctl = NULL;
-			jnlpool.repl_inst_filehdr = NULL;
-			jnlpool.gtmsrc_lcl_array = NULL;
-			jnlpool.gtmsource_local_array = NULL;
-			jnlpool.jnldata_base = NULL;
-			pool_init = FALSE;
 		}
 	} else
 		jnlpool_seqno = 0;
