@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2006-2016 Fidelity National Information	*
+ * Copyright (c) 2006-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -74,7 +74,7 @@
 
 /* Callers of this macro ensure that the maximum seqno which can be found until offset MAX_SEQNO_EOF_ADDR is MAX_SEQNO. */
 #define	CTL_SET_MAX_SEQNO(CTL, MAX_SEQNO, MAX_SEQNO_ADDR, MAX_SEQNO_EOF_ADDR, SKIP_EOF_ADDR_CHECK)		\
-{														\
+MBSTART {													\
 	assert(MAX_SEQNO_ADDR <= MAX_SEQNO_EOF_ADDR);								\
 	assert(SKIP_EOF_ADDR_CHECK || (MAX_SEQNO_EOF_ADDR <= CTL->repl_buff->fc->eof_addr));			\
 	assert(ctl->max_seqno <= MAX_SEQNO);									\
@@ -84,10 +84,40 @@
 	assert(MAX_SEQNO_EOF_ADDR >= MAX_SEQNO_ADDR);								\
 	assert(ctl->max_seqno_eof_addr <= MAX_SEQNO_EOF_ADDR);							\
 	ctl->max_seqno_eof_addr = MAX_SEQNO_EOF_ADDR;								\
-}
+} MBEND
+
+#define BUNCHING_TIME	(8 * MILLISECS_IN_SEC)
+
+#define	GTMSRC_DO_JNL_FLUSH_IF_POSSIBLE(CTL, CSA)									\
+MBSTART {														\
+	boolean_t		flush_done;										\
+	uint4			dskaddr, freeaddr, rsrv_freeaddr, saved_jpc_cycle;					\
+	jnl_private_control	*jpc;											\
+	int			rc;											\
+															\
+	jpc = CSA->jnl;													\
+	saved_jpc_cycle = jpc->cycle; /* remember current cycle */							\
+	DO_JNL_FLUSH_IF_POSSIBLE(CTL->reg, CSA, flush_done, dskaddr, freeaddr, rsrv_freeaddr);				\
+	if (flush_done)													\
+	{	/* Source server did a flush. Log that event for debugging purposes */					\
+		repl_log(gtmsource_log_fp, TRUE, TRUE, "REPL_INFO : Source server did flush of journal file %s "	\
+			"state %s : seqno %llu [0x%llx]. [dskaddr 0x%x freeaddr 0x%x rsrv_freeaddr 0x%x]\n",		\
+			CTL->jnl_fn, jnl_file_state_lit[CTL->file_state], CTL->seqno, CTL->seqno,			\
+			dskaddr, freeaddr, rsrv_freeaddr);								\
+		/* Since "flush_done" is TRUE, it means the source server has done a "jnl_ensure_open" inside the	\
+		 * DO_JNL_FLUSH_IF_POSSIBLE macro and has opened the journal file. It already has a copy of the fd in	\
+		 * ctl->repl_buff->fc->fd which it uses to read the journal file. Now that the jnl_flush (rare event)	\
+		 * is done, close the jpc->channel and continue to use the ctl's fd.					\
+		 */													\
+		assert(NOJNL != jpc->channel);										\
+		JNL_FD_CLOSE(jpc->channel, rc);	/* sets jpc->channel to NOJNL */					\
+		assert(NOJNL == jpc->channel);										\
+		jpc->cycle = saved_jpc_cycle; /* do not want cycle to be changed by our flushing so restore it */	\
+	}														\
+} MBEND
 
 GBLREF	unsigned char		*gtmsource_tcombuff_start;
-GBLREF	jnlpool_addrs		jnlpool;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool;
 GBLREF	repl_ctl_element	*repl_ctl_list;
 GBLREF	repl_rctl_elem_t	*repl_rctl_list;
 GBLREF	seq_num			gtmsource_save_read_jnl_seqno;
@@ -98,7 +128,6 @@ GBLREF	gd_region		*gv_cur_region;
 GBLREF	FILE			*gtmsource_log_fp;
 GBLREF	gtmsource_state_t	gtmsource_state;
 GBLREF	uint4			process_id;
-GBLREF	uint4			heartbeat_counter;
 
 LITREF char *jnl_file_state_lit[];
 
@@ -388,7 +417,7 @@ static	int open_newer_gener_jnlfiles(gd_region *reg, repl_ctl_element *reg_ctl_e
 	/* Note that at this point, journaling might have been turned OFF (e.g. REPL_WAS_ON state) in which case
 	 * JNL_GDID_PTR(csa) would have been nullified by jnl_file_lost. Therefore comparing with that is not a good idea
 	 * to use the "id" to check if the journal file remains same (this was done previously). Instead use the ID of
-	 * the current reg_ctl_end and the NAME of the newly opened journal file. Because we dont have crit, we cannot
+	 * the current reg_ctl_end and the NAME of the newly opened journal file. Because we don't have crit, we cannot
 	 * safely read the journal file name from the file header therefore we invoke repl_ctl_create unconditionally
 	 * (that has safety protections for crit) and use the new_ctl that it returns to access the journal file name
 	 * returned and use that for the ID to NAME comparison.
@@ -503,10 +532,10 @@ static	int update_eof_addr(repl_ctl_element *ctl, int *eof_change)
 			     csa->nl->jnl_file.u.device,  csa->nl->jnl_file.u.st_gen);
 		if (!ctl->eof_addr_final)
 		{
-			F_READ_BLK_ALIGNED(fc->fd, 0, fc->jfh, REAL_JNL_HDR_LEN, status);
+			F_READ_BLK_ALIGNED(fc->fd, 0, fc->jfh, ROUND_UP2(REAL_JNL_HDR_LEN, fc->fs_block_size), status);
 			if (SS_NORMAL != status)
-				rts_error_csa(CSA_ARG(csa) VARLSTCNT(9) ERR_REPLFILIOERR, 2, ctl->jnl_fn_len, ctl->jnl_fn,
-						ERR_TEXT, 2, LEN_AND_LIT("Error in reading jfh in update_eof_addr"), status);
+				rts_error_csa(CSA_ARG(csa) VARLSTCNT(9) ERR_REPLFILIOERR, 2, ctl->jnl_fn_len, ctl->jnl_fn, ERR_TEXT,
+						2, LEN_AND_LIT("Error reading journal fileheader to update EOF address"), status);
 			REPL_DPRINT2("Update EOF : Jnl file hdr refreshed from file for %s\n", ctl->jnl_fn);
 			ctl->eof_addr_final = TRUE; /* No more updates to fc->eof_addr for this journal file */
 		}
@@ -704,13 +733,17 @@ static	int update_max_seqno_info(repl_ctl_element *ctl)
 					return (SS_NORMAL);
 				}
 				SHORT_SLEEP(GTMSOURCE_WAIT_FOR_JNL_RECS);
-				if (0 == (wait_for_jnl += GTMSOURCE_WAIT_FOR_JNL_RECS) % LOG_WAIT_FOR_JNL_RECS_PERIOD)
+				wait_for_jnl += GTMSOURCE_WAIT_FOR_JNL_RECS;
+				if (0 == (wait_for_jnl % LOG_WAIT_FOR_JNL_FLUSH_PERIOD))
+					GTMSRC_DO_JNL_FLUSH_IF_POSSIBLE(ctl, csa); /* See if a "jnl_flush" might help nudge */
+				if (0 == (wait_for_jnl % LOG_WAIT_FOR_JNL_RECS_PERIOD))
 				{
-					repl_log(gtmsource_log_fp, TRUE, TRUE, "REPL_WARN : Source server waited %u seconds for "
+					repl_log(gtmsource_log_fp, TRUE, TRUE, "REPL_WARN : Source server waited %u seconds for"
 						" journal record(s) to be written to journal file %s while attempting to read "
-						"seqno %llu [0x%llx]. [dskaddr 0x%llx freeaddr 0x%llx]. Check for problems with "
-						"journaling\n", wait_for_jnl / 1000, ctl->jnl_fn, ctl->seqno, ctl->seqno,
-						csa->jnl->jnl_buff->dskaddr, csa->jnl->jnl_buff->freeaddr);
+						"seqno %llu [0x%llx]. [dskaddr 0x%x freeaddr 0x%x rsrv_freeaddr 0x%x]."
+						" Check for problems with journaling\n", wait_for_jnl / 1000, ctl->jnl_fn,
+						ctl->seqno, ctl->seqno, csa->jnl->jnl_buff->dskaddr,
+						csa->jnl->jnl_buff->freeaddr, csa->jnl->jnl_buff->rsrv_freeaddr);
 				}
 			} else
 			{
@@ -1011,13 +1044,17 @@ static	int read_transaction(repl_ctl_element *ctl, unsigned char **buff, int *bu
 				return 0;
 			}
 			SHORT_SLEEP(GTMSOURCE_WAIT_FOR_JNL_RECS);
-			if (0 == (total_wait_for_jnl_recs += GTMSOURCE_WAIT_FOR_JNL_RECS) % LOG_WAIT_FOR_JNL_RECS_PERIOD)
+			total_wait_for_jnl_recs += GTMSOURCE_WAIT_FOR_JNL_RECS;
+			if (0 == (total_wait_for_jnl_recs % LOG_WAIT_FOR_JNL_FLUSH_PERIOD))
+				GTMSRC_DO_JNL_FLUSH_IF_POSSIBLE(ctl, csa); /* See if a "jnl_flush" might help nudge */
+			if (0 == (total_wait_for_jnl_recs % LOG_WAIT_FOR_JNL_RECS_PERIOD))
 			{
 				repl_log(gtmsource_log_fp, TRUE, TRUE, "REPL_WARN : Source server waited %u seconds for journal "
 					"record(s) to be written to journal file %s while attempting to read seqno %llu [0x%llx]. "
-					"[dskaddr 0x%llx freeaddr 0x%llx]. Check for problems with journaling\n",
+					"[dskaddr 0x%x freeaddr 0x%x rsrv_freeaddr 0x%x]. Check for problems with journaling\n",
 					total_wait_for_jnl_recs / 1000, ctl->jnl_fn, ctl->seqno, ctl->seqno,
-					csa->jnl->jnl_buff->dskaddr, csa->jnl->jnl_buff->freeaddr);
+					csa->jnl->jnl_buff->dskaddr, csa->jnl->jnl_buff->freeaddr,
+					csa->jnl->jnl_buff->rsrv_freeaddr);
 			}
 		} else
 		{
@@ -1336,12 +1373,21 @@ static	tr_search_state_t position_read(repl_ctl_element *ctl, seq_num read_seqno
 			/* Since we know that read_seqno is LESSER than ctl->max_seqno, we know for sure we should not return
 			 * this function with found = TR_NOT_FOUND. If ever the binary/linear search invocation at the end of
 			 * this function returns with TR_NOT_FOUND, we have to change that to TR_WILL_NOT_BE_FOUND and also
-			 * adjust ctl->seqno to point to ctl->max_seqno that way we dont repeat binary search (wastefully) for
+			 * adjust ctl->seqno to point to ctl->max_seqno that way we don't repeat binary search (wastefully) for
 			 * all seqnos between read_seqno and the least seqno larger than read_seqno in this file.
 			 */
 			willnotbefound_addr = hi_addr;
 			assert(ctl->max_seqno_dskaddr < rb->fc->eof_addr);
 			willnotbefound_stop_at = rb->fc->eof_addr;
+			if (lo_addr == hi_addr)
+			{	/* Since hi_addr corresponds to ctl->max_seqno which is in turn > read_seqno, the above
+				 * "if" check succeeding implies we can never find read_seqno here. So skip calling
+				 * "do_binary_search" in that case as that and "do_linear_search" have asserts that expect
+				 * callers to never invoke them with the same value of lo_addr and hi_addr.
+				 */
+				found = TR_NOT_FOUND;
+				srch_func = NULL;
+			}
 		} else if (read_seqno == ctl->max_seqno)
 		{	/* For read_seqno == ctl->max_seqno, do not use linear search. Remember, max_seqno_dskaddr may be the
 			 * the address of the TCOM record of a transaction, and this TCOM record may be in a different block
@@ -1395,7 +1441,8 @@ static	tr_search_state_t position_read(repl_ctl_element *ctl, seq_num read_seqno
 #	endif
 	REPL_DPRINT6("position_read: Using %s search to locate %llu in %s between %u and %u\n",
 			(srch_func == do_linear_search) ? "linear" : "binary", read_seqno, ctl->jnl_fn, lo_addr, hi_addr);
-	found = srch_func(ctl, lo_addr, hi_addr, read_seqno, &srch_status);
+	if (NULL != srch_func)
+		found = srch_func(ctl, lo_addr, hi_addr, read_seqno, &srch_status);
 	if ((TR_NOT_FOUND == found) && (0 != willnotbefound_addr))
 	{	/* There is a block that contains a seqno larger than read_seqno; leave ctl positioned at this higher seqno.
 		 * If we don't do this, we could end up in an infinite loop if the caller of this function is "read_regions".
@@ -1455,34 +1502,38 @@ static	int read_and_merge(unsigned char *buff, int maxbufflen, seq_num read_jnl_
 				return 0;
 			}
 			SHORT_SLEEP(GTMSOURCE_WAIT_FOR_JNLOPEN); /* sleep for 10 msec between each iteration */
-			if (0 == ((total_wait_for_jnlopen += GTMSOURCE_WAIT_FOR_JNLOPEN) % LOG_WAIT_FOR_JNLOPEN_PERIOD))
+			total_wait_for_jnlopen += GTMSOURCE_WAIT_FOR_JNLOPEN;
+			if (0 == (total_wait_for_jnlopen % LOG_WAIT_FOR_JNL_FLUSH_PERIOD))
+			{
+				for (ctl = repl_ctl_list->next; NULL != ctl; ctl = ctl->next)
+				{
+					csa = &FILE_INFO(ctl->reg)->s_addrs;
+					GTMSRC_DO_JNL_FLUSH_IF_POSSIBLE(ctl, csa); /* See if a "jnl_flush" might help nudge */
+				}
+			}
+			if (0 == (total_wait_for_jnlopen % LOG_WAIT_FOR_JNLOPEN_PERIOD))
 			{	/* We have waited for 5000 intervals of 10 msec each, for a total of 50 seconds sleep.
-				 * Issue alert every 50 seconds. Print detail of all open journal files only in first alert.
+				 * Issue alert every 50 seconds.
 				 */
 				repl_log(gtmsource_log_fp, TRUE, TRUE, "REPL_WARN : Source server waited %u seconds for journal"
 					" file(s) to be opened, or updated while attempting to read seqno %llu [0x%llx]. Check"
 					" for problems with journaling\n", total_wait_for_jnlopen / 1000,
 					read_jnl_seqno, read_jnl_seqno);
-				if (LOG_WAIT_FOR_JNLOPEN_PERIOD == total_wait_for_jnlopen)
-				{	/* Print debug info only for first alert */
-					for (ctl = repl_ctl_list->next; NULL != ctl; ctl = ctl->next)
-					{
-						csa = &FILE_INFO(ctl->reg)->s_addrs;
-						repl_log(gtmsource_log_fp, TRUE, TRUE, "DBG_INFO: Journal File: %s for"
-							" Database File: %s; State: %s. Timer PIDs(%d): (%u %u %u)\n",
-							ctl->jnl_fn, ctl->reg->dyn.addr->fname,
-							jnl_file_state_lit[ctl->file_state], csa->nl->wcs_timers + 1,
-							csa->nl->wt_pid_array[0], csa->nl->wt_pid_array[1],
-							csa->nl->wt_pid_array[2]);
-						if (JNL_FILE_OPEN == ctl->file_state)
-						{
-							repl_log(gtmsource_log_fp, TRUE, TRUE, " "
-								"ctl->seqno = %llu [0x%llx]. [dskaddr = 0x%x,freeaddr = 0x%x]. "
-								"ctl->repl_rctl->read_complete = %d\n",
-								ctl->seqno, ctl->seqno, csa->jnl->jnl_buff->dskaddr,
-								csa->jnl->jnl_buff->freeaddr, ctl->repl_rctl->read_complete);
-						}
-					}
+				for (ctl = repl_ctl_list->next; NULL != ctl; ctl = ctl->next)
+				{
+					csa = &FILE_INFO(ctl->reg)->s_addrs;
+					repl_log(gtmsource_log_fp, TRUE, TRUE, "DBG_INFO: Journal File: %s for"
+						" Database File: %s; State: %s. Timer PIDs(%d): (%u %u %u)\n",
+						ctl->jnl_fn, ctl->reg->dyn.addr->fname,
+						jnl_file_state_lit[ctl->file_state], csa->nl->wcs_timers + 1,
+						csa->nl->wt_pid_array[0], csa->nl->wt_pid_array[1],
+						csa->nl->wt_pid_array[2]);
+					repl_log(gtmsource_log_fp, TRUE, TRUE, " "
+						"ctl->seqno = %llu [0x%llx]. [dskaddr = 0x%x,freeaddr = 0x%x,rsrv_freeaddr = 0x%x]."
+						" ctl->repl_rctl->read_complete = %d\n",
+						ctl->seqno, ctl->seqno, csa->jnl->jnl_buff->dskaddr,
+						csa->jnl->jnl_buff->freeaddr, csa->jnl->jnl_buff->rsrv_freeaddr,
+						ctl->repl_rctl->read_complete);
 				}
 			}
 		}
@@ -1528,7 +1579,8 @@ static	int read_regions(unsigned char **buff, int *buff_avail,
 	cumul_read = 0;
 	*brkn_trans = TRUE;
 	assert(repl_ctl_list->next != NULL);
-	jctl = jnlpool.jnlpool_ctl;
+	assert(NULL != jnlpool);
+	jctl = jnlpool->jnlpool_ctl;
 	/* For each region */
 	assert(repl_ctl_list->next == repl_rctl_list->ctl_start);
 	for (repl_rctl = repl_rctl_list; (NULL != repl_rctl) && !trans_read; repl_rctl = repl_rctl->next)
@@ -1636,7 +1688,7 @@ static	int read_regions(unsigned char **buff, int *buff_avail,
 					if (QWLT(read_jnl_seqno, jctl->jnl_seqno))
 					{
 						csa = &FILE_INFO(ctl->reg)->s_addrs;
-						freeaddr = csa->jnl->jnl_buff->freeaddr;
+						freeaddr = csa->jnl->jnl_buff->rsrv_freeaddr;
 						if ((ctl->repl_buff->fc->eof_addr == freeaddr) || (!JNL_ENABLED(csa->hdr)))
 						{	/* No more pending updates in the journal file. Next update to the
 							 * journal file will take the seqno jctl->jnl_seqno which will be
@@ -1878,10 +1930,11 @@ int gtmsource_readfiles(unsigned char *buff, int *data_len, int maxbufflen, bool
 	uint4			jnlpool_size;
 	boolean_t		file2pool;
 	unsigned int		start_heartbeat;
+	boolean_t		stop_bunching;
 	gtmsource_state_t	gtmsource_state_sav;
 
-	jctl = jnlpool.jnlpool_ctl;
-	gtmsource_local = jnlpool.gtmsource_local;
+	jctl = jnlpool->jnlpool_ctl;
+	gtmsource_local = jnlpool->gtmsource_local;
 	jnlpool_size = jctl->jnlpool_size;
 	max_read_seqno = jctl->jnl_seqno;
 	/* Note that we are fetching the value of "jctl->jnl_seqno" without a lock on the journal pool. This means we could
@@ -1889,7 +1942,7 @@ int gtmsource_readfiles(unsigned char *buff, int *data_len, int maxbufflen, bool
 	 * concurrent GT.M process updates this 8-byte field in a sequence of two 4-byte operations instead of one
 	 * atomic operation (possible in architectures where 8-byte operations are not native) AND if the pre-update and
 	 * post-update value differ in their most significant 4-bytes. Since that is considered a virtually impossible
-	 * rare occurrence and since we want to avoid the overhead of doing a "grab_lock", we dont do that here.
+	 * rare occurrence and since we want to avoid the overhead of doing a "grab_lock", we don't do that here.
 	 */
 	assert(REPL_MSG_HDRLEN == SIZEOF(jnldata_hdr_struct));
 	DEBUG_ONLY(loopcnt = 0;)
@@ -1909,12 +1962,15 @@ int gtmsource_readfiles(unsigned char *buff, int *data_len, int maxbufflen, bool
 			REPL_DPRINT1("REPL_HISTREC message first needs to be sent before any more seqnos can be sent across\n");
 			return 0;
 		}
-		start_heartbeat = heartbeat_counter;
+		TIMEOUT_INIT(stop_bunching, BUNCHING_TIME);
 		read_addr = gtmsource_local->read_addr;
 		GTMSOURCE_SAVE_STATE(gtmsource_state_sav);
 		first_tr_len = read_size = read_and_merge(buff, maxbufflen, read_jnl_seqno++) + REPL_MSG_HDRLEN;
 		if (GTMSOURCE_NOW_TRANSITIONAL(gtmsource_state_sav))
+		{
+			TIMEOUT_DONE(stop_bunching);
 			return 0;
+		}
 		tot_tr_len = 0;
 		do
 		{
@@ -1924,9 +1980,9 @@ int gtmsource_readfiles(unsigned char *buff, int *data_len, int maxbufflen, bool
 			if (gtmsource_save_read_jnl_seqno < read_jnl_seqno)
 			{
 				read_addr += read_size;
-				if (jnlpool_size >= (jctl->early_write_addr - read_addr))
+				if (jnlpool_size >= (jctl->rsrv_write_addr - read_addr))
 				{	/* No more overflow, switch to READ_POOL.  To avoid the expense of memory barrier
-					 * in jnlpool_hasnt_overflowed(), we use a possibly stale value of early_write_addr
+					 * in jnlpool_hasnt_overflowed(), we use a possibly stale value of rsrv_write_addr
 					 * to check if we can switch back to pool. The consequence is that we may switch
 					 * back and forth between file and pool read if we are in a situation wherein a GTM
 					 * process races with source server, writing transactions into the pool right when
@@ -1949,7 +2005,7 @@ int gtmsource_readfiles(unsigned char *buff, int *data_len, int maxbufflen, bool
 			 * of no-progress. In the worst case we could be unresponsive for 8 seconds (1 heartbeat period)
 			 * with this approach.
 			 */
-			read_multiple = read_multiple && (start_heartbeat == heartbeat_counter);
+			read_multiple = read_multiple && !stop_bunching;
 			if (read_multiple)
 			{	/* Ok to read multiple transactions. Limit the multiple reads until there is no more to be read
 				 * OR total read size reaches a fixed value MAX_TR_BUFFSIZE. This strikes a fine balance between
@@ -1966,7 +2022,10 @@ int gtmsource_readfiles(unsigned char *buff, int *data_len, int maxbufflen, bool
 					GTMSOURCE_SAVE_STATE(gtmsource_state_sav);
 					read_size = read_and_merge(buff, maxbufflen, read_jnl_seqno++) + REPL_MSG_HDRLEN;
 					if (GTMSOURCE_NOW_TRANSITIONAL(gtmsource_state_sav))
-						return 0;	/* Control message triggered state change */
+					{	/* Control message triggered state change */
+						TIMEOUT_DONE(stop_bunching);
+						return 0;
+					}
 					/* Don't use buff to assign type and len as buffer may have expanded.
 					 * Use gtmsource_msgp instead */
 					((repl_msg_ptr_t)((unsigned char *)gtmsource_msgp + tot_tr_len))->type = REPL_TR_JNL_RECS;
@@ -1979,9 +2038,10 @@ int gtmsource_readfiles(unsigned char *buff, int *data_len, int maxbufflen, bool
 			}
 			break;
 		} while (TRUE);
+		TIMEOUT_DONE(stop_bunching);
 		assert(read_jnl_seqno <= max_read_seqno);
 		if ((gtmsource_local->next_histinfo_num < gtmsource_local->num_histinfo)
-			|| (gtmsource_local->num_histinfo == jnlpool.repl_inst_filehdr->num_histinfo))
+			|| (gtmsource_local->num_histinfo == jnlpool->repl_inst_filehdr->num_histinfo))
 		{	/* We are either sending seqnos of a histinfo that is not the last one in the instance file OR
 			 * we are sending seqnos of the last histinfo (that is open-ended) but there has been no more histinfo
 			 * records concurrently added to this instance file compared to what is in our private memory. In either
@@ -2005,6 +2065,12 @@ int gtmsource_readfiles(unsigned char *buff, int *data_len, int maxbufflen, bool
 	} while (TRUE);
 	if (file2pool && !gtmsource_local->jnlfileonly)
 	{
+		/* Ahead of the transition to pool, force repl_phase2_cleanup() when write_addr is behind read_addr. This
+		 * condition happens frequently with replicating instances and instances with infrequent updates.
+		 */
+		if (jctl->write_addr < read_addr)
+			repl_phase2_cleanup(jnlpool);
+		assert(jctl->write_addr >= read_addr);
 		gtmsource_local->read = (uint4)(read_addr % jnlpool_size) ;
 		gtmsource_local->read_state = read_state = READ_POOL;
 	} else
@@ -2105,7 +2171,7 @@ int gtmsource_update_zqgblmod_seqno_and_tn(seq_num resync_seqno)
 			grab_crit(ctl->reg);
 		if (csa->onln_rlbk_cycle != csa->nl->onln_rlbk_cycle)
 		{
-			assert(process_id != jnlpool.gtmsource_local->gtmsource_srv_latch.u.parts.latch_pid);
+			assert(process_id != jnlpool->gtmsource_local->gtmsource_srv_latch.u.parts.latch_pid);
 			SYNC_ONLN_RLBK_CYCLES;
 			gtmsource_onln_rlbk_clnup(); /* would have set gtmsource_state accordingly */
 			if (!was_crit)
@@ -2119,18 +2185,18 @@ int gtmsource_update_zqgblmod_seqno_and_tn(seq_num resync_seqno)
 		if (max_zqgblmod_seqno < start_seqno)
 			max_zqgblmod_seqno = start_seqno;
 	}
-	assert(!jnlpool.jnlpool_ctl->max_zqgblmod_seqno || jnlpool.jnlpool_ctl->max_zqgblmod_seqno > resync_seqno);
+	assert(!jnlpool->jnlpool_ctl->max_zqgblmod_seqno || jnlpool->jnlpool_ctl->max_zqgblmod_seqno > resync_seqno);
 	assert(0 < max_zqgblmod_seqno);
 	assert(resync_seqno >= max_zqgblmod_seqno);
-	assert(!(FILE_INFO(jnlpool.jnlpool_dummy_reg)->s_addrs.now_crit));
-	grab_lock(jnlpool.jnlpool_dummy_reg, TRUE, HANDLE_CONCUR_ONLINE_ROLLBACK);
+	assert(!(FILE_INFO(jnlpool->jnlpool_dummy_reg)->s_addrs.now_crit));
+	grab_lock(jnlpool->jnlpool_dummy_reg, TRUE, HANDLE_CONCUR_ONLINE_ROLLBACK);
 	if (GTMSOURCE_HANDLE_ONLN_RLBK == gtmsource_state)
 	{
-		assert(process_id != jnlpool.gtmsource_local->gtmsource_srv_latch.u.parts.latch_pid);
+		assert(process_id != jnlpool->gtmsource_local->gtmsource_srv_latch.u.parts.latch_pid);
 		return -1; /* gtmsource_ctl_close will be done by gtmsource_process */
 	}
-	jnlpool.jnlpool_ctl->max_zqgblmod_seqno = max_zqgblmod_seqno;
-	rel_lock(jnlpool.jnlpool_dummy_reg);
+	jnlpool->jnlpool_ctl->max_zqgblmod_seqno = max_zqgblmod_seqno;
+	rel_lock(jnlpool->jnlpool_dummy_reg);
 	gtmsource_ctl_close(); /* close all structures now that we are done; if we have to read from journal files; we'll open
 				* the structures again */
 	return (SS_NORMAL);
