@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -45,13 +45,14 @@
 #include "gtm_utf8.h"
 #include "gtmimagename.h"
 #include "gtmio.h"
+#include "op.h"
+#include "indir_enum.h"
 
 #define  LOGNAME_LEN 255
 /* avoid calling getservbyname for shell and Kerberos shell */
 #define	RSHELL_PORT		514
 #define	KSHELL_PORT		544
 
-LITREF	unsigned char		io_params_size[];
 GBLREF	dev_dispatch_struct	io_dev_dispatch[];
 GBLREF	io_desc			*active_device;
 GBLREF	mstr			sys_input;
@@ -59,27 +60,28 @@ GBLREF	mstr			sys_output;
 GBLREF	bool			out_of_time;
 GBLREF	int4			outofband;
 GBLREF	int4			write_filter;
+
 LITREF	mstr			chset_names[];
+LITREF	unsigned char		io_params_size[];
 
 error_def(ERR_GETNAMEINFO);
 error_def(ERR_GTMEISDIR);
 error_def(ERR_TEXT);
 
-bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mval *mspace)	/* timeout in seconds */
+boolean_t io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 msec_timeout, mval *mspace)	/* timeout in msec */
 {
 	uint4		status;
-	int4		msec_timeout;	/* timeout in milliseconds */
 	mstr		tn;		/* translated name */
 	int		oflag;
 	params		*param;
 	unsigned char	ch;
 	bool		timed = FALSE;
 	int		file_des;
-#ifdef __MVS__
+#	ifdef __MVS__
 	int		file_des_w = -2;	/*	fifo write	*/
 	uint4		status_w;
 	d_rm_struct	*d_rm_out, *d_rm_in;
-#endif
+#	endif
 	boolean_t	filecreated = FALSE;
 	TID		timer_id;
 	struct stat	outbuf;
@@ -92,11 +94,9 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 	int		char_or_block_special;
 	int		stat_res;
 	int		fstat_res;
-
 	int		p_offset, len;
 	boolean_t	mknod_err , stat_err, dir_err;
 	int 		save_mknod_err, save_stat_err, save_gsn_err;
-
 	int		gso_stat, gsn_stat, sockoptval;
 	in_port_t	sockport;
 	GTM_SOCKLEN_TYPE	socknamelen;
@@ -106,7 +106,11 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 	int		errcode;
 	unsigned int	port_len;
 	char		port_buffer[NI_MAXSERV], *port_ptr;
+	io_desc		*iod, *temp_iod;
+	DCL_THREADGBL_ACCESS;
 
+	SETUP_THREADGBL_ACCESS;
+	iod = naml->iod;
 	mt_ptr = NULL;
 	char_or_block_special = FALSE;
 	out_of_time = FALSE;
@@ -123,77 +127,69 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 	namebuf[tn.len] = '\0';
 	buf = namebuf;
 	timer_id = (TID)io_open_try;
-	if (NO_M_TIMEOUT == timeout)
-	{
+	if (NO_M_TIMEOUT == msec_timeout)
 		timed = FALSE;
-		msec_timeout = NO_M_TIMEOUT;
-	} else
-	{
-		timed = TRUE;
-		msec_timeout = timeout2msec(timeout);
-		if (!msec_timeout)
-			timed = FALSE;
-	}
-	if (0 == naml->iod)
+	else
+		timed = (0 != msec_timeout);
+	if (0 == iod)
 	{
 		if (0 == tl->iod)
 		{
-			tl->iod =  (io_desc *)malloc(SIZEOF(io_desc));
-			memset((char*)tl->iod, 0, SIZEOF(io_desc));
-			tl->iod->pair.in  = tl->iod;
-			tl->iod->pair.out = tl->iod;
-			tl->iod->trans_name = tl;
-			tl->iod->type = n_io_dev_types;
+			temp_iod =  (io_desc *)malloc(SIZEOF(io_desc));
+			memset((char*)temp_iod, 0, SIZEOF(io_desc));
+			temp_iod->pair.in  = temp_iod;
+			temp_iod->pair.out = temp_iod;
+			temp_iod->trans_name = tl;
+			temp_iod->type = n_io_dev_types;
 			p_offset = 0;
 			while(iop_eol != *(pp->str.addr + p_offset))
 			{
 				ch = *(pp->str.addr + p_offset++);
 				if (iop_fifo == ch)
-					tl->iod->type = ff;
+					temp_iod->type = ff;
 				else  if (iop_sequential == ch)
-					tl->iod->type = rm;
+					temp_iod->type = rm;
 				p_offset += ((IOP_VAR_SIZE == io_params_size[ch]) ?
 					     (unsigned char)*(pp->str.addr + p_offset) + 1 : io_params_size[ch]);
 			}
-			if (ff == tl->iod->type)
+			if (ff == temp_iod->type)
 			{
 				/* fifo with RW permissions for owner, group, other */
 				if ((-1 != MKNOD(buf, FIFO_PERMISSION, 0))
-#ifdef __MVS__
-				    || (EEXIST == errno)
-#endif
-				    )
+#					ifdef __MVS__
+					|| (EEXIST == errno)
+#					endif
+					)
 				{
-#ifdef __MVS__
+#					ifdef __MVS__
 					if (EEXIST != errno)
 						filecreated = TRUE;
 					/*	create another one for fifo write	*/
-					tl->iod->pair.out = (io_desc *)malloc(SIZEOF(io_desc));
-					(tl->iod->pair.out)->pair.in = tl->iod;
-					(tl->iod->pair.out)->pair.out = (tl->iod->pair.out);
-					(tl->iod->pair.out)->trans_name = tl;
-					(tl->iod->pair.out)->type = ff;
-#else
+					temp_iod->pair.out = (io_desc *)malloc(SIZEOF(io_desc));
+					(temp_iod->pair.out)->pair.in = temp_iod;
+					(temp_iod->pair.out)->pair.out = (temp_iod->pair.out);
+					(temp_iod->pair.out)->trans_name = tl;
+					(temp_iod->pair.out)->type = ff;
+#					else
 					filecreated = TRUE;
-#endif
+#					endif
 				} else  if (EEXIST != errno)
 				{
 					mknod_err = TRUE;
 					save_mknod_err = errno;
 					/* This save_err will be checked immediately after error_handler is set;
-					   In that way error handler will get the chance to handle any error */
+					 * In that way error handler will get the chance to handle any error
+					 */
 				}
 			}
+			tl->iod = temp_iod;
 		}
 		if ((n_io_dev_types == tl->iod->type) && mspace && mspace->str.len)
 		{
 			lower_to_upper(dev_type, (uchar_ptr_t)mspace->str.addr, mspace->str.len);
-
-			if (((SIZEOF("SOCKET") - 1) == mspace->str.len)
-				 && (0 == MEMCMP_LIT(dev_type, "SOCKET")))
+			if (((SIZEOF("SOCKET") - 1) == mspace->str.len) && (0 == MEMCMP_LIT(dev_type, "SOCKET")))
 				tl->iod->type = gtmsocket;
-			else if (((SIZEOF("PIPE") - 1) == mspace->str.len)
-			    && (0 == MEMCMP_LIT(dev_type, "PIPE")))
+			else if (((SIZEOF("PIPE") - 1) == mspace->str.len) && (0 == MEMCMP_LIT(dev_type, "PIPE")))
 				tl->iod->type = pi;
 			else
 				tl->iod->type = us;
@@ -232,7 +228,6 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 				}
 			}
 		}
-
 		if (n_io_dev_types == tl->iod->type)
 		{
 			if (!stat_err)
@@ -275,11 +270,11 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 						{
 							socknamelen = SIZEOF(sockname);
 							gsn_stat = getsockname(file_des,
-									       (struct sockaddr *)&sockname,
-									       (GTM_SOCKLEN_TYPE *)&socknamelen);
+									(struct sockaddr *)&sockname,
+									(GTM_SOCKLEN_TYPE *)&socknamelen);
 							save_gsn_err = errno;
 							if (!gsn_stat && ((AF_INET == ((sockaddr_ptr)&sockname)->sa_family)
-									 || (AF_INET6 == ((sockaddr_ptr)&sockname)->sa_family)))
+									|| (AF_INET6 == ((sockaddr_ptr)&sockname)->sa_family)))
 							{
 								port_len = NI_MAXSERV;
 								if (0 != (errcode = getnameinfo((struct sockaddr *)&sockname,
@@ -298,16 +293,13 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 								}
 							}
 							else if ((!gsn_stat && ((AF_UNIX == ((sockaddr_ptr)&sockname)->sa_family)
-										|| (0 == socknamelen)
-										|| (0 == ((sockaddr_ptr)&sockname)->sa_family)))
-									|| (EOPNOTSUPP == save_gsn_err)
-#									if defined(_AIX)
-									|| (ENOTCONN == save_gsn_err)
-#									endif
-#									if defined(__sun) || defined(__hpux)
-									|| (EINVAL == save_gsn_err)
-#									endif
-									)
+								|| (0 == socknamelen)
+								|| (0 == ((sockaddr_ptr)&sockname)->sa_family)))
+								|| (EOPNOTSUPP == save_gsn_err)
+#								if defined(_AIX)
+								|| (ENOTCONN == save_gsn_err)
+#								endif
+								)
 							{
 								tl->iod->type = gtmsocket;
 								break;
@@ -328,13 +320,12 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 				}
 			}
 		}
-		naml->iod = tl->iod;
+		naml->iod = iod = tl->iod;
 	}
-	assert((0 <= naml->iod->state) && (n_io_dev_states > naml->iod->state));
-	active_device = naml->iod;
-
-	if ((-2 == file_des) && (dev_open != naml->iod->state) && (us != naml->iod->type)
-	    && (gtmsocket != naml->iod->type) && (pi != naml->iod->type))
+	assert((0 <= iod->state) && (n_io_dev_states > iod->state));
+	active_device = iod;
+	if ((-2 == file_des) && (dev_open != iod->state) && (us != iod->type)
+	    && (gtmsocket != iod->type) && (pi != iod->type))
 	{
 		oflag |= (O_RDWR | O_CREAT | O_NOCTTY);
 		p_offset = 0;
@@ -346,19 +337,18 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 			{
 				case iop_exception:
 				{
-					naml->iod->error_handler.len = *(pp->str.addr + p_offset);
-					naml->iod->error_handler.addr = (char *)(pp->str.addr + p_offset + 1);
-					s2pool(&naml->iod->error_handler);
+					DEF_EXCEPTION(pp, p_offset, iod);
 					break;
 				}
 				case iop_append:
 					/* this deviceparameter will move the file pointer to EOF in iorm_open.c
-					 but will no longer force write to the EOF, so nothing to do here */
+					 * but will no longer force write to the EOF, so nothing to do here
+					 */
 					break;
 				case iop_contiguous:
 					break;
 				case iop_newversion:
-					if ((dev_open != naml->iod->state) && (rm == naml->iod->type))
+					if ((dev_open != iod->state) && (rm == iod->type))
 						oflag |= O_TRUNC;
 					break;
 				case iop_readonly:
@@ -367,7 +357,7 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 					break;
 				case iop_writeonly:
 #					ifdef __MVS__
-					if (ff != naml->iod->type)
+					if (ff != iod->type)
 #					endif
 					{
 						oflag  &= ~(O_RDWR | O_RDONLY);
@@ -375,28 +365,28 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 					}
 					break;
 				case iop_ipchset:
-#ifdef KEEP_zOS_EBCDIC
-					if ( (iconv_t)0 != naml->iod->input_conv_cd )
+#					ifdef KEEP_zOS_EBCDIC
+					if ( (iconv_t)0 != iod->input_conv_cd )
 					{
-						ICONV_CLOSE_CD(naml->iod->input_conv_cd);
+						ICONV_CLOSE_CD(iod->input_conv_cd);
 					}
-					SET_CODE_SET(naml->iod->in_code_set, (char *)(pp->str.addr + p_offset + 1));
-					if (DEFAULT_CODE_SET != naml->iod->in_code_set)
-						ICONV_OPEN_CD(naml->iod->input_conv_cd,
-							      (char *)(pp->str.addr + p_offset + 1), INSIDE_CH_SET);
-#endif
+					SET_CODE_SET(iod->in_code_set, (char *)(pp->str.addr + p_offset + 1));
+					if (DEFAULT_CODE_SET != iod->in_code_set)
+						ICONV_OPEN_CD(iod->input_conv_cd,
+							(char *)(pp->str.addr + p_offset + 1), INSIDE_CH_SET);
+#					endif
 					break;
 				case iop_opchset:
-#ifdef KEEP_zOS_EBCDIC
-					if ( (iconv_t)0 != naml->iod->output_conv_cd)
+#					ifdef KEEP_zOS_EBCDIC
+					if ( (iconv_t)0 != iod->output_conv_cd)
 					{
-						ICONV_CLOSE_CD(naml->iod->output_conv_cd);
+						ICONV_CLOSE_CD(iod->output_conv_cd);
 					}
-					SET_CODE_SET(naml->iod->out_code_set, (char *)(pp->str.addr + p_offset + 1));
-					if (DEFAULT_CODE_SET != naml->iod->out_code_set)
-						ICONV_OPEN_CD(naml->iod->output_conv_cd, INSIDE_CH_SET,
+					SET_CODE_SET(iod->out_code_set, (char *)(pp->str.addr + p_offset + 1));
+					if (DEFAULT_CODE_SET != iod->out_code_set)
+						ICONV_OPEN_CD(iod->output_conv_cd, INSIDE_CH_SET,
 							      (char *)(pp->str.addr + p_offset + 1));
-#endif
+#					endif
 					break;
 				case iop_m:
 				case iop_utf8:
@@ -410,7 +400,6 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 			p_offset += ((IOP_VAR_SIZE == io_params_size[ch]) ?
 				     (unsigned char)*(pp->str.addr + p_offset) + 1 : io_params_size[ch]);
 		}
-
 		/* Check the saved error from mknod() for fifo, also saved error from fstat() or stat()
 		   so error handler (if set)  can handle it */
 		if (ff == tl->iod->type  && mknod_err)
@@ -421,18 +410,13 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 		/* Error from trying to open a dir */
 		if (dir_err)
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_GTMEISDIR, 2, LEN_AND_STR(buf));
-
-
 		if (timed)
 			start_timer(timer_id, msec_timeout, wake_alarm, 0, NULL);
-
 		/* RW permissions for owner and others as determined by umask. */
 		umask_orig = umask(000);	/* determine umask (destructive) */
 		(void)umask(umask_orig);	/* reset umask */
 		umask_creat = 0666 & ~umask_orig;
-
-		/*
-		 * no OPEN EINTR macros in the following while loop  due to complex error checks and processing between
+		/* no OPEN EINTR macros in the following while loop  due to complex error checks and processing between
 		 * top of while and calls to OPEN3
 		 */
 		while ((-1 == (file_des = OPEN3(buf, SETOCLOEXEC(oflag), umask_creat))) && !out_of_time)
@@ -441,15 +425,15 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 				out_of_time = TRUE;
 			if (outofband)
 				outofband_action(FALSE);
-			if (   EINTR == errno
-			       || ETXTBSY == errno
-			       || ENFILE == errno
-			       || EBUSY == errno
-			       || ((mb == naml->iod->type) && (ENXIO == errno)))
+			if (EINTR == errno
+					|| ETXTBSY == errno
+					|| ENFILE == errno
+					|| EBUSY == errno
+					|| ((mb == iod->type) && (ENXIO == errno)))
 				continue;
 			else
 			{
-#ifdef __MVS__
+#				ifdef __MVS__
 				if (EINVAL == errno && ff == tl->iod->type && (oflag & O_RDONLY) && (oflag & O_WRONLY))
 				{
 					int	oflag_clone;
@@ -463,18 +447,17 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 							out_of_time = TRUE;
 						if (outofband)
 							outofband_action(FALSE);
-						if (   EINTR == errno
+						if (EINTR == errno
 						       || ETXTBSY == errno
 						       || ENFILE == errno
 						       || EBUSY == errno
-						       || ((mb == naml->iod->type) && (ENXIO == errno)))
+						       || ((mb == iod->type) && (ENXIO == errno)))
 							continue;
 						else
 							break;
 					}
 					FCNTL2(file_des, F_GETFL, oflag_clone);
 					FCNTL3(file_des, F_SETFL, (oflag_clone & ~O_NONBLOCK), fcntl_res);
-
 					/* oflag was just made O_RDONLY, now set it to be O_WRONLY */
 					oflag |= O_WRONLY;
 					oflag &= ~O_RDONLY;
@@ -484,21 +467,23 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 							out_of_time = TRUE;
 						if (outofband)
 							outofband_action(FALSE);
-						if (   EINTR == errno
+						if (EINTR == errno
 						       || ETXTBSY == errno
 						       || ENFILE == errno
 						       || EBUSY == errno
-						       || ((mb == naml->iod->type) && (ENXIO == errno)))
+						       || ((mb == iod->type) && (ENXIO == errno)))
 							continue;
 						else
 							break;
 					}
 				}
-#endif
+#				endif
 				break;
 			}
 		}
 		SETFDCLOEXEC(file_des);
+		if (timed && !out_of_time)
+			cancel_timer(timer_id);
 		if (out_of_time && (-1 == file_des))
 		{
 			if ((ff == tl->iod->type) && filecreated)
@@ -510,32 +495,28 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 			return FALSE;
 		}
 	}
-	if (timed && !out_of_time)
-		cancel_timer(timer_id);
-
-#ifdef KEEP_zOS_EBCDIC
-	if (gtmsocket != naml->iod->type)
+#	ifdef KEEP_zOS_EBCDIC
+	if (gtmsocket != iod->type)
 	{
-		SET_CODE_SET(naml->iod->in_code_set, OUTSIDE_CH_SET);
-		if (DEFAULT_CODE_SET != naml->iod->in_code_set)
-			ICONV_OPEN_CD(naml->iod->input_conv_cd, OUTSIDE_CH_SET, INSIDE_CH_SET);
-		SET_CODE_SET(naml->iod->out_code_set, OUTSIDE_CH_SET);
-		if (DEFAULT_CODE_SET != naml->iod->out_code_set)
-			ICONV_OPEN_CD(naml->iod->output_conv_cd, INSIDE_CH_SET, OUTSIDE_CH_SET);
+		SET_CODE_SET(iod->in_code_set, OUTSIDE_CH_SET);
+		if (DEFAULT_CODE_SET != iod->in_code_set)
+			ICONV_OPEN_CD(iod->input_conv_cd, OUTSIDE_CH_SET, INSIDE_CH_SET);
+		SET_CODE_SET(iod->out_code_set, OUTSIDE_CH_SET);
+		if (DEFAULT_CODE_SET != iod->out_code_set)
+			ICONV_OPEN_CD(iod->output_conv_cd, INSIDE_CH_SET, OUTSIDE_CH_SET);
 	} else
 	{
-		SET_CODE_SET(naml->iod->out_code_set, INSIDE_CH_SET);
-		if (DEFAULT_CODE_SET != naml->iod->out_code_set)
-			ICONV_OPEN_CD(naml->iod->output_conv_cd, OUTSIDE_CH_SET, INSIDE_CH_SET);
-		SET_CODE_SET(naml->iod->in_code_set, INSIDE_CH_SET);
-		if (DEFAULT_CODE_SET != naml->iod->in_code_set)
-			ICONV_OPEN_CD(naml->iod->input_conv_cd, INSIDE_CH_SET, OUTSIDE_CH_SET);
+		SET_CODE_SET(iod->out_code_set, INSIDE_CH_SET);
+		if (DEFAULT_CODE_SET != iod->out_code_set)
+			ICONV_OPEN_CD(iod->output_conv_cd, OUTSIDE_CH_SET, INSIDE_CH_SET);
+		SET_CODE_SET(iod->in_code_set, INSIDE_CH_SET);
+		if (DEFAULT_CODE_SET != iod->in_code_set)
+			ICONV_OPEN_CD(iod->input_conv_cd, INSIDE_CH_SET, OUTSIDE_CH_SET);
 	}
-#endif
+#	endif
 	if (-1 == file_des)
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) errno);
-
-	if (n_io_dev_types == naml->iod->type)
+	if (n_io_dev_types == iod->type)
 	{
 		/* On AIX, /dev/{,*}random are of 'terminal type'.Hence define its type before
 		 * calling isatty()
@@ -544,23 +525,22 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 			|| (0 == memvcmp(tn.addr, tn.len, LIT_AND_LEN("/dev/urandom"))))
 				tl->iod->type = rm;
 		else if (isatty(file_des))
-			naml->iod->type = tt;
+			iod->type = tt;
 		else if (char_or_block_special && file_des > 2)
 			if (0 == memvcmp(tn.addr, tn.len, LIT_AND_LEN("/dev/zero")))
 				tl->iod->type = rm;
 			else
 				/* assume mag tape */
-				naml->iod->type = mt;
+				iod->type = mt;
 		else
-			naml->iod->type = rm;
+			iod->type = rm;
 	}
-	assert(naml->iod->type < n_io_dev_types);
-	naml->iod->disp_ptr = &io_dev_dispatch[naml->iod->type];
-
+	assert(iod->type < n_io_dev_types);
+	iod->disp_ptr = &io_dev_dispatch[iod->type];
 	/* Do this deviceparameter check only if type is rm, it is closed, and no_destroy is defined. If a deviceparameter other
 	 * than SEEK or APPEND is defined, then clear no_destroy as state will not be restored.
 	 */
-	if ((rm == naml->iod->type) && (dev_closed == naml->iod->state) && ((d_rm_struct *)naml->iod->dev_sp)->no_destroy)
+	if ((rm == iod->type) && (dev_closed == iod->state) && ((d_rm_struct *)iod->dev_sp)->no_destroy)
 	{
 		for (p_offset = 0; iop_eol != *(pp->str.addr + p_offset); p_offset += ((IOP_VAR_SIZE == io_params_size[ch])
 				? (unsigned char)*(pp->str.addr + p_offset) + 1 : io_params_size[ch]))
@@ -569,35 +549,36 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 			assert((params)ch < (params)n_iops);
 			if ((iop_seek != ch) && (iop_append != ch))
 			{
-				((d_rm_struct *)naml->iod->dev_sp)->no_destroy = FALSE;
+				((d_rm_struct *)iod->dev_sp)->no_destroy = FALSE;
 				break;
 			}
 		}
 	}
-
-	if (dev_never_opened == naml->iod->state)
+	if (dev_never_opened == iod->state)
 	{
-		naml->iod->wrap = DEFAULT_IOD_WRAP;
-		naml->iod->width = DEFAULT_IOD_WIDTH;
-		naml->iod->length = DEFAULT_IOD_LENGTH;
-		naml->iod->write_filter = write_filter;
+		iod->wrap = DEFAULT_IOD_WRAP;
+		iod->width = DEFAULT_IOD_WIDTH;
+		iod->length = DEFAULT_IOD_LENGTH;
+		iod->write_filter = write_filter;
 	}
-	if (dev_open != naml->iod->state)
-	{
-		/* initialize x and y unless it is closed and rm and no_destroy is TRUE */
-		if (!((dev_closed == naml->iod->state) && (rm == naml->iod->type) &&
-		    ((d_rm_struct *)naml->iod->dev_sp)->no_destroy))
+	if (dev_open != iod->state)
+	{	/* initialize x and y unless it is closed and rm and no_destroy is TRUE */
+		if (!((dev_closed == iod->state) && (rm == iod->type) &&
+		    ((d_rm_struct *)iod->dev_sp)->no_destroy))
 		{
-			naml->iod->dollar.x = 0;
-			naml->iod->dollar.y = 0;
+			iod->dollar.x = 0;
+			iod->dollar.y = 0;
 		}
-		naml->iod->dollar.za = 0;
-		naml->iod->dollar.zb[0] = 0;
-		naml->iod->dollar.key[0] = 0;
-		naml->iod->dollar.device[0] = 0;
+		iod->dollar.za = 0;
+		iod->dollar.zb[0] = 0;
+		iod->dollar.key[0] = 0;
+		iod->dollar.device[0] = 0;
+		if (iod->dollar.devicebuffer)
+			free(iod->dollar.devicebuffer);
+		iod->dollar.devicebuffer = NULL;
 	}
-#ifdef __MVS__
-	/*	copy over the content of tl->iod(naml->iod) to (tl->iod->pair.out)	*/
+#	ifdef __MVS__
+	/*	copy over the content of tl->iod(iod) to (tl->iod->pair.out)	*/
 	if (ff == tl->iod->type)
 	{
 		*(tl->iod->pair.out) = *(tl->iod);
@@ -608,41 +589,40 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 		{
 			io_desc		*io_ptr;
 			io_log_name	dev_name;	/*	dummy	*/
-
 			io_ptr = (tl->iod->pair.out);
 			assert(io_ptr != tl->iod->pair.in);
 			assert(NULL != io_ptr);
 			assert(io_ptr->state >= 0 && io_ptr->state < n_io_dev_states);
 			assert(ff == io_ptr->type);
 			dev_name.iod = tl->iod->pair.out;
-			if (TRUE == ioff_open(&dev_name, pp, file_des_w, mspace, timeout))
+			if (TRUE == ioff_open(&dev_name, pp, file_des_w, mspace, msec_timeout))
 				(tl->iod->pair.out)->state = dev_open;
 			else if (dev_open == (tl->iod->pair.out)->state)
 				(tl->iod->pair.out)->state = dev_closed;
 		}
 		else if ((-2 == file_des_w) && (O_WRONLY == (oflag & O_ACCMODE)))
-			active_device = naml->iod = tl->iod->pair.out;
+			active_device = naml->iod = iod = tl->iod->pair.out;
 		if (0 == file_des_w)
 			(tl->iod->pair.out)->dollar.zeof = TRUE;
 	}
-#endif
-	naml->iod->newly_created = filecreated;
-	status = (naml->iod->disp_ptr->open)(naml, pp, file_des, mspace, timeout);
+#	endif
+	iod->newly_created = filecreated;
+	status = (iod->disp_ptr->open)(naml, pp, file_des, mspace, msec_timeout);
 	if (TRUE == status)
-		naml->iod->state = dev_open;
+		iod->state = dev_open;
 	else
 	{
-		if ((dev_open == naml->iod->state) && (gtmsocket != naml->iod->type))
-			naml->iod->state = dev_closed;
-		if ((gtmsocket == naml->iod->type) && naml->iod->newly_created)
+		if ((dev_open == iod->state) && (gtmsocket != iod->type))
+			iod->state = dev_closed;
+		if ((gtmsocket == iod->type) && iod->newly_created)
 		{
-			assert (naml->iod->state != dev_open);
-			iosocket_destroy(naml->iod);
+			assert (iod->state != dev_open);
+			iosocket_destroy(iod);
 			active_device = 0;
 			return status;
 		}
 	}
-#ifdef __MVS__
+#	ifdef __MVS__
 	d_rm_out = tl->iod->pair.out->dev_sp;
 	d_rm_in = tl->iod->pair.in->dev_sp;
 	if ((ff == tl->iod->pair.in->type) || (ff == tl->iod->pair.out->type))
@@ -668,14 +648,12 @@ bool io_open_try(io_log_name *naml, io_log_name *tl, mval *pp, int4 timeout, mva
 		free(d_rm_in);
 		tl->iod->pair.in->dev_sp = d_rm_out;
 	}
-
-#endif
+#	endif
 	if (1 == file_des)
-		naml->iod->dollar.zeof = TRUE;
+		iod->dollar.zeof = TRUE;
 	active_device = 0;
-	naml->iod->newly_created = FALSE;
-
-	if ((NO_M_TIMEOUT != timeout) && IS_MCODE_RUNNING)
+	iod->newly_created = FALSE;
+	if ((NO_M_TIMEOUT != msec_timeout) && IS_MCODE_RUNNING)
 		return (status);
 	else
 		return TRUE;
