@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -18,15 +18,16 @@
 #define DEF_RM_WIDTH		32767
 #define DEF_RM_RECORDSIZE	32767
 #define DEF_RM_LENGTH		66
-#define CHUNK_SIZE		512
+#define CHUNK_SIZE		BUFSIZ
 
-#define	ONE_COMMA		"1,"
 #define	ONE_COMMA_UNAVAILABLE	"1,Resource temporarily unavailable"
-#define	ONE_COMMA_DEV_DET_EOF	"1,Device detected EOF"
+#define ONE_COMMA_DEV_DET_EOF	"1,Device detected EOF"
+#define	ONE_COMMA_DEV_DET_EOF_DOLLARDEVICE	"1,Device detected EOF ..... Now just exceed this line to more than DD_BUFLEN (80)."
+#define	ONE_COMMA_CRYPTBADWRTPOS	"1,Encrypted WRITE disallowed from a position different than where the last WRITE completed"
 
 #define	DEF_RM_PADCHAR		' '	/* SPACE */
 
-/*#define DEBUG_PIPE*/
+/*	#define DEBUG_PIPE	*/
 
 #ifdef DEBUG_PIPE
 #define PIPE_DEBUG(X) X
@@ -68,6 +69,12 @@ int pid;
 error_def(ERR_CLOSEFAIL);
 error_def(ERR_CRYPTBADWRTPOS);
 
+#define MEMSET_IF_DEFINED(BUFFER, CHAR, SIZE)				\
+{									\
+	if (NULL != BUFFER)						\
+		memset(BUFFER, CHAR, SIZE);				\
+}
+
 #define	IORM_FCLOSE(D_RM, FILDES, FILSTR)								\
 {													\
 	GBLREF int	process_exiting;								\
@@ -159,7 +166,10 @@ error_def(ERR_CRYPTBADWRTPOS);
 				stop_image_no_core();							\
 			}										\
 		}											\
-		DOLLAR_DEVICE_WRITE(IOD, write_status);							\
+		if (EAGAIN == write_status)								\
+			SET_DOLLARDEVICE_ERRSTR(IOD, ONE_COMMA_UNAVAILABLE);		\
+		else											\
+			SET_DOLLARDEVICE_ONECOMMA_STRERROR(IOD, write_status);		\
 		IOD->dollar.za = 9;									\
 		rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) write_status);					\
 	}												\
@@ -217,7 +227,7 @@ typedef struct
 	boolean_t	write_only;	/* WRITEONLY specified */
 	boolean_t	stream;
 	boolean_t	fifo;
-	boolean_t	pipe;		/* True if pipe device */
+	boolean_t	is_pipe;	/* True if pipe device */
 	boolean_t	independent; 	/* True if pipe process to live after pipe is closed */
 	boolean_t	parse;		/* True if pipe command is to be parsed */
 	boolean_t	done_1st_read;	/* If UTF16, we need to know if this is the first read or not to check for BOM */
@@ -260,9 +270,9 @@ typedef struct
 	FILE		*filstr;
 	off_t		file_pos;
 	long		pipe_buff_size;
-	char		utf_tmp_buffer[CHUNK_SIZE];	/* Buffer to store CHUNK bytes */
-	int		utf_tot_bytes_in_buffer;	/* Number of bytes read from device, it refers utf_tmp_buffer buffer */
-	int		utf_start_pos;			/* Current position in utf_tmp_buffer */
+	char		*tmp_buffer;			/* Buffer to store CHUNK_SIZE bytes */
+	int		tot_bytes_in_buffer;		/* Number of bytes read from device, it refers tmp_buffer buffer */
+	int		start_pos;			/* Current position in tmp_buffer */
 	boolean_t	write_occurred;			/* Flag indicating whether a write has occurred on this device. */
 	boolean_t	read_occurred;			/* Flag indicating whether a read has occurred on this device. */
 	boolean_t	input_encrypted;		/* Whether this device's input stream is encrypted or not. */
@@ -273,8 +283,11 @@ typedef struct
 	mstr		output_key;			/* Name that maps to an output encryption key on disk. */
 	gtmcrypt_key_t	input_cipher_handle;		/* Encryption cipher handle for this device. */
 	gtmcrypt_key_t	output_cipher_handle;		/* Decryption cipher handle for this device. */
+	gtm_chset_t	ichset_utf16_variant;		/* Used to determine UTF16 variant when CHSET is changed b/w UTF16 & M. */
+	gtm_chset_t	ochset_utf16_variant;		/* Used to determine UTF16 variant when CHSET is changed b/w UTF16 & M. */
 	uint4		fsblock_buffer_size;		/* I/O buffer size; 1 == default size; 0 == no buffering */
 	char		*fsblock_buffer;		/* I/O buffer for, erm, buffered I/O */
+	boolean_t	crlastbuff;			/* Whether CR was last char of the buffer */
 } d_rm_struct;	/*  rms		*/
 
 #ifdef KEEP_zOS_EBCDIC
