@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -32,11 +32,13 @@
 #include "gtmmsg.h"
 #include "iosp.h"	/* for SS_NORMAL */
 #include "gtmio.h"
+#include "sgtm_putmsg.h"
 #include "repl_sp.h"	/* for F_CLOSE used by the JNL_FD_CLOSE macro */
 #include "interlock.h"
 #include "lockconst.h"
 #include "aswp.h"
 #include "is_file_identical.h"
+#include "jnl_file_close_timer.h"
 #ifdef __MVS__
 #include "gtm_zos_io.h"
 #endif
@@ -47,6 +49,14 @@
 
 #include "wbox_test_init.h"
 
+#define SET_JPC_ERR_STR(ERR1, ERR2, BUF)							\
+{												\
+	jpc->status = ERR2;									\
+	sts = ERR1;										\
+	sgtm_putmsg(BUF, VARLSTCNT(7) sts, 4, JNL_LEN_STR(csd), DB_LEN_STR(reg), jpc->status);	\
+	jpc->err_str = BUF;									\
+}
+
 GBLREF	boolean_t		is_src_server;
 GBLREF	boolean_t		mupip_jnl_recover;
 
@@ -55,7 +65,7 @@ error_def(ERR_JNLMOVED);
 error_def(ERR_JNLOPNERR);
 error_def(ERR_JNLRDERR);
 
-uint4 jnl_file_open(gd_region *reg, bool init, void *dummy)	/* third argument for compatibility with VMS version */
+uint4 jnl_file_open(gd_region *reg, boolean_t init)
 {
 	sgmnt_addrs		*csa;
 	sgmnt_data_ptr_t	csd;
@@ -67,6 +77,7 @@ uint4 jnl_file_open(gd_region *reg, bool init, void *dummy)	/* third argument fo
 	int			fstat_res;
 	int			close_res;
 	boolean_t		switch_and_retry;
+	char			buff[OUT_BUFF_SIZE];
 	ZOS_ONLY(int		realfiletag;)
 
 	csa = &FILE_INFO(reg)->s_addrs;
@@ -76,21 +87,25 @@ uint4 jnl_file_open(gd_region *reg, bool init, void *dummy)	/* third argument fo
 	assert(NOJNL == jpc->channel);
 	sts = 0;
 	jpc->status = jpc->status2 = SS_NORMAL;
+	assert(NULL == jpc->err_str);
 	nameptr = csd->jnl_file_name;
 	assert('/' == csd->jnl_file_name[0]);
 	if (init)
 	{
 		assert(csd->jnl_file_len < JNL_NAME_SIZE);
+		/* csd->jnl_file_name[] is of size JNL_NAME_SIZE and must not change across GT.M versions hence the below asserts */
+		assert(JNL_NAME_SIZE == ARRAYSIZE(csd->jnl_file_name));
+		assert(256 == JNL_NAME_SIZE);
 		nameptr[csd->jnl_file_len] = 0;
 		cre_jnl_file_intrpt_rename(csa);
-		/* although jnl_file_close() would have reset jnl_file.u.inode and device to 0 and incremented cycle, it
+		/* although "jnl_file_close" would have reset jnl_file.u.inode and device to 0 and incremented cycle, it
 		 * might have got shot in the middle of executing those instructions. we redo it here just to be safe.
 		 */
 		csa->nl->jnl_file.u.inode = 0;
 		csa->nl->jnl_file.u.device = 0;
 		jb->cycle++;
 		/* Source Server only reads journal files so must never try to create and switch to a new journal file. */
-		switch_and_retry = (!is_src_server) ? TRUE : FALSE;
+		switch_and_retry = (!is_src_server);
 		for (;;)
 		{
 			/* D9E04-002445 MUPIP RECOVER always open journal file without O_SYNC, ignoring jnl_sync_io */
@@ -125,13 +140,13 @@ uint4 jnl_file_open(gd_region *reg, bool init, void *dummy)	/* third argument fo
 					JNL_FD_CLOSE(jpc->channel, close_res);	/* sets jpc->channel to NOJNL */
 					break;
 				} else
-					sts = jnl_file_open_common(reg, (off_jnl_t) stat_buf.st_size);
+					sts = jnl_file_open_common(reg, (off_jnl_t) stat_buf.st_size, buff);
 			}
-			DEBUG_ONLY(
-				/* Will fail if Source Server would need to switch journal files. */
-				assert((gtm_white_box_test_case_enabled && (WBTEST_JNL_SWITCH_EXPECTED ==
-					gtm_white_box_test_case_number)) || (0 == sts) || switch_and_retry);
-			)
+#			ifdef DEBUG
+			/* Will fail if Source Server would need to switch journal files. */
+			assert((gtm_white_box_test_case_enabled && (WBTEST_JNL_SWITCH_EXPECTED == gtm_white_box_test_case_number))
+					|| (0 == sts) || (!is_src_server));
+#			endif
 			if ((0 != sts) && switch_and_retry)
 			{	/* Switch to a new journal file and retry, but only once */
 				sts = jnl_file_open_switch(reg, sts);
@@ -197,19 +212,18 @@ uint4 jnl_file_open(gd_region *reg, bool init, void *dummy)	/* third argument fo
 						/* The dupfd check above should not reset our channel. */
 						assertpro(NOJNL != jpc->channel);
 					)
+					START_JNL_FILE_CLOSE_TIMER_IF_NEEDED;
 				}  /* if jnl_state */
 			} else
 			{	/* not init and file moved */
-				jpc->status = ERR_JNLMOVED;
-				sts = ERR_JNLOPNERR;
+				SET_JPC_ERR_STR(ERR_JNLOPNERR, ERR_JNLMOVED, buff);
 				assert(gtm_white_box_test_case_enabled
 					&& ((WBTEST_JNLOPNERR_EXPECTED == gtm_white_box_test_case_number)
 						|| (WBTEST_JNL_CREATE_FAIL == gtm_white_box_test_case_number)));
 			}
 		} else
 		{	/* stat failed */
-			jpc->status = errno;
-			sts = ERR_JNLFILOPN;
+			SET_JPC_ERR_STR(ERR_JNLFILOPN, errno, buff);
 		}
 	}  /* sts == 0 when we started */
 	if (0 != sts)
@@ -219,6 +233,7 @@ uint4 jnl_file_open(gd_region *reg, bool init, void *dummy)	/* third argument fo
 		assert(NOJNL == jpc->channel);
 		jnl_send_oper(jpc, sts);
 	}
+	jpc->err_str = NULL;
 	assert((0 != sts) || (NOJNL != jpc->channel));
 	return sts;
 }
