@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2015 Fidelity National Information 	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -24,7 +24,7 @@
 #include "gtm_un.h"
 
 #include <errno.h>
-#if defined(__sparc) || defined(__hpux) || defined(__MVS__) || defined(__linux__) || defined(__CYGWIN__)
+#if defined(__MVS__) || defined(__linux__) || defined(__CYGWIN__)
 #include "gtm_limits.h"
 #else
 #include <sys/limits.h>
@@ -64,32 +64,16 @@
 #endif
 
 #ifdef MUTEX_MSEM_WAKE
-#define MUTEX_MAX_HEARTBEAT_WAIT        2 /* so that total wait for both select and msem wait will be the same */
-#define MUTEX_LCKALERT_PERIOD		4
+#define MUTEX_MAX_WAIT        		(MUTEX_CONST_TIMEOUT_VAL * MILLISECS_IN_SEC)
 #endif
 
-/* The following PROBE_* macros invoke the corresponding * macros except in the case csa->hdr is NULL.
- * This is possible if the csa corresponds to the journal pool where there is no notion of a db hdr.
- * In that case, we skip invoking the * macros.
- */
-#define	PROBE_SET_TRACEABLE_VAR(CSA, VALUE)					\
-{										\
-	sgmnt_data_ptr_t	lcl_csd;					\
-										\
-	lcl_csd = CSA->hdr;							\
-	assert((NULL != lcl_csd)						\
-		|| (CSA == &FILE_INFO(jnlpool.jnlpool_dummy_reg)->s_addrs));	\
-	if (NULL != lcl_csd)							\
-		SET_TRACEABLE_VAR(CSA->nl->wc_blocked, TRUE);			\
-}
-
 #define	PROBE_BG_TRACE_PRO_ANY(CSA, EVENT)					\
 {										\
 	sgmnt_data_ptr_t	lcl_csd;					\
 										\
 	lcl_csd = CSA->hdr;							\
 	assert((NULL != lcl_csd)						\
-		|| (CSA == &FILE_INFO(jnlpool.jnlpool_dummy_reg)->s_addrs));	\
+		|| (CSA == &FILE_INFO(jnlpool->jnlpool_dummy_reg)->s_addrs));	\
 	if (NULL != lcl_csd)							\
 		BG_TRACE_PRO_ANY(CSA, EVENT);					\
 }
@@ -109,7 +93,7 @@ MBSTART {															\
 		(CSA)->critical->crit_cycle++;											\
 		MUTEX_DPRINT3("%d: Write %sACQUIRED\n", (PID), (MUTEX_LOCK_WRITE == (LOCK_TYPE)) ? "" : "IMMEDIATE ");		\
 		MUTEX_TEST_SIGNAL_HERE("WRTLCK NOW CRIT\n", FALSE);								\
-		(CSA)->now_crit = TRUE;												\
+		SET_CSA_NOW_CRIT_TRUE(CSA);											\
 		MUTEX_TEST_SIGNAL_HERE("WRTLCK SUCCESS\n", FALSE);								\
 		if (-1 != (SPIN_CNT))												\
 		{														\
@@ -151,6 +135,8 @@ MBSTART {															\
 			(CSA)->probecrit_rec.p_crit_yields = (gtm_uint64_t)(YIELDS);						\
 			(CSA)->probecrit_rec.p_crit_que_slps = (gtm_uint64_t)(Q_SLPS);						\
 		}														\
+		if (save_jnlpool != jnlpool)											\
+			jnlpool = save_jnlpool;											\
 		return STATUS;													\
 	}															\
 } MBEND
@@ -158,11 +144,10 @@ MBSTART {															\
 
 GBLREF int			num_additional_processors;
 GBLREF jnl_gbls_t		jgbl;
-GBLREF jnlpool_addrs		jnlpool;
+GBLREF jnlpool_addrs_ptr_t	jnlpool;
 GBLREF uint4			process_id;
-GBLREF uint4			image_count, mutex_per_process_init_pid;
+GBLREF uint4			mutex_per_process_init_pid;
 #ifdef MUTEX_MSEM_WAKE
-GBLREF volatile uint4           heartbeat_counter;
 #  ifdef POSIX_MSEM
 static sem_t			*mutex_wake_msem_ptr = NULL;
 #  else
@@ -364,7 +349,8 @@ static	enum cdb_sc mutex_long_sleep(mutex_struct_ptr_t addr, sgmnt_addrs *csa,
 	enum cdb_sc		status;
 	boolean_t		wakeup_status;
 #	ifdef MUTEX_MSEM_WAKE
-	uint4                   bad_heartbeat;
+	boolean_t		msem_timedout;
+	int			save_errno;
 #	else
 	struct timeval		timeout;
 	int			timeout_threshold;
@@ -398,40 +384,39 @@ static	enum cdb_sc mutex_long_sleep(mutex_struct_ptr_t addr, sgmnt_addrs *csa,
 	do
 	{
 #		ifdef MUTEX_MSEM_WAKE
-		/* My msemaphore is already used by another process.
-		 * In other words, I was woken up, but missed my wakeup call.
-		 * I should return immediately.
-		 */
 		if (msem_slot->pid != process_id)
+		{	/* My msemaphore is already used by another process.
+		   	 * In other words, I was woken up, but missed my wakeup call.
+			 * I should return immediately.
+			 */
 			wakeup_status = TRUE;
-		else
+		} else
 		{
-			bad_heartbeat = 0;
+			TIMEOUT_INIT(msem_timedout, MUTEX_MAX_WAIT);
 			/*
 			 * the check for EINTR below is valid and should not be converted to an EINTR
 			 * wrapper macro, because another condition is checked for the while loop.
 			 */
 			while (!(wakeup_status = (0 == MSEM_LOCKW(mutex_wake_msem_ptr))))
 			{
-				if (EINTR == errno)
+				save_errno = errno;
+				if (EINTR == save_errno)
 				{
-					if (bad_heartbeat)	/* to save memory reference and calc on fast path */
+					if (msem_timedout)
 					{
-						if (bad_heartbeat < heartbeat_counter)
-						{
-							MUTEX_DPRINT3("%d: msem sleep done, heartbeat_counter = %d\n",
-								     process_id, heartbeat_counter);
-							break;
-						}
-						MUTEX_DPRINT3("%d: msem sleep continue, heartbeat_counter = %d\n",
-							      process_id, heartbeat_counter);
-					} else
-						bad_heartbeat = heartbeat_counter + MUTEX_MAX_HEARTBEAT_WAIT - 1;
-					/* -1 since we were interrupted this time */
+						MUTEX_DPRINT3("%d: msem sleep done, heartbeat_counter = %d\n",
+							     process_id, heartbeat_counter);
+						break;
+					}
+					MUTEX_DPRINT3("%d: msem sleep continue, heartbeat_counter = %d\n",
+						      process_id, heartbeat_counter);
 				} else
+				{
 					rts_error_csa(CSA_ARG(csa) VARLSTCNT(7) ERR_MUTEXERR, 0, ERR_TEXT, 2,
-						RTS_ERROR_TEXT("Error with mutex wake msem"), errno);
+						RTS_ERROR_TEXT("Error with mutex wake msem"), save_errno);
+				}
 			}
+			TIMEOUT_DONE(msem_timedout);
 			/* wakeup_status is set to true, if I was able to lock...somebody woke me up;
 			 * wakeup_status is set to false, if I timed out and should go to recovery.
 			 */
@@ -663,15 +648,19 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 	node_local		*cnl;
 	uint4			in_crit_pid;
 	sgmnt_addrs		*csa;
+	jnlpool_addrs_ptr_t	save_jnlpool;
 	time_t			curr_time;
 	uint4			curr_time_uint4, next_alert_uint4;
 	ABS_TIME 		atstart;
 #	ifdef MUTEX_MSEM_WAKE
 	int			rc;
 #	endif
+        DCL_THREADGBL_ACCESS;
 
+        SETUP_THREADGBL_ACCESS;
 	csa = &FILE_INFO(reg)->s_addrs;
 	assert(!csa->now_crit);
+	save_jnlpool = jnlpool;
 	cnl = csa->nl;
 	/* Check that "mutex_per_process_init" has happened before we try to grab crit and that it was done with our current
 	 * pid (i.e. ensure that even in the case where parent did the mutex init with its pid and did a fork, the child process
@@ -679,13 +668,14 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 	 * know for sure there is no other pid accessing the database shared memory.
 	 */
 	assert((MUTEX_LOCK_WRITE_IMMEDIATE == mutex_lock_type) || (MUTEX_LOCK_WRITE == mutex_lock_type));
-	assert((mutex_per_process_init_pid == process_id) || (0 == mutex_per_process_init_pid) && in_mu_rndwn_file);
+	assert((mutex_per_process_init_pid == process_id) || ((0 == mutex_per_process_init_pid) && in_mu_rndwn_file));
 	MUTEX_TRACE_CNTR((MUTEX_LOCK_WRITE == mutex_lock_type) ? mutex_trc_lockw : mutex_trc_lockwim);
 	optimistic_attempts = MUTEX_MAX_OPTIMISTIC_ATTEMPTS;
 	queue_sleeps = csa->probecrit_rec.p_crit_que_full = 0;
 	spins = yields = 0;
 	local_crit_cycle = 0;	/* this keeps us from doing a MUTEXLCKALERT on the first cycle in case the time latch is stale */
 	try_recovery = jgbl.onlnrlbk; /* salvage lock the first time if we are online rollback thereby reducing unnecessary waits */
+	assert(cnl);
 	epoch_count = cnl->doing_epoch;
 	addr = csa->critical;
 	if (csa->crit_probe)
@@ -744,6 +734,9 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 		assert(MAXUINT4 > curr_time);
 		curr_time_uint4 = (uint4)curr_time;
 		next_alert_uint4 = csa->critical->stuckexec.cas_time;
+		assert(save_jnlpool == jnlpool);
+		assert(!csa->jnlpool || (csa->jnlpool == jnlpool));
+		assert((curr_time_uint4 <= next_alert_uint4) || (!csa->jnlpool || (csa->jnlpool == jnlpool)));
 		if ((curr_time_uint4 > next_alert_uint4) && !IS_REPL_INST_FROZEN)
 		{	/* We've waited long enough and the Instance is not frozen - might be time to send MUTEXLCKALERT */
 			if (COMPSWAP_LOCK(&csa->critical->stuckexec.time_latch, next_alert_uint4, 0,
@@ -753,7 +746,7 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 				if (process_id == in_crit_pid)
 				{	/* This is just a precaution - shouldn't ever happen and has no code to maintain gvstats */
 					assert(FALSE);
-					csa->now_crit = TRUE;
+					SET_CSA_NOW_CRIT_TRUE(csa);
 					return (cdb_sc_normal);
 				}
 				if (in_crit_pid && (in_crit_pid == cnl->in_crit) && is_proc_alive(in_crit_pid, 0))
@@ -882,7 +875,11 @@ enum cdb_sc gtm_mutex_lock(gd_region *reg,
 						mutex_wake_proc((sm_int_ptr_t)&wake_this_pid, wake_instance);
 #					endif
 					if (0 != (--sleep_spin_cnt))
+					{
+						if (save_jnlpool != jnlpool)
+							jnlpool = save_jnlpool;
 						return (cdb_sc_dbccerr);	/* Too many failures */
+					}
 					assert(!csa->now_crit);
 					GTM_REL_QUANT(mutex_spin_parms->mutex_spin_sleep_mask);
 				} while (sleep_spin_cnt);		/* actually terminated by the return three lines above */
@@ -925,15 +922,17 @@ enum cdb_sc mutex_unlockw(gd_region *reg, int crash_count)
 
 	uint4		already_clear;
 	sgmnt_addrs	*csa;
+        DCL_THREADGBL_ACCESS;
 
+        SETUP_THREADGBL_ACCESS;
 	csa = &FILE_INFO(reg)->s_addrs;
 	if (crash_count != csa->critical->crashcnt)
 		return (cdb_sc_critreset);
 	assert(csa->now_crit);
 	MUTEX_TEST_SIGNAL_HERE("WRTUNLCK NOW CRIT\n", FALSE);
-	csa->now_crit = FALSE;
 	assert(csa->critical->semaphore.u.parts.latch_pid == process_id);
 	RELEASE_SWAPLOCK(&csa->critical->semaphore);
+	SET_CSA_NOW_CRIT_FALSE(csa);
 	MUTEX_DPRINT2("%d: WRITE LOCK RELEASED\n", process_id);
 	return (mutex_wakeup(csa->critical, NULL != csa->hdr
 		? (mutex_spin_parms_ptr_t)(&csa->hdr->mutex_spin_parms)
@@ -949,7 +948,7 @@ void mutex_cleanup(gd_region *reg)
 	   the lock, go ahead and release it.
 	*/
 	csa = &FILE_INFO(reg)->s_addrs;
-	if (COMPSWAP_UNLOCK(&csa->critical->semaphore, process_id, image_count, LOCK_AVAILABLE, 0))
+	if (COMPSWAP_UNLOCK(&csa->critical->semaphore, process_id, CMPVAL2, LOCK_AVAILABLE, 0))
 	{
 		MUTEX_DPRINT2("%d  mutex_cleanup : released lock\n", process_id);
 	}
@@ -969,46 +968,228 @@ void mutex_seed_init(void)
 
 void mutex_salvage(gd_region *reg)
 {
-	sgmnt_addrs	*csa;
-	int		salvage_status;
-	uint4		holder_pid, onln_rlbk_pid;
-	boolean_t	mutex_salvaged;
-	VMS_ONLY(uint4	holder_imgcnt;)
+	sgmnt_addrs		*csa;
+	int			index1, index2, orig_index2, salvage_status;
+	uint4			holder_pid, onln_rlbk_pid, start_freeaddr, orig_freeaddr;
+	boolean_t		mutex_salvaged;
+	sgmnt_data_ptr_t	csd;
+	jnlpool_ctl_ptr_t	jpl;
+	jpl_phase2_in_prog_t	*lastJplCmt;
+	jbuf_phase2_in_prog_t	*lastJbufCmt;
+	node_local		*cnl;
+	seq_num			jnl_seqno, strm_seqno, strmSeqno60;
+	int			strmIndex;
+	jnl_buffer_ptr_t	jbp;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
 	csa = &FILE_INFO(reg)->s_addrs;
+	cnl = csa->nl;
 	if (0 != (holder_pid = csa->critical->semaphore.u.parts.latch_pid))
 	{
 		mutex_salvaged = FALSE;
-		VMS_ONLY(holder_imgcnt = csa->critical->semaphore.u.parts.latch_image_count);
-		if (holder_pid == process_id VMS_ONLY(&& holder_imgcnt == image_count))
-		{	/* We were trying to obtain a lock we already held -- very odd */
-			RELEASE_SWAPLOCK(&csa->critical->semaphore);
-			csa->nl->in_crit = 0;
-			/* Mutex crash repaired, want to do write cache recovery, just in case.
-			 * Take care not to do it for jnlpool which has no concept of a db cache.
-			 * In that case csa->hdr is NULL so use PROBE_SET_TRACEABLE_VAR macro.
+		/* If we were trying to obtain a lock we already held, it is an out-of-design situation. We cannot safely
+		 * release the lock in that case so create a core file even in pro.
+		 */
+		assertpro(holder_pid != process_id);
+		if (!is_proc_alive(holder_pid, 0))
+		{	/* Release the COMPSWAP lock AFTER setting cnl->in_crit to 0 as an assert in
+			 * grab_crit (checking that cnl->in_crit is 0) relies on this order.
 			 */
-			PROBE_SET_TRACEABLE_VAR(csa, TRUE);
-			mutex_salvaged = TRUE;
-			MUTEX_DPRINT2("%d : mutex salvaged, culprit was our own process\n", process_id);
-		} else if (!is_proc_alive(holder_pid, UNIX_ONLY(0) VMS_ONLY(holder_imgcnt)))
-		{	/* Release the COMPSWAP lock AFTER setting csa->nl->in_crit to 0 as an assert in
-			 * grab_crit (checking that csa->nl->in_crit is 0) relies on this order.
-			 */
-			send_msg_csa(CSA_ARG(csa) VARLSTCNT(5) ERR_MUTEXFRCDTERM, 3, holder_pid, REG_LEN_STR(reg));
-			csa->nl->in_crit = 0;
+			send_msg_csa(CSA_ARG(csa) VARLSTCNT(5) ERR_MUTEXFRCDTERM, 3, holder_pid, DB_LEN_STR(reg));
+			cnl->in_crit = 0;
 			/* Mutex crash repaired, want to do write cache recovery, in case previous holder of crit had set
 			 * some cr->in_cw_set to a non-zero value. Not doing cache recovery could cause incorrect
 			 * GTMASSERTs in PIN_CACHE_RECORD macro in t_end/tp_tend.				BYPASSOK(GTMASSERT)
-			 * Take care not to do it for jnlpool which has no concept of a db cache.
-			 * In that case csa->hdr is NULL so use PROBE_SET_TRACEABLE_VAR macro.
+			 * Take care not to do it for jnlpool (csa->hdr is NULL in that case) which has no concept of a db cache.
 			 */
-			PROBE_SET_TRACEABLE_VAR(csa, TRUE);
+			csd = csa->hdr;
+			assert((NULL != csd) || (NULL != jnlpool));
+			assert((NULL != csd) || (csa == &FILE_INFO(jnlpool->jnlpool_dummy_reg)->s_addrs));
+			if (NULL == csd)
+			{	/* This is a jnlpool. Check if a process in t_end/tp_tend was killed BEFORE
+				 * it incremented jpl->jnl_seqno. If so, undo any changes done in UPDATE_JPL_RSRV_WRITE_ADDR.
+				 */
+				jpl = jnlpool->jnlpool_ctl;
+				assert(NULL != jpl);
+				index1 = jpl->phase2_commit_index1;
+				index2 = jpl->phase2_commit_index2;
+				orig_index2 = index2;
+				assert(jpl->write_addr <= jpl->rsrv_write_addr);
+				DECR_PHASE2_COMMIT_INDEX(index2, JPL_PHASE2_COMMIT_ARRAY_SIZE);
+				lastJplCmt = &jpl->phase2_commit_array[index2];
+				/* This process could have been killed during a commit in t_end/tp_tend.
+				 *	a) In the middle of Step CMT03 (see secshr_db_clnup.c for CMTxx steps)
+				 *		UPDATE_JPL_RSRV_WRITE_ADDR macro OR
+				 *	b) After Step CMT03 but before Step CMT07 finished.
+				 * In either case, we need to reset jpl to what it was BEFORE Step CMT03 began i.e. roll back.
+				 * If the process gets killed AFTER CMT07 finishes, the transaction is rolled forward even
+				 *	if it means writing JRT_NULL and/or JRT_INCTN records in jnlpool and/or jnlbuff.
+				 * Note that there is still a small window of instructions after CMT07 is done but before
+				 *	CMT08 is done (for the first region in case of a multi-region TP transaction) if
+				 *	a process gets killed, we will roll forward the jnlpool but roll back the jnlbuff
+				 *	and so there would be a seqno with no corresponding journal records in the journal
+				 *	files. This is not easily handled so is left as a todo for the future.
+				 */
+				if ((index1 == orig_index2) || (lastJplCmt->process_id != holder_pid))
+				{	/* CMT02 < killed <= CMT03.
+					 * Kill could have happened before CMT03 finished so reset things.
+					 * This reset is a no-op if the kill happened even before CMT03 started.
+					 * This is Case (a).
+					 */
+					jpl->rsrv_write_addr = lastJplCmt->start_write_addr + lastJplCmt->tot_jrec_len;
+					assert(((lastJplCmt->jnl_seqno + 1) == jpl->jnl_seqno) || !lastJplCmt->jnl_seqno);
+					jpl->lastwrite_len = lastJplCmt->tot_jrec_len;
+				} else
+				{
+					assert((lastJplCmt->jnl_seqno == jpl->jnl_seqno)
+						|| ((lastJplCmt->jnl_seqno + 1) == jpl->jnl_seqno));
+					if (lastJplCmt->jnl_seqno == jpl->jnl_seqno)
+					{	/* CMT03 < killed < CMT07 */
+						jpl->rsrv_write_addr = lastJplCmt->start_write_addr;
+						jpl->lastwrite_len = lastJplCmt->prev_jrec_len;
+						SHM_WRITE_MEMORY_BARRIER; /* similar layout as UPDATE_JPL_RSRV_WRITE_ADDR */
+						jpl->phase2_commit_index2 = index2;	/* remove last commit entry */
+					}
+					/* else : CMT07 < killed and so no rollback needed */
+				}
+			} else
+			{	/* This is a database shm. Check if a process in t_end/tp_tend was killed BEFORE
+				 * Step CMT08 (see secshr_db_clnup.c) when it would have set cnl->update_underway_tn.
+				 * If so, undo any changes done in Step CMT06 (UPDATE_JBP_RSRV_FREEADDR).
+				 * Effectively rolling back the aborted commit in this region.
+				 */
+				assert((csd->trans_hist.early_tn == csd->trans_hist.curr_tn)
+					|| (csd->trans_hist.early_tn == (csd->trans_hist.curr_tn + 1)));
+				assert(cnl->update_underway_tn <= csd->trans_hist.curr_tn);
+				assert(csd->trans_hist.early_tn >= cnl->update_underway_tn);
+				if (JNL_ENABLED(csd) && (csd->trans_hist.early_tn != csd->trans_hist.curr_tn))
+				{	/* i.e. Process was killed after CMT04 but before CMT12. It is represented as
+					 *	CMT04 < killed < CMT12
+					 */
+					assert(NULL != csa->jnl);
+					assert(NULL != csa->jnl->jnl_buff);
+					jbp = csa->jnl->jnl_buff;
+					index1 = jbp->phase2_commit_index1;
+					index2 = jbp->phase2_commit_index2;
+					orig_index2 = index2;
+					assert(jbp->freeaddr <= jbp->rsrv_freeaddr);
+					DECR_PHASE2_COMMIT_INDEX(index2, JNL_PHASE2_COMMIT_ARRAY_SIZE);
+					lastJbufCmt = &jbp->phase2_commit_array[index2];
+					if (cnl->update_underway_tn != csd->trans_hist.curr_tn)
+					{	/* CMT04 < killed < CMT08.
+						 * ----------------------------------------------------------------
+						 * Roll-back the entire transaction's effect on this database file
+						 * ----------------------------------------------------------------
+						 */
+						start_freeaddr = lastJbufCmt->start_freeaddr;
+						if ((index1 == orig_index2) || (lastJbufCmt->process_id != holder_pid)
+							|| (lastJbufCmt->curr_tn != csd->trans_hist.curr_tn))
+						{	/* CMT04 < KILLED <= CMT06.
+							 * Kill could have happened before CMT06 finished so reset things.
+							 * This reset is a no-op if the kill happened even before CMT06 started.
+							 */
+							SET_JBP_RSRV_FREEADDR(jbp, start_freeaddr + lastJbufCmt->tot_jrec_len);
+						} else
+						{	/* CMTO6 < killed < CMT08 */
+							assert(lastJbufCmt->curr_tn == csd->trans_hist.curr_tn);
+							/* CMT06 finished. So undo it as a whole */
+							assert(lastJbufCmt->process_id == holder_pid);
+							/* If CMT06a was in progress when the process was KILLED, then it is
+							 * possible jbp->freeaddr was updated as CMT16 (which is what CMT06a
+							 * executes) progressed. So undo that too. Likewise for dskaddr,
+							 * fsync_dskaddr etc. And finally reset rsrv_freeaddr.
+							 */
+							assert(jbp->fsync_dskaddr <= jbp->dskaddr);
+							orig_freeaddr = jbp->freeaddr;
+							if (orig_freeaddr > start_freeaddr)
+							{
+								jbp->freeaddr = start_freeaddr;
+								jbp->free = start_freeaddr % jbp->size;
+							} else
+								assert(jbp->free == (orig_freeaddr % jbp->size));
+							if (jbp->dskaddr > start_freeaddr)
+							{
+								assert(!GLOBAL_LATCH_HELD_BY_US(&jbp->io_in_prog_latch));
+								grab_latch(&jbp->io_in_prog_latch, GRAB_LATCH_INDEFINITE_WAIT);
+								/* Fix jbp->dskaddr & jbp->dsk while holding io latch */
+								assert(orig_freeaddr > start_freeaddr);
+								jbp->dskaddr = start_freeaddr;
+								jbp->dsk = start_freeaddr % jbp->size;
+								/* Setting jbp->dskaddr to start_freeaddr is not enough.
+								 * We also need to re-read the partial filesystem-block-size
+								 * aligned block of data that precedes the new jbp->dskaddr
+								 * since that part is most likely no longer in the jnl buffer
+								 * (have been overwritten by the current aborted tn's jnl records).
+								 * We can try and optimize this by avoiding setting
+								 * jbp->re_read_dskaddr in case no overwrite happened. But it is
+								 * not straightforward to detect that and the risk is journal
+								 * file corruption. Given "mutex_salvage" is a rare occurrence,
+								 * it is safer to re-read unconditionally.
+								 */
+								jbp->re_read_dskaddr = start_freeaddr;
+								rel_latch(&jbp->io_in_prog_latch);
+								if (jbp->fsync_dskaddr > start_freeaddr)
+								{	/* Fix jbp->fsync_dskaddr while holding fsync io latch */
+									assert(!GLOBAL_LATCH_HELD_BY_US(&jbp->fsync_in_prog_latch));
+									grab_latch(&jbp->fsync_in_prog_latch,
+											GRAB_LATCH_INDEFINITE_WAIT);
+									jbp->fsync_dskaddr = start_freeaddr;
+									rel_latch(&jbp->fsync_in_prog_latch);
+								}
+							} else
+								assert(jbp->dsk == (jbp->dskaddr % jbp->size));
+							/* "jnl_write_phase2" is never called with JRT_EPOCH (see assert there
+							 * at function entry of possible rectype values and EPOCH is not in that
+							 * list). Therefore we are guaranteed a "jnl_write_epoch_rec" call never
+							 * happened since the first call to "jnl_write_reserve" happened in this
+							 * transaction. Therefore no UNDO of the effects of "jnl_write_epoch_rec"
+							 * needed here (e.g. jbp->post_epoch_freeaddr).
+							 */
+							assert(jbp->post_epoch_freeaddr <= start_freeaddr);
+							SET_JBP_RSRV_FREEADDR(jbp, start_freeaddr);
+							SHM_WRITE_MEMORY_BARRIER;	/* see corresponding
+											 * SHM_READ_MEMORY_BARRIER in
+											 * "jnl_phase2_cleanup".
+											 */
+							jbp->phase2_commit_index2 = index2; /* remove last commit entry */
+						}
+						csd->trans_hist.early_tn = csd->trans_hist.curr_tn;	/* Undo CMT04 */
+						/* CMT07 is jnlpool related, so no undo done here (in db mutex_salvage) for that */
+					} else
+					{	/* CMT08 < killed < CMT12.
+						 * -------------------------------------------------------------------
+						 * Roll-forward the entire transaction's effect on this database file
+						 * -------------------------------------------------------------------
+						 * In case process got killed before CMT09 occurred, redo it.
+						 * If the process got killed after CMT09, the below redo is a no-op.
+						 */
+						/* CMT09 redo : start */
+						jnl_seqno = lastJbufCmt->jnl_seqno + 1;
+						strm_seqno = lastJbufCmt->strm_seqno;
+						/* If "strm_seqno" is 0, we are guaranteed this is not a supplementary
+						 * instance (i.e. "supplementary" variable in t_end/tp_tend is FALSE).
+						 */
+						if (strm_seqno)
+						{
+							strmIndex = GET_STRM_INDEX(strm_seqno);
+							strmSeqno60 = GET_STRM_SEQ60(strm_seqno) + 1;
+						}
+						SET_REG_SEQNO(csa, jnl_seqno, strm_seqno, strmIndex, strmSeqno60, SKIP_ASSERT_TRUE);
+						/* CMT09 redo : end */
+						csd->trans_hist.curr_tn = csd->trans_hist.early_tn;	/* Redo CMT12 */
+					}
+				}
+				/* else: Step CMT04 did not happen OR Database is not journaled.
+				 *	 Nothing to undo in this db for Steps CMT01, CMT02 and CMT03.
+				 */
+				SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE); /* This will ensure we call "wcs_recover" which
+									   * will recover CMT04 and other CMTxx steps.
+									   */
+			}
 			COMPSWAP_UNLOCK(&csa->critical->semaphore, holder_pid, holder_imgcnt, LOCK_AVAILABLE, 0);
 			mutex_salvaged = TRUE;
-			/* Reset jb->blocked as well if the holder_pid had it set */
+			/* Reset jbp->blocked as well if the holder_pid had it set */
 			if ((NULL != csa->jnl) && (NULL != csa->jnl->jnl_buff) && (csa->jnl->jnl_buff->blocked == holder_pid))
 				csa->jnl->jnl_buff->blocked = 0;
 			MUTEX_DPRINT3("%d : mutex salvaged, culprit was %d\n", process_id, holder_pid);
@@ -1024,9 +1205,10 @@ void mutex_salvage(gd_region *reg)
 		 * Take care not to do it for jnlpool which has no concept of a db cache.
 		 * In that case csa->hdr is NULL so check accordingly.
 		 */
-		assert((NULL != csa->hdr) || (csa == &FILE_INFO(jnlpool.jnlpool_dummy_reg)->s_addrs));
+		assert((NULL != csa->hdr) || (jnlpool && (csa == &FILE_INFO(jnlpool->jnlpool_dummy_reg)->s_addrs)));
 		if (mutex_salvaged && (NULL != csa->hdr))
 		{
+			SET_TRACEABLE_VAR(cnl->wc_blocked, TRUE);
 			BG_TRACE_PRO_ANY(csa, wcb_mutex_salvage); /* no need to use PROBE_BG_TRACE_PRO_ANY macro
 								   * since we already checked for csa->hdr non-NULL.
 								   */
@@ -1050,7 +1232,7 @@ void	mutex_per_process_init(void)
 	mutex_seed_init();
 #	ifndef MUTEX_MSEM_WAKE
 	if (mutex_per_process_init_pid)
-	{	/* Close socket opened by the first call. But dont delete the socket file as the parent process will do that. */
+	{	/* Close socket opened by the first call. But don't delete the socket file as the parent process will do that. */
 		assert(FD_INVALID != mutex_sock_fd);
 		if (FD_INVALID != mutex_sock_fd)
 			CLOSEFILE_RESET(mutex_sock_fd, status);	/* resets "mutex_sock_fd" to FD_INVALID */
