@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2006, 2008 Fidelity Information Services, Inc	*
+ * Copyright (c) 2006-2015 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -12,6 +13,11 @@
 #include "mdef.h"
 #include "op.h"
 #include "gtm_utf8.h"
+#ifdef UNICODE_SUPPORTED
+# include "utfcgr.h"
+#else
+# include "utfcgr_trc.h"
+#endif
 
 #define OP_FNEXTRACT op_fnextract
 #include "op_fnextract.h"
