@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2013-2015 Fidelity National Information	*
+ * Copyright (c) 2013-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -48,6 +48,8 @@ error_def(ERR_ZPEEKNORPLINFO);
 
 #define FMTHEXDGT(spfree, digit) *spfree++ = digit + ((digit <= 9) ? '0' : ('A' - 0x0A))
 #define ARGUMENT_MAX_LEN	MAX_MIDENT_LEN
+#define	PASS1			1
+#define	PASS2			2
 
 /* Codes for peek operation mnemonics */
 typedef enum
@@ -70,15 +72,15 @@ typedef enum
 	PO_JBFREG	/* 15 Journal buffer information - jnl_buffer_ptr_t */
 } zpeek_mnemonic;
 
-GBLREF boolean_t        created_core;
-GBLREF sigset_t		blockalrm;
-GBLREF gd_addr		*gd_header;
-GBLREF boolean_t	pool_init;
-GBLREF boolean_t	jnlpool_init_needed;
-GBLREF jnlpool_addrs	jnlpool;
-GBLREF recvpool_addrs	recvpool;
+GBLREF boolean_t        	created_core;
+GBLREF sigset_t			blockalrm;
+GBLREF gd_addr			*gd_header;
+GBLREF int			pool_init;
+GBLREF boolean_t		jnlpool_init_needed;
+GBLREF jnlpool_addrs_ptr_t	jnlpool;
+GBLREF recvpool_addrs		recvpool;
 #ifdef DEBUG
-GBLREF	int		process_exiting;
+GBLREF	int			process_exiting;
 #endif
 
 LITREF unsigned char lower_to_upper_table[];
@@ -407,7 +409,7 @@ CONDITION_HANDLER(op_fnzpeek_getpool_ch)
 STATICFNDEF boolean_t op_fnzpeek_attach_jnlpool(void)
 {
 	ESTABLISH_RET(op_fnzpeek_getpool_ch, FALSE);
-	jnlpool_init(GTMRELAXED, FALSE, NULL);		/* Attach to journal pool */
+	jnlpool_init(GTMRELAXED, FALSE, NULL, NULL);		/* Attach to journal pool */
 	REVERT;
 	return pool_init;
 }
@@ -445,6 +447,7 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 	unsigned int		full_len;
 	unsigned char		argument_uc_buf[ARGUMENT_MAX_LEN];
 	sgmnt_addrs		*csa;
+	int			pass;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -481,20 +484,29 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_BADZPEEKARG, 2, RTS_ERROR_LITERAL("mnemonic argument"));
 		switch(mnemonic_opcode)
 		{
-			case PO_CSAREG:			/* These types have a region name argument */
-			case PO_FHREG:
-			case PO_GDRREG:
-			case PO_NLREG:
-			case PO_JNLREG:
-			case PO_JBFREG:
-				/* Uppercase the region name since that is what GDE does when creating them */
-				lower_to_upper(argument_uc_buf, argptr, arglen);
-				argptr = argument_uc_buf;	/* Reset now to point to upper case version */
+		case PO_CSAREG:			/* These types have a region name argument */
+		case PO_FHREG:
+		case PO_GDRREG:
+		case PO_NLREG:
+		case PO_JNLREG:
+		case PO_JBFREG:
+			/* Uppercase the region name since that is what GDE does when creating them.
+			 * But we want the ability to do $zpeek on statsdb regions (lower-case regions)
+			 * so first check if region as is does exist. If so use that. If not, do uppercase.
+			 */
+			assert(arglen);
+			pass = (ISLOWER_ASCII(argptr[0]) ? PASS1 : PASS2);
+			for ( ; ; )
+			{
+				if (PASS1 != pass)
+				{
+					lower_to_upper(argument_uc_buf, argptr, arglen);
+					argptr = argument_uc_buf;	/* Reset now to point to upper case version */
+				}
 				/* See if region recently used so can avoid lookup */
 				if ((arglen == TREF(zpeek_regname_len)) && (0 == memcmp(argptr, TADR(zpeek_regname), arglen)))
 				{	/* Fast path - no lookup necessary */
 					r_ptr = TREF(zpeek_reg_ptr);
-					assert(r_ptr->open && !r_ptr->was_open);	/* Make sure truly open */
 					break;
 				}
 				/* Region now defined - make sure it is open */
@@ -504,42 +516,67 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 				for (r_top = r_ptr + gd_header->n_regions; ; r_ptr++)
 				{
 					if (r_ptr >= r_top)
+					{
+						if (PASS1 == pass)
+							break;
 						rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_BADZPEEKARG, 2,
 							      RTS_ERROR_LITERAL("mnemonic argument (region name)"));
+					}
 					if ((r_ptr->rname_len == arglen) && (0 == memcmp(r_ptr->rname, argptr, arglen)))
 						break;
 				}
-				if (!r_ptr->open)
-					gv_init_reg(r_ptr);
+				if (r_ptr == r_top)
+				{	/* Could not find lower-case region specified. Try upper-casing it.
+					 * Note that even though GDE guarantees all upper-case region names have a corresponding
+					 * lower-case region name, it is possible the user specified a region name with
+					 * a lower-case letter as the first letter and at least one upper-case letter in the
+					 * region name. In that case, it is possible we find the region in the gld only when we
+					 * convert the entire region name into upper case.
+					 */
+					assert(PASS1 == pass);
+					pass = PASS2;
+					continue;
+				}
 				/* Cache new region access for followup references */
 				memcpy(TADR(zpeek_regname), argptr, arglen);
 				TREF(zpeek_regname_len) = arglen;
 				TREF(zpeek_reg_ptr) = r_ptr;
-				/* r_ptr now points to (open) region */
-				assert(r_ptr->open && !r_ptr->was_open);	/* Make sure truly open */
 				break;
-			case PO_GLFREPL:		/* These types have an array index argument */
-			case PO_GSLREPL:
-				arryidx = asc2i(argptr, arglen);
-				if ((0 > arryidx) || (NUM_GTMSRC_LCL <= arryidx))
+			}
+			/* PO_GDRREG opcode examines only the region's fields so does not need the region to be open.
+			 * All the rest need it to be open. If there are any errors in the open (e.g. statsdb specified
+			 * and gtm_statsdir env var is too long etc.) then handle it by issuing an error.
+			 */
+			if ((PO_GDRREG != mnemonic_opcode) && !r_ptr->open)
+			{
+				gv_init_reg(r_ptr, NULL);
+				if (!r_ptr->open)
 					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_BADZPEEKARG, 2,
-						      RTS_ERROR_LITERAL("mnemonic argument (array index)"));
-				break;
-			case PO_PEEK:			/* Argument is address of form 0Xhhhhhhhh[hhhhhhhh] */
-				if (('0' != *cptr++) || ('x' != *cptr) && ('X' != *cptr))
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_BADZPEEKARG, 2,
-						      RTS_ERROR_LITERAL("mnemonic argument (peek base address)"));
-				cptr++;			/* Bump past 'x' or 'X' - rest of arg should be hex value */
-				prmpeekadr = (UINTPTR_T)GTM64_ONLY(asc_hex2l)NON_GTM64_ONLY(asc_hex2i)(cptr, arglen - 2);
-				if (-1 == (INTPTR_T)prmpeekadr)
-					/* Either an error occurred or the user specified the maximum address. So it's
-					 * either an error from the conversion routine or an otherwise useless value.
-					 */
-					rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_BADZPEEKARG, 2,
-						      RTS_ERROR_LITERAL("mnemonic argument (peek base address)"));
-				break;
-			default:
-				assert(FALSE);		/* Only the above types should ever have an argument */
+						      RTS_ERROR_LITERAL("mnemonic argument (region name could not be opened)"));
+			}
+			break;
+		case PO_GLFREPL:		/* These types have an array index argument */
+		case PO_GSLREPL:
+			arryidx = asc2i(argptr, arglen);
+			if ((0 > arryidx) || (NUM_GTMSRC_LCL <= arryidx))
+				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_BADZPEEKARG, 2,
+					      RTS_ERROR_LITERAL("mnemonic argument (array index)"));
+			break;
+		case PO_PEEK:			/* Argument is address of form 0Xhhhhhhhh[hhhhhhhh] */
+			if (('0' != *cptr++) || ('x' != *cptr) && ('X' != *cptr))
+				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_BADZPEEKARG, 2,
+					      RTS_ERROR_LITERAL("mnemonic argument (peek base address)"));
+			cptr++;			/* Bump past 'x' or 'X' - rest of arg should be hex value */
+			prmpeekadr = (UINTPTR_T)GTM64_ONLY(asc_hex2l)NON_GTM64_ONLY(asc_hex2i)(cptr, arglen - 2);
+			if (-1 == (INTPTR_T)prmpeekadr)
+				/* Either an error occurred or the user specified the maximum address. So it's
+				 * either an error from the conversion routine or an otherwise useless value.
+				 */
+				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(4) ERR_BADZPEEKARG, 2,
+					      RTS_ERROR_LITERAL("mnemonic argument (peek base address)"));
+			break;
+		default:
+			assert(FALSE);		/* Only the above types should ever have an argument */
 		}
 	}
 	/* Figure out the address of each block to return */
@@ -570,7 +607,7 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 		case PO_NLREPL:
 		case PO_RIHREPL:
 			/* Make sure jnlpool_addrs are availble */
-			if (!REPL_INST_AVAILABLE)
+			if (!REPL_INST_AVAILABLE(NULL))
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_ZPEEKNORPLINFO);
 			if (!pool_init)
 			{
@@ -581,19 +618,19 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 			switch(mnemonic_opcode)
 			{
 				case PO_GLFREPL:	/* arryidx set by option processing */
-					zpeekadr = (jnlpool.gtmsrc_lcl_array + arryidx);
+					zpeekadr = (jnlpool->gtmsrc_lcl_array + arryidx);
 					break;
 				case PO_GSLREPL:	/* arryidx set by option processing */
-					zpeekadr = (jnlpool.gtmsource_local_array + arryidx);
+					zpeekadr = (jnlpool->gtmsource_local_array + arryidx);
 					break;
 				case PO_NLREPL:
-					zpeekadr = (&FILE_INFO(jnlpool.jnlpool_dummy_reg)->s_addrs)->nl;
+					zpeekadr = (&FILE_INFO(jnlpool->jnlpool_dummy_reg)->s_addrs)->nl;
 					break;
 				case PO_JPCREPL:
-					zpeekadr = jnlpool.jnlpool_ctl;
+					zpeekadr = jnlpool->jnlpool_ctl;
 					break;
 				case PO_RIHREPL:
-					zpeekadr = jnlpool.repl_inst_filehdr;
+					zpeekadr = jnlpool->repl_inst_filehdr;
 					break;
 				default:
 					assert(FALSE);
@@ -604,7 +641,7 @@ void	op_fnzpeek(mval *structid, int offset, int len, mval *format, mval *ret)
 		case PO_UPLREPL:
 		case PO_UHCREPL:
 			/* Make sure recvpool_addrs are available */
-			if (!REPL_INST_AVAILABLE)
+			if (!REPL_INST_AVAILABLE(NULL))
 				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(1) ERR_ZPEEKNORPLINFO);
 			if (NULL == recvpool.recvpool_ctl)
 			{
