@@ -1,6 +1,7 @@
 /****************************************************************
  *								*
- *	Copyright 2001, 2014 Fidelity Information Services, Inc	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
+ * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
  *	of its copyright holder(s), and is made available	*
@@ -11,17 +12,9 @@
 
 #include "mdef.h"
 
-#include <stddef.h>
-#include <errno.h>
-#include <signal.h>
-#include <sys/wait.h>
-#include <sys/time.h>
-#include <sys/sem.h>
-#include <sys/shm.h>
-#include <sys/param.h>
-
+#include "gtm_signal.h"
 #include "gtm_ctype.h"
-#include "gtm_stdlib.h"		/* for exit() */
+#include "gtm_stdlib.h"		/* for EXIT() */
 #include "gtm_string.h"
 #include "gtm_socket.h"
 #include "gtm_fcntl.h"
@@ -32,6 +25,14 @@
 #include "gtm_syslog.h"
 #include "gtm_ipc.h"
 
+#include <stddef.h>
+#include <errno.h>
+#include <sys/wait.h>
+#include <sys/time.h>
+#include <sys/sem.h>
+#include <sys/shm.h>
+#include <sys/param.h>
+
 #include "gt_timer.h"
 #include "gtmio.h"
 #include "io.h"
@@ -106,7 +107,7 @@ const static char readonly *secshrstart_error_code[] = {
 };
 
 #define MAX_COMM_ATTEMPTS		4	/* 1 to start secshr, 2 maybe slow, 3 maybe really slow, 4 outside max */
-#define CLIENT_ACK_TIMER		5
+#define CLIENT_ACK_TIMER		5 * MILLISECS_IN_SEC
 
 #define START_SERVER										\
 {												\
@@ -130,7 +131,7 @@ const static char readonly *secshrstart_error_code[] = {
 		}										\
 		/* For transient failures we will continue after printing out message */	\
 	}											\
-	hiber_start(3000); /* 3000 ms (3 sec) to allow server to come up */			\
+	hiber_start(500); /* half-a-second to allow server to come up */			\
 }
 
 #define SETUP_FOR_RECV										\
@@ -140,8 +141,8 @@ const static char readonly *secshrstart_error_code[] = {
 	client_timer_popped = FALSE;								\
 	recv_complete = FALSE;									\
 	save_errno = 0;										\
-	msec_timeout = timeout2msec(CLIENT_ACK_TIMER);						\
-	start_timer(timer_id, msec_timeout, client_timer_handler, 0, NULL);			\
+	assert(MAXPOSINT4 >= CLIENT_ACK_TIMER);						\
+	start_timer(timer_id, CLIENT_ACK_TIMER, client_timer_handler, 0, NULL);			\
 }
 
 error_def(ERR_GTMDISTUNVERIF);
@@ -233,8 +234,8 @@ int send_mesg2gtmsecshr(unsigned int code, unsigned int id, char *path, int path
 		} else if (FLUSH_DB_IPCS_INFO == code)
 		{
 			assert(GTM_PATH_MAX > db_ipcs.fn_len);
+			/* Most of the time file length is much smaller than GTM_PATH_MAX, hence the fn_len + 1 below */
 			memcpy(&mesg.mesg.db_ipcs, &db_ipcs, (offsetof(ipcs_mesg, fn[0]) + db_ipcs.fn_len + 1));
-			/* Most of the time file length is much smaller than GTM_PATH_MAX */
 			send_len += offsetof(ipcs_mesg, fn[0]);
 			send_len += mesg.mesg.db_ipcs.fn_len + 1;
 		} else
@@ -348,9 +349,9 @@ int send_mesg2gtmsecshr(unsigned int code, unsigned int id, char *path, int path
 				send_msg_csa(CSA_ARG(NULL) VARLSTCNT(13) ERR_GTMSECSHRSRVFIL, 7, RTS_ERROR_TEXT("Client"),
 					 process_id, mesg.pid, req_code, RTS_ERROR_TEXT(mesg.mesg.path),
 					 ERR_TEXT, 2, RTS_ERROR_STRING("Communicating with wrong GT.M version"));
-				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(13) ERR_GTMSECSHRSRVFIL, 7, RTS_ERROR_TEXT("Client"),
-					  process_id, mesg.pid, req_code, RTS_ERROR_TEXT(mesg.mesg.path),
-					  ERR_TEXT, 2, RTS_ERROR_STRING("Communicating with wrong GT.M version"));
+				rts_error_csa(CSA_ARG(NULL) VARLSTCNT(13) MAKE_MSG_ERROR(ERR_GTMSECSHRSRVFIL), 7,
+					 RTS_ERROR_TEXT("Client"), process_id, mesg.pid, req_code, RTS_ERROR_TEXT(mesg.mesg.path),
+					 ERR_TEXT, 2, RTS_ERROR_STRING("Communicating with wrong GT.M version"));
 				break;	/* rts_error should not return */
 			}
 			switch(req_code)
@@ -452,7 +453,7 @@ int create_server(void)
 		{
 			send_msg_csa(CSA_ARG(NULL) VARLSTCNT(9) ERR_GTMSECSHRSTART, 3, RTS_ERROR_TEXT("Client"), process_id,
 				ERR_TEXT, 2, RTS_ERROR_STRING(secshrstart_error_code[UNABLETOEXECGTMSECSHR]));
-			_exit(UNABLETOEXECGTMSECSHR);
+			UNDERSCORE_EXIT(UNABLETOEXECGTMSECSHR);
 		}
         } else
 	{
