@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2010-2015 Fidelity National Information 	*
+ * Copyright (c) 2010-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -31,6 +31,7 @@
 #include "trigger_update_protos.h"
 #include "trigger_compare_protos.h"
 #include "trigger_user_name.h"
+#include "gtm_trigger_trc.h"
 #include "gtm_string.h"
 #include "mv_stent.h"			/* for COPY_SUBS_TO_GVCURRKEY macro */
 #include "gvsub2str.h"			/* for COPY_SUBS_TO_GVCURRKEY */
@@ -65,6 +66,9 @@
 #include "change_reg.h"		/* for "change_reg" prototype */
 #include "gvnh_spanreg.h"	/* for "gvnh_spanreg_subs_gvt_init" prototype */
 #include "mu_interactive.h"	/* for prompt looping */
+#include "is_file_identical.h"
+#include "anticipatory_freeze.h"
+#include "gtm_repl_multi_inst.h" /* for DISALLOW_MULTIINST_UPDATE_IN_TP */
 
 GBLREF	sgmnt_data_ptr_t	cs_data;
 GBLREF	uint4			dollar_tlevel;
@@ -74,6 +78,8 @@ GBLREF	gd_region		*gv_cur_region;
 GBLREF	gv_key			*gv_currkey;
 GBLREF	gd_addr			*gd_header;
 GBLREF	io_pair			io_curr_device;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool_head;
 GBLREF	trans_num		local_tn;
 GBLREF	gv_namehead		*reset_gv_target;
 GBLREF	sgm_info		*sgm_info_ptr;
@@ -326,7 +332,7 @@ static	boolean_t		promptanswer = TRUE;
 /* This error macro is used for all definition errors where the target is ^#t(GVN,<#LABEL|#COUNT|#CYCLE>) */
 #define HASHT_DEFINITION_RETRY_OR_ERROR(SUBSCRIPT, MOREINFO, CSA)	\
 {									\
-	if (CDB_STAGNATE > t_tries)					\
+	if (UPDATE_CAN_RETRY(t_tries, t_fail_hist[t_tries]))		\
 		t_retry(cdb_sc_triggermod);				\
 	else								\
 	{								\
@@ -427,16 +433,17 @@ STATICFNDEF int4 update_trigger_name(char *trigvn, int trigvn_len, int trigger_i
  */
 boolean_t trigger_name_search(char *trigger_name, uint4 trigger_name_len, mval *val, gd_region **srch_reg)
 {
-	boolean_t	name_found;
-	char		*ptr, *ptr2;
-	gd_region	*reg, *reg_top;
-	gd_region	*save_gv_cur_region;
-	gv_key		save_currkey[DBKEYALLOC(MAX_KEY_SZ)];
-	gv_namehead	*save_gv_target;
-	gvnh_reg_t	*gvnh_reg;
-	int		len;
-	mname_entry	gvname;
-	sgm_info	*save_sgm_info_ptr;
+	boolean_t		name_found;
+	char			*ptr, *ptr2;
+	gd_region		*reg, *reg_top;
+	gd_region		*save_gv_cur_region;
+	gv_key			save_currkey[DBKEYALLOC(MAX_KEY_SZ)];
+	gv_namehead		*save_gv_target;
+	gvnh_reg_t		*gvnh_reg;
+	int			len;
+	mname_entry		gvname;
+	sgm_info		*save_sgm_info_ptr;
+	jnlpool_addrs_ptr_t	save_jnlpool;
 
 	/* Example trigger name could be x#1#A:BREG to indicate trigger on global ^x with an autogenerated name x#1
 	 * that exists in multiple regions and hence had a runtime disambiguator of x#1#A. The :BREG is a region-level
@@ -458,7 +465,7 @@ boolean_t trigger_name_search(char *trigger_name, uint4 trigger_name_len, mval *
 	 * on the max # of duplicated auto-generated names.
 	 */
 	assert(0 < trigger_name_len);
-	SAVE_TRIGGER_REGION_INFO(save_currkey);
+	SAVE_REGION_INFO(save_currkey, save_gv_target, save_gv_cur_region, save_sgm_info_ptr, save_jnlpool);
 	name_found = FALSE;
 	reg = *srch_reg;
 	if (NULL != reg)
@@ -471,6 +478,8 @@ boolean_t trigger_name_search(char *trigger_name, uint4 trigger_name_len, mval *
 	}
 	for ( ; reg < reg_top; reg++)
 	{
+		if (IS_STATSDB_REGNAME(reg))
+			continue;
 		GVTR_SWITCH_REG_AND_HASHT_BIND_NAME(reg);
 		if (NULL == cs_addrs)	/* not BG or MM access method */
 			continue;
@@ -485,7 +494,7 @@ boolean_t trigger_name_search(char *trigger_name, uint4 trigger_name_len, mval *
 		ptr2 = memchr(ptr, '\0', val->str.len);	/* Do it this way since "val" has multiple fields null separated */
 		if (NULL == ptr2)
 		{	/* We expect $c(0) in the middle of ptr. If we dont find it, this is a restartable situation */
-			if (CDB_STAGNATE > t_tries)
+			if (UPDATE_CAN_RETRY(t_tries, t_fail_hist[t_tries]))
 				t_retry(cdb_sc_triggermod);
 			assert(WBTEST_HELPOUT_TRIGDEFBAD == gtm_white_box_test_case_number);
 			rts_error_csa(CSA_ARG(NULL) VARLSTCNT(6) ERR_TRIGNAMBAD, 4, LEN_AND_LIT("\"#TNAME\""),
@@ -507,7 +516,7 @@ boolean_t trigger_name_search(char *trigger_name, uint4 trigger_name_len, mval *
 		name_found = TRUE;
 		break;
 	}
-	RESTORE_TRIGGER_REGION_INFO(save_currkey);
+	RESTORE_REGION_INFO(save_currkey, save_gv_target, save_gv_cur_region, save_sgm_info_ptr, save_jnlpool);
 	return name_found;
 }
 
@@ -515,15 +524,16 @@ boolean_t trigger_name_search(char *trigger_name, uint4 trigger_name_len, mval *
 boolean_t check_unique_trigger_name_full(char **values, uint4 *value_len, mval *val, boolean_t *new_match,
 				char *trigvn, int trigvn_len, stringkey *kill_trigger_hash, stringkey *set_trigger_hash)
 {
-	boolean_t	overall_name_found, this_name_found;
-	gd_region	*reg, *reg_top;
-	gv_key		save_currkey[DBKEYALLOC(MAX_KEY_SZ)];
-	gd_region	*save_gv_cur_region;
-	gv_namehead	*save_gv_target;
-	sgm_info	*save_sgm_info_ptr;
-	int		set_index, kill_index;
-	boolean_t	db_matched_set, db_matched_kill, full_match, trigger_exists;
-	mval		setname, killname;
+	boolean_t		overall_name_found, this_name_found;
+	gd_region		*reg, *reg_top;
+	gv_key			save_currkey[DBKEYALLOC(MAX_KEY_SZ)];
+	gd_region		*save_gv_cur_region;
+	gv_namehead		*save_gv_target;
+	sgm_info		*save_sgm_info_ptr;
+	jnlpool_addrs_ptr_t	save_jnlpool;
+	int			set_index, kill_index;
+	boolean_t		db_matched_set, db_matched_kill, full_match, trigger_exists;
+	mval			setname, killname;
 	DCL_THREADGBL_ACCESS;
 
 	SETUP_THREADGBL_ACCESS;
@@ -537,10 +547,12 @@ boolean_t check_unique_trigger_name_full(char **values, uint4 *value_len, mval *
 	*new_match = TRUE;
 	if (0 == value_len[TRIGNAME_SUB])
 		return TRUE;
-	SAVE_TRIGGER_REGION_INFO(save_currkey);
+	SAVE_REGION_INFO(save_currkey, save_gv_target, save_gv_cur_region, save_sgm_info_ptr, save_jnlpool);
 	overall_name_found = FALSE;
 	for (reg = gd_header->regions, reg_top = reg + gd_header->n_regions; reg < reg_top; reg++)
 	{
+		if (IS_STATSDB_REGNAME(reg))
+			continue;
 		GVTR_SWITCH_REG_AND_HASHT_BIND_NAME(reg);
 		if (NULL == cs_addrs)	/* not BG or MM access method */
 			continue;
@@ -565,7 +577,7 @@ boolean_t check_unique_trigger_name_full(char **values, uint4 *value_len, mval *
 			}
 		}
 	}
-	RESTORE_TRIGGER_REGION_INFO(save_currkey);
+	RESTORE_REGION_INFO(save_currkey, save_gv_target, save_gv_cur_region, save_sgm_info_ptr, save_jnlpool);
 	return !overall_name_found;
 }
 
@@ -1531,23 +1543,14 @@ STATICFNDEF trig_stats_t trigupdrec_reg(char *trigvn, int trigvn_len, boolean_t
 	if (gv_cur_region->read_only)
 		rts_error_csa(CSA_ARG(csa) VARLSTCNT(4) ERR_TRIGMODREGNOTRW, 2, REG_LEN_STR(gv_cur_region));
 	assert(cs_addrs == gv_target->gd_csa);
-	csa->incr_db_trigger_cycle = TRUE; /* so that we increment csd->db_trigger_cycle at commit time */
-	if (dollar_ztrigger_invoked)
-	{	/* increment db_dztrigger_cycle so that next gvcst_put/gvcst_kill in this transaction, on this region,
-		 * will re-read triggers. Note that the below increment happens for every record added. So, even if a
-		 * single trigger file loaded multiple triggers on the same region, db_dztrigger_cycle will be incremented
-		 * more than one for same transaction. This is considered okay since we only need db_dztrigger_cycle to
-		 * be equal to a different value than gvt->db_dztrigger_cycle.
-		 */
-		csa->db_dztrigger_cycle++;
-	}
+	DISALLOW_MULTIINST_UPDATE_IN_TP(dollar_tlevel, jnlpool_head, csa, first_sgm_info, TRUE);
 	if (!*jnl_format_done && JNL_WRITE_LOGICAL_RECS(csa))
 	{	/* Attach to jnlpool if replication is turned on. Normally SET or KILL of the ^#t records take care of this
 		 * but in case this is a NO-OP trigger operation that wont happen and we still want to write a
 		 * TLGTRIG/ULGTRIG journal record. Hence the need to do this. Also write a LGTRIG record in just one region
 		 * in case this is a global spanning multiple regions.
 		 */
-		JNLPOOL_INIT_IF_NEEDED(csa, csa->hdr, csa->nl);
+		JNLPOOL_INIT_IF_NEEDED(csa, csa->hdr, csa->nl, SCNDDBNOUPD_CHECK_TRUE);
 		assert(dollar_tlevel);
 		T_BEGIN_SETORKILL_NONTP_OR_TP(ERR_TRIGLOADFAIL);	/* needed to set update_trans TRUE on this region
 									 * even if NO db updates happen to ^#t nodes. */
@@ -1780,7 +1783,7 @@ STATICFNDEF trig_stats_t trigupdrec_reg(char *trigvn, int trigvn_len, boolean_t
 								&tmp_index, &db_matched_set, &db_matched_kill,
 								&full_match, trigname[oprtype], trigname[oprtype])))
 					{	/* SET trigger found previously is not found again */
-						if (CDB_STAGNATE > t_tries)
+						if (UPDATE_CAN_RETRY(t_tries, t_fail_hist[t_tries]))
 							t_retry(cdb_sc_triggermod);
 						assert(WBTEST_HELPOUT_TRIGDEFBAD == gtm_white_box_test_case_number);
 						util_out_print_gtmio("Error : Previously found SET trigger " \
@@ -2011,7 +2014,21 @@ STATICFNDEF trig_stats_t trigupdrec_reg(char *trigvn, int trigvn_len, boolean_t
 	}
 	assert((STATS_UNCHANGED_TRIGFILE == trigload_status) || (STATS_NOERROR_TRIGFILE == trigload_status));
 	if ((0 == trig_stats[STATS_ERROR_TRIGFILE]) && (STATS_NOERROR_TRIGFILE == trigload_status))
+	{
 		trigger_incr_cycle(trigvn, trigvn_len);	/* ^#t records changed in this function, increment cycle */
+		csa->incr_db_trigger_cycle = TRUE; /* so that we increment csd->db_trigger_cycle at commit time */
+		if (dollar_ztrigger_invoked)
+		{	/* increment db_dztrigger_cycle so that next gvcst_put/gvcst_kill in this transaction, on this region,
+			 * will re-read triggers. Note that the below increment happens for every record added. So, even if a
+			 * single trigger file loaded multiple triggers on the same region, db_dztrigger_cycle will be incremented
+			 * more than one for same transaction. This is considered okay since we only need db_dztrigger_cycle to
+			 * be equal to a different value than gvt->db_dztrigger_cycle.
+			 */
+			csa->db_dztrigger_cycle++;
+			DBGTRIGR((stderr, "trigupdrec_reg: dollar_ztrigger_invoked CSA->db_dztrigger_cycle=%d\n",
+						csa->db_dztrigger_cycle));
+		}
+	}
 	RETURN_AND_POP_MVALS(trigload_status);
 }
 
