@@ -1,6 +1,6 @@
  /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information	*
+ * Copyright (c) 2001-2017 Fidelity National Information	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -61,8 +61,8 @@ GBLREF	io_pair			io_std_device;
 GBLREF	boolean_t		blocksig_initialized;
 GBLREF	sigset_t		block_sigsent;
 GBLREF	boolean_t		err_same_as_out;
-GBLREF	jnlpool_ctl_ptr_t	jnlpool_ctl;
-GBLREF	jnlpool_addrs		jnlpool;
+GBLREF	gd_addr			*gd_header;
+GBLREF	jnlpool_addrs_ptr_t	jnlpool;
 GBLREF	boolean_t		is_src_server;
 GBLREF	boolean_t		is_rcvr_server;
 GBLREF	boolean_t		is_updproc;
@@ -396,7 +396,14 @@ caddr_t util_format(caddr_t message, va_list fao, caddr_t buff, ssize_t size, in
 							isprintable = TRUE;
 						}
 					}
-					assert('\0' != ch);	/* we dont expect <null> bytes in the middle of the string */
+					assert(('\0' != ch) || (2 != faocnt) || !STRNCMP_LIT(message, "!/!_!AD"));
+					/* the assert above is to detect <NUL> bytes in GT.M messages, which cause trouble;
+					 * however, we have no control over the content of user-supplied input: that may contain
+					 * <NUL> bytes, which, as of this writing, we believe is confined, at least in our testing,
+					 * to reporting source syntax errors; therefore the assert tries to side-step any such
+					 * syntax errors by their signature, which is unique to the messages for SRCLINE and
+					 * EXTSRCLIN and. although DBG only, avoids a GBLDEF or gtm_threadglb_def
+					 */
 					assert((c + chlen) <= ctop);
 					assert(0 < chlen);
 					assert((0 < chwidth) || (0 == chwidth) && gtm_utf8_mode);
@@ -640,7 +647,7 @@ void	util_out_send_oper(char *addr, unsigned int len)
 	uint4			ustatus;
 	int4			status;
 	unsigned int		bufsize, file_name_len, *fn_len;
-	boolean_t		ret;
+	boolean_t		ret, inst_from_gld;
 	repl_inst_hdr		replhdr;
 	int			fd;
 	upd_helper_ctl_ptr_t	upd_helper_ctl;
@@ -699,16 +706,15 @@ void	util_out_send_oper(char *addr, unsigned int len)
 				assertpro(FALSE);
 		}
 		BUILD_FACILITY(img_type);
-		if (NULL != jnlpool_ctl)
+		if ((NULL != jnlpool) && (NULL != jnlpool->jnlpool_ctl) && (NULL != jnlpool->repl_inst_filehdr))
 		{	/* Read instace file name from jnlpool */
 			INSERT_MARKER;
-			BUILD_FACILITY((char *)jnlpool.repl_inst_filehdr->inst_info.this_instname);
+			BUILD_FACILITY((char *)jnlpool->repl_inst_filehdr->inst_info.this_instname);
 		} else
 		{	/* Read instance name from instance file */
 			fn_len = &file_name_len;
 			bufsize = MAX_FN_LEN + 1;
-			log_nam.addr = GTM_REPL_INSTANCE;
-			log_nam.len = SIZEOF(GTM_REPL_INSTANCE) - 1;
+			SETUP_INST_INFO(gd_header, log_nam, inst_from_gld);	/* set log_nam from GLD or environment variable */
 			trans_name.addr = temp_inst_fn;
 			ret = FALSE;
 			GET_INSTFILE_NAME(dont_sendmsg_on_log2long, return_on_error);
