@@ -1,6 +1,6 @@
 /****************************************************************
  *								*
- * Copyright (c) 2001-2016 Fidelity National Information 	*
+ * Copyright (c) 2001-2018 Fidelity National Information 	*
  * Services, Inc. and/or its subsidiaries. All rights reserved.	*
  *								*
  *	This source code contains the intellectual property	*
@@ -402,7 +402,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* CCECLSTPRCS */
 	   0,	/* ZSHOWBADFUNC */
 	   0,	/* NOTALLJNLEN */
-	   0,	/* UNUSEDMSG570 */
+	   0,	/* BADLOCKNEST */
 	   0,	/* NOLBRSRC */
 	   0,	/* INVZSTEP */
 	   0,	/* ZSTEPARG */
@@ -502,7 +502,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* CETOOLONG */
 	   0,	/* CENOINDIR */
 	   0,	/* COLLATIONUNDEF */
-	   0,	/* UNUSEDMSG670 */
+	   0,	/* UNUSEDMSG673 */
 	   0,	/* GTMSECSHRSRVF */
 	   0,	/* FREEZECTRL */
 	   0,	/* JNLFLUSH */
@@ -608,8 +608,8 @@ const static readonly int error_ansi[] = {
 	   0,	/* MEMORYRECURSIVE */
 	   0,	/* FREEZEID */
 	   0,	/* BLKWRITERR */
-	   0,	/* STOPTIMEOUT */
-	   0,	/* UNUSEDMSG777 */
+	   0,	/* UNUSEDMSG779 */
+	   0,	/* PINENTRYERR */
 	   0,	/* BCKUPBUFLUSH */
 	   0,	/* NOFORKCORE */
 	   0,	/* JNLREAD */
@@ -1148,7 +1148,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* CRYPTDLNOOPEN */
 	   0,	/* CRYPTNOV4 */
 	   0,	/* CRYPTNOMM */
-	   0,	/* UNUSEDMSG1316 */
+	   0,	/* READONLYNOBG */
 	   0,	/* CRYPTKEYFETCHFAILED */
 	   0,	/* CRYPTKEYFETCHFAILEDNF */
 	   0,	/* CRYPTHASHGENFAILED */
@@ -1206,7 +1206,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* MUUSERLBK */
 	   0,	/* SETINSETTRIGONLY */
 	   0,	/* DZTRIGINTRIG */
-	   0,	/* UNUSEDMSG1374 */
+	   0,	/* LSINSERTED */
 	   0,	/* BOOLSIDEFFECT */
 	   0,	/* DBBADUPGRDSTATE */
 	   0,	/* WRITEWAITPID */
@@ -1218,7 +1218,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* JNLORDBFLU */
 	   0,	/* ZCCLNUPRTNMISNG */
 	   0,	/* ZCINVALIDKEYWORD */
-	   0,	/* UNUSEDMSG1386 */
+	   0,	/* REPLMULTINSTUPDATE */
 	   0,	/* DBSHMNAMEDIFF */
 	   0,	/* SHMREMOVED */
 	   0,	/* DEVICEWRITEONLY */
@@ -1359,7 +1359,7 @@ const static readonly int error_ansi[] = {
 	   0,	/* GETNAMEINFO */
 	   0,	/* SOCKBIND */
 	   0,	/* INSTFRZDEFER */
-	   0,	/* REGOPENRETRY */
+	   0,	/* UNUSEDMSG1532 */
 	   0,	/* REGOPENFAIL */
 	   0,	/* REPLINSTNOSHM */
 	   0,	/* DEVPARMTOOSMALL */
@@ -1369,8 +1369,8 @@ const static readonly int error_ansi[] = {
 	   0,	/* GBLNOMAPTOREG */
 	   0,	/* ISSPANGBL */
 	   0,	/* TPNOSUPPORT */
-	   0,	/* GVSUBSERR */
-	   0,	/* UNUSEDMSG1540 */
+	   0,	/* UNUSEDMSG1542 */
+	   0,	/* ZATRANSERR */
 	   0,	/* FILTERTIMEDOUT */
 	   0,	/* TLSDLLNOOPEN */
 	   0,	/* TLSINIT */
@@ -1451,4 +1451,55 @@ const static readonly int error_ansi[] = {
 	   0,	/* ENCRYPTCONFLT */
 	   0,	/* JNLPOOLRECOVERY */
 	   0,	/* LOCKTIMINGINTP */
+	   0,	/* PBNUNSUPTYPE */
+	   0,	/* DBFHEADLRU */
+	   0,	/* ASYNCIONOV4 */
+	   0,	/* AIOCANCELTIMEOUT */
+	   0,	/* DBGLDMISMATCH */
+	   0,	/* DBBLKSIZEALIGN */
+	   0,	/* ASYNCIONOMM */
+	   0,	/* RESYNCSEQLOW */
+	   0,	/* DBNULCOL */
+	   0,	/* UTF16ENDIAN */
+	   0,	/* OFRZACTIVE */
+	   0,	/* OFRZAUTOREL */
+	   0,	/* OFRZCRITREL */
+	   0,	/* OFRZCRITSTUCK */
+	   0,	/* OFRZNOTHELD */
+	   0,	/* AIOBUFSTUCK */
+	   0,	/* DBDUPNULCOL */
+	   0,	/* CHANGELOGINTERVAL */
+	   0,	/* DBNONUMSUBS */
+	   0,	/* AUTODBCREFAIL */
+	   0,	/* RNDWNSTATSDBFAIL */
+	   0,	/* STATSDBNOTSUPP */
+	   0,	/* TPNOSTATSHARE */
+	   0,	/* FNTRANSERROR */
+	   0,	/* NOCRENETFILE */
+	   0,	/* DSKSPCCHK */
+	   0,	/* NOCREMMBIJ */
+	   0,	/* FILECREERR */
+	   0,	/* RAWDEVUNSUP */
+	   0,	/* DBFILECREATED */
+	   0,	/* PCTYRESERVED */
+	   0,	/* REGFILENOTFOUND */
+	   0,	/* DRVLONGJMP */
+	   0,	/* INVSTATSDB */
+	   0,	/* STATSDBERR */
+	   0,	/* STATSDBINUSE */
+	   0,	/* STATSDBFNERR */
+	   0,	/* JNLSWITCHRETRY */
+	   0,	/* JNLSWITCHFAIL */
+	   0,	/* CLISTRTOOLONG */
+	   0,	/* LVMONBADVAL */
+	   0,	/* RESTRICTEDOP */
+	   0,	/* RESTRICTSYNTAX */
+	   0,	/* MUCREFILERR */
+	   0,	/* JNLBUFFPHS2SALVAGE */
+	   0,	/* JNLPOOLPHS2SALVAGE */
+	   0,	/* MURNDWNARGLESS */
+	   0,	/* DBFREEZEON */
+	   0,	/* DBFREEZEOFF */
+	   0,	/* STPCRIT */
+	   0,	/* STPOFLOW */
 	};
